#ifndef BC_ARRAY_HPP
#define BC_ARRAY_HPP
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Project:                 VB nach C/C++ Konverter Klassenbibliothek
// Modul:                   VB Klasse fuer VB Arrays
// Filename:                bc_array.cpp
// Version:                 1.0
// Status:                  beta
//
// Copyright:               Copyright (C) 1995
//                          Cicerosoft GmbH, 
//                          Ballweilerstr. 7,
//                          66399 Erfweiler-Ehlingen
//
// Operating System(s):     Win95
// Compiler:                VC++ 4.1
//
// Author:                  Jochen Diener JD
// Creation Date:           JD 14.03.95
// Last Modification:       JD 15.07.96
//
// Description:             Beinhaltet alle Klassen / Hilfsfkt. die im Zusammenhang mit der VB 
//                          Konvertierung, fuer die Bereitstellung von Arrays benoetigt 
//                          werden. 
//                          
// Dependencies:            bc_array.cpp    ->  Klassendef.
//                          bi_array.hpp    ->  Klassendekl.
//
// Remarks:                 Die Beschreibung zu den Klassenmethoden werden so kurz als moeglich
//                          gehalten um die Uebersichtlichkeit zu erhoehen, Eine genauere Beschreibung
//                          laesst sich ggf. in den VB Handbuechern finden
//
// Revision(s):   
//                          
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

//================================= Defintionsabhaengigkeiten ===============================================
//
//-- C-Header
#include <stdio.h>  // fuer ellipse
#include <stdarg.h> // fuer ellipse

//-- Lib-Header
#ifndef BC_BOOL_HPP
  #include "bc_bool.hpp"
#endif

#ifndef BC_BYTE_HPP
  #include "bc_byte.hpp"
#endif

#ifndef BC_LONG_HPP
  #include "bc_long.hpp"
#endif


template <class T, class ArrayContract> class B_Bounds;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1996
//     B_ArrayChecked     Behaviour-Klasse fuer die Body-Klasse B_Array mit Fehlerueberpruefung
// Hierachy:    
// Inheritance: 
// Errorhandling:
// Note: 
//     B_OutOfMem wird immer ausgeworfen und ist nicht mit Behaviour-Klassen steuerbar
//     Verwendung von Basic-Klassen B_Long fuer Index da wenn groesser Overflow was dann Typ schon macht
//     ansonsten immer LZF(9) ausgeworfen wird
class DLL_DECL B_ArrayChecked
  {
  //==================== INTERFACE ============================================================================
  //
  public: // normal protected, wegen template overhead public
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     dimRange  Check ob die Dimensionsangabe im aktuellen Array gueltig ist
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    static void dimRange(B_Shrt aktDim, B_Shrt nbrDims); 

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     bound Range  Check ob die Boundangaben im guletigen Wertebereich liegt
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    static void boundRange(B_Lng lower, B_Lng upper);
   
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     idxRange  Check ob die Indexangabe einer Dimension im guletigen Wertebereich liegt
    //               der durch die deklaration bestimmt wurde
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    static void idxRange(B_Lng idx, B_Lng lower, B_Lng upper); 
    
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     elementRange  Check ob die angegebene Elementposition einen gueltigen Wert im internen 
    //                   Verwaltungsarray hat
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    static void elementRange(B_ULng aktPos, B_ULng maxPos); // ist pos eine gueltige Position im eindimensionalen elementearray pos <= nbrElements && pos >= 0

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     validAssignment  derzeit sind keine Spalten oder Zeilenweise Zuweisungen erlaubt sondern nur
    //                      Elementweise Zuweisungen zwischen zwei Arrays
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    static void B_ArrayChecked::validAssignment(B_ULng aktDimArr1, B_ULng maxDimArr1, B_ULng aktDimArr2, B_ULng maxDimArr2); 

  // friend class B_Array<T, B_ArrayChecked>; // wenn protected
  }; // B_ArrayChecked


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1996
//     B_ArrayUnchecked     Behaviour-Klasse fuer die Body-Klasse B_Array ohne jegliche Fehlerueberpruefung
// Hierachy:    
// Inheritance: 
// Errorhandling:
// Note: 
//     s. B_ArrayChecked
class DLL_DECL B_ArrayUnchecked
  {
  public: // normal protected, wegen template overhead public
    static void dimRange(B_Shrt, B_Shrt); // ob kleiner nbrDims und groesser als 0

    static void boundRange(B_Lng, B_Lng);  // Ob lower < upper
   
    static void idxRange(B_Lng, B_Lng, B_Lng); // ob index im gueltigen Bereich liegt
    
    static void elementRange(B_ULng, B_ULng); // ist pos eine gueltige Position im eindimensionalen elementearray pos <= nbrElements && pos >= 0

    static void B_ArrayUnchecked::validAssignment(B_ULng aktDimArr1, B_ULng maxDimArr1, B_ULng aktDimArr2, B_ULng maxDimArr2); 

  // friend class B_Array<T, B_ArrayUnchecked>; // wenn protected
  }; // B_ArrayUnchecked

//==================== PROXY-CLASSES ====================================================================== 
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////
// Interne Verwaltungsstruktur fuer Dimensionsindexe
#if 1	//ab.. 25.3.97 Klasse, damit Initialisierung per Konstruktor moeglich wird
class BoundPair
  {
   public:
	BoundPair() {lower = -1; upper = -1; range = -1;}
	B_Lng lower;
	B_Lng upper;
	B_Lng range;
  };
#else
struct BoundPair
  {
  B_Lng lower;
  B_Lng upper;
  B_Lng range;
  };
#endif

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1996
//     B_Array     Emuliert VB-Arrays
// Hierachy:    
// Inheritance: 
// Errorhandling:
// Note: 
//   Achtung in C kann bei einem z.B. Dreidimensionalen B_Array a[2][3][4] auch der Zugriff a[1][2] sinnvoll
//   und erlaubt sein. In VB ist nur der Zugriff unter Angabe aller Dimensionen erlaubt, somit auch bei 
//   unserer B_Arrayklasse.
//   In VB kann keine Variable eines anderen Typs als dem Arraydatentyp zugewiesen werden. Fuehrt in VB zu 
//   LZF(13) bei uns zu Compilerfehler.
//   Operatoren +, += machen keinen Sinn da die Arrays weder reine Zahlarrays sind noch sind die Dimensionen
//   auf eine fixe anzahl festgelegt.
//   Die Dimensionsanzahl ist durch den Wertebereich von unsigend char (255), um Speicherplatz beim Objekt ein-
//   zusparen. Dieser Wert kann geaendert werden da Array dann nur noch vom verfuegbaren Speicher abhaengt
//   gehenden Hauptspeichers begrenzt. 
//   Keine Auswertung von 'option base' da dies konverter macht und spaeter bei Aenderungen im C++ Code die
//   Upper/Lower-B_Boundss explizit angegeben werden muessen
//   In anderen Klassenbibliotheken werden Konstruktoren zur Verfgung gestellt die die explizite Angabe der
//   Arraycapazitaet (!= anzahl Elemente) erlauben. Da diese Arrayklasse speziell fuer VB optimiert wurde
//   koennen diese Konstruktorvariationen entfallen.

/*
* ANMERKUNG CT230197 statische und dynamische Array's
*
* aus Q-Basic 7.0 Hilfe:
*
*     How array is declared                         Allocation
*     ------------------------------------------    ----------
*     _ Declared first in a COMMON statement        Dynamic
*     _ Implicitly dimensioned arrays               Static
*     _ Dimensioned with numeric constants          Static
*       or CONST statement constants
*     _ Dimensioned with variables as subscripts    Dynamic
*
* _ Examples of different DIM statements and results:
*
*     Statement        Result
*     -------------    --------------------------------------------------
*     DIM A(0 TO 9)    Array A is allocated as a static array if
*                      $DYNAMIC is not in effect.
*     DIM A(MAXDIM)    If MAXDIM is defined in a CONST statement, array A
*                      is a static array. If MAXDIM is a variable, then
*                      the array is a dynamic array and is only allocated
*                      when the program reaches the DIM statement.
*
* Problem COMMON Array's:
*
*_ Both static and dynamic arrays are placed in COMMON by using the
*  array name followed by parentheses. The dimensions for a static
*  array must be set with integer-constant subscripts in a DIM statement
*  preceding the COMMON statement. The dimensions for a dynamic array
*  must be set in a later DIM or REDIM statement. The elements of a
*  dynamic array are not allocated in the COMMON block. Only an array
*  descriptor is placed in common.
*
*/

struct B_PreInitArr {};
DLL_DECL extern struct B_PreInitArr B_preInit;

template <class T, class ArrayContract>
class B_Array
  {
  //==================== MANGAEMENT =========================================================================
  //
  public:
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     ctor		CT161296	Fuer dynamische Array's
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Array();

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     ctor
    // Parameter:  
    //     beide auf B_true -> keine Elementinitialisierung
    // Return Value:
    // Errorhandling:
    // Note:      
    //     wenn explicit, dann 2. Parameter weg
    B_Array(const B_Boolean&, const B_Boolean&); 

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     ctor		CT161296	Fuer statische Array's
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Array(const B_Shrt dims, ...) ;

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     ctor fuer OLE
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Array(const VARIANT FAR&);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     ctor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    // B_Array(const T*, dims); // zum initialiseren mit {} a la C
    B_Array(T, const struct B_PreInitArr&, const B_Shrt dims, ...);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     cctor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Array(const B_Array& rhs);
    
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     In VB koennen Arrays nicht einander zugewiesen werden. Ein Laufzeitfehler der IDE (can't assigen to 
    //     a non Byte Array) ist die Folge. Auch bei Byte-Arrays.
    //     Bei uns erhaelt das Array die Dimensions- und Boundangaben des zugewiesenen Arrays.
    B_Array& operator=(const B_Array&);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    ~B_Array();

  //==================== INTERFACE ============================================================================
  //
  public:
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     operator= fuer OLE-Variant
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Array& operator=(const VARIANT FAR&);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Kopiert den Arrayinhalt des Objektes in einen Variantparameter
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Derzeit muessen beide Arrays die gleiche Groesse und Boundwerte haben. Eine Redimensionierung
    //     waere nur in der letzten Dimension moeglich da OLE keine andere moeglichkeit derzeit bietet.
    copyToOle(const VARIANT FAR&) const;

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     redim   Redimensionierung des Arrays
    // Parameter:  
    //     der erste Parameter dims, bestimmt die Anzahl der Dimensionen. Fuer jede Dimension muss
    //     dann in der Ellipse die zugehoerigen Lower- und UpperBounds angegeben werden
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Enspricht der VB-Prozedur Redim()
    void redim(const B_UChar dims, ...); 

	//---------------------------------------------------------------------------------------------------------       
	// NEU: CT171296	Implementiert fuer streamr() + dynamic Array
    // Method:
    //     redim   Redimensionierung des Arrays
    // Parameter:  
    //     der erste Parameter dims, bestimmt die Anzahl der Dimensionen. Fuer jede Dimension muss
    //     dann die zugehoerigen Lower- und UpperBounds und Range angegeben werden
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Enspricht der VB-Prozedur Redim()
    void redim(const B_Shrt dims, BoundPair * bPair); 

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     pRedim Preserver Redim, hier wird das Array redimensioniert ohne die Elemente zu initialiseren
    // Parameter:  
    //     Da in VB nur die letzte Dimension redimensioniert werden darf bei der angabe von Preserve,
    //     braucht auch nur der neue Wert fuer den UpperBound der letzten Dimension angegeben werden und nicht
    //     wie in VB auch noch alle anderen Upper/Lowerbounds aller Dimensionen die eh nicht veraendert werden
    //     duerfen.
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Entspricht der VB Prozedur Preserve Redim() 
    void pRedim(const B_Lng uBoundLastDim);

	//---------------------------------------------------------------------------------------------------------       
    // Method:
    //     pRedim Preserver Redim, hier wird das Array redimensioniert ohne die Elemente zu initialiseren
    // Parameter:  
    //     Da in VB nur die letzte Dimension redimensioniert werden darf bei der angabe von Preserve,
    //     braucht auch nur der neue Wert fuer den UpperBound der letzten Dimension angegeben werden und nicht
    //     wie in VB auch noch alle anderen Upper/Lowerbounds aller Dimensionen die eh nicht veraendert werden
    //     duerfen.
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Es wird nur das UBound des letzten uebergebenen Parameters der Ellipse ausgewertet
    void pRedim(const B_UChar dims, ...); 

	//---------------------------------------------------------------------------------------------------------       
    // Method:
    //     dimStatic	Fall: DIM for COMMON in Includdatei
    // Parameter:  
    //     der erste Parameter dims, bestimmt die Anzahl der Dimensionen. Fuer jede Dimension muss
    //     dann in der Ellipse die zugehoerigen Lower- und UpperBounds angegeben werden
    // Return Value:
    // Errorhandling:
    // Note:      
    void dimStatic(const B_UChar dims, ...); 

	//---------------------------------------------------------------------------------------------------------       
    // Method:
    //     dimDynamic	Fall: COMMON ohne DIM in Icludedatei aber DIM im *.BAS - Code
    // Parameter:  
	//     der erste Parameter dims, bestimmt die Anzahl der Dimensionen. Fuer jede Dimension muss
    //     dann in der Ellipse die zugehoerigen Lower- und UpperBounds angegeben werden
    // Return Value:
    // Errorhandling:
    // Note:     
    void dimDynamic(const B_UChar dims, ...); 

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     uBound  bestimmt den UpperBound (oberen Grenzwert des Dimensionindexes) einer Dimension dim
    // Parameter:  
    //     dim gibt die Dimension an von der der UpperBound bestimmt werden soll
    // Return Value:
    //     Wert des spezifizierten Upperbounds
    // Errorhandling:
    // Note:      
    //    Die zaehlung der Dimensionen beginnt bei 1. Entspricht VB-Prozedur uBound()
#ifdef B_VB_COMPATIBLE
    B_Long uBound(B_Integer dim) const;
    B_Long uBound(B_Int dim) const;
#else
    B_Lng uBound(B_Int dim) const;
#endif

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     lBound bestimmt den LowerBound (unteren Grenzwert des Dimensionindexes) einer Dimension dim
    // Parameter:  
    //     dim gibt die Dimension an von der der LowerBound bestimmt werden soll
    // Return Value:
    //     Wert des spezifizierten Lowerbounds
    // Errorhandling:
    // Note:      
    //    Die zaehlung der Dimensionen beginnt bei 1. Entpricht VB-Prozedur lBound()
#ifdef B_VB_COMPATIBLE
    B_Long lBound(B_Integer dim) const;
    B_Long lBound(B_Int dim) const;
#else
    B_Lng lBound(B_Int dim) const;
#endif

	//---------------------------------------------------------------------------------------------------------       
    // Method:	CT161296	Neue Funktion ERASE
    //			ERASE
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:	Wichtige Funktion die bisher nicht implementiert wurde. Hiermit steht die
	//			neue Variable B_Array::staticFlag in direktem Zusammenhang. Array's
	//			die in VB statisch angelegt sind "DIM ArrVar(1 TO 100) AS INTEGER"
	//			werden in 2 Faellen anderst verwaltet als dynamische Array's
    //		
	void erase();

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //    getValue()  Gibt einen Zeiger auf das erste Element des Elementearrays zureuck.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //    Diese Low-Level Funktion ist mit vorsicht zu geniessen. Bei jeder Redimensionierung z.B. wird der
    //    erhaltenen Pointer ungueltig. Bei anderen MEthodenaufrufen kann ein Elementpointer auf eine falsche
    //    Stelle im reorganisierten Array zeigen.
    T const* getValue() const;

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     elements() Gibt die Anzahl der derzeit im Array enthaltenen Elemente zurueck
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_ULng elements() const;

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     dimensions() Gibt die Anzahl der derzeit im Array enthaltenen Dimensionen zurueck
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Shrt dimensions() const;

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     initalize() Initialisiert das Array mit einem vorgebenen Wert
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Array& initialize(T&);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     getValueSize()  Gibt die Groesse der Elemente des Objektes als entsprechenden C/C++ Datentyp zurueck
    //                     ohne Verwaltungsinformationen
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Lng getValueSize() const;  
    
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     getSize()   Gibt die Groesse des Objektes zurueck, diese entspricht der durch das Objekt belegten Speicherplatz
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Lng getSize() const;       

    //----------------------------------------------------------------------------------------------------       
    // Method: 
    //     Schreiben der der eigentlichen Objektdaten im binaeren Format in einen Stream
    // Return Value: 
    //     B_true  ==  Objekt und Stream sind gueltig
    //     B_false ==  Objekt oder Stream sind ungueltig
    // Note: 
    //
    B_Boolean streamw(ostream& ostr, B_Shrt openmode);

    //----------------------------------------------------------------------------------------------------       
    // Method: 
    //     Lesen der der eigentlichen Objektdaten im binaeren Format in einen Stream
    // Return Value: 
    //     B_true  ==  Objekt und Stream sind gueltig
    //     B_false ==  Objekt oder Stream sind ungueltig
    // Note: 
    //
    B_Boolean streamr(istream& istr, B_Shrt openmode);


  //==================== HELPER ===============================================================================    
  //
  public:
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Subskriptionsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Nur fuer die Klassenimpl. benoetigt, keine Anwenderfkt.
#ifdef B_VB_COMPATIBLE    
    B_Bounds<T, ArrayContract> operator[](const B_Long& i); // koennen auch negativ sein
#endif
    B_Bounds<T, ArrayContract> operator[](const B_Lng i); // koennen auch negativ sein

	//---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Subskriptionsop. als Funktion -> Zugriff auf Members eines TYP in einem Array
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:	(Anzahl Dimensionen, Index/Dimension, ...)
	//			Implementierung nur fuer B_Lng da Ellypsen mit Klassen nicht moeglich sind.
    //     
	T& gA(const B_Lng dims, ...);

  private:
    /*
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     getElemente()
    // Parameter:  
    //     Interne Arrayposition
    // Return Value:
    //     Element das sich an der interene Arrayposition pos befindet
    // Errorhandling:
    // Note:      
    //     
    T getElement(B_ULng pos); 

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     setElement()
    // Parameter:  
    //     Element das an aktueller interner Arraposition abgespeichert werden soll
    // Return Value:
    // Errorhandling:
    // Note:      
    //  
    void setElement(const T& rhs);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     aktuelle Dimension
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //  
    B_Shrt getAktDim();
    void   setAktDim(B_Shrt);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     anzahl Dimensionen des Arrays 
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //  
    B_Shrt getNbrDims();
    void   setNbrDims(B_Shrt);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     aktuelle poisition im internen array
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //  
    B_ULng getElementPos();
    void   setElementPos();
    */

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     initArray()
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     initialisierung des Array mit Defaultwerten
    void initArray();

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     initEl()
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     initialisierung eines Elementes eines Arrays mit Defaultwerten
    void initEl(T&);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     convertArray
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Ein OLE-VARINAT Array in ein B_Array konvertieren
    void convertArray(B_Int varType, const VARIANT FAR& oleArr);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     convertArray
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Ein B_Array in ein gleich Grosses OLE-VARINAT Array kopieren
    void toOle(B_Int varType, const VARIANT FAR& oleArr);

	//----------------------------------------------------------------------------------------------------       
    // Method: 
	//	   Helperfunktion fuer Klassen und finite Typen
    //     Schreiben der der eigentlichen Objektdaten im binaeren Format in einen Stream
    // Return Value: 
    //     B_true  ==  Objekt und Stream sind gueltig
    //     B_false ==  Objekt oder Stream sind ungueltig
    // Note: 
    //
    B_Boolean classStreamw(ostream& ostr, B_Shrt openmode);
	B_Boolean typeStreamw(ostream& ostr, B_Shrt openmode);

    //----------------------------------------------------------------------------------------------------       
    // Method: 
	//	   Helperfunktion fuer Klassen und finite Typen
    //     Lesen der der eigentlichen Objektdaten im binaeren Format in einen Stream
    // Return Value: 
    //     B_true  ==  Objekt und Stream sind gueltig
    //     B_false ==  Objekt oder Stream sind ungueltig
    // Note: 
    //
    B_Boolean classStreamr(istream& istr, B_Shrt openmode);
	B_Boolean typeStreamr(istream& istr, B_Shrt openmode);

  //==================== MEMBER ===============================================================================    
  //
  private:
    B_ULng nbrElements;  // Anzahl der aktuell enthaltenen Elemente
    B_ULng capacity;	 // Speicherkapazitaet des Arrays die Speichermanagement handelt fuer effiziente Arrayverw.
    T* pElements;        // Speicherbereich der Arrayelemente enthaelt
    B_BOOLEAN initFlag;	 // CT161296 wenn == B_TRUE dann wird der Konstruktor jedes Members aufgerufen
    //-- Boundinfos
    B_Bounds<T, ArrayContract> bounds;  // Verwaltungsopbjekt der Bounderys
    BoundPair* pBoundValues;  
    B_Shrt nbrDims;      //  Anzahl Dimensionen des B_Arrays von 1 bis n; 
	// CT161296	neu
	B_BOOLEAN staticFlag;// wenn == B_TRUE dann ist es ein Static-Array sh. ERASE, REDIM

  //==================== FRIENDS ============================================================================
  //
  friend class B_Bounds<T, ArrayContract>;

  }; // B_Array


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1996
//     B_Bounds    Indexverwaltung der Arraydimensionen 
// Hierachy:    
// Inheritance: 
// Errorhandling:
// Note: 
//     Nur fuer arrayinterne Implementierung von Interesse
template <class T, class ArrayContract>
class B_Bounds
  {
  //================ MANAGEMENT =========================================================================
  //
  public:
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     ctor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Bounds();

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     ctor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Bounds(const B_UChar dims);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     cctor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Bounds(const B_Bounds& b);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    ~B_Bounds();

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Bounds& operator=(const B_Bounds&);

  //==================== INTERFACE ============================================================================
  //
  public:
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Konvertierungop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    operator T();

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Subskriptionsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
#ifdef B_VB_COMPATIBLE    
    B_Bounds<T, ArrayContract> operator[](const B_Long& idx); // koennen auch negativ sein in VB30 int in VB40 long
#endif
    B_Bounds<T, ArrayContract> operator[](const B_Lng idx); // koennen auch negativ sein in VB30 int in VB40 long

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Zuweisungsop. fuer Arrayelemtndatentyp
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Bounds<T, ArrayContract>& operator=(const T& d);
    B_Bounds<T, ArrayContract>& operator+=(const T& d);

  //==================== MEMBER ===============================================================================    
  //
  public: // private:
    B_Array<T, ArrayContract> *pArr;                    //  Zugriff auf Arrayelemente 
    BoundPair* pBoundValues; //  Boundverwaltung
    B_Shrt aktDim;           //  Aktuelle Dimension beim iterieren durch Subskriptionsop. Aufrufe
    B_ULng elementPos;       //  Umrechnung der Dimensionsangeabe auf interne Elementposition

  //==================== FRIENDS ============================================================================
  //
  friend class B_Array<T, ArrayContract>;
  }; // B_Bounds


/*
template <class T>
B_Boolean operator==(const B_Array<T>&, const B_Array<T>&);

template <class T>
B_Boolean operator!=(const B_Array<T>&, const B_Array<T>&);
*/

//================================ Inlinefunktionen =========================================================
//
#undef B_NOINLINE   // Weil Templates zur Instanzierung immer eingebunden werden muessen, auch wenn nicht inline

#ifndef B_NOINLINE   // festlegen wenn Debuger keine Inlines debuggen kann
  #define B_INLINE inline
  #ifndef  BI_ARRAY_HPP
    #include "bi_array.hpp"
  #endif
#endif

//===========================================================================================================
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif // BC_ARRAY_HPP