#ifndef BC_BOOL_HPP
#define BC_BOOL_HPP
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Project:                 VB nach C/C++ Konverter Klassenbibliothek
// Modul:                   VB Klasse fuer den Datentyp Boolean
// Filename:                bc_bool.hpp
// Version:                 0.0
// Status:                  beta
//
// Copyright:               (C) 1995
//                          Cicerosoft GmbH
//                          Ballweilerstr. 7
//                          66399 Erfweiler-Ehlingen
//
// Operating System(s):     Win95
// Compiler:                VC++ 4.1
//
// Author:                  Jochen Diener JD
// Creation Date:           JD 20.11.95
// Last Modification:       JD 23.05.96
//
// Description:             Beinhaltet alle Klassen / Hilfskt. die im Zusammenhang mit der VB
//                          Konvertierung, fuer die Bereitstellung des Datentyps Boolean benoetigt
//                          werden.
//
// Dependencies:            bc_bool.cpp   -> Klasse B_Boolean
//                          bi_bool.hpp   -> Inline Datei der Klasse B_Boolean
//
// Remarks:
// CT111296	wird bei VB intern durch einen Integer repraesentiert
// --> value von B_Char auf B_Shrt wegen FileIO Get/Put
//
// Revision(s):
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

//================================= Defintionsabhaengigkeiten ===============================================
//
#include <iostream.h>
#include <stddef.h>               // wegen size_t

#ifndef BD_INTYP_HPP
  #include "bd_intyp.hpp"		  // alias der Standardatentypen
#endif

//================================= Export =================================================================
//

class B_Empty;
class B_Null;
class B_String;
class B_fstream;						// CT131296 Fuer FileIO abgeleitete fstream-Klasse

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_Boolean  Bool-Klasse
// Hierachy:    
// Inheritance: 
// Errorhandling:
// Note: 
class DLL_DECL B_Boolean
  {
  //================ MANAGEMENT =========================================================================
  //
  public:
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     ctor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Initialisieren einer definierten Variablen
    B_Boolean();

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     ctor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     fuer Zuweisungskompatibilitaet mit C-Datentpyen zu erreichen
    //     fuer Strings(literale) sind nur 'True' 'False' erlaubt sonst LZF(13)
    B_Boolean(B_cCStr,     const B_Explicit&);      
    B_Boolean(const B_Int, const B_Explicit&); 
    B_Boolean(const B_UInt,const B_Explicit&);
    B_Boolean(const B_Lng, const B_Explicit&);
    B_Boolean(const B_Flt, const B_Explicit&);
    B_Boolean(const B_Dbl, const B_Explicit&);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     nicht erforderlich
    ~B_Boolean();

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     cctor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     nicht erforderlich
    B_Boolean(const B_Boolean&); 

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     operator=()
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     nicht erforderlich
    B_Boolean& operator=(const B_Boolean&);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     operator=()
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     erforderlich da in VB auch Strings uebergeben werden koennen
    //     Alle Zahldatentpyen werden zu True wenn sie ungleich 0 sind
    //     Zahlstrings sind erlaubt und Strings die "False" oder "True" enthalten. Alle anderen Strings
    //     liefern LZF(13) Type mismatch (auch "Wahr", "Falsch")
    B_Boolean& operator=(B_cCStr); // throw(B_TypeMismatch         
    B_Boolean& operator=(const B_Int);
    B_Boolean& operator=(const B_Lng);
    B_Boolean& operator=(const B_Flt);
    B_Boolean& operator=(const B_Dbl);

    B_Boolean& operator=(const B_Null&); // throw(B_InvalidNull)
    B_Boolean& operator=(const B_Empty&);  
    B_Boolean& operator=(const B_String&); 

    //==================== INTERFACE ============================================================================
    //  

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     operator int()
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Implizite Konvertierung von B_Boolean nach int
    //     Fuer VB direkt benoetigt.
    //     Um direkt in log. Ausdruecken verwenden zu koennen. Reicht aus alles andere ueber
    //     integrale Promotion / Standardkonvertierung
    operator B_Int() const;

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     operator int()
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     noetig da VB -1 zurueckgibt fuer True anstatt 1 -> jemand koennte auf -1 abfragen
    B_Boolean operator !() const;


    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     operator int()
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     noetig da VB -1 zurueckgibt fuer True anstatt 1 -> jemand koennte auf -1 abfragen
    B_Int getValue() const;

    //----------------------------------------------------------------------------------------------------       
    // Method: 
    //     Schreiben der der eigentlichen Objektdaten im binaeren Format in einen Stream
    // Return Value: 
    //     B_true  ==  Objekt und Stream sind gueltig
    //     B_false ==  Objekt oder Stream sind ungueltig
    // Note: 
    //
    B_Boolean streamw(ostream& ostr, B_Shrt openmode) const;

    //----------------------------------------------------------------------------------------------------       
    // Method: 
    //     Lesen der der eigentlichen Objektdaten im binaeren Format in einen Stream
    // Return Value: 
    //     B_true  ==  Objekt und Stream sind gueltig
    //     B_false ==  Objekt oder Stream sind ungueltig
    // Note: 
    //
    B_Boolean streamr(istream& istr, B_Shrt openmode);

  //==================== HELPER ===============================================================================
  //

  //==================== TYPES & CONSTANTS ==================================================================
  //
  protected:

  //==================== MEMBER =============================================================================
  //
  private:
      //B_Char value;	// CT111296	wird bei VB intern durch einen Integer repraesentiert
	  B_Shrt value;

  //==================== FRIENDS ============================================================================
  //
  friend class B_Byte;
  friend class B_Integer;
  friend class B_Long;
  friend class B_Single;
  friend class B_Double;
  friend class B_Currency;

  DLL_DECL friend B_Boolean operator&& (const B_Boolean&, const B_Boolean&);  
  DLL_DECL friend B_Boolean operator|| (const B_Boolean&, const B_Boolean&);
  DLL_DECL friend B_Boolean operator^  (const B_Boolean&, const B_Boolean&);
  DLL_DECL friend B_Boolean operator== (const B_Boolean&, const B_Boolean&);
  DLL_DECL friend B_Boolean operator!= (const B_Boolean&, const B_Boolean&);
  DLL_DECL friend B_Boolean operator<  (const B_Boolean&, const B_Boolean&);  
  DLL_DECL friend B_Boolean operator<= (const B_Boolean&, const B_Boolean&);
  DLL_DECL friend B_Boolean operator>= (const B_Boolean&, const B_Boolean&);
  DLL_DECL friend B_Boolean operator>  (const B_Boolean&, const B_Boolean&);
  DLL_DECL friend B_Boolean operator&& (const B_Boolean&, const B_Boolean&);
  DLL_DECL friend B_Boolean operator|| (const B_Boolean&, const B_Boolean&);
  DLL_DECL friend B_Boolean operator^  (const B_Boolean&, const B_Boolean&);

#ifdef THALESPLUS	//ab.. 27.1.97
  DLL_DECL friend B_Boolean operator&& (const B_Boolean&, const B_Int);  
  DLL_DECL friend B_Boolean operator&& (const B_Int, const B_Boolean&);
  DLL_DECL friend B_Boolean operator&& (const B_Boolean&, const B_Lng);  
  DLL_DECL friend B_Boolean operator&& (const B_Lng, const B_Boolean&);
  DLL_DECL friend B_Boolean operator&& (const B_Boolean&, const B_Flt);  
  DLL_DECL friend B_Boolean operator&& (const B_Flt, const B_Boolean&);
  DLL_DECL friend B_Boolean operator&& (const B_Boolean&, const bool);  
  DLL_DECL friend B_Boolean operator&& (const bool, const B_Boolean&);
  DLL_DECL friend B_Boolean operator|| (const B_Boolean&, const B_Int);  
  DLL_DECL friend B_Boolean operator|| (const B_Int, const B_Boolean&);
  DLL_DECL friend B_Boolean operator|| (const B_Boolean&, const B_Lng);  
  DLL_DECL friend B_Boolean operator|| (const B_Lng, const B_Boolean&);
  DLL_DECL friend B_Boolean operator|| (const B_Boolean&, const B_Flt);  
  DLL_DECL friend B_Boolean operator|| (const B_Flt, const B_Boolean&);
  DLL_DECL friend B_Boolean operator|| (const B_Boolean&, const bool);  
  DLL_DECL friend B_Boolean operator|| (const bool, const B_Boolean&);
#endif
  
  //----------------------------------------------------------------------------------------------------       
  // Method: operator<<	INSERTION
  //     Schreiben der eigentlichen Objektdaten im Textformat in einen Stream
  // Return Value: 
  //	   Instanz der abgeleiten fstream-Klasse der Datei
  // Note: 
  //	   CT131296
  //	   ersetzt die alte Methode streamp()
  //
  DLL_DECL friend B_fstream& operator<< (B_fstream& ostr, B_Boolean& rhs);

  //----------------------------------------------------------------------------------------------------       
  // Method: operator>>	EXTRACTION
  //     Lesen der eigentlichen Objektdaten im Textformat aus einem Stream
  // Return Value: 
  //	   Instanz der abgeleiten fstream-Klasse der Datei
  // Note: 
  //	   CT131296
  //	   ersetzt die alte Methode streamg()
  //
  DLL_DECL friend B_fstream& operator>> (B_fstream& istr, B_Boolean& rhs);

  }; // B_Boolean

DLL_DECL extern const B_Boolean B_false;
DLL_DECL extern const B_Boolean B_true;

//============= Operatorfunktionen =======================================================================


//============= Logische Operatoren ======================================================================
//-- muessen imp. werden da sie boolsche Werte als Ergebnis liefern und nicht wie Zahlenwerte logische
//   Verknuepfungen. Die logischen Verknuepfungen werden bei den einzelnen Datentypen abgehandelt.
DLL_DECL B_Boolean operator&& (const B_Boolean&, const B_Boolean&);
DLL_DECL B_Boolean operator|| (const B_Boolean&, const B_Boolean&);
DLL_DECL B_Boolean operator^  (const B_Boolean&, const B_Boolean&);

#ifdef THALESPLUS	//ab.. 27.1.97
DLL_DECL B_Boolean operator&& (const B_Boolean&, const B_Int);  
DLL_DECL B_Boolean operator&& (const B_Int, const B_Boolean&);
DLL_DECL B_Boolean operator&& (const B_Boolean&, const B_Lng);  
DLL_DECL B_Boolean operator&& (const B_Lng, const B_Boolean&);
DLL_DECL B_Boolean operator&& (const B_Boolean&, const B_Flt);  
DLL_DECL B_Boolean operator&& (const B_Flt, const B_Boolean&);
DLL_DECL B_Boolean operator&& (const B_Boolean&, const bool);  
DLL_DECL B_Boolean operator&& (const bool, const B_Boolean&);
DLL_DECL B_Boolean operator|| (const B_Boolean&, const B_Int);  
DLL_DECL B_Boolean operator|| (const B_Int, const B_Boolean&);
DLL_DECL B_Boolean operator|| (const B_Boolean&, const B_Lng);  
DLL_DECL B_Boolean operator|| (const B_Lng, const B_Boolean&);
DLL_DECL B_Boolean operator|| (const B_Boolean&, const B_Flt);  
DLL_DECL B_Boolean operator|| (const B_Flt, const B_Boolean&);
DLL_DECL B_Boolean operator|| (const B_Boolean&, const bool);  
DLL_DECL B_Boolean operator|| (const bool, const B_Boolean&);
#endif

//============= Vergleichsoperatoren =====================================================================
//
DLL_DECL B_Boolean operator== (const B_Boolean&, const B_Boolean&);
DLL_DECL B_Boolean operator!= (const B_Boolean&, const B_Boolean&);
DLL_DECL B_Boolean operator<  (const B_Boolean&, const B_Boolean&);
DLL_DECL B_Boolean operator<= (const B_Boolean&, const B_Boolean&);
DLL_DECL B_Boolean operator>= (const B_Boolean&, const B_Boolean&);
DLL_DECL B_Boolean operator>  (const B_Boolean&, const B_Boolean&);


//================================ Inlinefunktionen ======================================================
//
#if defined _DEBUG || defined B_VB_CONTRACT
  #define B_NOINLINE   // da sonst zirkulaere Bezuege Trace und Boolean
#else
  #undef B_NOINLINE
#endif

#ifndef B_NOINLINE               
  #define B_INLINE inline
  #ifndef BI_BOOL_HPP
    #include "bi_bool.hpp"
  #endif
#endif

//===========================================================================================================
//================================ End Of Header ============================================================
#endif //BC_BOOL_HPP

