#ifdef B_VB_COMPATIBLE

#ifndef BC_BYTE_HPP
#define BC_BYTE_HPP
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Project:                 VB nach C/C++ Konverter Klassenbibliothek
// Modul:                   VB Klasse fuer VB Byte Datentyp
// Filename:                bc_byte.hpp
// Version:                 1.0
// Status:                  beta
//
// Copyright:               Copyright (C) 1995
//                          Cicerosoft GmbH, 
//                          Ballweilerstr. 7,
//                          66399 Erfweiler-Ehlingen
//
// Operating System(s):     Win95
// Compiler:                VC++ 4.1
//
// Author:                  Jochen Diener JD
// Creation Date:           JD 14.12.95
// Last Modification:       JD 14.12.95
//
// Description:             Beinhaltet alle Klassen / Hilfsfkt. die im Zusammenhang mit der VB 
//                          Konvertierung, fuer die Bereitstellung des Datentyps Byte benoetigt 
//                          werden. 
//                          
// Dependencies:            bc_byte.cpp    ->  Implementierung der Member/Hilfsfkt.
//                          bi_byte.hpp    ->  Inlinefkt. der Klasse
//
// Remarks:                 Die Beschreibung zu den Klassenmethoden werden so kurz als moeglich
//                          gehalten um die Uebersichtlichkeit zu erhoehen, Eine genauere Beschreibung
//                          laesst sich ggf. in den VB Handbuechern finden
//
// Revision(s):   
//                          
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

//================================= Defintionsabhaengigkeiten ===============================================
//
#include <iostream.h>

#ifndef BD_LIB_HPP
  #include "bd_lib.hpp"
#endif

#ifndef BD_INTYP_HPP       
  #include "bd_intyp.hpp"       // alias der Standardatentypen
#endif

#ifndef BC_EXPT_HPP
  #include "bc_excpt.hpp"
#endif

#ifndef BC_BOOL_HPP
  #include "bc_bool.hpp"        // VB Boolean Typ
#endif

#ifndef BC_EMPTY_HPP
  #include "bc_empty.hpp"
#endif

#ifndef BC_NULL_HPP
  #include "bc_null.hpp"
#endif

#ifndef BC_FILE_HPP
  #include "bc_file.hpp"
#endif

//================================= Export =================================================================
//
class B_Integer;
class B_Long;
class B_Single;
class B_Double;
class B_Currency;
class B_String;
class B_fstream;						// CT131296 Fuer FileIO abgeleitete fstream-Klasse

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_Byte     Emuliert VB-Datentyp Byte
// Hierachy:    
// Inheritance: 
// Errorhandling:
// Note: 
//
class DLL_DECL B_Byte
  {
  //==================== MANGAEMENT =========================================================================
  //
  public: 
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     ctor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Byte(); // Standardkonstruktor initialisiert Integer in VB auf 0

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     ctor's  fuer Bibliotheksdatentypen
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Byte(const B_Boolean&);
    B_Byte(const B_Empty&); 
    B_Byte(const B_Null&);  
    B_Byte(const B_Integer&);  // throw(B_Overflow)
    B_Byte(const B_Long&);     // throw(B_Overflow)
    B_Byte(const B_Single&);   // throw(B_Overflow)
    B_Byte(const B_Double&);   // throw(B_Overflow)
    B_Byte(const B_Currency&); // throw(B_Overflow)
    B_Byte(const B_String&);   // throw(B_TypeMismatch, B_Overflow)


    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     ctor fuer OLE-Datentypen
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Byte(const VARIANT FAR& rhs); // Call by Ref

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     ctor's  fuer C-Standarddatentypen
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Byte(const B_Int,  const B_Explicit&);  // throw(B_Overflow)
    B_Byte(const B_UInt, const B_Explicit&);  // throw(B_Overflow)
    B_Byte(const B_Lng,  const B_Explicit&);  // throw(B_Overflow)
    B_Byte(const B_Flt,  const B_Explicit&);  // throw(B_Overflow)
    B_Byte(const B_Dbl,  const B_Explicit&);  // throw(B_Overflow)
    B_Byte(B_cCStr,      const B_Explicit&);  // throw(B_TypeMismatch, B_Overflow)


    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     cctor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Byte(const B_Byte&);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    ~B_Byte();

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Byte& operator=(const B_Byte&);

  //==================== INTERFACE ============================================================================
  //
  public:
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Zuweisungsop. fuer C-Standardtypen
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Byte& operator=(const B_Int)  throw(B_Overflow);
    B_Byte& operator=(const B_UInt) throw(B_Overflow);
    B_Byte& operator=(const B_Lng)  throw(B_Overflow);
    B_Byte& operator=(const B_Flt)  throw(B_Overflow);
    B_Byte& operator=(const B_Dbl)  throw(B_Overflow);
    B_Byte& operator=(B_cCStr)      throw(B_TypeMismatch, B_Overflow);
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Zuweisungsop. fuer Bibliotheksdatentypen
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Byte& operator=(const B_String&)   throw(B_TypeMismatch, B_Overflow);
    B_Byte& operator=(const B_Currency&) throw(B_Overflow);
    B_Byte& operator=(const B_Double&)   throw(B_Overflow);
    B_Byte& operator=(const B_Single&)   throw(B_Overflow);
    B_Byte& operator=(const B_Long&)     throw(B_Overflow);
    B_Byte& operator=(const B_Integer&)  throw(B_Overflow);
    B_Byte& operator=(const B_Boolean&);
    B_Byte& operator=(const B_Empty&);   
    B_Byte& operator=(const B_Null&);    

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Zuweisungsop. fuer OLE-Datentypen
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Byte& operator=(const VARIANT FAR&);

    //----------------------------------------------------------------------------------------------------       
    // Method:
    //     Addition des Wertes selbst mit einem anderen Wert. Entspricht x = x + y
    // Parameter:  
    //     Wert der hinzuaddiert werden soll (Entspricht y)
    // Return Value:
    //     Das Ergebnis der Addition
    // Errorhandling:
    // Note:      
    B_Byte& operator+=(const B_Byte&)    throw (B_Overflow);
    B_Byte& operator+=(const B_Boolean&) throw (B_Overflow);
    B_Byte& operator+=(const B_Empty&);
    B_Byte& operator+=(const B_Null&);
    B_Byte& operator+=(const B_Int)      throw (B_InvalidArgument, B_Overflow);

    //----------------------------------------------------------------------------------------------------       
    // Method:
    //     Subtraktion des Wertes selbst mit einem anderen Wert. Entspricht x = x - y
    // Parameter: 
    //     Wert der subtrahiert werden soll (Entspricht y)
    // Return Value:
    //     Das Ergebnis der Subtraktion
    // Errorhandling:
    // Note:      
    B_Byte& operator-=(const B_Byte&)     throw(B_Overflow);
    B_Byte& operator-=(const B_Boolean&)  throw(B_Overflow);
    B_Byte& operator-=(const B_Empty&);
    B_Byte& operator-=(const B_Null&);
    B_Byte& operator-=(const B_Int)       throw(B_InvalidArgument, B_Overflow);
        
    //----------------------------------------------------------------------------------------------------       
    // Function: 
    //     Multiplikation des Wertes selbst mit einem anderen Wert. Entspricht x = x * y
    // Parameter:   
    //     Wert der multipliziert werden soll (Entspricht y)
    // Return Value:
    //     Das Ergebnis der Multiplikation
    // Note:      
    B_Byte& operator*=(const B_Byte&)     throw(B_Overflow);
    B_Byte& operator*=(const B_Boolean&)  throw(B_Overflow);
    B_Byte& operator*=(const B_Empty&);
    B_Byte& operator*=(const B_Null&);
    B_Byte& operator*=(const B_Int)       throw(B_InvalidArgument, B_Overflow);

    //----------------------------------------------------------------------------------------------------       
    // Method:
    //     Division des Wertes selbst mit einem anderen Wert. Entspricht x = x / y
    // Parameter:  
    //     Wert der dividiert werden soll (Entspricht y)
    // Return Value: 
    //     Das Ergebnis der Division
    // Note:      
    //     ACHTUNG VB-Progr.: es wird eine ganzahldivision durchgefuehrt
    B_Byte& operator/=(const B_Byte&)    throw(B_ZeroDivide); 
    B_Byte& operator/=(const B_Boolean&) throw(B_ZeroDivide, B_Overflow);
    B_Byte& operator/=(const B_Empty&)   throw(B_ZeroDivide);
    B_Byte& operator/=(const B_Null&)    throw(B_InvalidNull);
    B_Byte& operator/=(const B_Int)      throw(B_InvalidArgument, B_ZeroDivide);
    
    //----------------------------------------------------------------------------------------------------       
    // Method:
    //     Division des Wertes selbst mit einem anderen Wert. Entspricht x = x / y
    // Parameter:  
    //     Wert der dividiert werden soll (Entspricht y)
    // Return Value: 
    //     Das Ergebnis der Division
    // Note:      
    //     ACHTUNG VB-Progr.: es wird eine ganzahldivision durchgefuehrt
    B_Byte& operator%=(const B_Byte&)    throw(B_ZeroDivide); 
    B_Byte& operator%=(const B_Boolean&) throw(B_ZeroDivide, B_Overflow);
    B_Byte& operator%=(const B_Empty&)   throw(B_ZeroDivide);
    B_Byte& operator%=(const B_Null&)    throw(B_InvalidNull);
    B_Byte& operator%=(const B_Int)      throw(B_InvalidArgument, B_ZeroDivide);

    //----------------------------------------------------------------------------------------------------       
    // Method:
    //     Unaeres Minus
    // Parameter:  
    // Return Value: 
    // Note:      
    B_Integer B_Byte::operator-() const;

    //----------------------------------------------------------------------------------------------------       
    // Method:
    //     Praeefix Inkrement, entspricht x = x + 1
    // Parameter:  
    // Return Value: 
    //     die um 1 erhoehte Variable
    // Note:      
    B_Byte& operator++() throw(B_Overflow);
 
    //----------------------------------------------------------------------------------------------------       
    // Method:
    //     Praeefix Dekrement, entspricht x = x - 1
    // Parameter:  
    // Return Value: 
    //     die um 1 verminderte Variable
    // Note:      
    B_Byte& operator--() throw(B_Overflow);
 
    //----------------------------------------------------------------------------------------------------       
    // Method:
    //     operator!() 
    // Parameter:  
    // Return Value: 
    // Note:      
    //
    B_Byte operator!(); 

    //----------------------------------------------------------------------------------------------------       
    // Method:
    //     Konvertierungsoperatoren fuer die wichtigsten Typen
    // Parameter:  
    //     
    // Return Value: 
    //     
    // Note:      
    operator const B_UChar() const;
    operator const VARIANT() const;
    
    //----------------------------------------------------------------------------------------------------       
    // Method: 
    //     Wandelt den Wert des Objektes in einen Absoluten Wert um, d.h 
    //     negativer Objektwert -> positiv
    //     positiver Objektwert -> bleibt positiv
    // Return Value:
    //     Boolean Wert:  B_True   wenn konvertiert
    //                    B_False  wenn nicht konvertiert werden konnte respektive Fehler
    //                              aufgetreten sind
    // Note:      
    B_Byte& abs();

    //----------------------------------------------------------------------------------------------------       
    // Method:       
    //     Gibt den Wert des Objektes als entsprechenden C/C++ Datentyp zurueck
    // Return Value:
    //     Kopie des Objetkwertes als B_UChar
    // Note:
    //     Den genauen Typ von B_UChar kann man in bf_intyp.hpp feststellen, in der alle
    //     C/C++ Datentypen definiert sind.
    //     Diese Method soll den Datenaustausch zwischen Klassen der Klassenbibliothek
    //     und den mit den C++ Standarddatentypen geschriebenen Methoden ermoeglichen
    //     wenn diese keine Klassen als Parameter erlauben.
    B_UChar getValue() const;

    //----------------------------------------------------------------------------------------------------       
    // Method:
    //     Gibt die Groesse des Wert des Objektes als entsprechenden C/C++ Datentyp zurueck
    // Return Value:
    //     Groesse in Byte
    // Note:
    //     Den genauen Typ und somit seine groesse koennen zwischen den C/C++ Datentypen der
    //     Compiler unterschiedlich sein, deshalb wurde diese Method eingefuegt die eine
    //     portable Programmierung aehnlich sizeof() ermoeglichen soll.
    B_ULng  getValueSize() const;  
    
    //----------------------------------------------------------------------------------------------------       
    // Method: 
    //     Gibt die Groesse des Objktes zurueck, diese entspricht der durch das Objekt belegten
    //     Speicherplatz
    // Return Value: 
    //     Groesse in Byte
    // Note: 
    //     Den genauen Typ und somit seine groesse koennen zwischen den C/C++ Compiler 
    //     unterschiedlich sein. Auch koennte sich die Klassenimplementierung aendern so
    //     dass aeltere Programme nur mit Aenderungsprogramm lauffaehig waeren, deshalb 
    //     wurde diese Method eingefuegt die eine portable Programmierung aehnlich 
    //     sizeof() ermoeglichen soll.
    B_ULng getSize() const;       
    
    //----------------------------------------------------------------------------------------------------       
    // Method: 
    //     Setzt den internen Status des Objektes auf optional. Somit kann mit isMissing der Status
    //     abgefragt werden und die Emulation des gleichnamigen VB Befehls wird moeglich.
    // Return Value: 
    // Note: 
    //
    B_Byte& isOptional();

    //----------------------------------------------------------------------------------------------------       
    // Method: 
    //     Abfragen des internen Objektstatus ob das Argument einer Funktion optional ist oder nicht
    // Return Value: 
    // Note: 
    //
    B_Boolean isMissing();

    //----------------------------------------------------------------------------------------------------       
    // Method: 
    //     Schreiben der der eigentlichen Objektdaten im binaeren Format in einen Stream
    // Return Value: 
    //     B_true  ==  Objekt und Stream sind gueltig
    //     B_false ==  Objekt oder Stream sind ungueltig
    // Note: 
    //
    B_Boolean streamw(ostream& ostr, B_Shrt openmode) const;

    //----------------------------------------------------------------------------------------------------       
    // Method: 
    //     Lesen der der eigentlichen Objektdaten im binaeren Format in einen Stream
    // Return Value: 
    //     B_true  ==  Objekt und Stream sind gueltig
    //     B_false ==  Objekt oder Stream sind ungueltig
    // Note: 
    //
    B_Boolean streamr(istream& istr, B_Shrt openmode);

  private:
    //----------------------------------------------------------------------------------------------------       
    // Method:
    //     Postfix   Inkrement/Dekrement
    // Parameter:  
    // Return Value: 
    // Note:      
    //     Da es beim Ueberladen keinen wirkliche Postfixoperatoren gibt wurden diese gesperrt
    B_Byte& operator++(int);
    B_Byte& operator--(int);
    
  //==================== MEMBER ===============================================================================    
  //
  private:
    B_UChar value;    // da VB Wertebereich 0-255
    B_Flag  optional;
  
  //==================== FRIENDS ============================================================================
  //
  friend class B_Integer;
  friend class B_Long; 
  friend class B_Single;
  friend class B_Double;
  friend class B_Currency;
  friend class B_String;

  friend DLL_DECL  B_Byte    operator+  (const B_Byte&,   const B_Byte&);
  friend DLL_DECL  B_Byte    operator-  (const B_Byte&,   const B_Byte&);
  friend DLL_DECL  B_Byte    operator-  (const B_Empty&,  const B_Byte&);
  friend DLL_DECL  B_Byte    operator*  (const B_Byte&,   const B_Byte&);
  friend DLL_DECL  B_Byte    operator%  (const B_Byte&,   const B_Byte&);
  friend DLL_DECL  B_Byte    B_Div      (const B_Byte&,   const B_Byte&);
  friend DLL_DECL  B_Byte    operator&& (const B_Byte&,   const B_Byte&);
  friend DLL_DECL  B_Byte    operator|| (const B_Byte&,   const B_Byte&);
  friend DLL_DECL  B_Byte    operator^  (const B_Byte&,   const B_Byte&);
  friend DLL_DECL  B_Boolean operator== (const B_Byte&,   const B_Byte&);
  friend DLL_DECL  B_Boolean operator!= (const B_Byte&,   const B_Byte&);
  friend DLL_DECL  B_Boolean operator<  (const B_Byte&,   const B_Byte&);
  friend DLL_DECL  B_Boolean operator<= (const B_Byte&,   const B_Byte&);
  friend DLL_DECL  B_Boolean operator>  (const B_Byte&,   const B_Byte&);
  friend DLL_DECL  B_Boolean operator>= (const B_Byte&,   const B_Byte&);
  friend DLL_DECL  B_Boolean operator== (const B_Byte&,   const B_Boolean&);
  friend DLL_DECL  B_Boolean operator!= (const B_Byte&,   const B_Boolean&);
  friend DLL_DECL  B_Boolean operator<  (const B_Byte&,   const B_Boolean&);
  friend DLL_DECL  B_Boolean operator<= (const B_Byte&,   const B_Boolean&);
  friend DLL_DECL  B_Boolean operator>  (const B_Byte&,   const B_Boolean&);
  friend DLL_DECL  B_Boolean operator>= (const B_Byte&,   const B_Boolean&);
  friend DLL_DECL  B_Boolean operator== (const B_Boolean&,   const B_Byte&);
  friend DLL_DECL  B_Boolean operator!= (const B_Boolean&,   const B_Byte&);
  friend DLL_DECL  B_Boolean operator<  (const B_Boolean&,   const B_Byte&);
  friend DLL_DECL  B_Boolean operator<= (const B_Boolean&,   const B_Byte&);
  friend DLL_DECL  B_Boolean operator>  (const B_Boolean&,   const B_Byte&);
  friend DLL_DECL  B_Boolean operator>= (const B_Boolean&,   const B_Byte&);
  friend DLL_DECL  B_Boolean operator== (const B_Empty&,   const B_Byte&);
  friend DLL_DECL  B_Boolean operator!= (const B_Empty&,   const B_Byte&);
  friend DLL_DECL  B_Boolean operator<  (const B_Empty&,   const B_Byte&);
  friend DLL_DECL  B_Boolean operator<= (const B_Empty&,   const B_Byte&);
  friend DLL_DECL  B_Boolean operator>  (const B_Empty&,   const B_Byte&);
  friend DLL_DECL  B_Boolean operator>= (const B_Empty&,   const B_Byte&);
  friend DLL_DECL  B_Boolean operator== (const B_Byte&,   const B_Empty&);
  friend DLL_DECL  B_Boolean operator!= (const B_Byte&,   const B_Empty&);
  friend DLL_DECL  B_Boolean operator<  (const B_Byte&,   const B_Empty&);
  friend DLL_DECL  B_Boolean operator<= (const B_Byte&,   const B_Empty&);
  friend DLL_DECL  B_Boolean operator>  (const B_Byte&,   const B_Empty&);
  friend DLL_DECL  B_Boolean operator>= (const B_Byte&,   const B_Empty&);

  //----------------------------------------------------------------------------------------------------       
  // Method: operator<<	INSERTION
  //     Schreiben der eigentlichen Objektdaten im Textformat in einen Stream
  // Return Value: 
  //	   Instanz der abgeleiten fstream-Klasse der Datei
  // Note: 
  //	   CT131296
  //	   ersetzt die alte Methode streamp()
  //
  DLL_DECL friend B_fstream& operator<< (B_fstream& ostr, B_Byte& rhs);

  //----------------------------------------------------------------------------------------------------       
  // Method: operator>>	EXTRACTION
  //     Lesen der eigentlichen Objektdaten im Textformat aus einem Stream
  // Return Value: 
  //	   Instanz der abgeleiten fstream-Klasse der Datei
  // Note: 
  //	   CT131296
  //	   ersetzt die alte Methode streamg()
  //
  DLL_DECL friend B_fstream& operator>> (B_fstream& istr, B_Byte& rhs);
  }; // B_Byte

DLL_DECL extern const B_Byte  B_byteMin;
DLL_DECL extern const B_Byte  B_byteMax;


//----------------------------------Arithmetische Operatoren -----------------------------------------

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der +Operator = Addition
DLL_DECL B_Byte operator+ (const B_Byte&,   const B_Byte&) throw(B_Overflow);
//-- Bibliothekstypen
DLL_DECL B_Byte operator+ (const B_Empty&,  const B_Byte&);
DLL_DECL B_Byte operator+ (const B_Byte&,   const B_Empty&);
DLL_DECL B_Null operator+ (const B_Null&,   const B_Byte&);
DLL_DECL B_Null operator+ (const B_Byte&,   const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der -Operator = Subtraktion
DLL_DECL B_Byte operator- (const B_Byte&,   const B_Byte&) throw(B_Overflow);
//-- Bibliothekstypen
DLL_DECL B_Byte operator- (const B_Empty&,  const B_Byte&);
DLL_DECL B_Byte operator- (const B_Byte&,   const B_Empty&);
DLL_DECL B_Null operator- (const B_Null&,   const B_Byte&);
DLL_DECL B_Null operator- (const B_Byte&,   const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der *Operator = Multiplikation 
DLL_DECL B_Byte operator* (const B_Byte&,   const B_Byte&) throw(B_Overflow);
//-- Bibliothekstypen
DLL_DECL B_Byte operator* (const B_Empty&,  const B_Byte&);
DLL_DECL B_Byte operator* (const B_Byte&,   const B_Empty&);
DLL_DECL B_Null operator* (const B_Null&,   const B_Byte&);
DLL_DECL B_Null operator* (const B_Byte&,   const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der /Operator = Division deren Ergebniss immer ein Fliesskommawert ergibt
//   deshalb sind diese Operatoren bei B_Single definiert
//-- Bibliothekstypen
DLL_DECL B_Null operator/ (const B_Null&,  const B_Byte&);
DLL_DECL B_Null operator/ (const B_Byte&,  const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der %Operator = Modulo Operation, gibt den Wert einer ganzahligen Division zurueck
//   entsprich MOD-Operator in VB
DLL_DECL B_Byte operator% (const B_Byte&,  const B_Byte&) throw(B_ZeroDivide);
//-- Bibliothekstypen
DLL_DECL B_Null operator% (const B_Null&,  const B_Byte&);
DLL_DECL B_Null operator% (const B_Byte&,  const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Ganzahldivision entspricht dem \Operator in VB, da in C++ kein geeigneter Operator zu Verfuegung steht
//   wird dieser Operator als Method realisiert
DLL_DECL B_Byte B_Div(const B_Byte&,  const B_Byte&) throw(B_ZeroDivide);
//-- Bibliothekstypen      
DLL_DECL B_Null B_Div(const B_Null&,  const B_Byte&);
DLL_DECL B_Null B_Div(const B_Byte&,  const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Dient zum Potenzieren einer Zahl mit einem Exponenten
//   Erg = Zahl^Exponent  -> Erg = Pow(zahl, exponent)
//   Das Argument Zahl darf nur negativ sein, wenn Exponent ein ganzzahliger Wert ist. Werden in einem einzigen 
//   Ausdruck mehrere Potenzierungen ausgefhrt, so wird der Operator ^ jeweils in der Reihenfolge seines 
//   Auftretens von links nach rechts ausgewertet.
//   Ergebnis hat den Datentyp Double  Ist jedoch entweder Zahl oder Exponent ein Ausdruck mit dem Wert Null, 
//   so ist das Ergebnis ebenfalls Null.
//   Da das Ergebnis ein B_Double ist, sind diese auch dort definiert
//-- Bibliothekstypen
DLL_DECL B_Null B_Pow(const B_Null&,  const B_Byte&);
DLL_DECL B_Null B_Pow(const B_Byte&,  const B_Null&);

//----------------------------------------------------------------------------------------------------
//---------------------------------- Logische Operatoren ---------------------------------------------

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der &&Operator entspricht AND in VB , 
//   Bit in Ausdruck1 	Bit in Ausdruck2 	Ergebnis 
//       0	                  0            	0
//       0	                  1           	0
//       1                  	0           	0
//       1                   	1            	1
//   ACHTUNG alle C/C++ Programmierer: in VB werden numerische Typen immer logisch bitweise und verknpft
//                                     Der Einsatz von & war nicht moeglich da stellenweise erst zur Lauf-
//                                     zeit das Verhalten bestimmt werden kann was der Konverter nicht 
//                                     bestimmen kann
DLL_DECL B_Byte operator&& (const B_Byte&,    const B_Byte&);
//-- Bibliothekstypen
DLL_DECL B_Null operator&& (const B_Null&,    const B_Byte&);
DLL_DECL B_Null operator&& (const B_Byte&,    const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der ||Operator entspricht OR in VB , 
//   Bit in Ausdruck1 	Bit in Ausdruck2 	Ergebnis 
//       0	                  0            	0
//       0	                  1           	1
//       1                  	0           	1
//       1                   	1            	1
//   ACHTUNG alle C/C++ Programmierer: in VB werden numerische Typen immer logisch bitweise und verknpft
//                                     Der Einsatz von | war nicht moeglich da stellenweise erst zur Lauf-
//                                     zeit das Verhalten bestimmt werden kann was der Konverter nicht 
//                                     bestimmen kann
DLL_DECL B_Byte operator|| (const B_Byte&,    const B_Byte&);
//-- Bibliothekstypen
DLL_DECL B_Byte operator|| (const B_Null&,    const B_Byte&);
DLL_DECL B_Byte operator|| (const B_Byte&,    const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der ^Operator entspricht XOR in VB 
//   Bit in Ausdruck1 	Bit in Ausdruck2 	Ergebnis 
//       0	                  0            	0
//       0	                  1           	1
//       1                  	0           	1
//       1                   	1            	0
//   ACHTUNG alle VB Programmierer:    In C++ bedeutet der ^Operator ein bitweises XOR auf numerische
//                                     Operanden und nicht wie in VB eine Potenzierung. Der ^Operator
//                                     wurde nicht zur Potenzierung verwandt da dies nicht C++ Stil
//                                     entspricht.
//   ACHTUNG alle C/C++ Programmierer: In VB werden numerische Typen immer logisch bitweise und verknpft
//                                     Dies hat fuer ^Operator die Folge, dass er bei nicht numerischen
//                                     Ausdreucken sich wie ein logisches XOR verhaelt
DLL_DECL B_Byte operator^ (const B_Byte&,   const B_Byte&);
//-- Bibliothekstypen
DLL_DECL B_Null operator^ (const B_Null&,   const B_Byte&);
DLL_DECL B_Null operator^ (const B_Byte&,   const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der !Operator entspricht NOT in VB 
//   Bit in Ausdruck1 	 Ergebnis
//       0	                 1
//       1                   0
//   ACHTUNG alle C/C++ Programmierer: In VB werden numerische Typen immer logisch bitweise und verknpft
//                                     Dies hat fuer !Operator die Folge, dass er bei nicht numerischen
//                                     Ausdreucken sich wie ein logisches NOT verhaelt
// fuer den Fall, dass B_Emtpy uebergeben wird ist fuer alle Datentypen die Method 
// B_Integer operator! (const B_Empty) verantwortlich
// B_Byte operator! (const B_UChar); geht nicht muss Konverter evtl. durch Casts abfangen

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Die Operatoren Imp (=logische Implikation) und Eqv (=logische quivalenz) werden vom Konverter
// in folgende Basisoperatorkonstrukte umgesetzt, da in C++ keine quivalenten Operatoren zur Verfgung
// stehen.
// a Imp b    ->  (!a || b)
// a Eqv b    ->  !(a ^ b)


//----------------------------------------------------------------------------------------------------
//---------------------------------- Vergleichs Operatoren -------------------------------------------

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Verlgeich auf Gleichheit der Operanden, es findet ein numerischer Vergleich statt
//   Ausdruck1 == Ausdruck2 -> B_True
//   Ausdruck1 != Ausdruck2 -> B_False
//   ACHTUNG alle VB Programmierer:  In C++ wird mit '==' auf Gleichheit abgefragt und nicht mit '='
//                                 Der '=' Operator wird nur fuer Zuweisungen verwendet.
DLL_DECL B_Boolean operator== (const B_Byte&,     const B_Byte&);
//-- Bibliothekstypen
DLL_DECL B_Boolean operator== (const B_Byte&,     const B_Empty&);
DLL_DECL B_Boolean operator== (const B_Empty&,    const B_Byte&);
DLL_DECL B_Null    operator== (const B_Byte&,     const B_Null&);
DLL_DECL B_Null    operator== (const B_Null&,     const B_Byte&);
DLL_DECL B_Boolean operator== (const B_Byte&,     const B_Boolean&);
DLL_DECL B_Boolean operator== (const B_Boolean&,  const B_Byte&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Verlgeich auf Ungleichheit der Operanden, es findet ein numerischer Vergleich statt
//   Ausdruck1 != Ausdruck2 -> B_True
//   Ausdruck1 == Ausdruck2 -> B_False
//   ACHTUNG alle VB Programmierer:  In C++ wird mit '!=' auf Ungleichheit abgefragt und nicht mit '<>'
//                                   Der '<>' Operator existiert nicht in C++.
DLL_DECL B_Boolean operator!= (const B_Byte&,    const B_Byte&);
// Biblitohekstypen
DLL_DECL B_Boolean operator!= (const B_Byte&,    const B_Empty&);
DLL_DECL B_Boolean operator!= (const B_Empty&,   const B_Byte&);
DLL_DECL B_Null    operator!= (const B_Byte&,    const B_Null&);
DLL_DECL B_Null    operator!= (const B_Null&,    const B_Byte&);
DLL_DECL B_Boolean operator!=(const B_Byte&,     const B_Boolean&);
DLL_DECL B_Boolean operator!=(const B_Boolean&,  const B_Byte&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Verlgeich auf Kleiner der Operanden, es findet ein numerischer Vergleich statt
//   Ausdruck1 <  Ausdruck2 -> B_True
//   Ausdruck1 >= Ausdruck2 -> B_False
DLL_DECL B_Boolean operator< (const B_Byte&,   const B_Byte&);
//-- Bibliothekstypen
DLL_DECL B_Boolean operator< (const B_Byte&,   const B_Empty&);
DLL_DECL B_Boolean operator< (const B_Empty&,  const B_Byte&);
DLL_DECL B_Null    operator< (const B_Byte&,   const B_Null&);
DLL_DECL B_Null    operator< (const B_Null&,   const B_Byte&);
DLL_DECL B_Boolean operator<(const B_Byte&,    const B_Boolean&);
DLL_DECL B_Boolean operator<(const B_Boolean&, const B_Byte&);


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Verlgeich auf Kleiner/Gleich der Operanden, es findet ein numerischer Vergleich statt
//   Ausdruck1 <=  Ausdruck2 -> B_True
//   Ausdruck1 >   Ausdruck2 -> B_False
DLL_DECL B_Boolean operator<= (const B_Byte&,   const B_Byte&);
//-- Bibliothekstypen
DLL_DECL B_Boolean operator<= (const B_Byte&,   const B_Empty&);
DLL_DECL B_Boolean operator<= (const B_Empty&,  const B_Byte&);
DLL_DECL B_Null    operator<= (const B_Byte&,   const B_Null&);
DLL_DECL B_Null    operator<= (const B_Null&,   const B_Byte&);
DLL_DECL B_Boolean operator<=(const B_Byte&,    const B_Boolean&);
DLL_DECL B_Boolean operator<=(const B_Boolean&, const B_Byte&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Verlgeich auf Groesser der Operanden, es findet ein numerischer Vergleich statt
//   Ausdruck1 >   Ausdruck2 -> B_True
//   Ausdruck1 <=  Ausdruck2 -> B_False
DLL_DECL B_Boolean operator> (const B_Byte&,   const B_Byte&);
//-- Bibliothekstypen
DLL_DECL B_Boolean operator> (const B_Byte&,   const B_Empty&);
DLL_DECL B_Boolean operator> (const B_Empty&,  const B_Byte&);
DLL_DECL B_Null    operator> (const B_Byte&,   const B_Null&);
DLL_DECL B_Null    operator> (const B_Null&,   const B_Byte&);
DLL_DECL B_Boolean operator>(const B_Byte&,    const B_Boolean&);
DLL_DECL B_Boolean operator>(const B_Boolean&, const B_Byte&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Verlgeich auf Groesser/Gleich der Operanden, es findet ein numerischer Vergleich statt
// Ausdruck1 >=  Ausdruck2 -> B_True
// Ausdruck1 <   Ausdruck2 -> B_False
DLL_DECL B_Boolean operator>= (const B_Byte&,    const B_Byte&);
//-- Bibliothekstypen
DLL_DECL B_Boolean operator>= (const B_Byte&,    const B_Empty&);
DLL_DECL B_Boolean operator>= (const B_Empty&,   const B_Byte&);
DLL_DECL B_Null    operator>= (const B_Byte&,    const B_Null&);
DLL_DECL B_Null    operator>= (const B_Null&,    const B_Byte&);
DLL_DECL B_Boolean operator>= (const B_Byte&,    const B_Boolean&);
DLL_DECL B_Boolean operator>= (const B_Boolean&, const B_Byte&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Streamoperatoren
DLL_DECL ostream& operator<<(ostream& ostr, const B_Byte& rhs);
DLL_DECL istream& operator>>(istream& istr, B_Byte& rhs);

//================================ Inlinefunktionen =========================================================
//
#if defined _DEBUG || defined B_VB_CONTRACT
  #define B_NOINLINE
#else
  #undef B_NOINLINE
#endif

#ifndef B_NOINLINE                 // festlegen wenn Debuger keine Inlines debuggen kann
  #define B_INLINE inline
  #ifndef  BI_BYTE_HPP
    #include "bi_byte.hpp"
  #endif
#endif

//===========================================================================================================
//================================ End Of Header ============================================================
#endif // BC_BYTE_HPP

#endif // VB_COMPATIBLE
