#ifndef BC_EXCPT_HPP
#define BC_EXCPT_HPP
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Project:                 VB nach C/C++ Konverter Klassenbibliothek
// Modul:                   VB Klassen fuer die Exceptionhierachie der Bibliothek
// Filename:                bc_excpt.hpp
// Version:                 0.0
// Status:                  beta
//
// Copyright:               (C) 1995
//                          Cicerosoft GmbH
//                          Ballweilerstr. 7
//                          66399 Erfweiler-Ehlingen
//
// Operation System(s):     Win95
// Compiler:                VC++ 4.1
//
// Author:                  Jochen Diener JD
// Creation Date::          JD 06.02.96
// Last Modification:       JD 07.06.96
//
// Description:             Beinhaltet die Klassenhierachie die fuer die Emulation des VB EH benoetigt wird
//
// Dependencies:            bc_excpt.cpp   -> Klassenimpl.
//                          bi_excpt.hpp   -> Inline (Member)Funktionen der Klasse
//
// Remarks:
//
//
// Revision(s):
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

//================================= DefintionsDependencies ===============================================
//
#ifndef BD_LIB_HPP
  #include "bd_lib.hpp"       // alias der Standardatentypen
#endif

#ifndef BD_INTYP_HPP
  #include "bd_intyp.hpp"       // alias der Standardatentypen
#endif

#ifndef BC_BOOL_HPP
  #include "bc_bool.hpp"
#endif

#ifndef BD_EXCEPT_HPP
  #include "bd_excpt.hpp"
#endif

//================================= Export =================================================================
//

//-- aus kompatibilitaetsgruenden zu Draft erhaelt jeder Konstruktor als uebergabewert einen String als what_arg.
//   Dieser String ist optionales Argument. Wenn er nicht angegeben wird oder ein Leerstring ist wird eine Standard-
//   fehlermeldung fuer diese EH in der jeweiligen Landessprache ausgegeben.
//   Bei manchen Klassen wird noch ein zusaetzlicher optionaler PArameter eingefuehrt der eine genauere Bestimmung
//   zulaest fuer das Auftreten der EH und den Reuckagabewert von what beeinflussen kann.
//   Bei der enhancedLib wird das Err-Objekt von VB nicht mehr unterstuetzt und somit auch kein Hilfekontext und
//   Hilfedatei-attribut mehr gesetzt
//-- Das traceFlag Argument der Klaasenkonstruktoren wird nur bibliotheksintern fuer bestimmte Bibliotheksversionen benoetigt
//   und ist somit nur Bibliotheksintern von Interesse


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_Exception                       Basisklasse der Exception Hierachie
// Hierachy:    
// Inheritance: 
// Errorhandling:
// Note: 
// Impl.: 
//     ist so aufgebaut dass sie spaeter durch die ANSI-Draft Exceptionklasse ausgetauscht werden
//     kann
//     Es wird nicht B_String-Klasse verwendet um EH leicht in anderen Libs einsetzen zu koennen
class DLL_DECL B_Exception
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    B_Exception(B_cCStr whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    B_Exception(const B_Exception&); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_Exception() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Inhalt von what nach Zuweisung in Draf undefiniert und impl. abhaengig
    B_Exception& operator=(const B_Exception&) throw();

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    // Return Value:
    //     Defaultstring den das Objekt bei einer EH annimmt
    // Errorhandling:
    // Note:      
    //     Inhalt von what nach Zuweisung in Draf undefiniert und impl. abhaengig
    virtual B_CStr what() const throw();

  //==================================== HELPER =============================================================
  //
  protected:
    //-- keine virtuellen Funktionen da sie nur intern verwendet werden in der Exceptionhierachie um
    //   ADT zu verwirklichen und immer nur die geerbten Daten setzen oder abfragen so dass die
    //   Implementierung immer gleich bleibt.

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     getDescription()
    // Parameter:  
    // Return Value:
    //     liefert den Wert der Membervariable description
    // Errorhandling:
    // Note:      
    B_cCStr getDescription() const;

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     setDescription()
    // Parameter:  
    // Return Value:
    //     setzt den Wert der Membervariable description
    // Errorhandling:
    // Note:      
    void setDescription(B_cCStr  description);

  //=================================== MEMBER ==============================================================
  //
  private:    
      B_CStr description;
  }; // B_Exception

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_LogicError  Basisklasse der Logik-Fehler Hierachie innerhalb der Exception-Hierachie
// Hierachy:      
//     B_Exception <- B_LogicError
// Inheritance: 
//     Methods(public): operator=(), what()
// Errorhandling: 
// Note:  
// Impl.:    
//     ist so aufgebaut dass sie spaeter durch die ANSI-Draft Exceptionklasse ausgetauscht werden kann
//
class DLL_DECL B_LogicError : public B_Exception
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_LogicError(B_cCStr whatArg = "", const B_Boolean& traceFlag = B_true);
    
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_LogicError(const B_LogicError& rhs);  
   
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_LogicError() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_LogicError& operator=(const B_LogicError& rhs) throw();

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();

  }; // B_LogicError


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_LengthError  Es wurde ein Objekt einer Laenge definiert die gleich oder groesser als die maximal 
//                    erlaubte Groesse ist.
// Hierachy:      
//     B_Exception <- B_LogicError <- B_LengthError
// Inheritance: 
//     Methods (public):  operator=(), what()
// Errorhandling:
// Note:         
//     Fehlermeldung: "Length error (category)"
// Impl.:
//     ist so aufgebaut dass sie spaeter durch die ANSI-Draft Exceptionklasse ausgetauscht werden kann
//
class DLL_DECL B_LengthError : public B_LogicError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_LengthError(B_cCStr whatArg = "", const B_Boolean& traceFlag = B_true); // throw()

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_LengthError(const B_LengthError& rhs);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_LengthError() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_LengthError& operator=(const B_LengthError& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_LengthError


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_OutOfRange  Wenn der Wertebereich eines Typs nicht im gueltigen Bereich liegt
// Hierachy:     
//     B_Exception <- B_LogicError <- B_OutOfRange
// Inheritance: 
//     Methods (public):  operator=(), what()
// Errorhandling:
// Note:         
//     Fehlermeldung: "Out of Range (category)"
// Impl.:
//     ist so aufgebaut dass sie spaeter durch die ANSI-Draft Exceptionklasse ausgetauscht werden kann
//
class DLL_DECL B_OutOfRange : public B_LogicError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_OutOfRange(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); //throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_OutOfRange(const B_OutOfRange& rhs);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_OutOfRange() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_OutOfRange& operator=(const B_OutOfRange& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  };


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_SubscriptError Wenn der Bereich einer Dimensionsangabe (=Subscripttion) den gueltigen Wertebereich 
//                      ueber oder unterschreitet.
// Hierachy:     
//     B_Exception <- B_LogicError <- B_OutOfRange <- B_SubscriptError
// Inheritance: 
//     Methods (public): operator=(), what()
// Errorhandling:
// Note:         
//     entspricht LZF(9) "Subscript out of Range" in VB Fehlermeldung: "Subscript out of range"
// Impl.:         
//
class DLL_DECL B_SubscriptError : public B_OutOfRange
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Subscript Wert der zu Fehler fuehrte in whatArg aufnehmen
    B_SubscriptError(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_SubscriptError(const B_SubscriptError&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_SubscriptError() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_SubscriptError& operator=(const B_SubscriptError& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  };

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_DomainError  Allg. Wenn der Domain (Werte- / Gueltigkeitsbereich) nicht im gueltigen Bereich liegt.
// Hierachy:  
//     B_Exception <- B_LogicError <- B_OutOfRange <- B_DomainError
// Inheritance:
//     Methods (public): operator=(), what()
// Errorhandling:
// Note:         
//     entspr. LZF(5) "Illegal Function Call" in VB deshalb gibt es keine eigene Klasse B_IllegalFuncCall. 
//     Die Fehlermeldung wird auf die von VB eingestellt. Fehlermeldung: "Illegal function call (domain error)"
// Impl.:    
//     ist so aufgebaut dass sie spaeter durch die ANSI-Draft Exceptionklasse ausgetauscht werden kann
//
class DLL_DECL B_DomainError : public B_LogicError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    //     value ist wert der zu Bereichsueberschreitung fuehrte und in whatArg aufgenommen wird
    B_DomainError(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_DomainError(const B_DomainError&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_DomainError() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_DomainError& operator=(const B_DomainError& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_DomainError


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_InvalidArgument Wenn der funktionsaufruf nicht komplett abgeschlossen werden kann, weil ein Argument
//                       nicht vom erwarteten Typ. 
// Hierachy:  
//     B_Exception <- B_LogicError <- B_InvalidArgument
// Inheritance:
//     Methods (public): operator=(), what()
// Errorhandling:
// Note:         
//     entspr. LZF(13) "TypeMismatch" in VB deshalb gibt es keine eigene Klasse B_TypeMismatch. Die
//     Fehlermeldung wird auf die von VB eingestellt. Fehlermeldung: "Type Mismatch for invalid argument
//     (category)"
// Impl.:
//     ist so aufgebaut dass sie spaeter durch die ANSI-Draft Exceptionklasse ausgetauscht werden kann
//
class DLL_DECL B_InvalidArgument : public B_LogicError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    //     der Argumentname kann n den whatArg-String gepackt werden
    B_InvalidArgument(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_InvalidArgument(const B_InvalidArgument&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_InvalidArgument() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_InvalidArgument& operator=(const B_InvalidArgument& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; //B_InvalidArgument

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_InvalidPattern Allg. wenn ein Zeichenfolgemuster ungueltig ist
//         
// Hierachy:    
//     B_Exception <- B_LogicError <- B_InvalidArgument <- B_InvalidPattern
// Inheritance: 
//     Methods (public): operator=(), what()
// Errorhandling: 
// Note:         
//     entspr. LZF(93) "Invalid Pattern String"  Fehlermeldung: "Invalid pattern string"
// Impl.:  
//
class DLL_DECL B_InvalidPattern : public B_InvalidArgument
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Pattern in whatArg aufnehmen
    B_InvalidPattern(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_InvalidPattern(const B_InvalidPattern&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_InvalidPattern() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_InvalidPattern& operator=(const B_InvalidPattern& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_InvalidPattern

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_RuntimeError Basisklasse fuer die Kategorie der Laufzeitfehler         
// Hierachy:  
//     B_Exception <- B_RuntimeError
// Inheritance:  
//     Methods (public):  operator=(), what()
// Errorhandling: 
// Note:          
//     Fehlermeldung: "Runtime error (category)"
// Impl.:
//     ist so aufgebaut dass sie spaeter durch die ANSI-Draft Exceptionklasse ausgetauscht werden kann
//
class DLL_DECL B_RuntimeError : public B_Exception
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_RuntimeError(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_RuntimeError(const B_RuntimeError&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_RuntimeError() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_RuntimeError& operator=(const B_RuntimeError& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_RuntimeError


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_RangeError Wertebereichsfehler die generell erst zur Laufzeit ermittelbar sind.
// Hierachy:  
//     B_Exception <- B_RuntimeError <- B_RangeError
// Inheritance: 
//     Methods (public):  operator=(), what()
// Errorhandling:
// Note:         
//     Fehlermeldung: "Range error" Im Gegensatz zu B_DomainError sind dies Wertebereichsfehler die nur zur 
//     Laufzeit erkannt werden koennen.
// Impl.:     
//     ist so aufgebaut dass sie spaeter durch die ANSI-Draft Exceptionklasse ausgetauscht werden kann
//
class DLL_DECL B_RangeError : public B_RuntimeError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_RangeError(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_RangeError(const B_RangeError&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_RangeError() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_RangeError& operator=(const B_RangeError& rhs);
  
  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_RangeError

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_Overflow Ausgeloest von artihmet. Ueberlaufen
// Hierachy:
//     B_Exception <- B_RuntimeError <- B_Overflow
// Inheritance:
//     Methods (public):  operator=(), what()
// Errorhandling:
// Note: 
//     Entspricht LZF(6) "Overflow" in VB, dort wird er generell verwendet wenn eine Wertebereichs-
//     ueberschreitung eines Paramters, Zielvariablen ueberschritten wird.
//     In der Konverterlib werden auch Underflow's als Overflow's ausgeworfen
//     Fehlermeldung: "Overflow error"
// Impl.: 
//     ist so aufgebaut dass sie spaeter durch die ANSI-Draft Exceptionklasse ausgetauscht werden
//     kann
//
class DLL_DECL B_Overflow : public B_RuntimeError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_Overflow(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_Overflow(const B_Overflow&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_Overflow() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_Overflow& operator=(const B_Overflow& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_Overflow


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_Underflow  Ausgeloest von artihmet. Unterlaeufen
// Hierachy: 
//     B_Exception <- B_RuntimeError <- B_Overflow <- B_Underflow
// Inheritance:  
//     Methods (public): operator=(), what()
// Errorhandling:
// Note:  
//     Hat keine Entsprechung in VB und wird erst in der enhancedLib erzeugt.
//     Fehlermeldung: "Underflow error"
// Impl.:
//     von B_Overflow abgeleitet und nicht von B_RuntimeError da viele Bibliotheken Underflow
//     und Overflow Errors unter Overflow zusammenfassen und dies sich so am besten integrieren
//     liesse
//
class DLL_DECL B_Underflow : public B_Overflow
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_Underflow(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_Underflow(const B_Underflow&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_Underflow() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_Underflow& operator=(const B_Underflow& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_Underflow


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_NotSupported  Wenn ein Parameter oder eine Funktion nicht unerstuetzt wird, die angesprochen wurde.
// Hierachy: 
//     B_Exception <- B_RuntimeError <- B_NotSupported
// Inheritance:  
//     Methods (public):  operator=(), what()
// Errorhandling:
// Note:  
//     Entsprcht in VB LZF(35) "Sub or Function not defined" und ist hauptsaechlich ein VB-IDE Fehler.
//     Fehlermeldung: "Sub or Function not supported"
// Impl.:
//     Aufgenommen da ihn auch C++ Compiler (VC++) unterstuetzen
//
class DLL_DECL B_NotSupported : public B_RuntimeError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_NotSupported(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_NotSupported(const B_NotSupported&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_NotSupported() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_NotSupported& operator=(const B_NotSupported& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_NotSupported

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_InvalidNull  Wenn ein Parameter/Variable vom Typ B_Null einem Argument oder Variable zugewiesen 
//                    wurde, fuer die Null nicht erlaubt ist.
// Hierachy: 
//     B_Exception <- B_RuntimeError <- B_InvalidNull
// Inheritance:  
//     Methods (public):  operator=(), what()
// Errorhandling: -
// Note:  
//     Entsprcht in VB LZF(94) "Invalid use of Null"
// Impl.:
//
class DLL_DECL B_InvalidNull : public B_RuntimeError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_InvalidNull(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_InvalidNull(const B_InvalidNull&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_InvalidNull() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_InvalidNull& operator=(const B_InvalidNull& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_Invalid Null

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_ZeroDivide Wenn versucht wird durch 0 (=Null) zu dividieren.
// Hierachy: 
//     B_Exception <- B_RuntimeError <- B_ZeroDivide
// Inheritance:  
//     Methods (public): operator=(),  what()
// Errorhandling:
// Note:  
//     Entsprcht in VB LZF(11) "Division by zero". Dies kann die bibliothek nur fuer Ausdruecke
//     gewaehrleisten, die den Wertebereich eines C/C++ long double Datentyps nicht ueberschreiten.
//     Fehlermeldung: "Division by zero"
// Impl.:
//     Kann nicht alle abfangen da Signalhandling unter Windows nicht korrekt arbeitet und C++ dies
//     nicht unterstuetzt resp. nicht moelgich ist vgl. Meyer/Strou
//
class DLL_DECL B_ZeroDivide : public B_RuntimeError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_ZeroDivide(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_ZeroDivide(const B_ZeroDivide&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_ZeroDivide() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_ZeroDivide& operator=(const B_ZeroDivide& rhs);
  
  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_ZeroDivide



/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_TypeMismatch Wenn versucht wird Datentypen einander zuzuweisen die nicht ineinander konvertiert 
//                    werden koennen oder durch die Konvertierung ungueltig werden.
// Hierachy: 
//     B_Exception <- B_RuntimeError <- B_TypeMismatch
// Inheritance:  
//     Methods (public): operator=(),  what()
// Errorhandling:
// Note:  
//     Entsprcht in VB LZF(13) "Type Mismatch".
// Impl.:
//
class DLL_DECL B_TypeMismatch : public B_RuntimeError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_TypeMismatch(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_TypeMismatch(const B_TypeMismatch&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_TypeMismatch() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_TypeMismatch& operator=(const B_TypeMismatch& rhs);
  
  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_TypeMismatch




/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_Ressource Error Basisklasse fuer die Kategorie der Ressource-Fehler.
// Hierachy: 
//     B_Exception <- B_ResourceError
// Inheritance:  
//     Methods (public): operator=(),  what()
// Errorhandling:
// Note:  
//     keine direkte entspr. in VB. Faengt alle Ressourcefehler ab die nicht in speziellen Klassen untergebr.
//     sind. Fehlermeldung: "Ressource Error (category)"
// Impl.:
//
class DLL_DECL B_ResourceError : public B_RuntimeError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_ResourceError(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_ResourceError(const B_ResourceError&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_ResourceError() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_ResourceError& operator=(const B_ResourceError& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_RessourceError

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:   B_OutOfMem                                                             (C) CiceroSoft 1995
//     Es ist kein Speicherblock der geforderten groesse mehr auf dem Heap verfuegbar, oder es wurde eine
//     64KB Grenze ueberschritten (-> Segmentierungsfehler, nur unter Win16).
// Hierachy: 
//     B_Exception <- B_ResourceError <- B_OutOfMem
// Inheritance:  
//     Methods (public): operator=(), what()
// Errorhandling:
// Note:  
//     entspricht LZF(7) "Out of memory" in VB. Fehlermeldung: "Out of memory"
// Impl.:
//
class DLL_DECL B_OutOfMem : public B_ResourceError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    //     in whatArg angeben in welcher Funktion und wie gross angeforderte Speicherblock war
    B_OutOfMem(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_OutOfMem(const B_OutOfMem&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_OutOfMem() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_OutOfMem& operator=(const B_OutOfMem& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_OutOfMem

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_DiskFull Es ist kein Speicherblock der geforderten groesse mehr auf dem Datentraeger mehr verfuegbar
//     ist.
// Hierachy: 
//     B_Exception <- B_ResourceError <- B_DiskFull
// Inheritance:  
//     Methods (public): operator=(), what()
// Errorhandling:
// Note:  
//     entspricht LZF(61) "Disk full" in VB. Fehlermeldung: "Data medium full"
// Impl.:
//
class DLL_DECL B_DiskFull : public B_ResourceError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Angabe bei welcher Fkt und wieviel momentan bei abgebr. Funktion noch zur Verfuegung steht 
    //     (bei Dos auch partition angegben)
    B_DiskFull(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_DiskFull(const B_DiskFull&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_DiskFull() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_DiskFull& operator=(const B_DiskFull& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_DiskFull

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_DeviceUnavailable  Das Device/Medium ist nicht verfuegbar, existiert nicht oder ist nicht aktiv
// Hierachy: 
//     B_Exception <- B_ResourceError <- B_DeviceUnavailable
// Inheritance:  
//     Methods (public): operator=(), what()
// Errorhandling:
// Note:  
//     entspricht LZF(68) "Device unavailable" in VB. Fehlermeldung: "Device unavailable"
// Impl.:
//
class DLL_DECL B_DeviceUnavailable : public B_ResourceError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    //     in whatArg Geraet mit reinschreiben zumindestens Laufwerksbuchstaben/Mountpoint wenn moelich auch
    //     Typ z.B. CD-ROM, Tape Floppy usw.
    B_DeviceUnavailable(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_DeviceUnavailable(const B_DeviceUnavailable&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_DeviceUnavailable() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_DeviceUnavailable& operator=(const B_DeviceUnavailable& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_DeviceUnavailable


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_DiskNotReady Ein Diskettenlaufwerk (3.5", 5.2") enthaelt keine Diskette oder wurde nicht verriegelt.    
// Hierachy: 
//     B_Exception <- B_RessourceError <- B_DeviceUnavailabe <- B_DiskNotReady
// Inheritance:  
//     Methods (public): operator=(), what()
// Errorhandling:
// Note:  
//     entspricht LZF(71) "Disk not ready" in VB. Dieser Fehler ist nur in der konverterLib vorhanden und
//     wurd mit aus diesem Grunde von B_DeviceUnavailable abgeleitet. Fehlermeldung: "(Floppy-) Disk not ready"
// Impl.:
//     Bezieht sich in VB nur auf 3,5 und 5,25 Floppy LW deshalb in EH Lib rausnehmen
//
class DLL_DECL B_DiskNotReady : public B_DeviceUnavailable
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    //     in whatArg Geraet mit reinschreiben zumindestens Laufwerksbuchstaben/Mountpoint wenn moelich auch
    //     Typ z.B. CD-ROM, Tape Floppy usw.
    B_DiskNotReady(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_DiskNotReady(const B_DiskNotReady&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_DiskNotReady() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_DiskNotReady& operator=(const B_DiskNotReady& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_DiskNotReady

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_TooManyFiles  Es sind zuviele Dateien gleichzeitig geoeffnet
// Hierachy: 
//     B_Exception <- B_RessourceError <- B_TooManyFiles
// Inheritance:  
//     Methods (public):  operator=(),  what()
// Errorhandling:
// Note:  
//     entspricht LZF(67) "Too many files" in VB. Fehlermeldung: "Too many open files"
// Impl.:
//
class DLL_DECL B_TooManyFiles : public B_ResourceError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Angeben wieviel Files offen sind und wenn moeglich Wert von files erfragen bei OS und auch
    //     zurueckgeben
    B_TooManyFiles(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_TooManyFiles(const B_TooManyFiles&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_TooManyFiles() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_TooManyFiles& operator=(const B_TooManyFiles& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_TooManyFiles


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_DeviceIOError  Es sind zuviele Dateien gleichzeitig geoeffnet
// Hierachy: 
//     B_Exception <- B_RessourceError <- B_DeviceIOError
// Inheritance:  
//     Methods (public):  operator=(),  what()
// Errorhandling:
// Note:  
//     entspricht LZF(57) "Device I/O Error" in VB. Fehlermeldung: "Device I/O Error"
// Impl.:
//
class DLL_DECL B_DeviceIOError : public B_ResourceError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Angeben wieviel Files offen sind und wenn moeglich Wert von files erfragen bei OS und auch
    //     zurueckgeben
    B_DeviceIOError(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_DeviceIOError(const B_DeviceIOError&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_DeviceIOError() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_DeviceIOError& operator=(const B_DeviceIOError& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_DeviceIOError






/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_FatalError  Basisklasser der Kategorie fuer Bibliotheksfehler
// Hierachy: 
//     B_Exception <- B_FatalError
// Inheritance:  
//     Methods (public):   operator=(),  what()
// Errorhandling: 
// Note:  
//     keine Entsprechung in VB. Fehlermeldung: "Fatal Error (category)"
// Impl.:
//
class DLL_DECL B_FatalError : public B_Exception
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Angeben wieviel Files offen sind und wenn moeglich Wert von files erfragen bei OS und auch
    //     zurueckgeben
    B_FatalError(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_FatalError(const B_FatalError&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_FatalError() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_FatalError& operator=(const B_FatalError& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_FatalError

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_InternalError  Klasse fuer Bibliothekinterne Fehler die das Programm in einen inkonsitenten Zustand 
//     versetzen
// Hierachy: 
//     B_Exception <- B_FatalError <- B_InternalError
// Inheritance:  
//     Methods (public):  operator=(),  what()
// Errorhandling: 
// Note:  
//     entsprechricht in VB annaehernd LZF(51) "Internal Error" nur dass er hier kein VB sondern ein 
//     Bibliotheksfehler der Homer-Lib. Fehlermeldung: "Internal Lib Error"
// Impl.:
//
class DLL_DECL B_InternalError : public B_FatalError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Interner Bibliotheksfehler, CiceroSoft eigene Fehlercode vergeben und mitgegeben.
    B_InternalError(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_InternalError(const B_InternalError&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_InternalError() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_InternalError& operator=(const B_InternalError& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_InternalError

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_ContractError  Wenn ein Verstoss gegen eine Pre-, Postcondition oder eine Assertion vorliegt.
// Hierachy: 
//     B_Exception <- B_FatalError <- B_ContractError
// Inheritance:  
//     Methods (public): operator=(), what()
// Errorhandling:
// Note:  
//     keine entspr. in VB. Fehlermeldung: "Design by Contract violation"
// Impl.:
//
class DLL_DECL B_ContractError : public B_FatalError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    //     whatArg kommt die Post, Pre- oder Assertion die verletzt wurde rein.
    B_ContractError(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_ContractError(const B_ContractError&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_ContractError() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_ContractError& operator=(const B_ContractError& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_ContractError


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_CriticalError  Wenn ein Bibliotheksinterner Fehler vorliegt der, im Gegensatz zu InternalError's, 
//                      durch den Anwender wieder behoben werden ohne die Bibliothek in einem inkonsistenten
//                      Zustand zu lassen.
// Hierachy: 
//     B_Exception <- B_FatalError <- B_CriticalError
// Inheritance:  
//     Methods (public): operator=(),  what()
// Errorhandling:
// Note:  
//     keine entspr. in VB. Die Herstellung der Konsistenz kann nur durch den Anwender erfolgen die 
//     Bibliothek ist zu solchen Massnahmen nicht in der Lage.
//     Fehlermeldung: "Critical Lib error"
// Impl.:
//    nur in enhancedlib zur Unterscheidung von FatalErrors, bei CriticalError kann das
//    System durch den Anwedner nochmal in einen Konsistenten zustand gebracht werden
//    bei einem Fatal, Internal, Contract (post und assertion) ist dies nicht mehr moeglich
//    oder fuer den Anwender nicht entscheidbar.
//
class DLL_DECL B_CriticalError : public B_FatalError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_CriticalError(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_CriticalError(const B_CriticalError&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_CriticalError() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_CriticalError& operator=(const B_CriticalError& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_CriticalError


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_CollectionError Basisklasse der Kategorie fuer Collection Fehler (Array, Map, Set, Bag...)
// Hierachy: 
//     B_Exception <- B_CollectionError
// Inheritance:  
//     Methods (public):  operator=(),  what()
// Errorhandling:
// Note:  
//     keine entspr. in VB. Fehlermeldung: "Collection Error (category)"
// Impl.:
//
class DLL_DECL B_CollectionError : public B_Exception
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_CollectionError(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_CollectionError(const B_CollectionError&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_CollectionError() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_CollectionError& operator=(const B_CollectionError& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_CollectionError

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_ArrayError  Klasse die alle Array-Fehler auffaengt
// Hierachy: 
//     B_Exception <- B_CollectionError <- B_ArrayError
// Inheritance:  
//     Methods (public): operator=(), what()
// Errorhandling:
// Note:  
//     Beeinhaltet VB LZF(10) "This array is temporaily locked". In enhancedLib auch sonstige spezielle
//     Array-Fehler, evtl. Bounds, die nicht de collections allg. zugeordnet werden koennen.
//     Fehlermeldung: "Array error"
// Impl.:
//
class DLL_DECL B_ArrayError : public B_CollectionError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_ArrayError(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_ArrayError(const B_ArrayError&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_ArrayError() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_ArrayError& operator=(const B_ArrayError& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_ArrayError

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_GetPutNot4Arr  Klasse die Get/Put Fehler im zusammenhang mit Save-Arrays abfaengt
// Hierachy: 
//     B_Exception <- B_CollectionError <- B_ArrayError <- B_GetPutNot4Arr
// Inheritance:  
//     Methods (public): operator=(),  what()
// Errorhandling:
// Note:  
//     Entspricht LZF(456) "Get and Put cannot be used with arrays in Variants" von VB. Ist in der 
//     enhancedLib nicht mehr vorhanden.
//     Fehlermeldung: "Get and Put cannot be used with arrays in Variants"
// Impl.:
//
class DLL_DECL B_GetPutNot4Arr : public B_ArrayError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_GetPutNot4Arr(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_GetPutNot4Arr(const B_GetPutNot4Arr&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_GetPutNot4Arr() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_GetPutNot4Arr& operator=(const B_GetPutNot4Arr& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_GetPutNot4Arr

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_KeyInUse  Klasse die Collectiondatenstrukturen auswerfen die eine Key verwenden, wenn dieser falsch 
//               ist (z.B. Hash-Tabellen)
// Hierachy: 
//     B_Exception <- B_CollectionError <- B_KeyInUse
// Inheritance:  
//     Methods (public):  operator=(), what()
// Errorhandling:
// Note:  
//     Beinhalteet auch LZF(457) "This Key is already associated with an element of this collection"
//     von VB. Fehlermeldung: "This Key is already associated with an element of this collection"
// Impl.:
//
class DLL_DECL B_KeyInUse : public B_CollectionError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_KeyInUse(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_KeyInUse(const B_KeyInUse&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_KeyInUse() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_KeyInUse& operator=(const B_KeyInUse& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_KeyInUse

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_ApplicationError  Basiskalsse fuer Kategorie der Benutzerfehler.
// Hierachy: 
//     B_Exception <- B_ApplicationError
// Inheritance:  
//     Methods (public):  operator=(),  what()
// Errorhandling: 
// Note:  
//     Beinhaltet LZF(??) "User Defined Error" (VB30) oder "Application-defined or object-defined error"
//     in VB40. Unter dieser Basisklasse kann der Anwender seine eigene Klassenhierachie fuer Aplikations-
//     spezfische Fehler aufbauen. Fehlermeldung: "Application-defined or object-defined error"
// Impl.:
//
class DLL_DECL B_ApplicationError : public B_Exception
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_ApplicationError(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_ApplicationError(const B_ApplicationError&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_ApplicationError() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_ApplicationError& operator=(const B_ApplicationError& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_ApplicationError


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_DirFileIOError   Basisklasse der Verzeichniss und Dateifehler Kategorie
// Hierachy: 
//     B_Exception <- B_DirFileIOError
// Inheritance:  
//     Methods (public):  operator=(), what()
// Errorhandling:
// Note:  
//     Fehlermeldung: "Directory or File IO-Error (category)"
// Impl.:
//     diese EH wird auch in enhancedLib sein alle abgeleiteten nur in konverterLib da diese
//     auf Errorhandling in VB aufbauen und nicht C++ ueblich sind
//
class DLL_DECL B_DirFileIOError : public B_Exception
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_DirFileIOError(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_DirFileIOError(const B_DirFileIOError&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_DirFileIOError() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_DirFileIOError& operator=(const B_DirFileIOError& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_DirFileIOError


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_BadFileNbr   Dateinummer oder Dateiname ungueltig
// Hierachy: 
//     B_Exception <- B_DirFileIOError <- B_BadFileNbr
// Inheritance:  
//     Methods (public):   operator=(),  what()
// Errorhandling:
// Note:  
//     Entspricht LZF(52) "Bad file name or number" in VB. Nicht in enhancedLib.
//     Fehlermeldung: "Bad file name or number"
// Impl.:
//
class DLL_DECL B_BadFileNbr : public B_DirFileIOError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_BadFileNbr(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_BadFileNbr(const B_BadFileNbr&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_BadFileNbr() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_BadFileNbr& operator=(const B_BadFileNbr& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_BadFileNbr

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//    B_FileNotFound   Die angegebene Datei konnte nicht gefunden werden
// Hierachy: 
//     B_Exception <- B_DirFileIOError <- B_FileNotFound
// Inheritance:  
//     Methods (public):  operator=(),  what()
// Errorhandling:
// Note:  
//     Entspricht LZF(53) "File not found" in VB. Nicht in enhancedLib. Fehlermeldung: "File not found"
// Impl.:
//    Entweder weil die datei unter den gemachten Angaben im BS nicht gefunden werden kann
//    Weil eine mit einer declare-Anweisung angegebene DLL-Funktion nicht gefunden werden kann
//    -> faellt bei uns weg, wenn wir intern einen haben dann rufen wir nur die DLLExcpt Fehler auf
//
class DLL_DECL B_FileNotFound : public B_DirFileIOError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_FileNotFound(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_FileNotFound(const B_FileNotFound&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_FileNotFound() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_FileNotFound& operator=(const B_FileNotFound& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_FileNotFound

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_BadFileMode  Es wurde versucht eine Datei imt einem ungueltigen Modus zu oeffnen
// Hierachy: 
//     B_Exception <- B_DirFileIOError <- B_BadFileMode
// Inheritance:  
//     Methods (public):  operator=(),   what()
// Errorhandling:
// Note:  
//     Entspricht LZF(54) "Bad file mode" in VB. Nicht in enhancedLib. Fehlermeldung: "Bad file mode"
// Impl.:
//     Datei wird mit einem falschen Dateimodus geoeffnet z.B. Get/Put auf eine sequentielle Datei
//
class DLL_DECL B_BadFileMode : public B_DirFileIOError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_BadFileMode(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_BadFileMode(const B_BadFileMode&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_BadFileMode() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_BadFileMode& operator=(const B_BadFileMode& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_BadFileMode

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_FileOpen  Wenn eine Datei bereits geoffnet war und versucht wurde sie unter einem anderen oder 
//                 gleichen Modus wieder zu oeffnen.
// Hierachy: 
//     B_Exception <- B_DirFileIOError <- B_FileOpen
// Inheritance:  
//     Methods (public): operator=(),  what()
// Errorhandling:
// Note:  
//     Entspricht LZF(55) "File already open" in VB. Nicht in enhancedLib. Fehlermeldung: "File already open"
// Impl.:
//
class DLL_DECL B_FileOpen : public B_DirFileIOError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_FileOpen(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true);// throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_FileOpen(const B_FileOpen&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_FileOpen() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_FileOpen& operator=(const B_FileOpen& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_FileOpen

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_FileExists  Tritt zur Laufzeit auf, wenn der neue Dateiname mit einem bereits bestehenden Dateinamen
//                   uebereinstimmen.
// Hierachy: 
//     B_Exception <- B_DirFileIOError <- B_FileExists
// Inheritance:  
//     Methods (public):  operator=(),  what()
// Errorhandling:
// Note:  
//     Entspricht LZF(58) "File already exists" in VB. Nicht in enhancedLib.
//     Fehlermeldung: "File already exists"
// Impl.:
//
class DLL_DECL B_FileExists : public B_DirFileIOError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_FileExists(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true);// throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_FileExists(const B_FileExists&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_FileExists() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_FileExists& operator=(const B_FileExists& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_FileExists


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_BadRecLength   Angegeben Laenge eines Datensatzes der mit Get/Put bearbeitet werden soll, stimmt 
//                         nicht mit der bei Open angegeben laenge ueberein. Die Variable in einer 
//                         Put-Anweisung ist, oder enthaelt, eine Zeichenfolge variabler Laenge
// Hierachy: 
//     B_Exception <- B_DirFileIOError <- B_BadRecLength
// Inheritance:  
//     Methods (public):  operator=(),   what()
// Errorhandling:
// Note:  
//     Entspricht LZF(59) "Bad record length" in VB. Nicht in enhancedLib. Fehlermeldung: "Bad record length"
// Impl.:
//
class DLL_DECL B_BadRecLength : public B_DirFileIOError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_BadRecLength(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_BadRecLength(const B_BadRecLength&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_BadRecLength() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_BadRecLength& operator=(const B_BadRecLength& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_BadRecLength

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_InputPastEOF  Input liest aus einer Datei die leer ist oder aus der schon alle Daten gelesen wurden
// Hierachy: 
//     B_Exception <- B_DirFileIOError <- B_InputPastEOF
// Inheritance:  
//     Methods (public):  operator=(),   what()
// Errorhandling:
// Note:  
//     Entspricht LZF(62) "Input past end of file" in VB. Nicht in enhancedLib.
//     Fehlermeldung: "Input past end of file"
// Impl.:
//
class DLL_DECL B_InputPastEOF : public B_DirFileIOError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_InputPastEOF(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_InputPastEOF(const B_InputPastEOF&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_InputPastEOF() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_InputPastEOF& operator=(const B_InputPastEOF& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_InputPastEOF

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_BadRecNbr  Der Datensatz in Get/Put ist kleiner oder gelich Null
// Hierachy: 
//     B_Exception <- B_DirFileIOError <- B_BadRecNbr
// Inheritance:  
//     Methods (public):   operator=().  what()
// Errorhandling:
// Note:  
//     Entspricht LZF(63) "Bad record number" in VB. Nicht in enhancedLib.
//     Fehlermeldung: "Bad record number"
// Impl.:
//     
class DLL_DECL B_BadRecNbr : public B_DirFileIOError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_BadRecNbr(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_BadRecNbr(const B_BadRecNbr&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_BadRecNbr() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_BadRecNbr& operator=(const B_BadRecNbr& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_BadRecNbr


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_BadFileName  Wenn der Dateiname (Pfad) nicht den konventionen fuer eine gueltigen Dateinamen
//                    entspricht.
// Hierachy: 
//     B_Exception <- B_DirFileIOError <- B_BadFileName
// Inheritance:  
//     Methods (public):   operator=(),  what()
// Errorhandling:
// Note:  
//     Entspricht LZF(64) "?" in VB. Nicht in enhancedLib.  Fehlermeldung: "File name is bad"
// Impl.:
//     keine Beschreibung bei VB -> rausgefunden ueber Abfrage aller moelgichne Fehlermedlungen
//     Fehler auswerfen wenn Slash/Backslach konflikt, Sonderzeichen  usw.
//
class DLL_DECL B_BadFileName : public B_DirFileIOError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_BadFileName(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_BadFileName(const B_BadFileName&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_BadFileName() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_BadFileName& operator=(const B_BadFileName& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_BadFileName


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_PermissionDenied  Es wurde versucht auf einen schreibgeschuetzten Datentraeger zu schreiben oder es
//                         wird versucht auf eine gesperrte Datei zuzugreifen
// Hierachy: 
//     B_Exception <- B_DirFileIOError <- B_PermissionDenied
// Inheritance:  
//     Methods (public):  operator=(), what()
// Errorhandling:
// Note:  
//     Entspricht LZF(70) "Permission denied" in VB. Nicht in enhancedLib.
//     Fehlermeldung: "Permission denied"
// Impl.:
//
class DLL_DECL B_PermissionDenied : public B_DirFileIOError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_PermissionDenied(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_PermissionDenied(const B_PermissionDenied&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_PermissionDenied() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_PermissionDenied& operator=(const B_PermissionDenied& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_PermissionDenied

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_DifferentDriveError  Es wurde versucht mit der Name Anweisung eine Datei auf ein anderes Laufwerk 
//                            zu verschieben dies geht jedoch nur auf gl. LW (nur mit FileCopy und Kill 
//                            moeglich).
// Hierachy: 
//     B_Exception <- B_DirFileIOError <- B_DifferentDriveError
// Inheritance:  
//     Methods (public):  operator=(),  what()
// Errorhandling:
// Note:  
//     Entspricht LZF(74) "Can't rename with different drive" in VB. Nicht in enhancedLib.
//     Fehlermeldung: "Can't rename with different drive"
// Impl.:
//
class DLL_DECL B_DifferentDriveError : public B_DirFileIOError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_DifferentDriveError(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_DifferentDriveError(const B_DifferentDriveError&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_DifferentDriveError() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_DifferentDriveError& operator=(const B_DifferentDriveError& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_DifferentDriveError

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_BadAccess   Beim Zugriff auf eine Datei oder LAufwerk (z.B. mit Open, MkDir,ChDir oder RmDir) 
//                   konnte das BS keine Verbindung zwischen der Pfadangabe und dem Dateinamen herstellen.
// Hierachy: 
//     B_Exception <- B_DirFileIOError <- B_BadPathFileAccess
// Inheritance:  
//     Methods (public):  operator=(),  what()
// Errorhandling:
// Note:  
//     Entspricht LZF(75) "Path/File access error" in VB. Nicht in enhancedLib.
//     Fehlermeldung: "Path/File access error"
// Impl.:
//
class DLL_DECL B_BadPathFileAccess : public B_DirFileIOError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_BadPathFileAccess(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_BadPathFileAccess(const B_BadPathFileAccess&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_BadPathFileAccess() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_BadPathFileAccess& operator=(const B_BadPathFileAccess& rhs);
  
  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_BadPathFileAccess

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_PathNotFound  Beim Zugriff auf eine Datei oder ein Laufwerk konnte das BS den angegebenen Pfad
//                     nicht finden. Wenn der Pfad verkehrt waere waere es BadFileName Error
// Hierachy: 
//     B_Exception <- B_DirFileIOError <- B_PathNotFound
// Inheritance:  
//     Methods (public): operator=(),   what()
// Errorhandling:
// Note:  
//     Entspricht LZF(76) "Path not found" in VB. Nicht in enhancedLib.
//     Fehlermeldung: "Path not found"
// Impl.:
//
class DLL_DECL B_PathNotFound : public B_DirFileIOError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_PathNotFound(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_PathNotFound(const B_PathNotFound&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_PathNotFound() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_PathNotFound& operator=(const B_PathNotFound& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_PathNotFound


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_DllError  Basisklasse der Dll-Exception Kategorie
// Hierachy: 
//     B_Exception <- B_DllError
// Inheritance:  
//     Methods (public): operator=(),  what()
// Errorhandling:
// Note:  
//     Keine Entpsrechung in VB. Fehlermeldung: "DLL error (category)"
// Impl.:
//
class DLL_DECL B_DllError : public B_Exception
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_DllError(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_DllError(const B_DllError&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_DllError() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_DllError& operator=(const B_DllError& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_DllError

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_TooManyClients  Es greifen zuviele Host Anwendungen gleichzeitig auf die Dll zu (in VB max. 50)
// Hierachy: 
//     B_Exception <- B_DllError <- B_TooManyClients
// Inheritance:  
//     Methods (public):  operator=(),  what()
// Errorhandling:
// Note:  
//     Entpricht LZF(47) "Too many DLL application clients".
//     Fehlermeldung: "Too many DLL application clients"
// Impl.:
//
class DLL_DECL B_TooManyClients : public B_DllError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_TooManyClients(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_TooManyClients(const B_TooManyClients&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_TooManyClients() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_TooManyClients& operator=(const B_TooManyClients& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_TooManyClients

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_LoadDllError  Die Datei ist keine ausfhrbare Datei oder keine Windows Dll, eine alte Win Dll 
//                      die nicht im Protected Mode laufen kann oder es wurde keine Pascal-Aufrufkonvention
//                      eingehalten oder die Argumentanzahl stimmt nicht mit der in der declare Anweisung
//                      ueberein.
// Hierachy: 
//     B_Exception <- B_DllError <- B_LoadDllError
// Inheritance:  
//     Methods (public): operator=(),  what()
// Errorhandling: 
// Note:  
//     Entpricht LZF(48) "Error in loading DLL".
//     Fehlermeldung:  "Error in loading DLL"
// Impl.:
//
class DLL_DECL B_LoadDllError : public B_DllError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_LoadDllError(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_LoadDllError(const B_LoadDllError&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_LoadDllError() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_LoadDllError& operator=(const B_LoadDllError& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_LoadDllError

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_BadDllCall  Der Dll-Funktion wurde ein falscher Parametertyp oder eine falsche Parameteranzahl
//                   uebergeben
// Hierachy: 
//     B_Exception <- B_DllError <- B_BadDllCall
// Inheritance:  
//     Methods (public):   operator=(),  what()
// Errorhandling:
// Note:  
//     Entpricht LZF(49) "Bad DLL calling convention".
//     Fehlermeldung:   "Bad DLL calling convention".
// Impl.:
//
class DLL_DECL B_BadDllCall : public B_DllError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_BadDllCall(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_BadDllCall(const B_BadDllCall&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_BadDllCall() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_BadDllCall& operator=(const B_BadDllCall& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_BadDllCall

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_InvalidOrdinal   Die Ordnungzahl der Dll-Funktion ist falsch oder konnte aus dem uebergeben Ausdruck
//                        nicht ermittelt werden.
// Hierachy: 
//     B_Exception <- B_DllError <- B_InvalidOrdinal
// Inheritance:  
//     Methods (public): operator=(),  what()
// Errorhandling:
// Note:  
//     Entpricht LZF(452) "Invalid Ordinal".
//     Fehlermeldung:   "Invalid DLL-Function ordinal"
// Impl.:
//
class DLL_DECL B_InvalidOrdinal : public B_DllError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_InvalidOrdinal(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_InvalidOrdinal(const B_InvalidOrdinal&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_InvalidOrdinal() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_InvalidOrdinal& operator=(const B_InvalidOrdinal& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_InvalidOrdinal

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_NoDllFunc  Die Dll konnte gefunden und aufgerufen werden jedoch nicht die angegebene Funktion 
//                  innerhalb der Dll
// Hierachy: 
//     B_Exception <- B_DllError <- B_NoDllFunc
// Inheritance:  
//     Methods (public):  operator=(),  what()
// Errorhandling:
// Note:  
//     Entpricht LZF(453) "Specified DLL function not found".
//     Fehlermeldung:   "Specified DLL function not found"
// Impl.:
//
class DLL_DECL B_NoDllFunc : public B_DllError
  {
  //=================================== MANAGEMENT ==========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     ctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_NoDllFunc(B_cCStr  whatArg = "", const B_Boolean& traceFlag = B_true); // throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     cctor
    // Parameter:  
    //     String der an die Standardfehlermeldung der EH angehaengt wird, zur naeheren Beschreibung des
    //     Fehlers
    // Return Value:
    // Errorhandling:
    // Note:      
    B_NoDllFunc(const B_NoDllFunc&);

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    virtual ~B_NoDllFunc() throw();

    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Zuweisung ist hier normalerweise nicht sinnvoll. Aber in Draft erlaubt deshalb aus Kompatibilitaets-
    //     gruenden hier auch erlaubt.
    B_NoDllFunc& operator=(const B_NoDllFunc& rhs);

  //=================================== INTERFACE ===========================================================
  //
  public:
    //-------------------------------------------------------------------------------------------------------
    // Method:
    //     what()
    // Parameter:  
    //     EH-Message die sich aus Standardstring und whatArg des Konstruktors zusammensetzt
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Der Standardstring wird in der jeweiligen Landessprache in abhaengigkeit von B_Local::global()
    //     ausgegeben
    virtual B_CStr what() const throw();
  }; // B_NoDllFunc

//================================ Inlinefunktionen =========================================================
//
#ifdef _DEBUG
  #define B_NOINLINE
#else
  #undef B_NOINLINE
#endif

#ifndef B_NOINLINE                // festlegen wenn Debuger keine Inlines debuggen kann
  #define B_INLINE inline
  #ifndef BI_EXCPT_HPP
    #include "bi_excpt.hpp"
  #endif
#endif


//===========================================================================================================
//================================ End Of Header ============================================================
#endif // BC_EXCPT_HPP

