#ifdef B_VB_COMPATIBLE

#ifndef BC_INT_HPP
#define BC_INT_HPP
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Project:                 VB nach C/C++ Konverter Klassenbibliothek
// Modul:                   VB Klasse fuer VB Integer Datentyp
// Filename:                bc_int.hpp
// Version:                 1.0
// Status:                  beta
//
// Copyright:               Copyright (C) 1995
//                          Cicerosoft GmbH, 
//                          Ballweilerstr. 7,
//                          66399 Erfweiler-Ehlingen
//
// Operating System(s):     Win95
// Compiler:                VC++ 4.1
//
// Author:                  Jochen Diener JD
// Creation Date:           JD 14.12.95
// Last Modification:       JD 27.06.95
//
// Description:             Beinhaltet alle Klassen / Hilfsfkt. die im Zusammenhang mit der VB 
//                          Konvertierung, fuer die Bereitstellung des Datentyps Byte benoetigt 
//                          werden. 
//                          
// Dependencies:            bc_int.cpp    ->  Implementierung der Member/Hilfsfkt.
//                          bi_int.hpp    ->  Inlinefkt. der Klasse
//
// Remarks:                 Die Beschreibung zu den Klassenmethoden werden so kurz als moeglich
//                          gehalten um die Uebersichtlichkeit zu erhoehen, Eine genauere Beschreibung
//                          laesst sich ggf. in den VB Handbuechern finden
//
// Revision(s):   
//                          
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

//================================= Defintionsabhaengigkeiten ===============================================
//
#include <iostream.h>

#ifndef BD_INTYP_HPP       
  #include "bd_intyp.hpp"       // alias der Standardatentypen
#endif

#ifndef BC_BOOL_HPP
  #include "bc_bool.hpp"        // VB Boolean Typ
#endif

#ifndef BC_EMPTY_HPP
  #include "bc_empty.hpp"        // VB Boolean Typ
#endif

#ifndef BC_NULL_HPP
  #include "bc_null.hpp"        // VB Boolean Typ
#endif

#ifndef BC_BYTE_HPP
  #include "bc_byte.hpp"
#endif

//================================= Export =================================================================
//
class B_String;
class B_Currency;
class B_Double;
class B_Single;
class B_Long;
class B_fstream;						// CT131296 Fuer FileIO abgeleitete fstream-Klasse

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_Integer     Emuliert VB-Datentyp Integer
// Hierachy:    
// Inheritance: 
// Errorhandling:
// Note: 
//
class DLL_DECL B_Integer
  {
  //==================== MANGAEMENT =========================================================================
  //
  public:
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     ctor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Integer(); // Standardkonstruktor initialisiert Integer in VB auf 0

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     ctor's fuer Bibliotheksdatentypen
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Integer(const B_Boolean&);
    B_Integer(const B_Empty&); 
    B_Integer(const B_Null&);   // throw(B_InvalidNull);  
    B_Integer(const B_Currency&); // throw(B_Overflow)
    B_Integer(const B_String&);   // throw(B_Overflow)
    B_Integer(const B_Byte&);
    B_Integer(const B_Long&);     // throw(B_Overflow)
    B_Integer(const B_Single&);   // throw(B_Overflow)
    B_Integer(const B_Double&);   // throw(B_Overflow)

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     ctor's fuer Standarddatentypen
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Integer(const B_Int, const B_Explicit&); // throw(B_Overflow);
    B_Integer(const B_UInt,const B_Explicit&); // throw(B_Overflow);
    B_Integer(const B_Lng, const B_Explicit&); // throw(B_Overflow);
    B_Integer(const B_Flt, const B_Explicit&); // throw(B_Overflow);
    B_Integer(const B_Dbl, const B_Explicit&); // throw(B_Overflow);
    B_Integer(B_cCStr,     const B_Explicit&); // throw(B_Overflow);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     cctor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Integer(const B_Integer&);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    ~B_Integer();

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Integer& operator=(const B_Integer&);


  //==================== INTERFACE ============================================================================
  //
  public:
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Zuweisungsop. fuer C-Standardatentypen
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Integer& operator=(const B_Int)   throw(B_Overflow);
    B_Integer& operator=(const B_UInt)  throw(B_Overflow);
    B_Integer& operator=(const B_Lng)   throw(B_Overflow);
    B_Integer& operator=(const B_Flt)   throw(B_Overflow);
    B_Integer& operator=(const B_Dbl)   throw(B_Overflow);
    B_Integer& operator=(B_cCStr)       throw(B_TypeMismatch, B_Overflow);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Zuweisungsop. fuer Bibliotheksdatentypen
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Integer& operator=(const B_Currency&) throw(B_Overflow);
    B_Integer& operator=(const B_String&)   throw(B_TypeMismatch, B_Overflow);    
    B_Integer& operator=(const B_Boolean&);
    B_Integer& operator=(const B_Empty&);  
    B_Integer& operator=(const B_Null&)     throw(B_InvalidNull);    
    B_Integer& operator=(const B_Byte&);
    B_Integer& operator=(const B_Long&)     throw(B_Overflow);
    B_Integer& operator=(const B_Single&)   throw(B_Overflow);
    B_Integer& operator=(const B_Double&)   throw(B_Overflow);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Addition des Wertes selbst mit einem anderen Wert. Entspricht x = x + y
    // Parameter:  
    //     Wert der hinzuaddiert werden soll (Entspricht y)
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Integer& operator+=(const B_Integer&) throw(B_Overflow);
    B_Integer& operator+=(const B_Byte&)    throw(B_Overflow);
    B_Integer& operator+=(const B_Boolean&) throw(B_Overflow);
    B_Integer& operator+=(const B_Empty&);
    B_Integer& operator+=(const B_Null&)    throw(B_InvalidNull);
    B_Integer& operator+=(const B_Int)      throw(B_InvalidArgument, B_Overflow);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Subtraktion des Wertes selbst mit einem anderen Wert. Entspricht x = x - y
    // Parameter:  
    //     Wert der subtrahiert werden soll (Entspricht y)
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Integer& operator-=(const B_Integer&) throw(B_Overflow);
    B_Integer& operator-=(const B_Byte&)    throw(B_Overflow);
    B_Integer& operator-=(const B_Boolean&) throw(B_Overflow);
    B_Integer& operator-=(const B_Empty&);
    B_Integer& operator-=(const B_Null&)    throw(B_InvalidNull); 
    B_Integer& operator-=(const B_Int)      throw(B_InvalidArgument, B_Overflow);
    
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Multiplikation des Wertes selbst mit einem anderen Wert. Entspricht x = x * y
    // Parameter:  
    //     Wert der multipliziert werden soll (Entspricht y)
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Integer& operator*=(const B_Integer&) throw(B_Overflow);
    B_Integer& operator*=(const B_Byte&)    throw(B_Overflow);
    B_Integer& operator*=(const B_Boolean&) throw(B_Overflow);
    B_Integer& operator*=(const B_Empty&);
    B_Integer& operator*=(const B_Null&)    throw(B_InvalidNull);
    B_Integer& operator*=(const B_Int)      throw(B_InvalidArgument, B_Overflow);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Division des Wertes selbst mit einem anderen Wert. Entspricht x = x / y
    // Parameter:  
    //     Wert der dividiert werden soll (Entspricht y)
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Integer& operator/=(const B_Integer&) throw(B_ZeroDivide);
    B_Integer& operator/=(const B_Byte&)    throw(B_ZeroDivide);
    B_Integer& operator/=(const B_Boolean&) throw(B_ZeroDivide);
    B_Integer& operator/=(const B_Empty&)   throw(B_ZeroDivide);
    B_Integer& operator/=(const B_Null&)    throw(B_InvalidNull);
    B_Integer& operator/=(const B_Int)      throw(B_InvalidArgument, B_ZeroDivide);
    
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Division des Wertes selbst mit einem anderen Wert. Entspricht x = x / y
    // Parameter:  
    //     Wert der dividiert werden soll (Entspricht y)
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Integer& operator%=(const B_Integer&) throw(B_ZeroDivide);
    B_Integer& operator%=(const B_Byte&)    throw(B_ZeroDivide);
    B_Integer& operator%=(const B_Boolean&) throw(B_ZeroDivide);
    B_Integer& operator%=(const B_Empty&)   throw(B_ZeroDivide);
    B_Integer& operator%=(const B_Null&)    throw(B_InvalidNull);
    B_Integer& operator%=(const B_Int)      throw(B_InvalidArgument, B_ZeroDivide);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Unaeres Minus
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Integer operator-() const;

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Inkrement/dekrement des Objektes. Entspricht: x = x + 1;/ x = x -1
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Integer& operator++() throw(B_Overflow);
    B_Integer& operator--() throw(B_Overflow);
    
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Not -Operator
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Integer operator!() const;

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Konvetierungsoperator
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    operator B_Shrt() const;

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //      Wandelt den Wert des Objektes in einen Absoluten Wert um, d.h 
    //      negativer Objektwert -> positiv
    //      positiver Objektwert -> bleibt positiv
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Integer& abs();

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //      Gibt den Wert des Objektes als entsprechenden C/C++ Datentyp zurueck
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //    Den genauen Typ von B_Integer kann man in bd_intyp.hpp feststellen, in der alle
    //    C/C++ Datentypen definiert sind. Diese Funktion soll den Datenaustausch zwischen Klassen der Klassenbibliothek
    //    und den mit den C++ Standarddatentypen geschriebenen Funktionen ermoeglichen wenn diese keine Klassen als Parameter erlauben.
    B_Shrt getValue() const;

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Gibt die Groesse des Wert des Objektes als entsprechenden C/C++ Datentyp zurueck
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Den genauen Typ und somit seine groesse koennen zwischen den C/C++ Datentypen der
    //     Compiler unterschiedlich sein, deshalb wurde diese Funktion eingefuegt die eine
    //     portable Programmierung aehnlich sizeof() ermoeglichen soll.
    B_Lng getValueSize() const;  
    
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Gibt die Groesse des Objktes zurueck, diese entspricht der durch das Objekt belegten Speicherplatz
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //     Den genauen Typ und somit seine groesse koennen zwischen den C/C++ Compiler 
    //     unterschiedlich sein. Auch koennte sich die Klassenimplementierung aendern so
    //     dass aeltere Programme nur mit Aenderungsprogramm lauffaehig waeren, deshalb 
    //     wurde diese Funktion eingefuegt die eine portable Programmierung aehnlich 
    //     sizeof() ermoeglichen soll.
    B_Lng getSize() const;       

    //----------------------------------------------------------------------------------------------------       
    // Method: 
    //     Setzt den internen Status des Objektes auf optional. Somit kann mit isMissing der Status
    //     abgefragt werden und die Emulation des gleichnamigen VB Befehls wird moeglich.
    // Return Value: 
    // Note: 
    //
    B_Integer& isOptional();

    //----------------------------------------------------------------------------------------------------       
    // Method: 
    //     Abfragen des internen Objektstatus ob das Argument einer Funktion optional ist oder nicht
    // Return Value: 
    // Note: 
    //
    B_Boolean isMissing() const;

    //----------------------------------------------------------------------------------------------------       
    // Method: 
    //     Lesen der der eigentlichen Objektdaten im binaeren Format in einen Stream
    // Return Value: 
    //     B_true  ==  Objekt und Stream sind gueltig
    //     B_false ==  Objekt oder Stream sind ungueltig
    // Note: 
    //
    B_Boolean streamr(istream& istr, B_Shrt openmode);

	//----------------------------------------------------------------------------------------------------       
    // Method: 
    //     Schreiben der der eigentlichen Objektdaten im binaeren Format in einen Stream
    // Return Value: 
    //     B_true  ==  Objekt und Stream sind gueltig
    //     B_false ==  Objekt oder Stream sind ungueltig
    // Note: 
    //
    B_Boolean streamw(ostream& ostr, B_Shrt openmode);
    
  private:
    B_Integer& operator++(int);
    B_Integer& operator--(int);

  //==================== MEMBER ===============================================================================    
  //
  private:
    B_Shrt value;  // da VB Wertebereich  -32.768 bis 32.767
    B_Flag optional;
  
  //==================== FRIENDS ============================================================================
  //
  friend class B_Byte;  
  friend class B_Long;
  friend class B_Single;
  friend class B_Double;
  friend class B_Currency;
  friend class B_String;

  friend DLL_DECL  B_Integer operator+(const B_Integer&, const B_Integer&);
  friend DLL_DECL  B_Integer operator+(const B_Empty&,   const B_Integer&);
  friend DLL_DECL  B_Integer operator+(const B_Integer&, const B_Empty&);
  friend DLL_DECL  B_Integer operator+(const B_Byte&,    const B_Integer&);
  friend DLL_DECL  B_Integer operator+(const B_Integer&, const B_Byte&);
  friend DLL_DECL  B_Integer operator+(const B_Integer&, const B_Boolean&);
  friend DLL_DECL  B_Integer operator+(const B_Boolean&, const B_Integer&);
  friend DLL_DECL  B_Integer operator+(const B_Int,      const B_Integer&);
  friend DLL_DECL  B_Integer operator+(const B_Integer&, const B_Int);

  friend DLL_DECL  B_Integer operator-(const B_Integer&, const B_Integer&);
  friend DLL_DECL  B_Integer operator-(const B_Empty&,   const B_Integer&);
  friend DLL_DECL  B_Integer operator-(const B_Integer&, const B_Empty&);
  friend DLL_DECL  B_Integer operator-(const B_Byte&,    const B_Integer&);
  friend DLL_DECL  B_Integer operator-(const B_Integer&, const B_Boolean&);
  friend DLL_DECL  B_Integer operator-(const B_Boolean&, const B_Integer&);
  friend DLL_DECL  B_Integer operator-(const B_Integer&, const B_Byte&);
  friend DLL_DECL  B_Integer operator-(const B_Int,      const B_Integer&);
  friend DLL_DECL  B_Integer operator-(const B_Integer&, const B_Int);

  friend DLL_DECL  B_Integer operator*(const B_Integer&, const B_Integer&);
  friend DLL_DECL  B_Integer operator*(const B_Empty&,   const B_Integer&);
  friend DLL_DECL  B_Integer operator*(const B_Integer&, const B_Empty&);
  friend DLL_DECL  B_Integer operator*(const B_Byte&,    const B_Integer&);
  friend DLL_DECL  B_Integer operator*(const B_Integer&, const B_Boolean&);
  friend DLL_DECL  B_Integer operator*(const B_Boolean&, const B_Integer&);
  friend DLL_DECL  B_Integer operator*(const B_Integer&, const B_Byte&);
  friend DLL_DECL  B_Integer operator*(const B_Int,      const B_Integer&);
  friend DLL_DECL  B_Integer operator*(const B_Integer&, const B_Int);

  friend DLL_DECL  B_Integer operator%(const B_Integer&, const B_Integer&);
  friend DLL_DECL  B_Integer operator%(const B_Empty&,   const B_Integer&);
  friend DLL_DECL  B_Integer operator%(const B_Integer&, const B_Empty&);
  friend DLL_DECL  B_Integer operator%(const B_Byte&,    const B_Integer&);
  friend DLL_DECL  B_Integer operator%(const B_Integer&, const B_Byte&);
  friend DLL_DECL  B_Integer operator%(const B_Integer&, const B_Boolean&);
  friend DLL_DECL  B_Integer operator%(const B_Boolean&, const B_Integer&);
  friend DLL_DECL  B_Integer operator%(const B_Empty&,   const B_Integer&);
  friend DLL_DECL  B_Integer operator%(const B_Int,      const B_Integer&);
  friend DLL_DECL  B_Integer operator%(const B_Integer&, const B_Int);

  friend DLL_DECL  B_Integer B_Div(const B_Integer&, const B_Integer&);
  friend DLL_DECL  B_Integer B_Div(const B_Empty&,   const B_Integer&);
  friend DLL_DECL  B_Integer B_Div(const B_Integer&, const B_Empty&);
  friend DLL_DECL  B_Integer B_Div(const B_Byte&,    const B_Integer&);
  friend DLL_DECL  B_Integer B_Div(const B_Integer&, const B_Byte&);
  friend DLL_DECL  B_Integer B_Div(const B_Integer&, const B_Boolean&);
  friend DLL_DECL  B_Integer B_Div(const B_Boolean&, const B_Integer&);
  friend DLL_DECL  B_Integer B_Div(const B_Empty&,   const B_Integer&);
  friend DLL_DECL  B_Integer B_Div(const B_Int,      const B_Integer&);
  friend DLL_DECL  B_Integer B_Div(const B_Integer&, const B_Int);

  friend DLL_DECL  B_Integer operator&& (const B_Integer&, const B_Integer&);
  friend DLL_DECL  B_Integer operator&& (const B_Byte&,    const B_Integer&);
  friend DLL_DECL  B_Integer operator&& (const B_Integer&, const B_Byte&);
  friend DLL_DECL  B_Integer operator&& (const B_Int,      const B_Integer&);
  friend DLL_DECL  B_Integer operator&& (const B_Integer&, const B_Int);

  friend DLL_DECL  B_Integer operator|| (const B_Integer&, const B_Integer&);
  friend DLL_DECL  B_Integer operator|| (const B_Byte&,    const B_Integer&);
  friend DLL_DECL  B_Integer operator|| (const B_Integer&, const B_Byte&);
  friend DLL_DECL  B_Integer operator|| (const B_Int,      const B_Integer&);
  friend DLL_DECL  B_Integer operator|| (const B_Integer&, const B_Int);

  friend DLL_DECL  B_Integer operator^ (const B_Integer&, const B_Integer&);
  friend DLL_DECL  B_Integer operator^ (const B_Byte&,    const B_Integer&);
  friend DLL_DECL  B_Integer operator^ (const B_Integer&, const B_Byte&);
  friend DLL_DECL  B_Integer operator^ (const B_Boolean&, const B_Integer&);
  friend DLL_DECL  B_Integer operator^ (const B_Integer&, const B_Boolean&);
  friend DLL_DECL  B_Integer operator^ (const B_Int,      const B_Integer&);
  friend DLL_DECL  B_Integer operator^ (const B_Integer&, const B_Int);

  friend DLL_DECL  B_Boolean operator==(const B_Integer&, const B_Integer&);
  friend DLL_DECL  B_Boolean operator==(const B_Byte&,    const B_Integer&);
  friend DLL_DECL  B_Boolean operator==(const B_Integer&, const B_Byte&);
  friend DLL_DECL  B_Boolean operator==(const B_Empty&,   const B_Integer&);
  friend DLL_DECL  B_Boolean operator==(const B_Integer&, const B_Empty&);
  friend DLL_DECL  B_Boolean operator==(const B_Boolean&, const B_Integer&);
  friend DLL_DECL  B_Boolean operator==(const B_Integer&, const B_Boolean&);
  friend DLL_DECL  B_Boolean operator==(const B_Int,      const B_Integer&);
  friend DLL_DECL  B_Boolean operator==(const B_Integer&, const B_Int);

  friend DLL_DECL  B_Boolean operator!=(const B_Integer&, const B_Integer&);
  friend DLL_DECL  B_Boolean operator!=(const B_Byte&,    const B_Integer&);
  friend DLL_DECL  B_Boolean operator!=(const B_Integer&, const B_Byte&);
  friend DLL_DECL  B_Boolean operator!=(const B_Empty&,   const B_Integer&);
  friend DLL_DECL  B_Boolean operator!=(const B_Integer&, const B_Empty&);
  friend DLL_DECL  B_Boolean operator!=(const B_Boolean&, const B_Integer&);
  friend DLL_DECL  B_Boolean operator!=(const B_Integer&, const B_Boolean&);
  friend DLL_DECL  B_Boolean operator!=(const B_Int,      const B_Integer&);
  friend DLL_DECL  B_Boolean operator!=(const B_Integer&, const B_Int);

  friend DLL_DECL  B_Boolean operator<(const B_Integer&, const B_Integer&);
  friend DLL_DECL  B_Boolean operator<(const B_Byte&,    const B_Integer&);
  friend DLL_DECL  B_Boolean operator<(const B_Integer&, const B_Byte&);
  friend DLL_DECL  B_Boolean operator<(const B_Empty&,   const B_Integer&);
  friend DLL_DECL  B_Boolean operator<(const B_Integer&, const B_Empty&);
  friend DLL_DECL  B_Boolean operator<(const B_Boolean&, const B_Integer&);
  friend DLL_DECL  B_Boolean operator<(const B_Integer&, const B_Boolean&);
  friend DLL_DECL  B_Boolean operator<(const B_Int,      const B_Integer&);
  friend DLL_DECL  B_Boolean operator<(const B_Integer&, const B_Int);

  friend DLL_DECL  B_Boolean operator<=(const B_Integer&, const B_Integer&);
  friend DLL_DECL  B_Boolean operator<=(const B_Byte&,    const B_Integer&);
  friend DLL_DECL  B_Boolean operator<=(const B_Integer&, const B_Byte&);
  friend DLL_DECL  B_Boolean operator<=(const B_Empty&,   const B_Integer&);
  friend DLL_DECL  B_Boolean operator<=(const B_Integer&, const B_Empty&);
  friend DLL_DECL  B_Boolean operator<=(const B_Boolean&, const B_Integer&);
  friend DLL_DECL  B_Boolean operator<=(const B_Integer&, const B_Boolean&);
  friend DLL_DECL  B_Boolean operator<=(const B_Int,      const B_Integer&);
  friend DLL_DECL  B_Boolean operator<=(const B_Integer&, const B_Int);

  friend DLL_DECL  B_Boolean operator>(const B_Integer&, const B_Integer&);
  friend DLL_DECL  B_Boolean operator>(const B_Byte&,    const B_Integer&);
  friend DLL_DECL  B_Boolean operator>(const B_Integer&, const B_Byte&);
  friend DLL_DECL  B_Boolean operator>(const B_Empty&,   const B_Integer&);
  friend DLL_DECL  B_Boolean operator>(const B_Integer&, const B_Empty&);
  friend DLL_DECL  B_Boolean operator>(const B_Boolean&, const B_Integer&);
  friend DLL_DECL  B_Boolean operator>(const B_Integer&, const B_Boolean&);
  friend DLL_DECL  B_Boolean operator>(const B_Int,      const B_Integer&);
  friend DLL_DECL  B_Boolean operator>(const B_Integer&, const B_Int);

  friend DLL_DECL  B_Boolean operator>=(const B_Integer&, const B_Integer&);
  friend DLL_DECL  B_Boolean operator>=(const B_Byte&,    const B_Integer&);
  friend DLL_DECL  B_Boolean operator>=(const B_Integer&, const B_Byte&);
  friend DLL_DECL  B_Boolean operator>=(const B_Empty&,   const B_Integer&);
  friend DLL_DECL  B_Boolean operator>=(const B_Integer&, const B_Empty&);
  friend DLL_DECL  B_Boolean operator>=(const B_Boolean&, const B_Integer&);
  friend DLL_DECL  B_Boolean operator>=(const B_Integer&, const B_Boolean&);
  friend DLL_DECL  B_Boolean operator>=(const B_Int,      const B_Integer&);
  friend DLL_DECL  B_Boolean operator>=(const B_Integer&, const B_Int);

  //----------------------------------------------------------------------------------------------------       
  // Method: operator<<	INSERTION
  //     Schreiben der eigentlichen Objektdaten im Textformat in einen Stream
  // Return Value: 
  //	   Instanz der abgeleiten fstream-Klasse der Datei
  // Note: 
  //	   CT131296
  //	   ersetzt die alte Methode streamp()
  //
  friend DLL_DECL  B_fstream& operator<< (B_fstream& ostr, B_Integer& rhs);

  //----------------------------------------------------------------------------------------------------       
  // Method: operator>>	EXTRACTION
  //     Lesen der eigentlichen Objektdaten im Textformat aus einem Stream
  // Return Value: 
  //	   Instanz der abgeleiten fstream-Klasse der Datei
  // Note: 
  //	   CT131296
  //	   ersetzt die alte Methode streamg()
  //
  friend DLL_DECL  B_fstream& operator>> (B_fstream& istr, B_Integer& rhs);
  }; // B_Integer

DLL_DECL extern const B_Integer  B_IntegerMin;
DLL_DECL extern const B_Integer  B_IntegerMax;


//----------------------------------Arithmetische Operatoren -----------------------------------------

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der +Operator = Addition
DLL_DECL B_Integer  operator+ (const B_Integer&,   const B_Integer&) throw(B_Overflow);
DLL_DECL B_Integer  operator+ (const B_Int,        const B_Integer&) throw(B_Overflow);
DLL_DECL B_Integer  operator+ (const B_Integer&,   const B_Int)      throw(B_Overflow);

DLL_DECL B_Integer  operator+ (const B_Byte&,      const B_Integer&) throw(B_Overflow);
DLL_DECL B_Integer  operator+ (const B_Integer&,   const B_Byte&)    throw(B_Overflow);
DLL_DECL B_Integer  operator+ (const B_Integer&,   const B_Boolean&) throw(B_Overflow);
DLL_DECL B_Integer  operator+ (const B_Boolean&,   const B_Integer&) throw(B_Overflow);

DLL_DECL B_Integer  operator+ (const B_Empty&,     const B_Integer&); 
DLL_DECL B_Integer  operator+ (const B_Integer&,   const B_Empty&);   
DLL_DECL B_Null     operator+ (const B_Null&,      const B_Integer&);
DLL_DECL B_Null     operator+ (const B_Integer&,   const B_Null&);

DLL_DECL B_Integer  operator+ (const B_Boolean&,   const B_Boolean&);
DLL_DECL B_Integer  operator+ (const B_Byte&,      const B_Boolean&);
DLL_DECL B_Integer  operator+ (const B_Boolean&,   const B_Byte&);
DLL_DECL B_Integer  operator+ (const B_Empty&,     const B_Boolean&);
DLL_DECL B_Integer  operator+ (const B_Boolean&,   const B_Empty&);


DLL_DECL B_Integer  operator+ (const B_Int,        const B_Empty&);
DLL_DECL B_Integer  operator+ (const B_Empty&,     const B_Int);
DLL_DECL B_Integer  operator+ (const B_Int,        const B_Integer&) throw(B_Overflow);
DLL_DECL B_Integer  operator+ (const B_Integer&,   const B_Int)      throw(B_Overflow);
DLL_DECL B_Integer  operator+ (const B_Int,        const B_Byte&)    throw(B_Overflow);
DLL_DECL B_Integer  operator+ (const B_Byte&,      const B_Int)      throw(B_Overflow);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der -Operator = Subtraktion
DLL_DECL B_Integer  operator- (const B_Integer&,  const B_Integer&) throw(B_Overflow);
DLL_DECL B_Integer  operator- (const B_Byte&,     const B_Integer&) throw(B_Overflow);
DLL_DECL B_Integer  operator- (const B_Integer&,  const B_Byte&)    throw(B_Overflow);
DLL_DECL B_Integer  operator- (const B_Empty&,    const B_Integer&);
DLL_DECL B_Integer  operator- (const B_Integer&,  const B_Empty&);  
DLL_DECL B_Integer  operator- (const B_Boolean&,  const B_Boolean&);
DLL_DECL B_Integer  operator- (const B_Integer&,  const B_Boolean&) throw(B_Overflow);
DLL_DECL B_Integer  operator- (const B_Boolean&,  const B_Integer&) throw(B_Overflow);
DLL_DECL B_Null     operator- (const B_Null&,     const B_Integer&);
DLL_DECL B_Null     operator- (const B_Integer&,  const B_Null&);

DLL_DECL B_Integer  operator- (const B_Byte&,     const B_Boolean&);
DLL_DECL B_Integer  operator- (const B_Boolean&,  const B_Byte&);
DLL_DECL B_Integer  operator- (const B_Empty&,    const B_Boolean&);
DLL_DECL B_Integer  operator- (const B_Boolean&,  const B_Empty&);

DLL_DECL B_Integer  operator- (const B_Int,       const B_Empty&);
DLL_DECL B_Integer  operator- (const B_Empty&,    const B_Int);
DLL_DECL B_Integer  operator- (const B_Integer&,  const B_Int)      throw(B_Overflow);
DLL_DECL B_Integer  operator- (const B_Int,       const B_Integer&) throw(B_Overflow);
DLL_DECL B_Integer  operator- (const B_Byte&,     const B_Int)      throw(B_Overflow);
DLL_DECL B_Integer  operator- (const B_Int,       const B_Byte&)    throw(B_Overflow);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der *Operator = Multiplikation 
DLL_DECL B_Integer  operator* (const B_Integer&,  const B_Integer&)  throw(B_Overflow);
DLL_DECL B_Integer  operator* (const B_Byte&,     const B_Integer&)  throw(B_Overflow);
DLL_DECL B_Integer  operator* (const B_Integer&,  const B_Byte&)     throw(B_Overflow);
DLL_DECL B_Integer  operator* (const B_Empty&,    const B_Integer&);  
DLL_DECL B_Integer  operator* (const B_Integer&,  const B_Empty&);    
DLL_DECL B_Integer  operator* (const B_Integer&,  const B_Boolean&);
DLL_DECL B_Integer  operator* (const B_Boolean&,  const B_Integer&);
DLL_DECL B_Null     operator* (const B_Null&,     const B_Integer&);
DLL_DECL B_Null     operator* (const B_Integer&,  const B_Null&);

DLL_DECL B_Integer  operator* (const B_Boolean&,  const B_Boolean&);
DLL_DECL B_Integer  operator* (const B_Byte&,     const B_Boolean&);
DLL_DECL B_Integer  operator* (const B_Boolean&,  const B_Byte&);
DLL_DECL B_Integer  operator* (const B_Empty&,    const B_Boolean&);
DLL_DECL B_Integer  operator* (const B_Boolean&,  const B_Empty&);

DLL_DECL B_Integer  operator* (const B_Int,       const B_Empty&);
DLL_DECL B_Integer  operator* (const B_Empty&,    const B_Int);
DLL_DECL B_Integer  operator* (const B_Integer&,  const B_Int)       throw(B_Overflow);
DLL_DECL B_Integer  operator* (const B_Int,       const B_Integer&)  throw(B_Overflow);
DLL_DECL B_Integer  operator* (const B_Byte&,     const B_Int)       throw(B_Overflow);
DLL_DECL B_Integer  operator* (const B_Int,       const B_Byte&)     throw(B_Overflow);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der /Operator = Division deren Ergebniss immer ein Fliesskommawert ergibt
//   Fehler wenn Bereich von Single ueberschritten wird
DLL_DECL B_Null operator/ (const B_Null&,      const B_Integer&);
DLL_DECL B_Null operator/ (const B_Integer&,   const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der %Operator = Modulo Operation, gibt den Wert einer ganzahligen Division zurueck
//   entsprich MOD-Operator in VB
DLL_DECL B_Integer  operator% (const B_Integer&, const B_Integer&)  throw(B_ZeroDivide);
DLL_DECL B_Integer  operator% (const B_Byte&,    const B_Integer&)  throw(B_ZeroDivide);
DLL_DECL B_Integer  operator% (const B_Integer&, const B_Byte&)     throw(B_ZeroDivide);
DLL_DECL B_Integer  operator% (const B_Empty&,   const B_Integer&)  throw(B_ZeroDivide);  
DLL_DECL B_Integer  operator% (const B_Integer&, const B_Empty&)    throw(B_ZeroDivide);   
DLL_DECL B_Integer  operator% (const B_Boolean&, const B_Boolean&)  throw(B_ZeroDivide);
DLL_DECL B_Integer  operator% (const B_Integer&, const B_Boolean&)  throw(B_ZeroDivide);
DLL_DECL B_Integer  operator% (const B_Boolean&, const B_Integer&);
DLL_DECL B_Null     operator% (const B_Null&,    const B_Integer&);
DLL_DECL B_Null     operator% (const B_Integer&, const B_Null&);

DLL_DECL B_Integer  operator% (const B_Boolean&, const B_Boolean&)  throw(B_ZeroDivide);
DLL_DECL B_Integer  operator% (const B_Byte&,    const B_Boolean&)  throw(B_ZeroDivide);
DLL_DECL B_Integer  operator% (const B_Boolean&, const B_Byte&)     throw(B_ZeroDivide);
DLL_DECL B_Integer  operator% (const B_Empty&,   const B_Boolean&)  throw(B_ZeroDivide);
DLL_DECL B_Integer  operator% (const B_Boolean&, const B_Empty&)    throw(B_ZeroDivide);
DLL_DECL B_Integer  operator% (const B_Byte&,    const B_Empty&)    throw(B_ZeroDivide);
DLL_DECL B_Integer  operator% (const B_Empty&,   const B_Byte&)     throw(B_ZeroDivide);

DLL_DECL B_Integer  operator% (const B_Int,      const B_Empty&)    throw(B_ZeroDivide);
DLL_DECL B_Integer  operator% (const B_Empty&,   const B_Int)       throw(B_ZeroDivide);
DLL_DECL B_Integer  operator% (const B_Int,      const B_Integer&)  throw(B_ZeroDivide);
DLL_DECL B_Integer  operator% (const B_Integer&, const B_Int)       throw(B_ZeroDivide);
DLL_DECL B_Integer  operator% (const B_Byte&,    const B_Int)       throw(B_ZeroDivide);
DLL_DECL B_Integer  operator% (const B_Int,      const B_Byte&)     throw(B_ZeroDivide);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Ganzahldivision entspricht dem \Operator in VB, da in C++ kein geeigneter Operator zu Verfuegung steht
//   wird dieser Operator als Funktion realisiert
DLL_DECL B_Integer B_Div(const B_Integer&,  const B_Integer&)  throw(B_ZeroDivide);
DLL_DECL B_Integer B_Div(const B_Byte&,     const B_Integer&)  throw(B_ZeroDivide);
DLL_DECL B_Integer B_Div(const B_Integer&,  const B_Byte&)     throw(B_ZeroDivide);
DLL_DECL B_Integer B_Div(const B_Empty&,    const B_Integer&)  throw(B_ZeroDivide); 
DLL_DECL B_Integer B_Div(const B_Integer&,  const B_Empty&)    throw(B_ZeroDivide);   
DLL_DECL B_Integer B_Div(const B_Integer&,  const B_Boolean&)  throw(B_ZeroDivide);
DLL_DECL B_Integer B_Div(const B_Boolean&,  const B_Integer&)  throw(B_ZeroDivide);
DLL_DECL B_Null    B_Div(const B_Null&,     const B_Integer&);
DLL_DECL B_Null    B_Div(const B_Integer&,  const B_Null&);

DLL_DECL B_Integer B_Div(const B_Boolean&,  const B_Boolean&)  throw(B_ZeroDivide);
DLL_DECL B_Integer B_Div(const B_Empty&,    const B_Boolean&)  throw(B_ZeroDivide);
DLL_DECL B_Integer B_Div(const B_Boolean&,  const B_Empty&)    throw(B_ZeroDivide);
DLL_DECL B_Integer B_Div(const B_Byte&,     const B_Boolean&)  throw(B_ZeroDivide);
DLL_DECL B_Integer B_Div(const B_Boolean&,  const B_Byte&)     throw(B_ZeroDivide);
DLL_DECL B_Integer B_Div(const B_Byte&,     const B_Empty&)    throw(B_ZeroDivide); 
DLL_DECL B_Integer B_Div(const B_Empty&,    const B_Byte&)     throw(B_ZeroDivide);

DLL_DECL B_Integer B_Div(const B_Int,       const B_Empty&)    throw(B_ZeroDivide);
DLL_DECL B_Integer B_Div(const B_Empty&,    const B_Int)       throw(B_ZeroDivide);
DLL_DECL B_Integer B_Div(const B_Byte&,     const B_Int)       throw(B_ZeroDivide);
DLL_DECL B_Integer B_Div(const B_Int,       const B_Byte&)     throw(B_ZeroDivide);
DLL_DECL B_Integer B_Div(const B_Int,       const B_Integer&)  throw(B_ZeroDivide);
DLL_DECL B_Integer B_Div(const B_Integer&,  const B_Int)       throw(B_ZeroDivide);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Dient zum Potenzieren einer Zahl mit einem Exponenten
//   Erg = Zahl^Exponent  -> Erg = Pow(zahl, exponent)
//   Das Argument Zahl darf nur negativ sein, wenn Exponent ein ganzzahliger Wert ist. Werden in einem einzigen 
//   Ausdruck mehrere Potenzierungen ausgefhrt, so wird der Operator ^ jeweils in der Reihenfolge seines 
//   Auftretens von links nach rechts ausgewertet.
//   Ergebnis hat den Datentyp Double  Ist jedoch entweder Zahl oder Exponent ein Ausdruck mit dem Wert Null, 
//   so ist das Ergebnis ebenfalls Null.
DLL_DECL B_Null B_Pow(const B_Null&,      const B_Integer&);
DLL_DECL B_Null B_Pow(const B_Integer&,   const B_Null&);


//----------------------------------------------------------------------------------------------------
//---------------------------------- Logische Operatoren ---------------------------------------------

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der !Operator entspricht NOT in VB , 
//   Bit in Ausdruck1 		Ergebnis 
//       0	                  1
//       1                  	0  
DLL_DECL B_Integer operator!(const B_Empty&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der &&Operator entspricht AND in VB , 
//   Bit in Ausdruck1 	Bit in Ausdruck2 	Ergebnis 
//       0	                  0            	0
//       0	                  1           	0
//       1                  	0           	0
//       1                   	1            	1
//   ACHTUNG alle C/C++ Programmierer: in VB werden numerische Typen immer logisch bitweise und verknpft
//                                     Der Einsatz von & war nicht moeglich da stellenweise erst zur Lauf-
//                                     zeit das Verhalten bestimmt werden kann was der Konverter nicht 
//                                     bestimmen kann
DLL_DECL B_Integer operator&& (const B_Integer&, const B_Integer&);
DLL_DECL B_Integer operator&& (const B_Byte&,    const B_Integer&);
DLL_DECL B_Integer operator&& (const B_Integer&, const B_Byte&);
DLL_DECL B_Integer operator&& (const B_Integer&, const B_Boolean&);
DLL_DECL B_Integer operator&& (const B_Boolean&, const B_Integer&);
DLL_DECL B_Null    operator&& (const B_Null&,    const B_Integer&);
DLL_DECL B_Null    operator&& (const B_Integer&, const B_Null&);

DLL_DECL B_Integer operator&& (const B_Byte&,    const B_Boolean&);
DLL_DECL B_Integer operator&& (const B_Boolean&, const B_Byte&);

DLL_DECL B_Integer operator&& (const B_Int,      const B_Integer&);
DLL_DECL B_Integer operator&& (const B_Integer&, const B_Int);
DLL_DECL B_Integer operator&& (const B_Byte&,    const B_Int);
DLL_DECL B_Integer operator&& (const B_Int,      const B_Byte&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der ||Operator entspricht OR in VB , 
//   Bit in Ausdruck1 	Bit in Ausdruck2 	Ergebnis 
//       0	                  0            	0
//       0	                  1           	1
//       1                  	0           	1
//       1                   	1            	1
//   ACHTUNG alle C/C++ Programmierer: in VB werden numerische Typen immer logisch bitweise und verknpft
//                                     Der Einsatz von | war nicht moeglich da stellenweise erst zur Lauf-
//                                     zeit das Verhalten bestimmt werden kann was der Konverter nicht 
//                                     bestimmen kann
DLL_DECL B_Integer operator|| (const B_Integer&,   const B_Integer&);
DLL_DECL B_Integer operator|| (const B_Byte&,      const B_Integer&);
DLL_DECL B_Integer operator|| (const B_Integer&,   const B_Byte&);
DLL_DECL B_Integer operator|| (const B_Integer&,   const B_Boolean&);
DLL_DECL B_Integer operator|| (const B_Boolean&,   const B_Integer&);
DLL_DECL B_Integer operator|| (const B_Null&,      const B_Integer&);
DLL_DECL B_Integer operator|| (const B_Integer&,   const B_Null&);

DLL_DECL B_Integer operator|| (const B_Byte&,      const B_Boolean&);
DLL_DECL B_Integer operator|| (const B_Boolean&,   const B_Byte&);
DLL_DECL B_Integer operator|| (const B_Int,        const B_Integer&);
DLL_DECL B_Integer operator|| (const B_Integer&,   const B_Int);
DLL_DECL B_Integer operator|| (const B_Byte&,      const B_Int);
DLL_DECL B_Integer operator|| (const B_Int,        const B_Byte&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der ^Operator entspricht XOR in VB 
//   Bit in Ausdruck1 	Bit in Ausdruck2 	Ergebnis 
//       0	                  0            	0
//       0	                  1           	1
//       1                  	0           	1
//       1                   	1            	0
//   ACHTUNG alle VB Programmierer:    In C++ bedeutet der ^Operator ein bitweises XOR auf numerische
//                                     Operanden und nicht wie in VB eine Potenzierung. Der ^Operator
//                                     wurde nicht zur Potenzierung verwandt da dies nicht C++ Stil
//                                     entspricht.
//   ACHTUNG alle C/C++ Programmierer: In VB werden numerische Typen immer logisch bitweise und verknpft
//                                     Dies hat fuer ^Operator die Folge, dass er bei nicht numerischen
//                                     Ausdreucken sich wie ein logisches XOR verhaelt
DLL_DECL B_Integer operator^ (const B_Integer&, const B_Integer&);
DLL_DECL B_Integer operator^ (const B_Byte&,    const B_Integer&);
DLL_DECL B_Integer operator^ (const B_Integer&, const B_Byte&);
DLL_DECL B_Integer operator^ (const B_Integer&, const B_Boolean&);
DLL_DECL B_Integer operator^ (const B_Boolean&, const B_Integer&);
DLL_DECL B_Null    operator^ (const B_Null&,    const B_Integer&);
DLL_DECL B_Null    operator^ (const B_Integer&, const B_Null&);

DLL_DECL B_Integer operator^ (const B_Byte&,    const B_Boolean&);
DLL_DECL B_Integer operator^ (const B_Boolean&, const B_Byte&);
DLL_DECL B_Integer operator^ (const B_Int,      const B_Integer&);
DLL_DECL B_Integer operator^ (const B_Integer&, const B_Int);
DLL_DECL B_Integer operator^ (const B_Byte&,    const B_Int);
DLL_DECL B_Integer operator^ (const B_Int,      const B_Byte&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der !Operator entspricht NOT in VB 
//   Bit in Ausdruck1 	 Ergebnis
//       0	                 1
//       1                   0
//   ACHTUNG alle C/C++ Programmierer: In VB werden numerische Typen immer logisch bitweise und verknpft
//                                     Dies hat fuer !Operator die Folge, dass er bei nicht numerischen
//                                     Ausdreucken sich wie ein logisches NOT verhaelt
// fuer den Fall, dass B_Emtpy uebergeben wird ist fuer alle Datentypen die Funktion 
// B_Integer operator! (const B_Empty) in bc_int.hpp verantwortlich
// Die Variante B_Null operator!( const B_Null&) ist in B_Byte-Klasse abgehandelt


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Die Operatoren Imp (=logische Implikation) und Eqv (=logische quivalenz) werden vom Konverter
// in folgende Basisoperatorkonstrukte umgesetzt, da in C++ keine quivalenten Operatoren zur Verfgung
// stehen.
// a Imp b    ->  (!a || b)
// a Eqv b    ->  !(a ^ b)


//----------------------------------------------------------------------------------------------------
//---------------------------------- Vergleichs Operatoren -------------------------------------------

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Verlgeich auf Gleichheit der Operanden, es findet ein numerischer Vergleich statt
// Ausdruck1 == Ausdruck2 -> B_True
// Ausdruck1 != Ausdruck2 -> B_False
// ACHTUNG alle VB Programmierer:  In C++ wird mit '==' auf Gleichheit abgefragt und nicht mit '='
//                                 Der '=' Operator wird nur fuer Zuweisungen verwendet.
DLL_DECL B_Boolean operator== (const B_Integer&,  const B_Integer&);
DLL_DECL B_Boolean operator== (const B_Byte&,     const B_Integer&);
DLL_DECL B_Boolean operator== (const B_Integer&,  const B_Byte&);
DLL_DECL B_Boolean operator== (const B_Integer&,  const B_Empty&);     // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator== (const B_Empty&,    const B_Integer&);   // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator== (const B_Integer&,  const B_Boolean&);
DLL_DECL B_Boolean operator== (const B_Boolean&,  const B_Integer&);
DLL_DECL B_Null    operator== (const B_Integer&,  const B_Null&);
DLL_DECL B_Null    operator== (const B_Null&,     const B_Integer&);
DLL_DECL B_Boolean operator== (const B_Byte&,     const B_Int);
DLL_DECL B_Boolean operator== (const B_Int,       const B_Byte&);
DLL_DECL B_Boolean operator== (const B_Int,       const B_Integer&);
DLL_DECL B_Boolean operator== (const B_Integer&,  const B_Int);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Verlgeich auf Ungleichheit der Operanden, es findet ein numerischer Vergleich statt
// Ausdruck1 != Ausdruck2 -> B_True
// Ausdruck1 == Ausdruck2 -> B_False
// ACHTUNG alle VB Programmierer:  In C++ wird mit '!=' auf Ungleichheit abgefragt und nicht mit '<>'
//                                 Der '<>' Operator existiert nicht in C++.
DLL_DECL B_Boolean operator!= (const B_Integer&,  const B_Integer&);
DLL_DECL B_Boolean operator!= (const B_Byte&,     const B_Integer&);
DLL_DECL B_Boolean operator!= (const B_Integer&,  const B_Byte&);
DLL_DECL B_Boolean operator!= (const B_Integer&,  const B_Empty&);     // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator!= (const B_Empty&,    const B_Integer&);   // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator!= (const B_Integer&,  const B_Boolean&);
DLL_DECL B_Boolean operator!= (const B_Boolean&,  const B_Integer&);
DLL_DECL B_Null    operator!= (const B_Integer&,  const B_Null&);
DLL_DECL B_Null    operator!= (const B_Null&,     const B_Integer&);
DLL_DECL B_Boolean operator!= (const B_Byte&,     const B_Int);
DLL_DECL B_Boolean operator!= (const B_Int,       const B_Byte&);
DLL_DECL B_Boolean operator!= (const B_Int,       const B_Integer&);
DLL_DECL B_Boolean operator!= (const B_Integer&,  const B_Int);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Verlgeich auf Kleiner der Operanden, es findet ein numerischer Vergleich statt
// Ausdruck1 <  Ausdruck2 -> B_True
// Ausdruck1 >= Ausdruck2 -> B_False
DLL_DECL B_Boolean operator< (const B_Integer&,  const B_Integer&);
DLL_DECL B_Boolean operator< (const B_Byte&,     const B_Integer&);
DLL_DECL B_Boolean operator< (const B_Integer&,  const B_Byte&);
DLL_DECL B_Boolean operator< (const B_Integer&,  const B_Empty&);     // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator< (const B_Empty&,    const B_Integer&);   // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator< (const B_Integer&,  const B_Boolean&);
DLL_DECL B_Boolean operator< (const B_Boolean&,  const B_Integer&);
DLL_DECL B_Null    operator< (const B_Integer&,  const B_Null&);
DLL_DECL B_Null    operator< (const B_Null&,     const B_Integer&);
DLL_DECL B_Boolean operator< (const B_Int,       const B_Integer&);
DLL_DECL B_Boolean operator< (const B_Integer&,  const B_Int);
DLL_DECL B_Boolean operator< (const B_Int,       const B_Byte&);
DLL_DECL B_Boolean operator< (const B_Byte&,     const B_Int);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Verlgeich auf Kleiner/Gleich der Operanden, es findet ein numerischer Vergleich statt
// Ausdruck1 <=  Ausdruck2 -> B_True
// Ausdruck1 >   Ausdruck2 -> B_False
DLL_DECL B_Boolean operator<= (const B_Integer&,  const B_Integer&);
DLL_DECL B_Boolean operator<= (const B_Byte&,     const B_Integer&);
DLL_DECL B_Boolean operator<= (const B_Integer&,  const B_Byte&);
DLL_DECL B_Boolean operator<= (const B_Integer&,  const B_Empty&);     // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator<= (const B_Empty&,    const B_Integer&);   // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator<= (const B_Integer&,  const B_Boolean&);
DLL_DECL B_Boolean operator<= (const B_Boolean&,  const B_Integer&);
DLL_DECL B_Null    operator<= (const B_Integer&,  const B_Null&);
DLL_DECL B_Null    operator<= (const B_Null&,     const B_Integer&);
DLL_DECL B_Boolean operator<= (const B_Int,       const B_Integer&);
DLL_DECL B_Boolean operator<= (const B_Integer&,  const B_Int);
DLL_DECL B_Boolean operator<= (const B_Byte&,     const B_Int);
DLL_DECL B_Boolean operator<= (const B_Int,       const B_Byte&);


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Verlgeich auf Groesser der Operanden, es findet ein numerischer Vergleich statt
// Ausdruck1 >   Ausdruck2 -> B_True
// Ausdruck1 <=  Ausdruck2 -> B_False
DLL_DECL B_Boolean operator> (const B_Integer&,  const B_Integer&);
DLL_DECL B_Boolean operator> (const B_Byte&,     const B_Integer&);
DLL_DECL B_Boolean operator> (const B_Integer&,  const B_Byte&);
DLL_DECL B_Boolean operator> (const B_Integer&,  const B_Empty&);     // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator> (const B_Empty&,    const B_Integer&);   // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator> (const B_Integer&,  const B_Boolean&);
DLL_DECL B_Boolean operator> (const B_Boolean&,  const B_Integer&);
DLL_DECL B_Null    operator> (const B_Integer&,  const B_Null&);
DLL_DECL B_Null    operator> (const B_Null&,     const B_Integer&);
DLL_DECL B_Boolean operator> (const B_Byte&,     const B_Int);
DLL_DECL B_Boolean operator> (const B_Int,       const B_Byte&);
DLL_DECL B_Boolean operator> (const B_Int,       const B_Integer&);
DLL_DECL B_Boolean operator> (const B_Integer&,  const B_Int);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Verlgeich auf Groesser/Gleich der Operanden, es findet ein numerischer Vergleich statt
// Ausdruck1 >=  Ausdruck2 -> B_True
// Ausdruck1 <   Ausdruck2 -> B_False
DLL_DECL B_Boolean operator>= (const B_Integer&,  const B_Integer&);
DLL_DECL B_Boolean operator>= (const B_Byte&,     const B_Integer&);
DLL_DECL B_Boolean operator>= (const B_Integer&,  const B_Byte&);
DLL_DECL B_Boolean operator>= (const B_Integer&,  const B_Empty&);     // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator>= (const B_Empty&,    const B_Integer&);   // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator>= (const B_Integer&,  const B_Boolean&);
DLL_DECL B_Boolean operator>= (const B_Boolean&,  const B_Integer&);
DLL_DECL B_Null    operator>= (const B_Integer&,  const B_Null&);
DLL_DECL B_Null    operator>= (const B_Null&,     const B_Integer&);
DLL_DECL B_Boolean operator>= (const B_Byte&,     const B_Int);
DLL_DECL B_Boolean operator>= (const B_Int,       const B_Byte&);
DLL_DECL B_Boolean operator>= (const B_Int,       const B_Integer&);
DLL_DECL B_Boolean operator>= (const B_Integer&,  const B_Int);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Streamoperatoren
DLL_DECL ostream& operator<<(ostream& ostr, const B_Integer& rhs);
DLL_DECL istream& operator>>(istream& istr, B_Integer& rhs);

//================================ Inlinefunktionen =========================================================
//
#if defined _DEBUG || defined B_VB_CONTRACT
  #define B_NOINLINE
#else
  #undef B_NOINLINE
#endif

#ifndef B_NOINLINE                 // festlegen wenn Debuger keine Inlines debuggen kann
  #define B_INLINE inline
  #ifndef  BI_INT_HPP
    #include "bi_int.hpp"
  #endif
#endif

//===========================================================================================================
//================================ End Of Header ============================================================
#endif // BC_INT_HPP

#endif // B_VB_COMPATIBLE
