#ifdef B_VB_COMPATIBLE

#ifndef BC_LONG_HPP
#define BC_LONG_HPP
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Project:                 VB nach C/C++ Konverter Klassenbibliothek
// Modul:                   VB Klasse fuer VB Long Datentyp
// Filename:                bc_long.hpp
// Version:                 1.0
// Status:                  beta
//
// Copyright:               (C)
//                          Cicerosoft GmbH,
//                          Ballweilerstr. 7,
//                          66399 Erfweiler-Ehlingen
//
// Operating System(s):     Win95 
// Compiler:                VC++ 4.1
//
// Author:                  Jochen Diener JD
// Creation Date:           14.11.95
// Last modification:       JD 29.06.96
//
// Description:             Beinhaltet die Klassendeklaration der B_Long Klasse
//
//
// Dependencies:            bi_long.hpp   -> Inline Fkt.
//                          bc_long.hpp   -> Deklaration von B_Byte
//
// Remarks:
//
// Revision(s):
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

//================================= Defintionsabhaengigkeiten ===============================================
//
#include <iostream.h>

#ifndef BD_LIB_HPP
  #include "bd_lib.hpp"
#endif

#ifndef BD_INTYP_HPP       
  #include "bd_intyp.hpp"       // alias der Standardatentypen
#endif

#ifndef BC_BOOL_HPP
  #include "bc_bool.hpp"        // VB Boolean Typ
#endif

#ifndef BC_EMPTY_HPP
  #include "bc_empty.hpp"
#endif

#ifndef BC_NULL_HPP
  #include "bc_null.hpp"
#endif

#ifndef BC_BYTE_HPP
  #include "bc_byte.hpp"
#endif

#ifndef BC_INT_HPP
  #include "bc_int.hpp"
#endif

//================================= Export =================================================================
//
class B_String;
class B_fstream;						// CT131296 Fuer FileIO abgeleitete fstream-Klasse

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_Long     Emuliert VB-Datentyp Long
// Hierachy:    
// Inheritance: 
// Errorhandling:
// Note: 
//
class DLL_DECL B_Long
  {
  //==================== MANGAEMENT =========================================================================
  //
  public: 
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     ctor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Long(); // Standardkonstruktor initialisiert Integer in VB auf 0


    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     cctor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Long(const B_Long&);    

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     ctor's fuer Bibliotheksdatentypen
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Long(const B_String&);   // throw(B_TypeMismatch, B_Overflow);
    B_Long(const B_Currency&); // throw(B_Overflow);
    B_Long(const B_Double&);   // throw(B_Overflow);
    B_Long(const B_Single&);   // throw(B_Overflow);
    B_Long(const B_Integer&);
    B_Long(const B_Byte&);
    B_Long(const B_Boolean&);
    B_Long(const B_Empty&); 
    B_Long(const B_Null&); // throw(B_InvalidNull);
    
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     ctor's fuer C-Standarddatentypen
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Long(const B_Lng, const B_Explicit&); 
    B_Long(const B_UInt,const B_Explicit&);   
    B_Long(const B_Int, const B_Explicit&);
    B_Long(const B_Flt, const B_Explicit&); // throw(B_Overflow);
    B_Long(const B_Dbl, const B_Explicit&); // throw(B_Overflow);
    B_Long(B_cCStr,     const B_Explicit&); // throw(B_TypeMismatch, B_Overflow);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    ~B_Long();

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Long& operator=(const B_Long&);

  //==================== INTERFACE ============================================================================
  //
  public:
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Zuweisungsop. fuer Bibliotheksdatenypen
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Long& operator=(const B_String&)   throw(B_TypeMismatch, B_Overflow);
    B_Long& operator=(const B_Currency&) throw(B_Overflow);
    B_Long& operator=(const B_Double&)   throw(B_Overflow);
    B_Long& operator=(const B_Single&)   throw(B_Overflow);
    B_Long& operator=(const B_Integer&);
    B_Long& operator=(const B_Byte&);
    B_Long& operator=(const B_Boolean&);
    B_Long& operator=(const B_Empty&);  
    B_Long& operator=(const B_Null&)    throw(B_InvalidNull);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Zuweisungsop. fuer C-Standardatentypen
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Long& operator=(const B_Lng);   
    B_Long& operator=(const B_Int);   
    B_Long& operator=(const B_UInt);    
    B_Long& operator=(const B_Flt)  throw(B_Overflow);
    B_Long& operator=(const B_Dbl)  throw(B_Overflow);
    B_Long& operator=(B_cCStr)      throw(B_TypeMismatch, B_Overflow);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Addition des Wertes selbst mit einem anderen Wert. Entspricht x = x + y
    // Parameter:  
    //     Wert der hinzuaddiert werden soll (Entspricht y)
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Long& operator+=(const B_Long&)     throw(B_Overflow);
    B_Long& operator+=(const B_Integer&)  throw(B_Overflow);
    B_Long& operator+=(const B_Byte&)     throw(B_Overflow);
    B_Long& operator+=(const B_Boolean&)  throw(B_Overflow);
    B_Long& operator+=(const B_Empty&);
    B_Long& operator+=(const B_Null&)     throw(B_InvalidNull);
    //-- C-Standartypen
    B_Long& operator+=(const B_Lng)       throw(B_Overflow);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Subtraktion des Wertes selbst mit einem anderen Wert. Entspricht x = x - y
    // Parameter:  
    //     Wert der subtrahiert werden soll (Entspricht y)
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Long& operator-=(const B_Long&)    throw(B_Overflow);
    B_Long& operator-=(const B_Integer&) throw(B_Overflow);
    B_Long& operator-=(const B_Byte&)    throw(B_Overflow);
    B_Long& operator-=(const B_Boolean&) throw(B_Overflow);
    B_Long& operator-=(const B_Empty&);
    B_Long& operator-=(const B_Null&)    throw(B_InvalidNull);
    //-- C-Standartypen
    B_Long& operator-=(const B_Lng)      throw(B_Overflow);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Multiplikation des Wertes selbst mit einem anderen Wert. Entspricht x = x * y
    // Parameter:  
    //     Wert der multipliziert werden soll (Entspricht y)
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Long& operator*=(const B_Long&)    throw(B_Overflow);
    B_Long& operator*=(const B_Integer&) throw(B_Overflow);
    B_Long& operator*=(const B_Byte&)    throw(B_Overflow);
    B_Long& operator*=(const B_Boolean&);
    B_Long& operator*=(const B_Empty&);
    B_Long& operator*=(const B_Null&)    throw(B_InvalidNull);
    //-- C-Standartypen    
    B_Long& operator*=(const B_Lng)      throw(B_Overflow);

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Division des Wertes selbst mit einem anderen Wert. Entspricht x = x / y
    // Parameter:  
    //     Wert der dividiert werden soll (Entspricht y)
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Long& operator/=(const B_Long&)    throw(B_ZeroDivide);
    B_Long& operator/=(const B_Integer&) throw(B_ZeroDivide);
    B_Long& operator/=(const B_Byte&)    throw(B_ZeroDivide);
    B_Long& operator/=(const B_Boolean&) throw(B_ZeroDivide);
    B_Long& operator/=(const B_Empty&);
    B_Long& operator/=(const B_Null&)    throw(B_InvalidNull);
    //-- C-Standartypen    
    B_Long& operator/=(const B_Lng)      throw(B_ZeroDivide); 

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Division des Wertes selbst mit einem anderen Wert. Entspricht x = x / y
    // Parameter:  
    //     Wert der dividiert werden soll (Entspricht y)
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Long& operator%=(const B_Long&)    throw(B_ZeroDivide);
    B_Long& operator%=(const B_Integer&) throw(B_ZeroDivide);
    B_Long& operator%=(const B_Byte&)    throw(B_ZeroDivide);
    B_Long& operator%=(const B_Boolean&) throw(B_ZeroDivide);
    B_Long& operator%=(const B_Empty&);
    B_Long& operator%=(const B_Null&)    throw(B_InvalidNull);
    //-- C-Standartypen    
    B_Long& operator%=(const B_Lng)      throw(B_ZeroDivide); 

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     operator B_Lng() Konvertierungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    operator B_Lng() const;

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     operator!   Nicht-Operator
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Long operator!() const;

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Unaeres Minus
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Long operator-() const;

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Inkrement/dekrement des Objektes. Entspricht: x = x + 1;/ x = x -1
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:      
    //
    B_Long& operator++() throw(B_Overflow);
    B_Long& operator--() throw(B_Overflow);

    //----------------------------------------------------------------------------------------------------       
    // Method: 
    //     Wandelt den Wert des Objektes in einen Absoluten Wert um, d.h 
    //     negativer Objektwert -> positiv
    //     positiver Objektwert -> bleibt positiv
    // Return Value:
    //     Boolean Wert:  B_True   wenn konvertiert
    //                    B_False  wenn nicht konvertiert werden konnte respektive Fehler
    //                              aufgetreten sind
    // Note:      
    B_Long& abs();

    //----------------------------------------------------------------------------------------------------       
    // Method:       
    //     Gibt den Wert des Objektes als entsprechenden C/C++ Datentyp zurueck
    // Return Value:
    //     Kopie des Objetkwertes als B_UChar
    // Note:
    //     Den genauen Typ von B_Lng kann man in bd_intyp.hpp feststellen, in der alle
    //     C/C++ Datentypen definiert sind.
    //     Diese Method soll den Datenaustausch zwischen Klassen der Klassenbibliothek
    //     und den mit den C++ Standarddatentypen geschriebenen Methoden ermoeglichen
    //     wenn diese keine Klassen als Parameter erlauben.
    B_Lng  getValue() const;

    //----------------------------------------------------------------------------------------------------       
    // Method:
    //     Gibt die Groesse des Wert des Objektes als entsprechenden C/C++ Datentyp zurueck
    // Return Value:
    //     Groesse in Byte
    // Note:
    //     Den genauen Typ und somit seine groesse koennen zwischen den C/C++ Datentypen der
    //     Compiler unterschiedlich sein, deshalb wurde diese Method eingefuegt die eine
    //     portable Programmierung aehnlich sizeof() ermoeglichen soll.
    B_ULng  getValueSize() const;  
    
    //----------------------------------------------------------------------------------------------------       
    // Method: 
    //     Gibt die Groesse des Objktes zurueck, diese entspricht der durch das Objekt belegten
    //     Speicherplatz
    // Return Value: 
    //     Groesse in Byte
    // Note: 
    //     Den genauen Typ und somit seine groesse koennen zwischen den C/C++ Compiler 
    //     unterschiedlich sein. Auch koennte sich die Klassenimplementierung aendern so
    //     dass aeltere Programme nur mit Aenderungsprogramm lauffaehig waeren, deshalb 
    //     wurde diese Method eingefuegt die eine portable Programmierung aehnlich 
    //     sizeof() ermoeglichen soll.
    B_ULng getSize() const;       

    //----------------------------------------------------------------------------------------------------       
    // Method: 
    //     Setzt den internen Status des Objektes auf optional. Somit kann mit isMissing der Status
    //     abgefragt werden und die Emulation des gleichnamigen VB Befehls wird moeglich.
    // Return Value: 
    // Note: 
    //
    B_Long& isOptional();

    //----------------------------------------------------------------------------------------------------       
    // Method: 
    //     Abfragen des internen Objektstatus ob das Argument einer Funktion optional ist oder nicht
    // Return Value: 
    // Note: 
    //
    B_Boolean isMissing() const;

    //----------------------------------------------------------------------------------------------------       
    // Method: 
    //     Schreiben der der eigentlichen Objektdaten im binaeren Format in einen Stream
    // Return Value: 
    //     B_true  ==  Objekt und Stream sind gueltig
    //     B_false ==  Objekt oder Stream sind ungueltig
    // Note: 
    //
    B_Boolean streamw(ostream& ostr, B_Shrt openmode) const;

    //----------------------------------------------------------------------------------------------------       
    // Method: 
    //     Lesen der der eigentlichen Objektdaten im binaeren Format in einen Stream
    // Return Value: 
    //     B_true  ==  Objekt und Stream sind gueltig
    //     B_false ==  Objekt oder Stream sind ungueltig
    // Note: 
    //
    B_Boolean streamr(istream& istr, B_Shrt openmode);
	
  private:
    //----------------------------------------------------------------------------------------------------       
    // Method:
    //     Postfix   Inkrement/Dekrement
    // Parameter:  
    // Return Value: 
    // Note:      
    //     Da es beim ueberladen keinen wirkliche Postfixoperatoren gibt wurden diese gesperrt
    B_Long& operator++(int);
    B_Long& operator--(int);


  //==================== MEMBER ===============================================================================    
  //
  private:
    B_Lng value;  // da VB Wertebereich   -2.147.483.648 bis 2.147.483.647    
    B_Flag optional;

  //==================== FRIENDS ============================================================================
  //
  // friend class B_Err; noetig wenn Err umgestellt wird auf B_Long
  friend class B_Byte;
  friend class B_Integer;
  friend class B_Single;
  friend class B_Double;
  friend class B_Currency;
  friend class B_String;

  friend DLL_DECL B_Long  operator+ (const B_Long&,    const B_Long&);
  friend DLL_DECL B_Long  operator+ (const B_Integer&, const B_Long&);
  friend DLL_DECL B_Long  operator+ (const B_Long&,    const B_Integer&);
  friend DLL_DECL B_Long  operator+ (const B_Byte&,    const B_Long&);
  friend DLL_DECL B_Long  operator+ (const B_Long&,    const B_Byte&);
  friend DLL_DECL B_Long  operator+ (const B_Boolean&, const B_Long&);
  friend DLL_DECL B_Long  operator+ (const B_Long&,    const B_Boolean&);
  friend DLL_DECL B_Long  operator+ (const B_Lng,      const B_Long&);
  friend DLL_DECL B_Long  operator+ (const B_Long&,    const B_Lng);
  friend DLL_DECL B_Long  operator+ (const B_Int,      const B_Long&);
  friend DLL_DECL B_Long  operator+ (const B_Long&,    const B_Int);
  
  friend DLL_DECL B_Long  operator- (const B_Long&,    const B_Long&);
  friend DLL_DECL B_Long  operator- (const B_Integer&, const B_Long&);
  friend DLL_DECL B_Long  operator- (const B_Long&,    const B_Integer&);
  friend DLL_DECL B_Long  operator- (const B_Byte&,    const B_Long&);
  friend DLL_DECL B_Long  operator- (const B_Long&,    const B_Byte&);
  friend DLL_DECL B_Long  operator- (const B_Boolean&, const B_Long&);
  friend DLL_DECL B_Long  operator- (const B_Long&,    const B_Boolean&);
  friend DLL_DECL B_Long  operator- (const B_Empty&,   const B_Long&);
  friend DLL_DECL B_Long  operator- (const B_Long&,    const B_Empty&);
  friend DLL_DECL B_Long  operator- (const B_Lng,      const B_Long&);
  friend DLL_DECL B_Long  operator- (const B_Long&,    const B_Lng);
  friend DLL_DECL B_Long  operator- (const B_Int,      const B_Long&);
  friend DLL_DECL B_Long  operator- (const B_Long&,    const B_Int);
  
  friend DLL_DECL B_Long  operator* (const B_Long&,    const B_Long&);
  friend DLL_DECL B_Long  operator* (const B_Integer&, const B_Long&);
  friend DLL_DECL B_Long  operator* (const B_Long&,    const B_Integer&);
  friend DLL_DECL B_Long  operator* (const B_Byte&,    const B_Long&);
  friend DLL_DECL B_Long  operator* (const B_Long&,    const B_Byte&);
  friend DLL_DECL B_Long  operator* (const B_Boolean&, const B_Long&);
  friend DLL_DECL B_Long  operator* (const B_Long&,    const B_Boolean&);
  friend DLL_DECL B_Long  operator* (const B_Lng,      const B_Long&);
  friend DLL_DECL B_Long  operator* (const B_Long&,    const B_Lng);
  friend DLL_DECL B_Long  operator* (const B_Int,      const B_Long&);
  friend DLL_DECL B_Long  operator* (const B_Long&,    const B_Int);

  friend DLL_DECL B_Long  operator% (const B_Long&,    const B_Long&);
  friend DLL_DECL B_Long  operator% (const B_Integer&, const B_Long&);
  friend DLL_DECL B_Long  operator% (const B_Long&,    const B_Integer&);
  friend DLL_DECL B_Long  operator% (const B_Byte&,    const B_Long&);
  friend DLL_DECL B_Long  operator% (const B_Long&,    const B_Byte&);
  friend DLL_DECL B_Long  operator% (const B_Long&,    const B_Boolean&);
  friend DLL_DECL B_Long  operator% (const B_Boolean&, const B_Long&);
  friend DLL_DECL B_Long  operator% (const B_Empty&,   const B_Long&);
  friend DLL_DECL B_Long  operator% (const B_Lng,      const B_Long&);
  friend DLL_DECL B_Long  operator% (const B_Long&,    const B_Lng);
  friend DLL_DECL B_Long  operator% (const B_Int,      const B_Long&);
  friend DLL_DECL B_Long  operator% (const B_Long&,    const B_Int);
  
  friend DLL_DECL B_Long B_Div(const B_Long&,    const B_Long&);
  friend DLL_DECL B_Long B_Div(const B_Integer&, const B_Long&);
  friend DLL_DECL B_Long B_Div(const B_Long&,    const B_Integer&);
  friend DLL_DECL B_Long B_Div(const B_Byte&,    const B_Long&);
  friend DLL_DECL B_Long B_Div(const B_Long&,    const B_Byte&);
  friend DLL_DECL B_Long B_Div(const B_Long&,    const B_Boolean&);
  friend DLL_DECL B_Long B_Div(const B_Boolean&, const B_Long&);
  friend DLL_DECL B_Long B_Div(const B_Empty&,   const B_Long&);
  friend DLL_DECL B_Long B_Div(const B_Lng,      const B_Long&);
  friend DLL_DECL B_Long B_Div(const B_Long&,    const B_Lng);
  friend DLL_DECL B_Long B_Div(const B_Int,      const B_Long&);
  friend DLL_DECL B_Long B_Div(const B_Long&,    const B_Int);
  
  friend DLL_DECL B_Long operator&& (const B_Long&,    const B_Long&);
  friend DLL_DECL B_Long operator&& (const B_Integer&, const B_Long&);
  friend DLL_DECL B_Long operator&& (const B_Long&,    const B_Integer&);
  friend DLL_DECL B_Long operator&& (const B_Byte&,    const B_Long&);
  friend DLL_DECL B_Long operator&& (const B_Long&,    const B_Byte&);
  friend DLL_DECL B_Long operator&& (const B_Boolean&, const B_Long&);
  friend DLL_DECL B_Long operator&& (const B_Long&,    const B_Boolean&);
  friend DLL_DECL B_Long operator&& (const B_Lng,      const B_Long&);
  friend DLL_DECL B_Long operator&& (const B_Long&,    const B_Lng);
  friend DLL_DECL B_Long operator&& (const B_Int,      const B_Long&);
  friend DLL_DECL B_Long operator&& (const B_Long&,    const B_Int);
  
  friend DLL_DECL B_Long operator|| (const B_Long&,    const B_Long&);
  friend DLL_DECL B_Long operator|| (const B_Integer&, const B_Long&);
  friend DLL_DECL B_Long operator|| (const B_Long&,    const B_Integer&);
  friend DLL_DECL B_Long operator|| (const B_Byte&,    const B_Long&);
  friend DLL_DECL B_Long operator|| (const B_Long&,    const B_Byte&);
  friend DLL_DECL B_Long operator|| (const B_Boolean&, const B_Long&);
  friend DLL_DECL B_Long operator|| (const B_Long&,    const B_Boolean&);
  friend DLL_DECL B_Long operator|| (const B_Lng,      const B_Long&);
  friend DLL_DECL B_Long operator|| (const B_Long&,    const B_Lng);
  friend DLL_DECL B_Long operator|| (const B_Int,      const B_Long&);
  friend DLL_DECL B_Long operator|| (const B_Long&,    const B_Int);
  
  friend DLL_DECL B_Long operator^ (const B_Long&,    const B_Long&);
  friend DLL_DECL B_Long operator^ (const B_Integer&, const B_Long&);
  friend DLL_DECL B_Long operator^ (const B_Long&,    const B_Integer&);
  friend DLL_DECL B_Long operator^ (const B_Byte&,    const B_Long&);
  friend DLL_DECL B_Long operator^ (const B_Long&,    const B_Byte&);
  friend DLL_DECL B_Long operator^ (const B_Long&,    const B_Boolean&);
  friend DLL_DECL B_Long operator^ (const B_Boolean&, const B_Long&);
  friend DLL_DECL B_Long operator^ (const B_Lng,      const B_Long&);
  friend DLL_DECL B_Long operator^ (const B_Long&,    const B_Lng);
  friend DLL_DECL B_Long operator^ (const B_Int,      const B_Long&);
  friend DLL_DECL B_Long operator^ (const B_Long&,    const B_Int);

  friend DLL_DECL B_Boolean operator== (const B_Long&,    const B_Long&);
  friend DLL_DECL B_Boolean operator== (const B_Integer&, const B_Long&);
  friend DLL_DECL B_Boolean operator== (const B_Long&,    const B_Integer&);
  friend DLL_DECL B_Boolean operator== (const B_Byte&,    const B_Long&);
  friend DLL_DECL B_Boolean operator== (const B_Long&,    const B_Byte&);
  friend DLL_DECL B_Boolean operator== (const B_Long&,    const B_Empty&);
  friend DLL_DECL B_Boolean operator== (const B_Empty&,   const B_Long&); 
  friend DLL_DECL B_Boolean operator== (const B_Long&,    const B_Boolean&);
  friend DLL_DECL B_Boolean operator== (const B_Boolean&, const B_Long&);
  friend DLL_DECL B_Boolean operator== (const B_Lng,      const B_Long&);
  friend DLL_DECL B_Boolean operator== (const B_Long&,    const B_Lng);
  friend DLL_DECL B_Boolean operator== (const B_Int,      const B_Long&);
  friend DLL_DECL B_Boolean operator== (const B_Long&,    const B_Int);

  friend DLL_DECL B_Boolean operator!= (const B_Long&,    const B_Long&);
  friend DLL_DECL B_Boolean operator!= (const B_Integer&, const B_Long&);
  friend DLL_DECL B_Boolean operator!= (const B_Long&,    const B_Integer&);
  friend DLL_DECL B_Boolean operator!= (const B_Byte&,    const B_Long&);
  friend DLL_DECL B_Boolean operator!= (const B_Long&,    const B_Byte&);
  friend DLL_DECL B_Boolean operator!= (const B_Long&,    const B_Empty&);
  friend DLL_DECL B_Boolean operator!= (const B_Empty&,   const B_Long&); 
  friend DLL_DECL B_Boolean operator!= (const B_Long&,    const B_Boolean&);
  friend DLL_DECL B_Boolean operator!= (const B_Boolean&, const B_Long&);
  friend DLL_DECL B_Boolean operator!= (const B_Lng,      const B_Long&);
  friend DLL_DECL B_Boolean operator!= (const B_Long&,    const B_Lng);
  friend DLL_DECL B_Boolean operator!= (const B_Int,      const B_Long&);
  friend DLL_DECL B_Boolean operator!= (const B_Long&,    const B_Int);

  friend DLL_DECL B_Boolean operator< (const B_Long&,    const B_Long&);
  friend DLL_DECL B_Boolean operator< (const B_Integer&, const B_Long&);
  friend DLL_DECL B_Boolean operator< (const B_Long&,    const B_Integer&);
  friend DLL_DECL B_Boolean operator< (const B_Byte&,    const B_Long&);
  friend DLL_DECL B_Boolean operator< (const B_Long&,    const B_Byte&);
  friend DLL_DECL B_Boolean operator< (const B_Long&,    const B_Empty&);
  friend DLL_DECL B_Boolean operator< (const B_Empty&,   const B_Long&); 
  friend DLL_DECL B_Boolean operator< (const B_Long&,    const B_Boolean&);
  friend DLL_DECL B_Boolean operator< (const B_Boolean&, const B_Long&);
  friend DLL_DECL B_Boolean operator< (const B_Lng,      const B_Long&);
  friend DLL_DECL B_Boolean operator< (const B_Long&,    const B_Lng);
  friend DLL_DECL B_Boolean operator< (const B_Int,      const B_Long&);
  friend DLL_DECL B_Boolean operator< (const B_Long&,    const B_Int);

  friend DLL_DECL B_Boolean operator<= (const B_Long&,    const B_Long&);
  friend DLL_DECL B_Boolean operator<= (const B_Integer&, const B_Long&);
  friend DLL_DECL B_Boolean operator<= (const B_Long&,    const B_Integer&);
  friend DLL_DECL B_Boolean operator<= (const B_Byte&,    const B_Long&);
  friend DLL_DECL B_Boolean operator<= (const B_Long&,    const B_Byte&);
  friend DLL_DECL B_Boolean operator<= (const B_Long&,    const B_Empty&);
  friend DLL_DECL B_Boolean operator<= (const B_Empty&,   const B_Long&); 
  friend DLL_DECL B_Boolean operator<= (const B_Long&,    const B_Boolean&);
  friend DLL_DECL B_Boolean operator<= (const B_Boolean&, const B_Long&);
  friend DLL_DECL B_Boolean operator<= (const B_Lng,      const B_Long&);
  friend DLL_DECL B_Boolean operator<= (const B_Long&,    const B_Lng);
  friend DLL_DECL B_Boolean operator<= (const B_Int,      const B_Long&);
  friend DLL_DECL B_Boolean operator<= (const B_Long&,    const B_Int);

  friend DLL_DECL B_Boolean operator> (const B_Long&,    const B_Long&);
  friend DLL_DECL B_Boolean operator> (const B_Integer&, const B_Long&);
  friend DLL_DECL B_Boolean operator> (const B_Long&,    const B_Integer&);
  friend DLL_DECL B_Boolean operator> (const B_Byte&,    const B_Long&);
  friend DLL_DECL B_Boolean operator> (const B_Long&,    const B_Byte&);
  friend DLL_DECL B_Boolean operator> (const B_Long&,    const B_Empty&);
  friend DLL_DECL B_Boolean operator> (const B_Empty&,   const B_Long&); 
  friend DLL_DECL B_Boolean operator> (const B_Long&,    const B_Boolean&);
  friend DLL_DECL B_Boolean operator> (const B_Boolean&, const B_Long&);
  friend DLL_DECL B_Boolean operator> (const B_Lng,      const B_Long&);
  friend DLL_DECL B_Boolean operator> (const B_Long&,    const B_Lng);
  friend DLL_DECL B_Boolean operator> (const B_Int,      const B_Long&);
  friend DLL_DECL B_Boolean operator> (const B_Long&,    const B_Int);

  friend DLL_DECL B_Boolean operator>= (const B_Long&,    const B_Long&);
  friend DLL_DECL B_Boolean operator>= (const B_Integer&, const B_Long&);
  friend DLL_DECL B_Boolean operator>= (const B_Long&,    const B_Integer&);
  friend DLL_DECL B_Boolean operator>= (const B_Byte&,    const B_Long&);
  friend DLL_DECL B_Boolean operator>= (const B_Long&,    const B_Byte&);
  friend DLL_DECL B_Boolean operator>= (const B_Long&,    const B_Empty&);
  friend DLL_DECL B_Boolean operator>= (const B_Empty&,   const B_Long&); 
  friend DLL_DECL B_Boolean operator>= (const B_Long&,    const B_Boolean&);
  friend DLL_DECL B_Boolean operator>= (const B_Boolean&, const B_Long&);
  friend DLL_DECL B_Boolean operator>= (const B_Lng,      const B_Long&);
  friend DLL_DECL B_Boolean operator>= (const B_Long&,    const B_Lng);
  friend DLL_DECL B_Boolean operator>= (const B_Int,      const B_Long&);
  friend DLL_DECL B_Boolean operator>= (const B_Long&,    const B_Int);

  //----------------------------------------------------------------------------------------------------       
  // Method: operator<<	INSERTION
  //     Schreiben der eigentlichen Objektdaten im Textformat in einen Stream
  // Return Value: 
  //	   Instanz der abgeleiten fstream-Klasse der Datei
  // Note: 
  //	   CT131296
  //	   ersetzt die alte Methode streamp()
  //
  friend DLL_DECL B_fstream& operator<< (B_fstream& ostr, B_Long& rhs);

  //----------------------------------------------------------------------------------------------------       
  // Method: operator>>	EXTRACTION
  //     Lesen der eigentlichen Objektdaten im Textformat aus einem Stream
  // Return Value: 
  //	   Instanz der abgeleiten fstream-Klasse der Datei
  // Note: 
  //	   CT131296
  //	   ersetzt die alte Methode streamg()
  //
  friend DLL_DECL B_fstream& operator>> (B_fstream& istr, B_Long& rhs);

  }; // B_Long

DLL_DECL extern const B_Long  B_LongMin;
DLL_DECL extern const B_Long  B_LongMax;

//----------------------------------Arithmetische Operatoren -----------------------------------------

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der +Operator = Addition
DLL_DECL B_Long  operator+ (const B_Long&,    const B_Long&)    throw(B_Overflow);
DLL_DECL B_Long  operator+ (const B_Integer&, const B_Long&)    throw(B_Overflow);
DLL_DECL B_Long  operator+ (const B_Long&,    const B_Integer&) throw(B_Overflow);
DLL_DECL B_Long  operator+ (const B_Byte&,    const B_Long&)    throw(B_Overflow);
DLL_DECL B_Long  operator+ (const B_Long&,    const B_Byte&)    throw(B_Overflow);
DLL_DECL B_Long  operator+ (const B_Empty&,   const B_Long&);
DLL_DECL B_Long  operator+ (const B_Long&,    const B_Empty&);
DLL_DECL B_Long  operator+ (const B_Boolean&, const B_Long&)    throw(B_Overflow);
DLL_DECL B_Long  operator+ (const B_Long&,    const B_Boolean&) throw(B_Overflow);
DLL_DECL B_Null  operator+ (const B_Null&,    const B_Long&);
DLL_DECL B_Null  operator+ (const B_Long&,    const B_Null&);
//-- C-Standardtypen
DLL_DECL B_Long  operator+ (const B_Empty&,   const B_Lng);
DLL_DECL B_Long  operator+ (const B_Lng,      const B_Empty&);
DLL_DECL B_Long  operator+ (const B_UInt,     const B_Empty&);
DLL_DECL B_Long  operator+ (const B_Empty&,   const B_UInt);
DLL_DECL B_Long  operator+ (const B_Long&,    const B_Lng)      throw(B_Overflow);
DLL_DECL B_Long  operator+ (const B_Lng,      const B_Long&)    throw(B_Overflow);
DLL_DECL B_Long  operator+ (const B_Integer&, const B_Lng)      throw(B_Overflow);
DLL_DECL B_Long  operator+ (const B_Lng,      const B_Integer&) throw(B_Overflow);
DLL_DECL B_Long  operator+ (const B_Byte&,    const B_Lng)      throw(B_Overflow);
DLL_DECL B_Long  operator+ (const B_Lng,      const B_Byte&)    throw(B_Overflow);
DLL_DECL B_Long  operator+ (const B_Long&,    const B_Int)      throw(B_Overflow);
DLL_DECL B_Long  operator+ (const B_Int,      const B_Long&)    throw(B_Overflow);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der -Operator = Subtraktion
DLL_DECL B_Long  operator- (const B_Long&,    const B_Long&)    throw(B_Overflow);
DLL_DECL B_Long  operator- (const B_Integer&, const B_Long&)    throw(B_Overflow);
DLL_DECL B_Long  operator- (const B_Long&,    const B_Integer&) throw(B_Overflow);
DLL_DECL B_Long  operator- (const B_Byte&,    const B_Long&)    throw(B_Overflow);
DLL_DECL B_Long  operator- (const B_Long&,    const B_Byte&)    throw(B_Overflow);
DLL_DECL B_Long  operator- (const B_Empty&,   const B_Long&);
DLL_DECL B_Long  operator- (const B_Long&,    const B_Empty&);
DLL_DECL B_Long  operator- (const B_Boolean&, const B_Long&)    throw(B_Overflow);
DLL_DECL B_Long  operator- (const B_Long&,    const B_Boolean&) throw(B_Overflow);
DLL_DECL B_Null  operator- (const B_Null&,    const B_Long&);
DLL_DECL B_Null  operator- (const B_Long&,    const B_Null&);
DLL_DECL B_Long  operator- (const B_Empty&,   const B_Lng);
DLL_DECL B_Long  operator- (const B_Lng,      const B_Empty&);
DLL_DECL B_Long  operator- (const B_UInt,     const B_Empty&);
DLL_DECL B_Long  operator- (const B_Empty&,   const B_UInt);
DLL_DECL B_Long  operator- (const B_Lng,      const B_Long&)    throw(B_Overflow);
DLL_DECL B_Long  operator- (const B_Long&,    const B_Lng)      throw(B_Overflow);
DLL_DECL B_Long  operator- (const B_Integer&, const B_Lng)      throw(B_Overflow);
DLL_DECL B_Long  operator- (const B_Lng,      const B_Integer&) throw(B_Overflow);
DLL_DECL B_Long  operator- (const B_Byte&,    const B_Lng)      throw(B_Overflow);
DLL_DECL B_Long  operator- (const B_Lng,      const B_Byte&)    throw(B_Overflow);
DLL_DECL B_Long  operator- (const B_Int,      const B_Long&)    throw(B_Overflow);
DLL_DECL B_Long  operator- (const B_Long&,    const B_Int)      throw(B_Overflow);


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der *Operator = Multiplikation   
DLL_DECL B_Long  operator* (const B_Long&,    const B_Long&)    throw(B_Overflow);
DLL_DECL B_Long  operator* (const B_Integer&, const B_Long&)    throw(B_Overflow);
DLL_DECL B_Long  operator* (const B_Long&,    const B_Integer&) throw(B_Overflow);
DLL_DECL B_Long  operator* (const B_Byte&,    const B_Long&)    throw(B_Overflow);
DLL_DECL B_Long  operator* (const B_Long&,    const B_Byte&)    throw(B_Overflow);
DLL_DECL B_Long  operator* (const B_Empty&,   const B_Long&);
DLL_DECL B_Long  operator* (const B_Long&,    const B_Empty&);
DLL_DECL B_Long  operator* (const B_Boolean&, const B_Long&);
DLL_DECL B_Long  operator* (const B_Long&,    const B_Boolean&);
DLL_DECL B_Null  operator* (const B_Null&,    const B_Long&);
DLL_DECL B_Null  operator* (const B_Long&,    const B_Null&);
//-- C-Standardtypen
DLL_DECL B_Long  operator* (const B_Empty&,   const B_Lng);
DLL_DECL B_Long  operator* (const B_Lng,      const B_Empty&);
DLL_DECL B_Long  operator* (const B_UInt,     const B_Empty&);
DLL_DECL B_Long  operator* (const B_Empty&,   const B_UInt);
DLL_DECL B_Long  operator* (const B_Lng,      const B_Long&)    throw(B_Overflow);
DLL_DECL B_Long  operator* (const B_Long&,    const B_Lng)      throw(B_Overflow);
DLL_DECL B_Long  operator* (const B_Integer&, const B_Lng)      throw(B_Overflow);
DLL_DECL B_Long  operator* (const B_Lng,      const B_Integer&) throw(B_Overflow);
DLL_DECL B_Long  operator* (const B_Byte&,    const B_Lng)      throw(B_Overflow);
DLL_DECL B_Long  operator* (const B_Lng,      const B_Byte&)    throw(B_Overflow);
DLL_DECL B_Long  operator* (const B_Int,      const B_Long&)    throw(B_Overflow);
DLL_DECL B_Long  operator* (const B_Long&,    const B_Int)      throw(B_Overflow);


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der /Operator = Division deren Ergebniss immer ein Fliesskommawert ergibt
//   Fehler wenn Bereich von Single ueberschritten wird
DLL_DECL B_Null operator/ (const B_Null&,     const B_Long&);
DLL_DECL B_Null operator/ (const B_Long&,     const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der %Operator = Modulo Operation, gibt den Wert einer ganzahligen Division zurueck
//   entsprich MOD-Operator in VB
DLL_DECL B_Long  operator% (const B_Long&,    const B_Long&)    throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Integer&, const B_Long&)    throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Long&,    const B_Integer&) throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Byte&,    const B_Long&)    throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Long&,    const B_Byte&)    throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Empty&,   const B_Long&)    throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Long&,    const B_Empty&)   throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Empty&,   const B_Empty&)   throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Long&,    const B_Boolean&) throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Boolean&, const B_Long&)    throw(B_ZeroDivide);
DLL_DECL B_Null  operator% (const B_Null&,    const B_Long&);
DLL_DECL B_Null  operator% (const B_Long&,    const B_Null&);
//-- C-Standardtypen
DLL_DECL B_Long  operator% (const B_Dbl,      const B_Empty&)   throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Empty&,   const B_Dbl)      throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Flt,      const B_Empty&)   throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Empty&,   const B_Flt)      throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Empty&,   const B_Lng)      throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Lng,      const B_Empty&)   throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_UInt,     const B_Empty&)   throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Empty&,   const B_UInt)     throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Lng,      const B_Long&)    throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Long&,    const B_Lng)      throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Integer&, const B_Lng)      throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Lng,      const B_Integer&) throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Byte&,    const B_Lng)      throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Lng,      const B_Byte&)    throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Dbl,      const B_Long&)    throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Long&,    const B_Dbl)      throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Integer&, const B_Dbl)      throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Dbl,      const B_Integer&) throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Byte&,    const B_Dbl)      throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Dbl,      const B_Byte&)    throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Flt,      const B_Long&)    throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Long&,    const B_Flt)      throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Integer&, const B_Flt)      throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Flt,      const B_Integer&) throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Byte&,    const B_Flt)      throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Flt,      const B_Byte&)    throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Int,      const B_Long&)    throw(B_ZeroDivide);
DLL_DECL B_Long  operator% (const B_Long&,    const B_Int)      throw(B_ZeroDivide);


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Ganzahldivision entspricht dem \Operator in VB, da in C++ kein geeigneter Operator zu Verfuegung steht
//   wird dieser Operator als Funktion realisiert
DLL_DECL B_Long B_Div(const B_Long&,    const B_Long&)    throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Integer&, const B_Long&)    throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Long&,    const B_Integer&) throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Byte&,    const B_Long&)    throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Long&,    const B_Byte&)    throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Empty&,   const B_Long&)    throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Long&,    const B_Empty&)   throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Long&,    const B_Boolean&) throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Boolean&, const B_Long&)    throw(B_ZeroDivide);
DLL_DECL B_Null B_Div(const B_Long&,    const B_Null&);
DLL_DECL B_Null B_Div(const B_Null&,    const B_Long&);
DLL_DECL B_Long B_Div(const B_Empty&,   const B_Empty&)   throw(B_ZeroDivide);
//-- C-Standardtypen
DLL_DECL B_Long B_Div(const B_Dbl,      const B_Empty&)   throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Empty&,   const B_Dbl)      throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Flt,      const B_Empty&)   throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Empty&,   const B_Flt)      throw(B_ZeroDivide); 
DLL_DECL B_Long B_Div(const B_Empty&,   const B_Lng)      throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Lng,      const B_Empty&)   throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_UInt,     const B_Empty&)   throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Empty&,   const B_UInt)     throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Lng,      const B_Long&)    throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Long&,    const B_Lng)      throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Integer&, const B_Lng)      throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Lng,      const B_Integer&) throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Byte&,    const B_Lng)      throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Lng,      const B_Byte&)    throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Flt,      const B_Long&)    throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Long&,    const B_Flt)      throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Integer&, const B_Flt)      throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Flt,      const B_Integer&) throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Byte&,    const B_Flt)      throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Flt,      const B_Byte&)    throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Dbl,      const B_Long&)    throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Long&,    const B_Dbl)      throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Integer&, const B_Dbl)      throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Dbl,      const B_Integer&) throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Byte&,    const B_Dbl)      throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Dbl,      const B_Byte&)    throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Int,      const B_Long&)    throw(B_ZeroDivide);
DLL_DECL B_Long B_Div(const B_Long&,    const B_Int)      throw(B_ZeroDivide);


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Dient zum Potenzieren einer Zahl mit einem Exponenten
//   Erg = Zahl^Exponent  -> Erg = Pow(zahl, exponent)
//   Das Argument Zahl darf nur negativ sein, wenn Exponent ein ganzzahliger Wert ist. Werden in einem einzigen 
//   Ausdruck mehrere Potenzierungen ausgefhrt, so wird der Operator ^ jeweils in der Reihenfolge seines 
//   Auftretens von links nach rechts ausgewertet.
//   Ergebnis hat den Datentyp Double  Ist jedoch entweder Zahl oder Exponent ein Ausdruck mit dem Wert Null, 
//   so ist das Ergebnis ebenfalls Null.
DLL_DECL B_Null B_Pow(const B_Null&, const B_Long&);
DLL_DECL B_Null B_Pow(const B_Long&, const B_Null&);

//----------------------------------------------------------------------------------------------------
//---------------------------------- Logische Operatoren ---------------------------------------------
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der !Operator entspricht NOT in VB , 
//   Bit in Ausdruck1  	Ergebnis 
//       0	                1
//       1	                0
//   Die Operatoren sind bei den jeweiligen Klassen implementiert
// DLL_DECL B_Long operator!(const B_Single&) const;
// DLL_DECL B_Long operator!(const B_Double&) const;
// DLL_DECL B_Long operator!(const B_Currency&) const;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der &&Operator entspricht AND in VB , 
//   Bit in Ausdruck1 	Bit in Ausdruck2 	Ergebnis 
//       0	                  0            	0
//       0	                  1           	0
//       1                  	0           	0
//       1                   	1            	1
//   ACHTUNG alle C/C++ Programmierer: in VB werden numerische Typen immer logisch bitweise und verknpft
//                                     Der Einsatz von & war nicht moeglich da stellenweise erst zur Lauf-
//                                     zeit das Verhalten bestimmt werden kann was der Konverter nicht 
//                                     bestimmen kann
DLL_DECL B_Long operator&& (const B_Long&,    const B_Long&);
DLL_DECL B_Long operator&& (const B_Integer&, const B_Long&);
DLL_DECL B_Long operator&& (const B_Long&,    const B_Integer&);
DLL_DECL B_Long operator&& (const B_Byte&,    const B_Long&);
DLL_DECL B_Long operator&& (const B_Long&,    const B_Byte&);
DLL_DECL B_Long operator&& (const B_Empty&,   const B_Long&);     // Empty als 0 interpretiert wird
DLL_DECL B_Long operator&& (const B_Long&,    const B_Empty&);    // Empty als 0 interpretiert wird
DLL_DECL B_Long operator&& (const B_Boolean&, const B_Long&);
DLL_DECL B_Long operator&& (const B_Long&,    const B_Boolean&);
DLL_DECL B_Null operator&& (const B_Null&,    const B_Long&);
DLL_DECL B_Null operator&& (const B_Long&,    const B_Null&);
//-- Bibliothekstypen
DLL_DECL B_Long operator&& (const B_Empty&,   const B_Empty&);
DLL_DECL B_Long operator&& (const B_Empty&,   const B_Byte&);
DLL_DECL B_Long operator&& (const B_Byte&,    const B_Empty&);
DLL_DECL B_Long operator&& (const B_Empty&,   const B_Integer&);
DLL_DECL B_Long operator&& (const B_Integer&, const B_Empty&);
DLL_DECL B_Long operator&& (const B_Empty&,   const B_Boolean&);
DLL_DECL B_Long operator&& (const B_Boolean&, const B_Empty&);
DLL_DECL B_Long operator&& (const B_Null&,    const B_Empty&);
DLL_DECL B_Long operator&& (const B_Empty&,   const B_Null&);
//-- C-Standarddatentypen
DLL_DECL B_Long operator&& (const B_Int,      const B_Empty&);
DLL_DECL B_Long operator&& (const B_Empty&,   const B_Int);
DLL_DECL B_Long operator&& (const B_UInt,     const B_Empty&);
DLL_DECL B_Long operator&& (const B_Empty&,   const B_UInt);
DLL_DECL B_Long operator&& (const B_Lng,      const B_Empty&);
DLL_DECL B_Long operator&& (const B_Empty&,   const B_Lng);
DLL_DECL B_Long operator&& (const B_Flt,      const B_Empty&);
DLL_DECL B_Long operator&& (const B_Empty&,   const B_Flt);
DLL_DECL B_Long operator&& (const B_Dbl,      const B_Empty&);
DLL_DECL B_Long operator&& (const B_Empty&,   const B_Dbl);
DLL_DECL B_Long operator&& (const B_Lng,      const B_Long&);
DLL_DECL B_Long operator&& (const B_Long&,    const B_Lng);
DLL_DECL B_Long operator&& (const B_Integer&, const B_Lng);
DLL_DECL B_Long operator&& (const B_Lng,      const B_Integer&);
DLL_DECL B_Long operator&& (const B_Byte&,    const B_Lng);
DLL_DECL B_Long operator&& (const B_Lng,      const B_Byte&);
DLL_DECL B_Long operator&& (const B_Flt,      const B_Long&);
DLL_DECL B_Long operator&& (const B_Long&,    const B_Flt);
DLL_DECL B_Long operator&& (const B_Integer&, const B_Flt);
DLL_DECL B_Long operator&& (const B_Flt,      const B_Integer&);
DLL_DECL B_Long operator&& (const B_Byte&,    const B_Flt);
DLL_DECL B_Long operator&& (const B_Flt,      const B_Byte&);
DLL_DECL B_Long operator&& (const B_Dbl,      const B_Long&);
DLL_DECL B_Long operator&& (const B_Long&,    const B_Dbl);
DLL_DECL B_Long operator&& (const B_Integer&, const B_Dbl);
DLL_DECL B_Long operator&& (const B_Dbl,      const B_Integer&);
DLL_DECL B_Long operator&& (const B_Byte&,    const B_Dbl);
DLL_DECL B_Long operator&& (const B_Dbl,      const B_Byte&);
DLL_DECL B_Long operator&& (const B_Int,      const B_Long&);
DLL_DECL B_Long operator&& (const B_Long&,    const B_Int);



/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der ||Operator entspricht OR in VB , 
//   Bit in Ausdruck1 	Bit in Ausdruck2 	Ergebnis 
//       0	                  0            	0
//       0	                  1           	1
//       1                  	0           	1
//       1                   	1            	1
//   ACHTUNG alle C/C++ Programmierer: in VB werden numerische Typen immer logisch bitweise und verknpft
//                                     Der Einsatz von | war nicht moeglich da stellenweise erst zur Lauf-
//                                     zeit das Verhalten bestimmt werden kann was der Konverter nicht 
//                                     bestimmen kann
DLL_DECL B_Long operator|| (const B_Long&,    const B_Long&);
DLL_DECL B_Long operator|| (const B_Integer&, const B_Long&);
DLL_DECL B_Long operator|| (const B_Long&,    const B_Integer&);
DLL_DECL B_Long operator|| (const B_Byte&,    const B_Long&);
DLL_DECL B_Long operator|| (const B_Long&,    const B_Byte&);
DLL_DECL B_Long operator|| (const B_Empty&,   const B_Long&);    // B_Long ohne Aenderung zurueckgeben, 
DLL_DECL B_Long operator|| (const B_Long&,    const B_Empty&);   // B_Long ohne Aenderung zurueckgeben
DLL_DECL B_Long operator|| (const B_Boolean&, const B_Long&);
DLL_DECL B_Long operator|| (const B_Long&,    const B_Boolean&); 
DLL_DECL B_Long operator|| (const B_Null&,    const B_Long&);
DLL_DECL B_Long operator|| (const B_Long&,    const B_Null&);
//-- Bibliothekstypen
DLL_DECL B_Long operator|| (const B_Empty&,   const B_Empty&);
DLL_DECL B_Long operator|| (const B_Empty&,   const B_Byte&);
DLL_DECL B_Long operator|| (const B_Byte&,    const B_Empty&);
DLL_DECL B_Long operator|| (const B_Empty&,   const B_Boolean&);
DLL_DECL B_Long operator|| (const B_Boolean&, const B_Empty&);
DLL_DECL B_Long operator|| (const B_Empty&,   const B_Long&);
DLL_DECL B_Long operator|| (const B_Long&,    const B_Empty&);
DLL_DECL B_Long operator|| (const B_Empty&,   const B_Integer&);
DLL_DECL B_Long operator|| (const B_Integer&, const B_Empty&);
//-- C-Standardtypen
DLL_DECL B_Long operator|| (const B_Int,      const B_Empty&);
DLL_DECL B_Long operator|| (const B_Empty&,   const B_Int);
DLL_DECL B_Long operator|| (const B_UInt,     const B_Empty&);
DLL_DECL B_Long operator|| (const B_Empty&,   const B_UInt);
DLL_DECL B_Long operator|| (const B_Empty&,   const B_Lng);
DLL_DECL B_Long operator|| (const B_Lng,      const B_Empty&);
DLL_DECL B_Long operator|| (const B_Flt,      const B_Empty&);
DLL_DECL B_Long operator|| (const B_Empty&,   const B_Flt);
DLL_DECL B_Long operator|| (const B_Dbl,      const B_Empty&);
DLL_DECL B_Long operator|| (const B_Empty&,   const B_Dbl);
DLL_DECL B_Long operator|| (const B_Dbl,   const B_Null&);
DLL_DECL B_Long operator|| (const B_Null&, const B_Dbl);
DLL_DECL B_Long operator|| (const B_Flt,   const B_Null&);
DLL_DECL B_Long operator|| (const B_Null&, const B_Flt);
DLL_DECL B_Long operator|| (const B_Null&, const B_Lng);
DLL_DECL B_Long operator|| (const B_Lng,   const B_Null&);
DLL_DECL B_Long operator|| (const B_UInt,  const B_Null&);
DLL_DECL B_Long operator|| (const B_Null&, const B_UInt);
DLL_DECL B_Long operator|| (const B_Int,   const B_Null&);
DLL_DECL B_Long operator|| (const B_Null&, const B_Int);

DLL_DECL B_Long operator|| (const B_Lng,      const B_Long&);
DLL_DECL B_Long operator|| (const B_Long&,    const B_Lng);
DLL_DECL B_Long operator|| (const B_Integer&, const B_Lng);
DLL_DECL B_Long operator|| (const B_Lng,      const B_Integer&);
DLL_DECL B_Long operator|| (const B_Byte&,    const B_Lng);
DLL_DECL B_Long operator|| (const B_Lng,      const B_Byte&);
DLL_DECL B_Long operator|| (const B_Flt,      const B_Long&);
DLL_DECL B_Long operator|| (const B_Long&,    const B_Flt);
DLL_DECL B_Long operator|| (const B_Integer&, const B_Flt);
DLL_DECL B_Long operator|| (const B_Flt,      const B_Integer&);
DLL_DECL B_Long operator|| (const B_Byte&,    const B_Flt);
DLL_DECL B_Long operator|| (const B_Flt,      const B_Byte&);
DLL_DECL B_Long operator|| (const B_Dbl,      const B_Long&);
DLL_DECL B_Long operator|| (const B_Long&,    const B_Dbl);
DLL_DECL B_Long operator|| (const B_Integer&, const B_Dbl);
DLL_DECL B_Long operator|| (const B_Dbl,      const B_Integer&);
DLL_DECL B_Long operator|| (const B_Byte&,    const B_Dbl);
DLL_DECL B_Long operator|| (const B_Dbl,      const B_Byte&);
DLL_DECL B_Long operator|| (const B_Int,      const B_Long&);
DLL_DECL B_Long operator|| (const B_Long&,    const B_Int);


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Der ^Operator entspricht XOR in VB 
//   Bit in Ausdruck1 	Bit in Ausdruck2 	Ergebnis 
//       0	                  0            	0
//       0	                  1           	1
//       1                  	0           	1
//       1                   	1            	0
//   ACHTUNG alle VB Programmierer:    In C++ bedeutet der ^Operator ein bitweises XOR auf numerische
//                                     Operanden und nicht wie in VB eine Potenzierung. Der ^Operator
//                                     wurde nicht zur Potenzierung verwandt da dies nicht C++ Stil
//                                     entspricht.
//   ACHTUNG alle C/C++ Programmierer: In VB werden numerische Typen immer logisch bitweise und verknpft
//                                     Dies hat fuer ^Operator die Folge, dass er bei nicht numerischen
//                                     Ausdreucken sich wie ein logisches XOR verhaelt
DLL_DECL B_Long operator^ (const B_Long&,    const B_Long&);
DLL_DECL B_Long operator^ (const B_Integer&, const B_Long&);
DLL_DECL B_Long operator^ (const B_Long&,    const B_Integer&);
DLL_DECL B_Long operator^ (const B_Byte&,    const B_Long&);
DLL_DECL B_Long operator^ (const B_Long&,    const B_Byte&);
DLL_DECL B_Long operator^ (const B_Empty&,   const B_Long&);    // Empty als 0 interpretiert wird
DLL_DECL B_Long operator^ (const B_Long&,    const B_Empty&);   // Empty als 0 interpretiert wird
DLL_DECL B_Long operator^ (const B_Long&,    const B_Boolean&);
DLL_DECL B_Long operator^ (const B_Boolean&, const B_Long&);
DLL_DECL B_Null operator^ (const B_Null&,    const B_Long&);
DLL_DECL B_Null operator^ (const B_Long&,    const B_Null&);
//-- Bibliothekstypen
DLL_DECL B_Long operator^ (const B_Empty&,   const B_Empty&);
DLL_DECL B_Long operator^ (const B_Empty&,   const B_Byte&);
DLL_DECL B_Long operator^ (const B_Byte&,    const B_Empty&);
DLL_DECL B_Long operator^ (const B_Empty&,   const B_Integer&);
DLL_DECL B_Long operator^ (const B_Integer&, const B_Empty&);
DLL_DECL B_Long operator^ (const B_Empty&,   const B_Boolean&);
DLL_DECL B_Long operator^ (const B_Boolean&, const B_Empty&);
//-- C-Standardtypen
DLL_DECL B_Long operator^ (const B_Empty&,   const B_Int);
DLL_DECL B_Long operator^ (const B_UInt,     const B_Empty&);
DLL_DECL B_Long operator^ (const B_Empty&,   const B_UInt);
DLL_DECL B_Long operator^ (const B_Empty&,   const B_Lng);
DLL_DECL B_Long operator^ (const B_Lng,      const B_Empty&);
DLL_DECL B_Long operator^ (const B_Flt,      const B_Empty&);
DLL_DECL B_Long operator^ (const B_Empty&,   const B_Flt);
DLL_DECL B_Long operator^ (const B_Dbl,      const B_Empty&);
DLL_DECL B_Long operator^ (const B_Empty&,   const B_Dbl);

DLL_DECL B_Long operator^ (const B_Lng,      const B_Long&);
DLL_DECL B_Long operator^ (const B_Long&,    const B_Lng);
DLL_DECL B_Long operator^ (const B_Integer&, const B_Lng);
DLL_DECL B_Long operator^ (const B_Lng,      const B_Integer&);
DLL_DECL B_Long operator^ (const B_Byte&,    const B_Lng);
DLL_DECL B_Long operator^ (const B_Lng,      const B_Byte&);
DLL_DECL B_Long operator^ (const B_Int,      const B_Empty&);
DLL_DECL B_Long operator^ (const B_Empty&,   const B_Int);
DLL_DECL B_Long operator^ (const B_UInt,     const B_Empty&);
DLL_DECL B_Long operator^ (const B_Empty&,   const B_UInt);
DLL_DECL B_Long operator^ (const B_Empty&,   const B_Lng);
DLL_DECL B_Long operator^ (const B_Lng,      const B_Empty&);
DLL_DECL B_Long operator^ (const B_Flt,      const B_Empty&);
DLL_DECL B_Long operator^ (const B_Empty&,   const B_Flt);
DLL_DECL B_Long operator^ (const B_Dbl,      const B_Empty&);
DLL_DECL B_Long operator^ (const B_Empty&,   const B_Dbl);
DLL_DECL B_Long operator^ (const B_Int,      const B_Long&);
DLL_DECL B_Long operator^ (const B_Long&,    const B_Int);


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Die Operatoren Imp (=logische Implikation) und Eqv (=logische quivalenz) werden vom Konverter
// in folgende Basisoperatorkonstrukte umgesetzt, da in C++ keine quivalenten Operatoren zur Verfgung
// stehen.
// a Imp b    ->  (!a || b)
// a Eqv b    ->  !(a ^ b)


//----------------------------------------------------------------------------------------------------
//---------------------------------- Vergleichs Operatoren -------------------------------------------

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Verlgeich auf Gleichheit der Operanden, es findet ein numerischer Vergleich statt
// Ausdruck1 == Ausdruck2 -> B_True
// Ausdruck1 != Ausdruck2 -> B_False
// ACHTUNG alle VB Programmierer:  In C++ wird mit '==' auf Gleichheit abgefragt und nicht mit '='
//                                 Der '=' Operator wird nur fuer Zuweisungen verwendet.
DLL_DECL B_Boolean operator== (const B_Long&,    const B_Long&);
DLL_DECL B_Boolean operator== (const B_Integer&, const B_Long&);
DLL_DECL B_Boolean operator== (const B_Long&,    const B_Integer&);
DLL_DECL B_Boolean operator== (const B_Byte&,    const B_Long&);
DLL_DECL B_Boolean operator== (const B_Long&,    const B_Byte&);
DLL_DECL B_Boolean operator== (const B_Long&,    const B_Empty&);  // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator== (const B_Empty&,   const B_Long&);   // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator== (const B_Long&,    const B_Boolean&);
DLL_DECL B_Boolean operator== (const B_Boolean&, const B_Long&);
DLL_DECL B_Null    operator== (const B_Long&,    const B_Null&);
DLL_DECL B_Null    operator== (const B_Null&,    const B_Long&);

DLL_DECL B_Boolean operator== (const B_Lng,      const B_Long&);
DLL_DECL B_Boolean operator== (const B_Long&,    const B_Lng);
DLL_DECL B_Boolean operator== (const B_Integer&, const B_Lng);
DLL_DECL B_Boolean operator== (const B_Lng,      const B_Integer&);
DLL_DECL B_Boolean operator== (const B_Byte&,    const B_Lng);
DLL_DECL B_Boolean operator== (const B_Lng,      const B_Byte&);
DLL_DECL B_Boolean operator== (const B_Int,      const B_Long&);
DLL_DECL B_Boolean operator== (const B_Long&,    const B_Int);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Verlgeich auf Ungleichheit der Operanden, es findet ein numerischer Vergleich statt
// Ausdruck1 != Ausdruck2 -> B_True
// Ausdruck1 == Ausdruck2 -> B_False
// ACHTUNG alle VB Programmierer:  In C++ wird mit '!=' auf Ungleichheit abgefragt und nicht mit '<>'
//                                 Der '<>' Operator existiert nicht in C++.
DLL_DECL B_Boolean operator!= (const B_Long&,    const B_Long&);
DLL_DECL B_Boolean operator!= (const B_Integer&, const B_Long&);
DLL_DECL B_Boolean operator!= (const B_Long&,    const B_Integer&);
DLL_DECL B_Boolean operator!= (const B_Byte&,    const B_Long&);
DLL_DECL B_Boolean operator!= (const B_Long&,    const B_Byte&);
DLL_DECL B_Boolean operator!= (const B_Long&,    const B_Empty&);  // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator!= (const B_Empty&,   const B_Long&);   // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator!= (const B_Long&,    const B_Boolean&);
DLL_DECL B_Boolean operator!= (const B_Boolean&, const B_Long&);
DLL_DECL B_Null    operator!= (const B_Long&,    const B_Null&);
DLL_DECL B_Null    operator!= (const B_Null&,    const B_Long&);

DLL_DECL B_Boolean operator!= (const B_Integer&, const B_Lng);
DLL_DECL B_Boolean operator!= (const B_Lng,      const B_Integer&);
DLL_DECL B_Boolean operator!= (const B_Lng,      const B_Long&);
DLL_DECL B_Boolean operator!= (const B_Long&,    const B_Lng);
DLL_DECL B_Boolean operator!= (const B_Byte&,    const B_Lng);
DLL_DECL B_Boolean operator!= (const B_Lng,      const B_Byte&);
DLL_DECL B_Boolean operator!= (const B_Int,      const B_Long&);
DLL_DECL B_Boolean operator!= (const B_Long&,    const B_Int);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Verlgeich auf Kleiner der Operanden, es findet ein numerischer Vergleich statt
// Ausdruck1 <  Ausdruck2 -> B_True
// Ausdruck1 >= Ausdruck2 -> B_False
DLL_DECL B_Boolean operator< (const B_Long&,    const B_Long&);
DLL_DECL B_Boolean operator< (const B_Integer&, const B_Long&);
DLL_DECL B_Boolean operator< (const B_Long&,    const B_Integer&);
DLL_DECL B_Boolean operator< (const B_Byte&,    const B_Long&);
DLL_DECL B_Boolean operator< (const B_Long&,    const B_Byte&);
DLL_DECL B_Boolean operator< (const B_Long&,    const B_Empty&);  // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator< (const B_Empty&,   const B_Long&);   // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator< (const B_Long&,    const B_Boolean&);
DLL_DECL B_Boolean operator< (const B_Boolean&, const B_Long&);
DLL_DECL B_Null    operator< (const B_Long&,    const B_Null&);
DLL_DECL B_Null    operator< (const B_Null&,    const B_Long&);

DLL_DECL B_Boolean operator< (const B_Lng,      const B_Long&);
DLL_DECL B_Boolean operator< (const B_Long&,    const B_Lng);
DLL_DECL B_Boolean operator< (const B_Integer&, const B_Lng);
DLL_DECL B_Boolean operator< (const B_Lng,      const B_Integer&);
DLL_DECL B_Boolean operator< (const B_Byte&,    const B_Lng);
DLL_DECL B_Boolean operator< (const B_Lng,      const B_Byte&);
DLL_DECL B_Boolean operator< (const B_Int,      const B_Long&);
DLL_DECL B_Boolean operator< (const B_Long&,    const B_Int);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Verlgeich auf Kleiner/Gleich der Operanden, es findet ein numerischer Vergleich statt
// Ausdruck1 <=  Ausdruck2 -> B_True
// Ausdruck1 >   Ausdruck2 -> B_False
DLL_DECL B_Boolean operator<= (const B_Long&,    const B_Long&);
DLL_DECL B_Boolean operator<= (const B_Integer&, const B_Long&);
DLL_DECL B_Boolean operator<= (const B_Long&,    const B_Integer&);
DLL_DECL B_Boolean operator<= (const B_Byte&,    const B_Long&);
DLL_DECL B_Boolean operator<= (const B_Long&,    const B_Byte&);
DLL_DECL B_Boolean operator<= (const B_Long&,    const B_Empty&);  // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator<= (const B_Empty&,   const B_Long&);   // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator<= (const B_Long&,    const B_Boolean&);
DLL_DECL B_Boolean operator<= (const B_Boolean&, const B_Long&);
DLL_DECL B_Null    operator<= (const B_Long&,    const B_Null&);
DLL_DECL B_Null    operator<= (const B_Null&,    const B_Long&);

DLL_DECL B_Boolean operator<= (const B_Lng,      const B_Long&);
DLL_DECL B_Boolean operator<= (const B_Long&,    const B_Lng);
DLL_DECL B_Boolean operator<= (const B_Integer&, const B_Lng);
DLL_DECL B_Boolean operator<= (const B_Lng,      const B_Integer&);
DLL_DECL B_Boolean operator<= (const B_Byte&,    const B_Lng);
DLL_DECL B_Boolean operator<= (const B_Lng,      const B_Byte&);
DLL_DECL B_Boolean operator<= (const B_Int,      const B_Long&);
DLL_DECL B_Boolean operator<= (const B_Long&,    const B_Int);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Verlgeich auf Groesser der Operanden, es findet ein numerischer Vergleich statt
// Ausdruck1 >   Ausdruck2 -> B_True
// Ausdruck1 <=  Ausdruck2 -> B_False
DLL_DECL B_Boolean operator> (const B_Long&,     const B_Long&);
DLL_DECL B_Boolean operator> (const B_Integer&,  const B_Long&);
DLL_DECL B_Boolean operator> (const B_Long&,     const B_Integer&);
DLL_DECL B_Boolean operator> (const B_Byte&,     const B_Long&);
DLL_DECL B_Boolean operator> (const B_Long&,     const B_Byte&);
DLL_DECL B_Boolean operator> (const B_Long&,     const B_Empty&);     // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator> (const B_Empty&,    const B_Long&);   // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator> (const B_Long&,     const B_Boolean&);
DLL_DECL B_Boolean operator> (const B_Boolean&,  const B_Long&);
DLL_DECL B_Null    operator> (const B_Long&,     const B_Null&);
DLL_DECL B_Null    operator> (const B_Null&,     const B_Long&);

DLL_DECL B_Boolean operator> (const B_Lng,       const B_Long&);
DLL_DECL B_Boolean operator> (const B_Long&,     const B_Lng);
DLL_DECL B_Boolean operator> (const B_Integer&,  const B_Lng);
DLL_DECL B_Boolean operator> (const B_Lng,       const B_Integer&);
DLL_DECL B_Boolean operator> (const B_Byte&,     const B_Lng);
DLL_DECL B_Boolean operator> (const B_Lng,       const B_Byte&);
DLL_DECL B_Boolean operator> (const B_Int,       const B_Long&);
DLL_DECL B_Boolean operator> (const B_Long&,     const B_Int);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Verlgeich auf Groesser/Gleich der Operanden, es findet ein numerischer Vergleich statt
// Ausdruck1 >=  Ausdruck2 -> B_True
// Ausdruck1 <   Ausdruck2 -> B_False
DLL_DECL B_Boolean operator>= (const B_Long&,     const B_Long&);
DLL_DECL B_Boolean operator>= (const B_Integer&,  const B_Long&);
DLL_DECL B_Boolean operator>= (const B_Long&,     const B_Integer&);
DLL_DECL B_Boolean operator>= (const B_Byte&,     const B_Long&);
DLL_DECL B_Boolean operator>= (const B_Long&,     const B_Byte&);
DLL_DECL B_Boolean operator>= (const B_Long&,     const B_Empty&);  // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator>= (const B_Empty&,    const B_Long&);   // B_Empty wird als 0 interprtiert
DLL_DECL B_Boolean operator>= (const B_Long&,     const B_Boolean&);
DLL_DECL B_Boolean operator>= (const B_Boolean&,  const B_Long&);
DLL_DECL B_Null    operator>= (const B_Long&,     const B_Null&);
DLL_DECL B_Null    operator>= (const B_Null&,     const B_Long&);

DLL_DECL B_Boolean operator>= (const B_Lng,       const B_Long&);
DLL_DECL B_Boolean operator>= (const B_Long&,     const B_Lng);
DLL_DECL B_Boolean operator>= (const B_Integer&,  const B_Lng);
DLL_DECL B_Boolean operator>= (const B_Lng,       const B_Integer&);
DLL_DECL B_Boolean operator>= (const B_Byte&,     const B_Lng);
DLL_DECL B_Boolean operator>= (const B_Lng,       const B_Byte&);
DLL_DECL B_Boolean operator>= (const B_Int,       const B_Long&);
DLL_DECL B_Boolean operator>= (const B_Long&,     const B_Int);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Streamoperatoren
DLL_DECL ostream& operator<<(ostream& ostr, const B_Long& rhs);
DLL_DECL istream& operator>>(istream& istr, B_Long& rhs);


//================================ Inlinefunktionen =========================================================
//
#if defined _DEBUG || defined B_VB_CONTRACT
  #define B_NOINLINE
#else
  #undef B_NOINLINE
#endif

#ifndef B_NOINLINE                 // festlegen wenn Debuger keine Inlines debuggen kann
  #define B_INLINE inline
  #ifndef BI_LONG_HPP
    #include "bi_long.hpp"
  #endif
#endif


//===========================================================================================================
//================================ End Of Header ============================================================
#endif // BC_LONG_HPP

#endif // B_VB_COMPATIBLE
