#ifndef BC_NULL_HPP
#define BC_NULL_HPP
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Project:                 VB nach C/C++ Konverter Klassenbibliothek
// Modul:                   VB Klasse fuer den Datentyp Boolean
// Filename:                bc_null.hpp
// Version:                 0.0
// Status:                  beta
//
// Copyright:               (C) 1995
//                          Cicerosoft GmbH
//                          Ballweilerstr. 7
//                          66399 Erfweiler-Ehlingen
//
// Operating System(s):     Win95
// Compiler:                VC++ 4.1
//
// Author:                  Jochen Diener JD
// Creation Date:           JD 20.12.95
// Last Modification:       JD 22.06.95
//
// Description:             Beinhaltet alle Klassen / Hilfskt. die im Zusammenhang mit der VB
//                          Konvertierung, fuer die Bereitstellung des Datentyps B_Null benoetigt
//                          werden.
//
// Dependencies:            bc_null.cpp   -> Klassenimpl.
//                          bi_null.hpp   -> Inline (Member)Funktionen der Klasse
//
// Remarks:
//
//
// Revision(s):
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

//================================= Defintionsabhaengigkeiten ===============================================
//
#ifndef BD_INTYP_HPP
  #include "bd_intyp.hpp"       // alias der Standardatentypen
#endif

#ifndef BC_BOOL_HPP
  #include "bc_bool.hpp"
#endif

#ifndef BC_EMPTY_HPP
  #include "bc_empty.hpp"       // alias der Standardatentypen
#endif


//================================= Export =================================================================
//

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Description:                                                                           (C) CiceroSoft 1995
//     B_Null  Null-Datentyp von VB emulieren
// Hierachy:    
// Inheritance: 
// Errorhandling:
// Note: 
//     Null ist in VB in logischen Ausdruecken und nur dort im boolschen Sinne 'False'. In allen anderen 
//     loest Null LZF(94) aus (s.auch B_Boolean) Soll also ein Null Wert in logischen Ausdruecken auf
//     False geprueft werden so ist dies explizit auszuschreiben (z.B. 'if (variantVar == B_Null) then ...'
//     und nicht 'if (variantVar) then ...')
// Impl.: 
class DLL_DECL B_Null
  {
  //================ MANAGEMENT =========================================================================
  //
  public:
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     ctor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:  
    //     default
    B_Null(); 

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     dtor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:  
    //     default
    ~B_Null(); 

    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     cctor
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:  
    //     default, in VB Syntax error
    B_Null(const B_Null&);

  private:
    //---------------------------------------------------------------------------------------------------------       
    // Method:
    //     Zuweisungsop.
    // Parameter:  
    // Return Value:
    // Errorhandling:
    // Note:  
    //     default, in VB Syntax Error
    B_Null& operator=(const B_Null&);

    B_Null& operator=(const B_Int);
    B_Null& operator=(const B_Lng);
    B_Null& operator=(const B_Flt);
    B_Null& operator=(const B_Dbl);
    B_Null& operator=(const B_Empty&);
  
  //==================== FRIENDS ============================================================================
  //
  }; // B_Null

DLL_DECL extern const B_Null B_null; // globales Objekt das fuer alle B_Null-Operationen verwandt wird

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//-- Die Operatoren werden nicht mit B_Null und B_UChar& konstruiert da dies mit Standard
//   C++ Datentypen wenig Sinn macht da sie Basicspezifisch sind. Allerdings werden Literale akzeptiert
//   und somit muss auch fuer C-Standardtypen ueberladen werden. 
//   Jegliche Kombination mit Benutzerdefinierten Typen (Strukturen) in Vb fuehrt zu einem IDE TypeMismatch Error
//   Da bei der Konvertierung von Syntaxfehlern frei laufenden Programmen ausgegangen wird muss nicht
//   notwendigerweise alle Typen explizit ueberladen werden um Standarddatentypen mit einem Compilerfehler
//   abzuhandeln. Sollte dies erforderlich/gewuenscht werden muss halt fuer alle Bibliotheks und
//   C-Standarddatentypen (wegen mehrdeutigkeit) ueberladen werden
DLL_DECL B_Null operator+(const B_Null&, const B_Null&);

// C-Standardtypen
DLL_DECL B_Null operator+ (const B_Dbl,   const B_Null&);
DLL_DECL B_Null operator+ (const B_Null&, const B_Dbl);
DLL_DECL B_Null operator+ (const B_Flt,   const B_Null&);
DLL_DECL B_Null operator+ (const B_Null&, const B_Flt);
DLL_DECL B_Null operator+ (const B_Null&, const B_Lng);
DLL_DECL B_Null operator+ (const B_Lng,   const B_Null&);
DLL_DECL B_Null operator+ (const B_UInt,  const B_Null&);
DLL_DECL B_Null operator+ (const B_Null&, const B_UInt);
DLL_DECL B_Null operator+ (const B_Int,   const B_Null&);
DLL_DECL B_Null operator+ (const B_Null&, const B_Int);

// Bibliothekstypen
DLL_DECL B_Null operator+ (const B_Null&,      const B_Boolean&);
DLL_DECL B_Null operator+ (const B_Boolean&,   const B_Null&);
DLL_DECL B_Null operator+ (const B_Null&,      const B_Empty&);
DLL_DECL B_Null operator+ (const B_Empty&,     const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// fuer alle gleich, s.h. opertor+()
DLL_DECL B_Null operator- (const B_Null&, const B_Null&);

// C-Standardtypen
DLL_DECL B_Null operator- (const B_Dbl,   const B_Null&);
DLL_DECL B_Null operator- (const B_Null&, const B_Dbl);
DLL_DECL B_Null operator- (const B_Flt,   const B_Null&);
DLL_DECL B_Null operator- (const B_Null&, const B_Flt);
DLL_DECL B_Null operator- (const B_Null&, const B_Lng);
DLL_DECL B_Null operator- (const B_Lng,   const B_Null&);
DLL_DECL B_Null operator- (const B_UInt,  const B_Null&);
DLL_DECL B_Null operator- (const B_Null&, const B_UInt);
DLL_DECL B_Null operator- (const B_Int,   const B_Null&);
DLL_DECL B_Null operator- (const B_Null&, const B_Int);

// Bibliothekstypen
DLL_DECL B_Null operator- (const B_Null&,     const B_Boolean&);
DLL_DECL B_Null operator- (const B_Boolean&,  const B_Null&);
DLL_DECL B_Null operator- (const B_Null&,     const B_Empty&);
DLL_DECL B_Null operator- (const B_Empty&,    const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// fuer alle gleich, s.h. opertor+()
DLL_DECL B_Null operator* (const B_Null&, const B_Null&);

// C-Standardtypen
DLL_DECL B_Null operator* (const B_Dbl,   const B_Null&);
DLL_DECL B_Null operator* (const B_Null&, const B_Dbl);
DLL_DECL B_Null operator* (const B_Flt,   const B_Null&);
DLL_DECL B_Null operator* (const B_Null&, const B_Flt);
DLL_DECL B_Null operator* (const B_Null&, const B_Lng);
DLL_DECL B_Null operator* (const B_Lng,   const B_Null&);
DLL_DECL B_Null operator* (const B_UInt,  const B_Null&);
DLL_DECL B_Null operator* (const B_Null&, const B_UInt);
DLL_DECL B_Null operator* (const B_Int,   const B_Null&);
DLL_DECL B_Null operator* (const B_Null&, const B_Int);

// Bibliothekstypen
DLL_DECL B_Null operator* (const B_Null&,    const B_Boolean&);
DLL_DECL B_Null operator* (const B_Boolean&, const B_Null&);
DLL_DECL B_Null operator* (const B_Null&,    const B_Empty&);
DLL_DECL B_Null operator* (const B_Empty&,   const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// fuer alle gleich, s.h. opertor+()
DLL_DECL B_Null operator/ (const B_Null&, const B_Null&);

// C-Standardtypen
DLL_DECL B_Null operator/ (const B_Dbl,   const B_Null&);
DLL_DECL B_Null operator/ (const B_Null&, const B_Dbl);
DLL_DECL B_Null operator/ (const B_Flt,   const B_Null&);
DLL_DECL B_Null operator/ (const B_Null&, const B_Flt);
DLL_DECL B_Null operator/ (const B_Null&, const B_Lng);
DLL_DECL B_Null operator/ (const B_Lng,   const B_Null&);
DLL_DECL B_Null operator/ (const B_UInt,  const B_Null&);
DLL_DECL B_Null operator/ (const B_Null&, const B_UInt);
DLL_DECL B_Null operator/ (const B_Int,   const B_Null&);
DLL_DECL B_Null operator/ (const B_Null&, const B_Int);

// Bibliothekstypen
DLL_DECL B_Null operator/ (const B_Null&,    const B_Boolean&);
DLL_DECL B_Null operator/ (const B_Boolean&, const B_Null&);
DLL_DECL B_Null operator/ (const B_Null&,    const B_Empty&);
DLL_DECL B_Null operator/ (const B_Empty&,   const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// fuer alle gleich, s.h. opertor+()
DLL_DECL B_Null operator% (const B_Null&, const B_Null&);

// C-Standardtypen
DLL_DECL B_Null operator% (const B_Dbl,   const B_Null&);
DLL_DECL B_Null operator% (const B_Null&, const B_Dbl);
DLL_DECL B_Null operator% (const B_Flt,   const B_Null&);
DLL_DECL B_Null operator% (const B_Null&, const B_Flt);
DLL_DECL B_Null operator% (const B_Null&, const B_Lng);
DLL_DECL B_Null operator% (const B_Lng,   const B_Null&);
DLL_DECL B_Null operator% (const B_UInt,  const B_Null&);
DLL_DECL B_Null operator% (const B_Null&, const B_UInt);
DLL_DECL B_Null operator% (const B_Int,   const B_Null&);
DLL_DECL B_Null operator% (const B_Null&, const B_Int);

// Bibliothekstypen
DLL_DECL B_Null operator% (const B_Null&,    const B_Boolean&);
DLL_DECL B_Null operator% (const B_Boolean&, const B_Null&);
DLL_DECL B_Null operator% (const B_Null&,    const B_Empty&);
DLL_DECL B_Null operator% (const B_Empty&,   const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// fuer alle gleich, s.h. opertor+()
DLL_DECL B_Null B_Div(const B_Null&, const B_Null&);

// C-Standardtypen
DLL_DECL B_Null B_Div(const B_Dbl,   const B_Null&);
DLL_DECL B_Null B_Div(const B_Null&, const B_Dbl);
DLL_DECL B_Null B_Div(const B_Flt,   const B_Null&);
DLL_DECL B_Null B_Div(const B_Null&, const B_Flt);
DLL_DECL B_Null B_Div(const B_Null&, const B_Lng);
DLL_DECL B_Null B_Div(const B_Lng,   const B_Null&);
DLL_DECL B_Null B_Div(const B_UInt,  const B_Null&);
DLL_DECL B_Null B_Div(const B_Null&, const B_UInt);
DLL_DECL B_Null B_Div(const B_Int,   const B_Null&);
DLL_DECL B_Null B_Div(const B_Null&, const B_Int);

// Bibliothekstypen
DLL_DECL B_Null B_Div(const B_Null&,    const B_Boolean&);
DLL_DECL B_Null B_Div(const B_Boolean&, const B_Null&);
DLL_DECL B_Null B_Div(const B_Null&,    const B_Empty&);
DLL_DECL B_Null B_Div(const B_Empty&,   const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// fuer alle gleich, s.h. opertor+()
DLL_DECL B_Null B_Pow(const B_Null&, const B_Null&);

// C-Standardtypen
DLL_DECL B_Null B_Pow(const B_Dbl,   const B_Null&);
DLL_DECL B_Null B_Pow(const B_Null&, const B_Dbl);
DLL_DECL B_Null B_Pow(const B_Flt,   const B_Null&);
DLL_DECL B_Null B_Pow(const B_Null&, const B_Flt);
DLL_DECL B_Null B_Pow(const B_Null&, const B_Lng);
DLL_DECL B_Null B_Pow(const B_Lng,   const B_Null&);
DLL_DECL B_Null B_Pow(const B_UInt,  const B_Null&);
DLL_DECL B_Null B_Pow(const B_Null&, const B_UInt);
DLL_DECL B_Null B_Pow(const B_Int,   const B_Null&);
DLL_DECL B_Null B_Pow(const B_Null&, const B_Int);

// Bibliothekstypen
DLL_DECL B_Null B_Pow(const B_Null&,    const B_Boolean&);
DLL_DECL B_Null B_Pow(const B_Boolean&, const B_Null&);
DLL_DECL B_Null B_Pow(const B_Null&,    const B_Empty&);
DLL_DECL B_Null B_Pow(const B_Empty&,   const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Verkettungsoperator.
// Bei Null & X wird Null als Null-String gewertet ist also bei Bibliotheksdatentypen impl.
DLL_DECL B_Null operator&(const B_Null&, const B_Null&);


//===================== L O G I K   O P E R A T O R E N  =================================================

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// logisch UND
DLL_DECL B_Null operator&&(const B_Null&,  const B_Null&);
//-- muss fuer alle Standarddatentypen ueberladen werden da diese sonst von B_Boolean ueber Konstruktoren er-
//   fasst wuerden
DLL_DECL B_Null operator&&(const B_Int,    const B_Null&);
DLL_DECL B_Null operator&&(const B_Null&,  const B_Int);
DLL_DECL B_Null operator&&(const B_UInt,   const B_Null&);
DLL_DECL B_Null operator&&(const B_Null&,  const B_UInt);
DLL_DECL B_Null operator&&(const B_Lng,    const B_Null&);
DLL_DECL B_Null operator&&(const B_Null&,  const B_Lng);
DLL_DECL B_Null operator&&(const B_Flt,    const B_Null&);
DLL_DECL B_Null operator&&(const B_Null&,  const B_Flt);
DLL_DECL B_Null operator&&(const B_Dbl,    const B_Null&);
DLL_DECL B_Null operator&&(const B_Null&,  const B_Dbl);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// logisch ODER
DLL_DECL B_Null operator||(const B_Null&, const B_Null&);

DLL_DECL B_Null operator|| (const B_Null&,       const B_Empty&);
DLL_DECL B_Null operator|| (const B_Empty&,      const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// XOR
DLL_DECL B_Null operator^ (const B_Null&, const B_Null&);

// C-Standardtypen
DLL_DECL B_Null operator^ (const B_Dbl,   const B_Null&);
DLL_DECL B_Null operator^ (const B_Null&, const B_Dbl);
DLL_DECL B_Null operator^ (const B_Flt,   const B_Null&);
DLL_DECL B_Null operator^ (const B_Null&, const B_Flt);
DLL_DECL B_Null operator^ (const B_Null&, const B_Lng);
DLL_DECL B_Null operator^ (const B_Lng,   const B_Null&);
DLL_DECL B_Null operator^ (const B_UInt,  const B_Null&);
DLL_DECL B_Null operator^ (const B_Null&, const B_UInt);
DLL_DECL B_Null operator^ (const B_Int,   const B_Null&);
DLL_DECL B_Null operator^ (const B_Null&, const B_Int);

// Bibliothekstypen
DLL_DECL B_Null operator^ (const B_Null&,    const B_Boolean&);
DLL_DECL B_Null operator^ (const B_Boolean&, const B_Null&);
DLL_DECL B_Null operator^ (const B_Null&,    const B_Empty&);
DLL_DECL B_Null operator^ (const B_Empty&,   const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// NOT-Operator
DLL_DECL B_Null operator! (const B_Null&); // fuer alle Klassen

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// fuer alle gleich
DLL_DECL B_Null operator== (const B_Null&, const B_Null&);

// C-Standardtypen
DLL_DECL B_Null operator== (const B_Dbl,   const B_Null&);
DLL_DECL B_Null operator== (const B_Null&, const B_Dbl);
DLL_DECL B_Null operator== (const B_Flt,   const B_Null&);
DLL_DECL B_Null operator== (const B_Null&, const B_Flt);
DLL_DECL B_Null operator== (const B_Null&, const B_Lng);
DLL_DECL B_Null operator== (const B_Lng,   const B_Null&);
DLL_DECL B_Null operator== (const B_UInt,  const B_Null&);
DLL_DECL B_Null operator== (const B_Null&, const B_UInt);
DLL_DECL B_Null operator== (const B_Int,   const B_Null&);
DLL_DECL B_Null operator== (const B_Null&, const B_Int);

// Bibliothekstypen
DLL_DECL B_Null operator== (const B_Null&,    const B_Boolean&);
DLL_DECL B_Null operator== (const B_Boolean&, const B_Null&);
DLL_DECL B_Null operator== (const B_Null&,    const B_Empty&);
DLL_DECL B_Null operator== (const B_Empty&,   const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// fuer alle gleich
DLL_DECL B_Null operator!= (const B_Null&, const B_Null&);

// C-Standardtypen
DLL_DECL B_Null operator!= (const B_Dbl,   const B_Null&);
DLL_DECL B_Null operator!= (const B_Null&, const B_Dbl);
DLL_DECL B_Null operator!= (const B_Flt,   const B_Null&);
DLL_DECL B_Null operator!= (const B_Null&, const B_Flt);
DLL_DECL B_Null operator!= (const B_Null&, const B_Lng);
DLL_DECL B_Null operator!= (const B_Lng,   const B_Null&);
DLL_DECL B_Null operator!= (const B_UInt,  const B_Null&);
DLL_DECL B_Null operator!= (const B_Null&, const B_UInt);
DLL_DECL B_Null operator!= (const B_Int,   const B_Null&);
DLL_DECL B_Null operator!= (const B_Null&, const B_Int);

// Bibliothekstypen
DLL_DECL B_Null operator!= (const B_Null&,    const B_Boolean&);
DLL_DECL B_Null operator!= (const B_Boolean&, const B_Null&);
DLL_DECL B_Null operator!= (const B_Null&,    const B_Empty&);
DLL_DECL B_Null operator!= (const B_Empty&,   const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// fuer alle gleich
DLL_DECL B_Null operator< (const B_Null&, const B_Null&);

// C-Standardtypen
DLL_DECL B_Null operator< (const B_Dbl,   const B_Null&);
DLL_DECL B_Null operator< (const B_Null&, const B_Dbl);
DLL_DECL B_Null operator< (const B_Flt,   const B_Null&);
DLL_DECL B_Null operator< (const B_Null&, const B_Flt);
DLL_DECL B_Null operator< (const B_Null&, const B_Lng);
DLL_DECL B_Null operator< (const B_Lng,   const B_Null&);
DLL_DECL B_Null operator< (const B_UInt,  const B_Null&);
DLL_DECL B_Null operator< (const B_Null&, const B_UInt);
DLL_DECL B_Null operator< (const B_Int,   const B_Null&);
DLL_DECL B_Null operator< (const B_Null&, const B_Int);

// Bibliothekstypen
DLL_DECL B_Null operator< (const B_Null&,     const B_Boolean&);
DLL_DECL B_Null operator< (const B_Boolean&,  const B_Null&);
DLL_DECL B_Null operator< (const B_Null&,     const B_Empty&);
DLL_DECL B_Null operator< (const B_Empty&,    const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// fuer alle gleich
DLL_DECL B_Null operator<= (const B_Null&, const B_Null&);

// C-Standardtypen
DLL_DECL B_Null operator<= (const B_Dbl,   const B_Null&);
DLL_DECL B_Null operator<= (const B_Null&, const B_Dbl);
DLL_DECL B_Null operator<= (const B_Flt,   const B_Null&);
DLL_DECL B_Null operator<= (const B_Null&, const B_Flt);
DLL_DECL B_Null operator<= (const B_Null&, const B_Lng);
DLL_DECL B_Null operator<= (const B_Lng,   const B_Null&);
DLL_DECL B_Null operator<= (const B_UInt,  const B_Null&);
DLL_DECL B_Null operator<= (const B_Null&, const B_UInt);
DLL_DECL B_Null operator<= (const B_Int,   const B_Null&);
DLL_DECL B_Null operator<= (const B_Null&, const B_Int);

// Bibliothekstypen
DLL_DECL B_Null operator<= (const B_Null&,     const B_Boolean&);
DLL_DECL B_Null operator<= (const B_Boolean&,  const B_Null&);
DLL_DECL B_Null operator<= (const B_Null&,     const B_Empty&);
DLL_DECL B_Null operator<= (const B_Empty&,    const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// fuer alle gleich
DLL_DECL B_Null operator>(const B_Null&, const B_Null&);

// C-Standardtypen
DLL_DECL B_Null operator> (const B_Dbl,   const B_Null&);
DLL_DECL B_Null operator> (const B_Null&, const B_Dbl);
DLL_DECL B_Null operator> (const B_Flt,   const B_Null&);
DLL_DECL B_Null operator> (const B_Null&, const B_Flt);
DLL_DECL B_Null operator> (const B_Null&, const B_Lng);
DLL_DECL B_Null operator> (const B_Lng,   const B_Null&);
DLL_DECL B_Null operator> (const B_UInt,  const B_Null&);
DLL_DECL B_Null operator> (const B_Null&, const B_UInt);
DLL_DECL B_Null operator> (const B_Int,   const B_Null&);
DLL_DECL B_Null operator> (const B_Null&, const B_Int);

// Bibliothekstypen
DLL_DECL B_Null operator> (const B_Null&,    const B_Boolean&);
DLL_DECL B_Null operator> (const B_Boolean&, const B_Null&);
DLL_DECL B_Null operator> (const B_Null&,    const B_Empty&);
DLL_DECL B_Null operator> (const B_Empty&,   const B_Null&);

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// fuer alle gleich
DLL_DECL B_Null operator>= (const B_Null&, const B_Null&);

// C-Standardtypen
DLL_DECL B_Null operator>= (const B_Dbl,   const B_Null&);
DLL_DECL B_Null operator>= (const B_Null&, const B_Dbl);
DLL_DECL B_Null operator>= (const B_Flt,   const B_Null&);
DLL_DECL B_Null operator>= (const B_Null&, const B_Flt);
DLL_DECL B_Null operator>= (const B_Null&, const B_Lng);
DLL_DECL B_Null operator>= (const B_Lng,   const B_Null&);
DLL_DECL B_Null operator>= (const B_UInt,  const B_Null&);
DLL_DECL B_Null operator>= (const B_Null&, const B_UInt);
DLL_DECL B_Null operator>= (const B_Int,   const B_Null&);
DLL_DECL B_Null operator>= (const B_Null&, const B_Int);

// Bibliothekstypen
DLL_DECL B_Null operator>= (const B_Null&,    const B_Boolean&);
DLL_DECL B_Null operator>= (const B_Boolean&, const B_Null&);
DLL_DECL B_Null operator>= (const B_Null&,    const B_Empty&);
DLL_DECL B_Null operator>= (const B_Empty&,   const B_Null&);

//================================ Inlinefunktionen =========================================================
//
//#if defined _DEBUG || defined B_VB_CONTRACT
  #define B_NOINLINE
//#else
//  #undef B_NOINLINE
//#endif

#ifndef B_NOINLINE                // festlegen wenn Debuger keine Inlines debuggen kann
  #define B_INLINE inline
  #ifndef BI_NULL_HPP
    #include "bi_null.hpp"
  #endif
#endif

//===========================================================================================================
//================================ End Of Header ============================================================
#endif // BC_NULL_HPP

