#ifndef BD_EXCPT_HPP
#define BD_EXCPT_HPP
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Project:                 VB nach C/C++ Konverter Klassenbibliothek
// Modul:                   Makros und Konstanten fuer die Emulation des VB Fehlerhandlings
// Filename:                bd_excpt.hpp
// Version:                 0.0
// Status:                  alpha
//
// Copyright:               (C) 1995
//                          Cicerosoft GmbH
//                          Ballweilerstr. 7
//                          66399 Erfweiler-Ehlingen
//
// Operation System(s):     Win95
// Compiler:                VC++ 4.1 
//
// Author:                  Jochen Diener JD
// Creation Date::          JD 15.02.96
// Last Modification:       JD 18.06.96
//
// Description:             Deklarationen von Variablen, Konstanten usw. die die Klassen der 
//                          Exceptionhierachie oder verwandte Klassen/Modulen benoetigen die mit EH
//                          arbeiten
//
// Dependencies:            bc_excpt.cpp   -> Klassenimpl.
//                          bc_excpt.hpp   -> Klassendekl.
//                          bi_excpt.hpp   -> Inline (Member)Funktionen der Klasse
//
// Remarks:
//
//
// Revision(s):
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

//================================= DefintionsDependencies ===============================================
//
#ifndef BD_INTYP_HPP
  #include "bd_intyp.hpp"       // alias der Standardatentypen
#endif


//================================= Export =================================================================
//
// Die Emulation der On Error Goto Konstruktes aus VB basiert darauf dass nur eine Exception fuer alle Faelle ausgeworfen
// wird. Diese Exception ist entweder eine richtige C++ Exception oder bei Systemen auf denen keine Compiler verfuegbar
// sind die dies unterstuetzen eine einfache Integerexception die mittels der Emulation der Telia Research-Modells 
// implementiert wird. 
// Bei Auswurf einer Exception wird das B_Err Objekt auf den gleichen Wert gesetzt wie auch VB dies mit der Err-Variablen
// machen wuerde. Somit kann der Quelltext im Catch-Block (= in VB hinter der Springmarke) unveraendert uebernommen werden.
// Einzig der Block zwischen der ON Errror.. Anweisung und der Sprungmarke muss vom Konverter gesondert behandelt werden.
//
// Der Einsatz eines, im Sinne von C++, richtigen EH (keine verkrueppelte realisierung wie in VB) erfordert den manuellen
// Eingriff eines C++ Programmieres in das Konvertat, der weis was er tut.

//==== Konvertermakros

// On Error Goto 0 -> muss konverter behandlen da folgende Zeilen nicht mehr in Try-Block aufgenommern
//   werden duerfen und catch-Block einzufuehren ist
//-----------------
// On Error Resume Next
// Der MArkoparamter ist notig da in einer Funktione auch mehrmals On Error Resume Next abwechselnd mit 
// On Error Goto x stehen kann.
#define INIT_VB_ERRORHANDLING B_Shrt b_ResumeLine
#define ON_ERROR_RESUME_NEXT B_err.clear(); b_ResumeLine = __LINE__; b_ResumeLabel: try { switch ( b_ResumeLine ) { case __LINE__:
// am Ende der Funktion dann.
// catch(...) faengt alle auf catch (B_Exception&) nur unsere und den Rest als unexpected da diese von anderen
// Bibliotheken kommmen koennten
#define ON_RESUME_NEXT } } catch (B_Exception&) { b_ResumeLine++; goto b_ResumeLabel; }
// geht da wenn neue Prozedur aufgerufen wird in VB die On Error Resume Next Anweisung inaktiv wird

//-----------------
// On Error Goto anyMarke
#define ON_ERROR_GOTO(LabelX) b_ResumeLine = __LINE__; LabelX: try { switch(b_ResumeLine) {case __LINE__:
// wo normal anyMarke stuende
#define GOTO_MARKE } } catch(B_Exception&) {
// #define GOTOMARKE } catch (...) { mit oder ohne Ellipse
// in Catch-Block den Code schreiben der hinter der VB-Marke steht bis zu End Sub, End Function,
// Exit Sub, Exit Function. Somit koennen auch mehrere Handler in einer Prozedur/funktion stehen
//
// Auch der hirnrissige Fall das der Errorhandler immer am Funktionsende ausgefuehrt ist waere denkbar wenn
// keine Marke mit vorangender Exit Sub, Exit Function Methode bestehen wuerde
#define END_GOTO_MARKE } // ende catch-Block; keine entsprechung in VB
// Resume
#define RESUME(LabelX) goto LabelX;
// Resume Next
#define RESUME_NEXT(LabelX) b_ResumeLine++; goto LabelX
// Resume 'Line'
#define RESUME_LINE(LineNbr, LabelX) b_ResumeLine = LineNbr; goto LabelX;
// der Block hinter 'On Error Goto anyMarke' ist jede Statementzeile wie folgt zu praeparieren:
// An den Anfang jeder Statementzeile wird folgendes Praeprozessormakro eingefuegt, vom Konverter.
#define B_EH case __LINE__ : b_ResumeLine = __LINE__;


//-- Bei "Exit Sub", "Exit Function" , "Exit Property" muss der Konverter die Funktion korrekt beenden und die
//   Err.Clear() MEhtode aufrufen. Der Aufruf von Err.Clear() ist nur erforderlich wenn EH eingesetzt wird.
//   Die Reuckgabewerte von Funktionen werden bei Exit Function auf die defaultwerte zureuckgesetzt. D.h. alle
//   Zahlwerte auf 0 gesetzt alle Strings auf einen Leerstring.
//   D.h. der Konvverter muesste folgenden Code generieren:
//     Exit Sub    ->    B_err.Clear(); return;
//     Exit Function ->  B_err.Clear(); return 0;  // bei Zahlrueckgaben
//                       B_err.Clear(); return ""; // bei Stringwerten
//                       B_err.Clear(); return 0;  // bei Datumswerten
//                       B_err.Clear(); return B_False;  // bei Wahrheitswerten

                                  
//=========== Bibliotheks EH (enhancedLib)
//-- Liegt ein Fehler < 65535 vor kann der Anwender davon ausgehen, dass es keine VB Systemfehler ist
//   und dass er alle anderen Werte setzten muss (description usw.) ansonsten uebernimmt dies VB
//   Da derzeit nur ca. 1000 Fehler im Bereich von 0 - 3300 definiert. Da das Err-Objekt auch bei
//   unseren Bibliotheksfehlern korrekt gesetzt wird verwenden wir Fehlernummern im negativen Bereich 
//   damit keine Konflikte mit VB-Programmen auftreten
//   Diese werden zur bibliotheksinternen Implementierung der Exceptionhierachie und der Err-Klassen benoetigt
//   und koennen ferner auch vom Kunden in seinen Programmen eingesetzt werden. Dies hat den Vorteil dass wenn
//   die Nummern von uns angepasst werden muessen weil eine zukunftige VB-Version diese belegt der Kunde nur
//   seine Programme neu uebersetzen muss

#define EXCEPTION_LZF  0  // Lazfzeitfehler die nur in LIB-EH-Hierachie vorhanden sind und keine direkte VB Entsprechung haben
#define LOGIC_LZF     -1
#define LENGTH_LZF    -2
#define OUTOFRANGE_LZF -3
#define DOMAIN_LZF     -4
#define RUNTIME_LZF -6
#define RANGE_LZF -7
#ifdef B_ENHANCED_LIB
  #define UNDERFLOW_LZF -9
#else
  #define UNDERFLOW_LZF 6 // in VB keine Unterscheidung beides Overflow
#endif
#define RESOURCE_LZF -10
#define FATAL_LZF -11
#define CONTRACT_LZF -12
#define CRITICAL_LZF -13
#define COLLECTION_LZF -14
#define DIR_FILE_LZF -15
#define DLL_EXCEPT_LZF -16
#define APPLICATION_LZF 0 // noch nummer ausdenken -> "Bentuzerdefinierter oder Anwendungsfehler"
#define INVALIDARGUMENT_LZF 5
#define OVERFLOW_LZF 6
#define OUT_OF_MEM_LZF 7
#define SUBSCRIPT_LZF  9
#define ARRAY_LZF 10
#define ZERO_DIVIDE_LZF 11
#define TYPE_MISMATCH_LZF 13
#define NOT_SUPPORTED_LZF 35
#define BAD_DLL_CALL_LZF 49
#define TOO_MANY_CLIENTS_LZF 47
#define ERROR_LOADING_DLL_LZF 48
#define INTERNAL_LZF 51
#define BAD_FILE_NBR_LZF 52
#define FILE_NOT_FOUND_LZF 53
#define BAD_FILE_MODE_LZF 54
#define FILE_OPEN_LZF 55
#define DEVICE_IO_LZF 57
#define FILE_EXISTS_LZF 58
#define BAD_REC_LENGTH_LZF 59
#define DISK_FULL_LZF 61
#define INPUT_PAST_EOF_LZF 62
#define BAD_REC_NBR_LZF 63
#define BAD_FILE_NAME_LZF 64
#define TOO_MANY_FILES_LZF 67
#define DEVICE_UNAVAILABLE_LZF 68
#define PERMISSION_DENIED_LZF 70
#define DISK_NOT_READY_LZF 71
#define DIFFERENT_DRIVE_LZF 74
#define BAD_PATH_FILE_LZF 75
#define PATH_NOT_FOUND_LZF 76
#define INVALID_PATTERN_LZF 93
#define INVALID_NULL_LZF 94
#define INVALID_ORDINAL_LZF 452
#define NO_DLL_FUNC_LZF 453
#define GETPUTARR_LZF 456
#define KEY_USE_LZF 457

                                 
//=========== Ende EH-Makros

//===========================================================================================================
//================================ End Of Header ============================================================
#endif // BD_EXCPT_HPP
