#ifndef BD_INTYP_HPP
#define BD_INTYP_HPP
///////////////////////////////////////////////////////////////////////////////////////////////////
// Project:                 VB nach C/C++ Konverter Klassenbibliothek
// Modul:                   VB Klasse fuer Datum und Zeit
// Filename:                bd_intyp.hpp
// Version:                 0.0
// Status:                  alpha
//
// Copyright:               (C) Cicerosoft GmbH,
//                          Ballweilerstr. 7,
//                          66399 Erfweiler Ehlingen
//
// Operating System:        Win95
// Compiler:                VC++ 4.1
//
// Author:                  Jochen Diener JD
// Creation Date:           14.11.95
// Last Modification:       JD 27.11.95
//
// Description:             Beinhaltet alle Klassen / Hilfskt. die im Zusammenhang mit der VB
//                          Konvertierung, fuer die Bereitstellung der mathematischen Datentypen benoetigt
//                          werden.
//
//                          Steht in direkter Abhaengigkeit zu 'bf_math.h' das die mathematischen
//                          Berechnungsfkt. enthaelt.
//
// Remarks:                 Zuerst nur Currency spaeter noch bel. genaue Integer/Floats -> wenn LEDA zum
//                          Einsatz kommt
//
// Revisions:
//
///////////////////////////////////////////////////////////////////////////////////////////////////

//================================= Defintionsabhaengigkeiten ===============================================
//
#include <limits.h>  // fuer eigene typedef-Limits benoetigt
#include <float.h>   // fuer eigene typedef-Limits benoetigt
// #include <windows.h>

// #include "bflt.hpp"
// #include <gstring.h>

#include "bd_lib.hpp"

//================================= Export =================================================================
//


// Standarddatentypen von C/C++ zur leichteren Portierung alias eingefuehrt
// Die Datentypen muessen den im Kommentar angegebenen Wertebereich darstellen, da diese Wertebereiche 
// in der Implementierung vorausgesetzt werden und fuer ein fehlerfreies arbeiten der Klassen noetig sind
typedef char           B_Char;    //            -128 bis 127              16bit und 32bit (8 Bits)
typedef unsigned char  B_UChar;   //               0 bis 255              16bit und 32bit (8 Bits)
typedef short          B_Shrt;    //         -32.768 bis 32.767
typedef unsigned short B_UShrt;   //               0 bis 65.535
typedef int            B_Int;     //         -32.768 bis 32.767           Die Verwendung von int sollte vermieden werden 
                                  //                                      wegen den unterschiedlichen groessen je nachdem
                                  //                                      ob 16/32bit. 16bit short int; 32bit long int
typedef unsigned int   B_UInt;    //               0 bis 65.535           (32bit 0- 4,294,967,295
typedef long           B_Lng;     //  -2.147.483.648 bis 2.147.483.647    16bit und 32bit (32 Bits)
typedef unsigned long  B_ULng;    //               0 bis 4.294.967.295    16bit und 32bit (32 Bits)
typedef float          B_Flt;     //        3.4E+/-38  7Digits            (VC++22, BC++45) bei Compilern gibt es unterschiede!
                                  //                                      Sicher sind nur die Werte 1.175494E-38 bis
                                  //                                      3.402823E+38 mit 6Digits nach IEEE (WC++105,SC++72, VC++40)    
                                                                    
typedef double         B_Dbl;     //  2.2250738585072014E-308	bis 1.7976931348623158E+308   16bit und 32bit 15Digits (VC++40, WC++105, SC++72)
                                  //   	    1.7E-308 bis 1.7E+308         16bit und 32bit (64 bits)  15Digits (VC++22, BC++45)
typedef long double    B_LDbl;    //  	   3.4E-4932 bis 1.1E+4932        16bit und 32bit (80 bits) 15Digits (BC++45, )
                                  //  	   1.2E-4932 bis 1.2E+4932        16bit und 32bit (80 bits) 19Digits (VC++22/40)
                                  //  bei WC++ und SC++ sind long double und double identisch

typedef char           B_Flag;     //  Flag zum merken vershiedener Zustaende auch fuer Boolean geignet wenn es auf Speicherplatz
                                  //  ankommt und bei log. Ausdruecken keine Probleme mit der integralen Promotion gibt

#ifdef B_WIN31
  typedef char HUGE*  B_CStr;
  typedef const char HUGE* B_cCStr;
#elif defined B_WIN95
  typedef char* B_CStr;
  typedef const char* B_cCStr;
#endif


// Als typedefs damit spaeter auch als Klasse implementierbar falls LEDA einsatz
//-- VB-Typen siehe in bd_vbtyp.hpp, hier C-Version
typedef int  B_BOOLEAN;
const int B_TRUE = 1;
const int B_FALSE = 0;
 //#ifdef B_BDLL
// #define B_String gStr      //  Bis Stringklasse zur Verfueung steht gStr
 //#else
 // Dummy fuer gStr dait anwender nicht die Generic-Header braucht
 //  struct gStrDummy {char b1; char b2; char b3; char b4; char b5; char b6; char b7; char b8;};
 //  #define B_String gStrDummy
 //#endif
// #define B_Array gDyA     //  G_DynArray von Generic gDyA
#ifdef _DEBUG
  #define B_ARRAY_CONTRACT B_ArrayChecked
  #define B_ARRAY(T) B_Array<T, B_ArrayChecked>
#else
  #define B_ARRAY_CONTRACT B_ArrayUnchecked
  #define B_ARRAY(T) B_Array<T, B_ArrayUnchecked>
#endif

// typedef unsigned char  B_Byte;
// typedef short          B_Integer;
// typedef signed long    B_Long;
// typedef float          B_Single;
// typedef double         B_Double; // ACHTUNG: vb akzeptiert 4.94065645841247E-324 jedoch bei C ist
                                    // Untergrenze 1.7E +/- 308 (15 digits). Jedoch nicht jeder Compiler macht long double
                                    // groesser wie double -> vgl. Watcom -> Deshalb ist Wertebereich eingeschraenkt.

// Grenzwerte der Implemetnierten Datentypen
const B_Char       B_CharMin        =  SCHAR_MIN;
const B_Char       B_CharMax        =  SCHAR_MAX;
const B_UChar      B_UCharMin       =  0;
const B_UChar      B_UCharMax       =  UCHAR_MAX;
const B_Shrt       B_ShrtMin        =  SHRT_MIN;
const B_Shrt       B_ShrtMax        =  SHRT_MAX;
const B_UShrt      B_UShrtMin       =  0;
const B_UShrt      B_UShrtMax       =  USHRT_MAX;
const B_Int        B_IntMin         =  INT_MIN;
const B_Int        B_IntMax         =  INT_MAX;
const B_UInt       B_UIntMin        =  0;
const B_UInt       B_UIntMax        =  UINT_MAX;
const B_Lng        B_LngMin         =  LONG_MIN;
const B_Lng        B_LngMax         =  LONG_MAX;
const B_ULng       B_ULngMin        =  0;
const B_ULng       B_ULngMax        =  ULONG_MAX;
// nicht alle FP werte aufgenommen bei bedarf ergaenzen
const B_Flt        B_FltMin         =  FLT_MIN;
const B_Flt        B_FltMax         =  FLT_MAX;
const B_UChar      B_FltDigits      =  FLT_DIG;      // genauigkeit
const B_Flt        B_FltEpsilon     =  FLT_EPSILON;  // kleinster Wert x fuer den 1.0 + x != 1.0 gilt
const B_LDbl       B_LDblMin        =  LDBL_MIN;
const B_LDbl       B_LDblMax        =  LDBL_MAX;
const B_UChar      B_LDblDigits     =  LDBL_DIG;
const B_LDbl       B_LDblEpsilon    =  LDBL_EPSILON;
// unsinn da nur fuer Nachkommastellen ineressant  B_CurrMin        =  -922337203685477.5808;  // die richtigen Currency Werte sind bei der Klasse dekl.
const B_Dbl        B_CurrMax        =  922337203685477.5807;
const B_UChar      B_CurrDigits     =  DBL_DIG;
const B_Dbl        B_CurrEpsilon    =  DBL_EPSILON;
const B_Dbl        B_DblMin         =  DBL_MIN; 
const B_Dbl        B_DblMax         =  DBL_MAX;
const B_UChar      B_DblDigits      =  DBL_DIG;
const B_Dbl        B_DblEpsilon     =  DBL_EPSILON;

struct B_Explicit
  {
  };

//-- Option exlicit emulation bis ale Compiler es koennen
DLL_DECL extern const B_Explicit B_explicit;

//-- Fuer bc_local.hpp  B_Locale::B_Messages
//    typedef int   catalog; fuer Draft
#define Catalog HINSTANCE

// #define LCID unsigned long

//-- fuer Statische Stringspeicher s. Bc_dbg*
#define MAX_FILE_NAME 256

//===========================================================================================================
//================================ End Of Header ============================================================
#endif // BD_INTYP_HPP

//================================== Infos wie machen es andere Klassenbibliotheken =========================
//

