#ifndef BF_CONVT_HPP
#define BF_CONVT_HPP
//////////////////////////////////////////////////////////////////////////////////////////////////////////
// Project:                 VB nach C/C++ Konverter Klassenbibliothek
// Modul:                   VB Funktionen zur Variantbearbeitung
// Filename:                bf_vanrt.h
// Version:                 0.0
// Status:                  alpha
//
// Copyright:               (c)
//                          Cicerosoft GmbH, 
//                          Ballweilerstr. 7,
//                          66399 Erfweiler-Ehlingen
//
// Operating Systems:       Win95 
// Compiler:                VC++ 4.1
//
// Author:                  Jochen Diener JD
// Creation Date:           10.01.96
// last Modification:       JD 12.01.96
//
// Description:             Beinhaltet alle Hilfskt. die im Zusammenhang mit der VB Konvertierung,
//                          fuer die Konvertierungsfkt. von die VB anbietet im Datentypen zu formatieren und
//                          zu konvertieren
//                          
// Dependencies:            bf_convt.hpp    ->  Deklaration der Funktionen
//                          bf_convt.cpp    ->  Implementierung der Funktionen
//
//
// Remarks:            
//
// Revision(s):   
//                          
///////////////////////////////////////////////////////////////////////////////////////////////////////////*/

//==========================================================================================================
//================================= Defintionsabhaengigkeiten ===============================================
//
#ifndef BD_INTYP_HPP       
  #include "bd_intyp.hpp"       // alias der Standardatentypen
#endif

#ifndef BC_NULL_HPP
  #include "bc_null.hpp"
#endif

#ifndef BC_EMPTY_HPP
  #include "bc_empty.hpp"
#endif

#ifndef BC_BOOL_HPP
  #include "bc_bool.hpp"
#endif

#ifndef BC_BYTE_HPP
  #include "bc_byte.hpp"
#endif

#ifndef BC_INT_HPP
  #include "bc_int.hpp"
#endif

#ifndef BC_LONG_HPP
  #include "bc_long.hpp"
#endif

#ifndef BC_SNGLE_HPP
  #include "bc_sngle.hpp"
#endif

#ifndef BC_DBL_HPP
  #include "bc_dbl.hpp"
#endif

#ifndef BC_CURR_HPP
  #include "bc_curr.hpp"
#endif

#ifndef BC_STR_HPP
  #include "bc_str.hpp"
#endif


//==========================================================================================================
//================================= Export =================================================================
//

//================================= Hilfsfunktionen fuer den Konverter =====================================
//

//============= Typverifizierungs Fkt.======================================================================
//========================================================================
// B_strConv(); 
// B_strS();    
// B_asc();     
// B_strConv(); 
// B_format();  
//========================================================================
// -------------> IsObject noch aufnhemen <-----------------

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function:                                                                             (C) CiceroSoft 1995
//     B_isDate    Gibt einen boolschen Wert zurueck der angibt ob der uebergebene Ausdruck in einen B_Date
//                 Datentyp konvertiert werden kann.
// See also:
//     B_date, B_cDate, B_typeName, B_vartype
// Parameter:
//     expression:
//         ein beliebiger String- oder Datumsausdruck der in ein Datum vom Typ B_Date konvertiert
//         werden kann.
// Return Value:
//     boolscher Wert:
//         True  -> wenn expression ein gueltiges Datum ist oder darin umgewandelt werden kann
//         False -> in allen anderen Faellen
// Errorhandling:
//     keines noetig
//         wird ein falscher Parameter wie z.B. ein Array mit Strings fester Laenge oder mit 
//         Element von usertypen enthaelt liefert die IDE eine Warnungsmeldung.
// Notes:  
//     Unter MS-Win?? liegt der gueltige Wertebereich von einem Datum zwischen dem 1.1.100 bis
//     zum 31.12.9999. Dieser Wert kann unter verschiedenen Betriebsystemen varieren.
//     Wird ein Array uebergeben wird es ohne Fehlermeldung angenommen und False zurueckgegeben
//     (auch wenn die Elemente des Arrays vom Typ date sind
// B_Boolean B_isDate(const B_Variant& expression);

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function::                                                                            (C) CiceroSoft 1995
//     B_isEmpty  Gibt einen boolschen Wert zurueck der angibt ob eine Variable intialisiert ist.
// See also:
//     B_typeName, B_vartype
// Parameter: 
//     expression:
//        eine beliebige gueltige numerischer oder Stringausdruck. Wird hauptsaechlich dazu ver-
//        wendet um festzustellen ob eine Variable initialisiert wurde. Somit ist expression
//        meist kein Ausdruck sondern nur ein Variablenname.
// Return Value: 
//     boolscher Wert:
//        True  -> wenn eine Variable uninitialisiert ist oder explizit auf Empty gesetzt wurde
//        False -> in allen anderen Faellen, insbesondere wenn mehr als eine Variable angegeben wurde
// Errorhandling: 
//     s. isDate
// Notes:
//     B_isEmpty liefert nur aussagekraeftige Werte fuer variants
//
// B_Boolean B_isEmpty(const B_Variant& expression);

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function:                                                                             (C) CiceroSoft 1995
//     B_isNull  Gibt einen boolschen Wert zurueck der angibt ob ein Ausdruck ungueltige Daten (=Null)
//               enthaelt
// See also:  
//     B_typeName, B_varType
// Parameter:
//     expression:
//         ein beliebiger numerischer oder string Ausdruck
// Return Value: 
//     boolscher Wert:
//         True  -> wenn der Audruck Null ist, also keine gueltigen Daten enthaelt
//         False -> sonst
// Errorhandling: 
//     s. isDate
// Notes:  
//     Wenn expression mehrere Variablen enthaelt und nur eine davon Null wird ist das Ergebnis
//     des Ausdurkcs Null und somit der Reuckgabewert von B_isNull() = True
//     Null gibt an dass die Variable keine gueltigen Werte enthaelt. Null ist somit nicht mit
//     Empty vergleichbar, da Empty eine noch nicht initialisierte Variable anzeigt. Ferner
//     ist Null auch nicht mit einem zero-lenght String (= "") zu verwechselen der manchmal
//     auch als null-String bezeichnet wird.
//     WICHTIG: Die isNull Funktion ist immer dann nazuwenden wenn bei einem Ausdruck oder einer
//              geprueft werden soll ob diese den Null-Wert enthaelt.
//              Hierbei ist zu beachten dass Ausruecke die Null enthalten nie True=Wahr werden
//              koennen. Es ist z.B. nicht moeglich
//              if var = Null
//              abzufragen da dieser Ausdruck nie Wahr werden kann.
//
// B_Boolean B_isNull(const B_Variant& expression);

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function:                                                                             (C) CiceroSoft 1995
//     B_isNumeric   Gibt einen boolschen Wert zurueck der angibt ob ein Ausdruck als Nummmer interpretiert
//                   werden kann
// See also:     
// Parameter: 
//     expression:
//         der Ausdruck kann ein beliebiger gueltiger numerischer oder Zeichenkettenausdruck
//         sein
// Return Value:  
//     boolscher Wert: 
//         True  -> Wenn der ganze Ausdruck als numerischer Wert erkannt/konvertiert wurde
//         False -> sonst
// Errorhandling:
//     s. isDate
// Notes:  
//     liefert False: bei Error, Null, Date, Arrays oder Strings wenn diese keine reinen
//     Zahlstrings sind. Hierbei wird auch Date nicht umgewandelt obwohl Date-Variabeln 
//     nomalerweise in Double Werte konvertiert werden koennen.
//
// B_Boolean B_isNumeric(const B_Variant& expression);
DLL_DECL B_Boolean B_isNumeric(const B_String&);
DLL_DECL B_Boolean B_isNumeric(const B_Boolean&);
DLL_DECL B_Boolean B_isNumeric(const B_Empty&);
DLL_DECL B_Boolean B_isNumeric(const B_Null&);
#ifdef B_VB_COMPATIBLE
DLL_DECL B_Boolean B_isNumeric(const B_Byte&);
DLL_DECL B_Boolean B_isNumeric(const B_Integer&);
DLL_DECL B_Boolean B_isNumeric(const B_Long&);
DLL_DECL B_Boolean B_isNumeric(const B_Single&);
DLL_DECL B_Boolean B_isNumeric(const B_Currency&);
DLL_DECL B_Boolean B_isNumeric(const B_Double&);
#endif

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function:                                                                             (C) CiceroSoft 1995
//     B_val
// See also:     
// Parameter: 
// Return Value:  
// Errorhandling:
// Notes:  
//
#ifdef B_VB_COMPATIBLE
DLL_DECL B_Double B_val(const B_String&);
#else
DLL_DECL B_Dbl B_val(const B_String&);
DLL_DECL B_Int B_valint(const B_String&);
#endif

#ifdef THALESPLUS
DLL_DECL B_Dbl B_val(const B_TStringStruct);
DLL_DECL B_Int B_valint(const B_TStringStruct);
#endif


////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function::  B_isError                                                             (C) CiceroSoft 1995
//                 Gibt einen boolschen Wert zurueck der angibt ob der uebergebene Ausdruck den Wert
//                 Error repraesentiert
//
// See also:     B_varType, B_typeName
//
// Parameter:      expression:
//                    muss ein Variant sein mit dem VarType vbError.
//
// Return Value:  boolscher Wert:
//                    True  -> der Ausdruck bildet einen Error Wert
//                    False -> in allen anderen Faellen
//
// Errorhandling:  s. isDate
//
// Notes:      Error Wert werden erzeugt indem realzahlen in error Werte mittels der B_cvErr Funktion
//                 konvertiert werden. die IsError Funktion wird also dazu verwendet um festzusetellen ob 
//                 ein numerischer Ausdruck einen Fehler darstellt.
//                 B_isError liefert nur fuer Variants von VarTpye vbError sinnvolle Werte.
//
//  B_Boolean B_isError(const B_Variant& expression);

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function::  B_isArray                                                             (C) CiceroSoft 1995
//                 Gibt einen boolschen Wert zurueck der angibt ob die uebergeben Variable ein Array
//                 ist.
//
// See also:     B_varType, B_typeName, B_array, B_varType, B_reDim, B_reDimP, B_lBound, B_uBound,
//                 B_is*-Funtkionen
//
// Parameter:      variable:
//                     eine Variable beliebigen Typs
//
// Return Value:  boolscher Wert:
//                    True  -> wenn die variable ein Array ist
//                    False -> sonst
// 
// Errorhandling:  s. isDate
//                 Da einer Variant Variablen in VB keine Array mit Elementen vom Typ fixed String oder
//                 eigener Typen (= Type .. End Type) sein darf koennen diese Werte auch nicht an die
//                 Funktion uebergeben werden. VB erkennt dies in seiner IDE oder bei der compilierung
//                 dies entspricht dem Verhalten der Bibliothek in C++, da dort der C++ Compiler einen
//                 Error beim Compilieren melden wuerde.
//                 Ansonsten werden alle WErte auch CVErr und Error werden akzeptiert
// Notes:      B_isArray ist besonders nuetzlich wenn getestet werden muss ob eine Variantvariable ein
//                 Array enthaelt.
//                 B_isArray gibt B_true zurueck wenn die Variable anyVar ein Array ist. Andernfalls wird
//                 B_True zurueckgegeben. B_isArray ist insbesondere fuer Variablen vom Typ Variant
//                 interessat die eine Array beinhalten.
//                 Erlaubt uebergabewerte sind: Error-Werte, Null, Empty, boolsche Werte, Zeichenketten
//                 Zahlwerte
//
 //-- Um das unter Errorhandling beschriebene verhalten zu erreichen muesste der Konverter die beiden Array-
 // formen erkennen koennen und dann eine spezielle Arrayklasse aufrufen damit der Compiler eine Typunter-
 // scheidung liefern kann.
 // alle is* Funktionen muessen eigentlich auc fuer die C++ Datentypen gueltig gemacht werden
 // da es bei Standarkonvertierungen ueber die Konstruktoren z.B bei Variants zu mehrdeutigkeiten
 //   kommen koennte. Eigentlich waeren auch templates sinnvoll die dann einfach immer False zuruckliefert
 //   da alle Werte fuer die die Funktion True liefert schon abgefangen werden.
 // Eventuell noch eine Ueberladung fuer den Array-Typ selbst weil sonst eine konvertierung eines Arrays
 //   per CopyKonstruktor in einen Variant zu aufwendig waere. Einzigstes Problem ist dann dann noch
 //   die Elemente auf den Typ getetstet werden muessen -> sie duerfen keine Array mit String fixer laenge 
 //   sein respektive Arrays mit benutzereigenen Typen.
// B_Boolean B_isArray(const B_Variant& variable);
// B_Boolean B_isArray(const B_Arrray& variable);

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function::  B_isMissing                                                           (C) CiceroSoft 1995
//                 Gibt einen boolschen Wert zurueck der angibt ob ein optional uebergebener Parameter
//                 an eine Funktion/Sub uebergeben wurde oder nicht
//
// See also:     
//
// Parameter:      variable:
//                     eine Variable beliebigen gueltigen Variant Untertyps
//
// Return Value:  boolscher Wert:
//                    True  -> wenn das Argument nicht uebergeben wurde
//                    False -> sonst
// 
// Errorhandling:  - keines - eigentlich keines Noetig da er alles akzeptiert und die Arrays von der VB IDE 
//                 abgefangen werden.
//
// Notes:      Das optionale Argument das sie als Funtkionsparameter angeben muss vom Typ Variant sein.
//                 Aus diesem Grunde koennen auch keine Array von Benutzerdefienierten Typen un dmit Strings
//                 fester Laenge uebergeben werden.
//                 Erlaubt hingegen sind Varianten Array, also Arrays deren Elemente von einem gueltigen
//                 Variantuntertyp sind.
//                 Wird kein Parameter uebergeben bekommt der Variantparameter den Untertyp Error.
//                 Dieser wird dann von IsMissing abgefragt.
//                 Der Wert von Arg kan auch mehrmals mit IsMissing abgefragt werden.
//                 Wird isMissing auf eine normale Variable (also keinen Funktion oder Sub Parameter) ange-
//                 wendet liefert die Funktion immer False. D.h die Funktion erkennt ob es ein Parameter ist
//                 oder nicht.
//                 Fuer Optionale Argumente gelten die gleichen Bedingungen wie unter C++:
//                   - Das/die optionalen Parameter muessen hinter den festen Paramter stehen
//                   - Es duerfen mehrere Optionale PArameter aueinander folgen
//                   
//
 //-- Bei impl. eigener Fehlerwert als default C++ argument euebergeben an dem is Missing erkenne kann ob
 // es ishc um einen Paramter handelt oder nicht.
 // z.B. FEhlernummer 2000 (bel. freie) -> vbncOptionalArg
 //  funktionXY ( B_Variant optarg = CVErr(vbncOptionalArg) )
 // somit wird es fuer die Funktion isMissing moeglich die Unterscheidung zu treffen
// B_Boolean B_isMissing(const B_Variant& parameter);

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function::  B_varTpye                                                             (C) CiceroSoft 1995
//                 Gibt eine Konstante/Wert zurueck der angibt von welchem (unter-)Typ die Variable ist
//
// See also:     B_tpyeName
//
// Parameter:      variable:
//                    eine beliebige Varibale, ausser eine von eines benutzerdefinierten Typs (= Type ..)
//
// Return Value:  Einen der folgenden Wert:
//                   Konstante   	Wert   	Variablentyp Beschreibung
//                   vbEmpty    	0	      Empty (uninitialized).
//                   vbNull	      1     	Null (no valid data).
//                   vbInteger	  2      	Integer.
//                   vbLong	      3     	Long integer.
//                   vbSingle   	4	      Single-precision floating-point number.
//                   vbDouble	    5     	Double-precision floating-point number.
//                   vbCurrency  	6     	Currency.
//                   vbDate	      7     	Date.
//                   vbString	    8     	String.
//                   vbObject	    9     	OLE Automation object.
//                   vbError	    10	    Error.
//                   vbBoolean	  11	    Boolean.
//                   vbVariant	  12    	Variant (used only with arrays of Variants).
//                   vbDataObject	13	    Non-OLE Automation object.
//                   vbByte     	17	    Byte
//                   vbArray	    8192	  Array.
//                  Diese Konstanten sind unter VB durch VB for Application spezifiziert. Die Bibliothek 
//                  stellt Ihnen diese symbolischen Konstanten auch standarmaessig zur Verfuegung.
//
// Errorhandling:  s. isDate
//
// Notes:      Die B_varType Funktion gibt niemals den Wert fuer vbArray alleine zurueck.
//                 Dieser Wert ist immer mit einem anderen Wert verbunden der den Typ der Variablen des
//                 Array beschreibt.
//                 Die Konstante vbVariant wird nur zurueckgeggen in Verbindung mit vbArray um anzuzeigen
//                 dass das Argument der B_varType Funktion ein Array von Variant-Elementen ist. z.B.
//                 wird er Wert fuer ein Integerarray wie folgt berechnet:
//                     vbInteger + vbArray -> 8194
//                 Wenn ein Objekt einen default Property hat gibt VarType(objectvar) den Typ des default
//                 Properties zurueck.
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////*/
 // evtl Return Value in enum verpacken wenn sonst nicht damit gerechnet werden kann
// B_Integer B_varType(const B_Variant& variable);

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function::  B_typeName                                                            (C) CiceroSoft 1995
//                 Gibt einen String zurueck der Informationen ueber die Variable liefert
//
// See also:     B_varType
//
// Parameter:      varName:
//                    kann eine Variable sein, ausgenommen eine Variable von einerm Benutzerdefinierten
//                    Typ
//
// Return Value:  Einer der folgenden vordefinierten Zeichenketten:
//                 zurueckgegebene Zeichenkette     die Variable enthaelt
//                 objecttype                     	Ein OLE Automatation Objekt vom Typ objecttype.
//                 Byte	                            Ein byte
//                 Integer                         	Ein integer.
//                 Long	                            Einen long integer.
//                 Single	                          Eine Fliesskommazahl einfacher Genauigkeit
//                 Double                          	Eine Fliesskommazahl doppelter Genauigkeit
//                 Currency	                        Einen currency    (Finanzmathematischer Datentyp)
//                 Date	                            Ein date          (Datum / Uhrzeit)
//                 String	                          Eine Zeichenkette
//                 Boolean	                        Einen boolschen Wert
//                 Error	                          Einen error Wert
//                 Empty	                          Einen uninitialisierten Wert
//                 Null	                            Keine gueltigen Daten
//                 Object	                          Ein Objekt das OLE Automation unterstuetzt
//                 Unknown	                        Ein OLE Automationsobjekt dessen Typ unbekannt ist
//                 Nothing	                        Eine Objektvariable die kein Objekt referenziert
//
// Errorhandling:  Bei Arrays mit Elementen vom Typ benutzerdefiniert oder Strings fester Laenge gibt
//                 die VB IDE eine Fehlermeldung zurueck ( Argument ungueltig, bei Benutzerdefinierten
//                 keine Werte by Ref erlaubt mit Schluesselwort ByVal unerlaubter Parameter)
//                 Ansonsten werden alle akzeptiert.
//
// Notes:      Wenn varName ein Array ist, so ist der Rueckgabestring einer der obigen aufgefuehrten
//                 moeglichen Strings (oder Variants) mit leeren Klammern angefuegt. Z.B. wenn varName 
//                 ein Array mit Elementen vom Typ integer spezifiziert gibt B_typeName "Integer()"
//                 zurueck.
//                 typeName gibts es erst seit VB40
///////////////////////////////////////////////////////////////////////////////////////////////////////////*/
// B_String B_typeName(const B_Variant& varName);

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function::  B_cCur                                                                (C) CiceroSoft 1995
//                 Wandelt einen Ausdruck in den Typ Currency um.
//
// See also:     alle B_c*-Fkt., B_format, B_hex, B_oct, B_val
//
// Parameter:      expression
//                    ein beliebiger zulaessiger numerischer Ausdruck oder Zeichenfolgenausdruck
// 
// Return Value:  ein Wert vom Typ Currency der die konvertierte expression darstellt
//
// Errorhandling:  (in VB Laufzeitfehler(13) Datentypen unvertraeglich)
//                     - Wenn ein String uebergeben wird der kein reiner Zahlstring ist.
//                 (in VB Laufzeitfehler(94) Verwednung von Null unzulaessig)
//                     - Wenn Null uebergeben wird
//                 (in VB Laufzeitfehler(6) Ueberlauf
//                     - Wenn expression auerhalb des zulaaessigen Wertebereich von Currency liegt
//                 Arrays koennen nicht uebergeben werden (egal welche) diese Fehler erkennt schon die VB IDE
//                 und gibt einen entsprechende Warnung aus.
//
// Notes:      Sie koennen Ihren Code mit Hilfe der Funktionen zum Umwandeln von Datentypen dokumentieren,
//                 um zu zeigen, da das Ergebnis einer Operation einen bestimmten, vom Standarddatentyp 
//                 abweichenden Datentyp haben soll. Mit CCur koennen Sie zum Beispiel Berechnungen explizit
//                 im Waehrungsformat durchfuehren, die normalerweise ganzzahlige Werte oder eine einfache 
//                 bzw. doppelte Genauigkeit verwenden wuerden.
//                 Die CCur-Funktion stellt eine Alternative zu Val dar, die international gueltige 
//                 Umwandlungen eines beliebigen Datentyps in den Typ Currency ermoeglicht. So werden zum 
//                 Beispiel verschiedene, vom Gebietsschema des Computers abhaengige Dezimalzeichen ebenso 
//                 erkannt wie unterschiedliche Tausendertrennzeichen und Waehrungseinstellungen.
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////*/
 // evtl asl Parameter auch einfach B_Currency angeben -> Compiler muss ueber Konstruktoren und Konvertierungs-
 // operatoren der einzelnen Datentypen entscheiden ob die Konvertierung geht und wuerde so schon zur 
 // compilierungszeit erkennen ob es geht und nicht erst zur Laufzeit. Dies waere auch ein Performancezuwachs
 // Superbible: c@ = CCur(Dbl#)
 // B_Currency B_cCur(const B_Variant& expression); 
  
 
////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function::  B_cDbl                                                                (C) CiceroSoft 1995
//                 Wandelt einen Ausdruck in den Typ Double um.
//
// See also:     alle B_c*-Fkt., B_format, B_hex, B_oct, B_val
//
// Parameter:      expression:
//                     ein beliebiger zulaessiger numerischer Ausdruck oder Zeichenfolgenausdruck
//
// Return Value:  ein Wert vom Typ Double der die konvertierte expression darstellt
//
// Errorhandling:  (in VB Laufzeitfehler(13) Datentypen unvertraeglich)
//                     - Wenn ein String uebergeben wird der kein reiner Zahlstring ist.
//                 (in VB Laufzeitfehler(94) Verwednung von Null unzulaessig)
//                     - Wenn Null uebergeben wird
//                 Arrays koennen nicht uebergeben werden (egal welche) diese Fehler erkennt schon die VB IDE
//                 und gibt einen entsprechende Warnung aus.
//
// Notes:      Sie koennen Ihren Code mit Hilfe der Funktionen zum Umwandeln von Datentypen dokumentieren,
//                 um zu zeigen, da das Ergebnis einer Operation einen bestimmten, vom Standarddatentyp 
//                 abweichenden Datentyp haben soll. Mit CDbl oder CSng koennen Sie zum Beispiel Berechnungen
//                 mit einfacher oder doppelter Genauigkeit durchfuehren, die sonst Waehrungswerte oder ganz-
//                 zahlige Werte verwenden wuerden. 
//                 Die CDbl-Funktion stellt eine Alternative zu Val da, die international gueltige 
//                 Umwandlungen eines beliebigen Datentyps in den Typ Double ermoeglicht. So werden zum
//                 Beispiel verschiedene, vom Gebietsschema des Computers abhaengige Dezimalzeichen ebenso
//                 erkannt wie unterschiedliche Tausendertrennzeichen.
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////*/
 // Parameter s. Impl. Bem. b_CCur
 // Sperbible: d# = CDbl(Vnt)
 // B_Double B_cDbl();    

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function::  B_cInt                                                                (C) CiceroSoft 1995
//                 Wandelt einen Ausdruck in den Typ Integer um.
//
// See also:     B_int, B_fix, alle B_c*-Fkt., B_format, B_hex, B_oct, B_val
//
// Parameter:      expression:
//                     ein beliebiger zulaessiger numerischer Ausdruck oder Zeichenfolgenausdruck
//
// Return Value:  ein Wert vom Typ Integer der die konvertierte expression darstellt
//
// Errorhandling:  (in VB Laufzeitfehler(13) Datentypen unvertraeglich)
//                     - Wenn ein String uebergeben wird der kein reiner Zahlstring ist.
//                 (in VB Laufzeitfehler(94) Verwednung von Null unzulaessig)
//                     - Wenn Null uebergeben wird
//                 (in VB Laufzeitfehler(6) Ueberlauf
//                     - Wenn expression auerhalb des zulaessigen Wertebereich von Integer liegt.
//                       D.h. z.B. wenn ein konvertierter Datumswert ausserhalb des Wertebereiches liegt,
//                       was bei Integer durchaus sein kann. Somit muss sich dieser Fehler nicht auf einen
//                       numerischen Datentyp beziehen
//                 Arrays koennen nicht uebergeben werden (egal welche) diese Fehler erkennt schon die VB IDE
//                 und gibt einen entsprechende Warnung aus.
//
// Notes:      Sie koennen Ihren Code mit Hilfe der Funktionen zum Umwandeln von Datentypen dokumentieren,
//                 um zu zeigen, da das Ergebnis einer Operation einen bestimmten, vom Standarddatentyp
//                 abweichenden Datentyp haben soll. Mit CInt oder CLng koennen Sie zum Beispiel Berechnungen
//                 mit ganzzahligen Werten durchfuehren, die normalerweise Waehrungswerte oder Werte mit
//                 einfacher oder doppelter Genauigkeit verwenden wuerden.
//                 Die CInt-Funktion stellt eine Alternative zu Val dar, die international gueltige
//                 Umwandlungen eines beliebigen Datentyps in den Typ Integer ermoeglicht. So werden zum 
//                 Beispiel verschiedene, vom Gebietsschema des Computers abhaengige Dezimalzeichen ebenso
//                 erkannt wie unterschiedliche Tausendertrennzeichen.
//                 Hinweis: CInt unterscheidet sich von den Funktionen Fix und Int, die die Nachkommastellen
//                          einer Zahl abschneiden, anstatt die Zahl zu runden. Ist der Nachkommateil einer
//                          Zahl genau gleich 0,5, so rundet die CInt-Funktion immer auf die naechste gerade
//                          Zahl. So wird zum Beispiel 0,5 auf 0 gerundet, waehrend 1,5 auf 2 gerundet wird.
//                Value   Cint(value)
//                 2.7     3
//                 2.5     ?
//                 2.2     2
//                 2       2
//                -2      -2
//                -2.2    -2
//                -2.5     ?
//                -2.7    -3
//                Vgl. B_int(), B_fix()
///////////////////////////////////////////////////////////////////////////////////////////////////////////*/
 // Parameter s. Impl. Bem. b_CCur
 // Superbible: i% = Cint(Lng&)
 // B_Integer B_cInt();    

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function::  B_cLng                                                                (C) CiceroSoft 1995
//                 Wandelt einen Ausdruck in den Typ Long um.
//
// See also:     B_int, B_fix, alle B_c*-Fkt., B_format, B_hex, B_oct, B_val
//
// Parameter:      expression:
//                    ein beliebiger zulaessiger numerischer Ausdruck oder Zeichenfolgenausdruck.
//
// Return Value:  ein Wert vom Typ Long der die konvertierte expression darstellt
//
// Errorhandling:  (in VB Laufzeitfehler(13) Datentypen unvertraeglich)
//                     - Wenn ein String uebergeben wird der kein reiner Zahlstring ist.
//                 (in VB Laufzeitfehler(94) Verwednung von Null unzulaessig)
//                     - Wenn Null uebergeben wird
//                 (in VB Laufzeitfehler(6) Ueberlauf
//                     - Wenn expression auerhalb des zulaessigen Wertebereich von Long liegt.
//                 Arrays koennen nicht uebergeben werden (egal welche) diese Fehler erkennt schon die VB IDE
//                 und gibt einen entsprechende Warnung aus.
//
// Notes:      Sie koennen Ihren Code mit Hilfe der Funktionen zum Umwandeln von Datentypen dokumentieren,
//                 um zu zeigen, da das Ergebnis einer Operation einen bestimmten, vom Standarddatentyp 
//                 abweichenden Datentyp haben soll. Mit CInt oder CLng koennen Sie zum Beispiel Berechnungen
//                 mit ganzzahligen Werten durchfuehren, die normalerweise Waehrungswerte oder Werte mit 
//                 einfacher oder doppelter Genauigkeit verwenden wuerden.
//                 Die CLng-Funktion stellt eine Alternative zu Val dar, die international gueltige 
//                 Umwandlungen eines beliebigen Datentyps in den Typ Long ermoeglicht. So werden zum Beispiel
//                 verschiedene, vom Gebietsschema des Computers abhaengige Dezimalzeichen ebenso erkannt wie
//                 unterschiedliche Tausendertrennzeichen.
//                 Hinweis: CLng unterscheidet sich von den Funktionen Fix und Int, die die Nachkommastellen
//                          einer Zahl abschneiden, statt die Zahl zu runden. Ist der Nachkommateil einer
//                          Zahl genau gleich 0,5, so rundet die CLng-Funktion immer auf die naechste gerade
//                          Zahl. So wird zum Beispiel 0,5 auf 0 gerundet, waehrend 1,5 auf 2 gerundet wird. 
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////*/
 // Wertebereich reicht fuer alle Datumswerte aus -> kein test noetig
 // Parameter s. Impl. Bem. b_CCur 
 // Superbible: l& = CLng(Vnt)
 // B_Long B_cLng();    

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function::  B_cSng                                                                (C) CiceroSoft 1995
//                 Wandelt einen Ausdruck in den Typ Single um
//
// See also:     alle B_c*-Fkt., B_format, B_hex, B_oct, B_val
//
// Parameter:      expression
//                    ein beliebiger zulaessiger numerischer Ausdruck oder Zeichenfolgenausdruck.  
//
// Return Value:  ein Wert vom Typ Single der die konvertierte expression darstellt
//
// Errorhandling:  (in VB Laufzeitfehler(13) Datentypen unvertraeglich)
//                     - Wenn ein String uebergeben wird der kein reiner Zahlstring ist.
//                     - 
//                 (in VB Laufzeitfehler(94) Verwednung von Null unzulaessig)
//                     - Wenn Null uebergeben wird
//                 (in VB Laufzeitfehler(6) Ueberlauf
//                     - Wenn expression auerhalb des zulaessigen Wertebereich von Single liegt.
//                       D.h. z.B. wenn ein konvertierter Datumswert ausserhalb des Wertebereiches liegt,
//                       somit muss sich dieser Fehler nicht auf einen numerischen Datentyp beziehen
//                 Arrays koennen nicht uebergeben werden (egal welche) diese Fehler erkennt schon die VB IDE
//                 und gibt einen entsprechende Warnung aus.
//
// Notes:      Sie koennen Ihren Code mit Hilfe der Funktionen zum Umwandeln von Datentypen dokumentieren,
//                 um zu zeigen, da das Ergebnis einer Operation einen bestimmten, vom Standarddatentyp
//                 abweichenden Datentyp haben soll. Mit CDbl oder CSng koennen Sie zum Beispiel Berechnungen
//                 mit Werten einfacher oder doppelter Genauigkeit durchfuehren, die normalerweise Waehrungs-
//                 werte oder ganzzahlige Werte verwenden wuerden.
//                 Die CSng-Funktion stellt eine Alternative zu Val dar, die international gueltige
//                 Umwandlungen eines beliebigen Datentyps in den Typ Single ermoeglicht. So werden zum 
//                 Beispiel verschiedene, vom Gebietsschema des Computers abhaengige Dezimalzeichen ebenso 
//                 erkannt wie unterschiedliche Tausendertrennzeichen.
//                 Wenn Ausdruck auerhalb des zulaessigen Bereichs fuer den Datentyp Single liegt, tritt ein
//                 Fehler auf.
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////*/
 // Wertebereich reicht fuer alle Datumswerte aus kein test noetig
 // Parameter s. Impl. Bem. b_CCur 
 // Superbible: s! = CSng(Cnc@)
 // B_Single B_cSng();   
 
////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function::  B_cStr                                                                (C) CiceroSoft 1995
//                 Wandelt einen Ausdruck in den Typ String um.
//
// See also:     B_str, alle B_c*-Fkt., B_format
//
// Parameter:      expression:
//                     ein beliebiger zulaessiger numerischer Ausdruck oder Zeichenfolgenausdruck.
//
// Return Value:  eine Zeichenkette die die konvertierte expression enthaelt
//
// Errorhandling:  (in VB Laufzeitfehler(13) Datentypen unvertraeglich)
//                    - Wenn Return Value der CVErr-Fkt uebergeben wird.
//
// Notes:      Sie koennen Ihren Code mit Hilfe der Funktionen zum Umwandeln von Datentypen dokumentieren,
//                 um zu zeigen, da das Ergebnis einer Operation einen bestimmten, vom Standarddatentyp 
//                 abweichenden Datentyp haben soll. Mit CStr koennen Sie zum Beispiel explizit den Typ String
//                 als Ergebnistyp angeben.
//                 Die CStr-Funktion stellt eine Alternative zu Str dar, die international gueltige
//                 Umwandlungen eines beliebigen Datentyps in den Typ String ermoeglicht. So werden zum
//                 Beispiel verschiedene, vom Gebietsschema des Computers abhaengige Dezimalzeichen erkannt.
//                 Die Daten in Ausdruck legen die Rueckgabewerte von CStr fest, wie in der folgenden Tabelle
//                 aufgefuehrt:
//                 Datentyp von Ausdruck	    Rueckgabe von CStr
//                 Boolean                   	Datentyp String mit dem Wert True oder False.
//                                            ( nicht Wahr oder Falsch bei deutscher Landeseinstellung)
//                 Date	                      Datentyp String mit einem Datum im eingestellten kurzen
//                                            Datumsformat.
//                 Null                       Laufzeitfehler.
//                 Empty	                    Eine leere Zeichenfolge vom Typ String ("").
//                 Error	                    Eine Zeichenfolge vom Typ String mit dem Wort "Fehler" und
//                                            der Fehlernummer (lt. OnlineHilfe).
//                                            Liefert in VB40 nur den String "Error" ohen Fehlernummer
//                                            zurueck, egal ob VBFehler oder Benutzerdefinierter
//                                            
//                 Anderer numerischer Wert	  Eine Zeichenfolge vom Typ String mit der Zahl.
//                                            z.B. wird 2E+40 nicht expandiert sonder als "2E+40" belassen.
//                                                 genauso werden ausgeschrieben Zahlenwerte nicht in 
//                                                 Exponentialschreibweise konvertiert, ausser den Konver-
//                                                 tierungen die VB schon in der IDE vornimmt.
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////*/
 // Parameter s. Impl. Bem. b_CCur 
 // Superbible: str$ = CStr(Dlb#)
 // B_String B_cStr();    

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function::  B_cVar                                                                (C) CiceroSoft 1995
//                 Wandelt einen Ausdruck in den Typ Variant um
//
// See also:     alle B_c*-Fkt.
//
// Parameter:      expression
//                     ein beliebiger zulaessiger numerischer Ausdruck oder Zeichenfolgenausdruck.
//
// Return Value:  Ein Variant von dem Untertyp in den umgewandelt werden konnte
//
// Errorhandling:  (in VB Laufzeitfehler(13) Datentypen unvertraeglich)
//                     - Wenn als expression ein Return Value vom Typ CVerr() uebergeben wird
//                     - Bei uebergabe eines Arrays mit Untertyen eines Variant. Bei Arrays von
//                       benutzerdefinierten Typen oder von Strings fester laenge meldet die VB IDE
//                       eine Warnung noch vor Programmausfuehrung
//
// Notes:      Sie koennen Ihren Code mit Hilfe der Funktionen zum Umwandeln von Datentypen dokumentieren,
//                 um zu zeigen, da das Ergebnis einer Operation einen bestimmten, vom Standarddatentyp
//                 abweichenden Datentyp haben soll. Mit CVar koennen Sie zum Beispiel explizit den Datentyp
//                 Variant fuer einen Ergebniswert angeben.
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////*/
 // Return Value B_Variant oder aber ueberladen fuer alle Typen und compiler muss sehen was er macht
 // somit habe ich mir Arbeit gespart Problem ist ob dies ohne mehrdeutigkeiten ueber die Buehne gehen kann
 // B_Bool B_cVar(B_Bool)  // Tja konstruktor von Bool nimmt schon mal Integers an -> Testen
 // Superbible: vnt = CVar(Lng&)
 // B_Variant B_cVar(); 

 ////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function::  B_cvDate                                                               (C) CiceroSoft 1995
//                 Wandelt einen Ausdruck in den Typ Variant mit Untertyp Date (=7) um.
//
// See also:     B_dateSerial, B_isDate, alle B_c*-Fkt.
//
// Parameter:      expression:
//                    Das Argument Datum ist ein beliebiger zulaessiger Datumsausdruck, also eine Zeichenkette
//                    oder ein numerischer Ausdruck der als Datum interpretiert werden kann.
//
// Return Value:  Wert von expression in ein Datum konvertiert
//
// Errorhandling:  (in VB Laufzeitfehler(13) Datentypen unvertraeglich)
//                     - Wenn Wertebereich einer Double oder Single variabeln ueber den gueltigen
//                       Datumswertebereich hinausgeht -> Scheiss Meldung iin jedemfall aus enhanced
//                       Lib rausnehmen besser Ueberlauf meldung
//                     - wenn expression uebergabewert von Error() oder CVErr() erhaelt
//                     - Wenn kein Zahl oder Datumstring uebergeben wird, z.B. ein Zeichenstring
//
// Notes:      Konvertierungen der CVDate-Fkt.:
//                   Typ des Parameters       Operation von CVDate
//                   numerischer Ausdruck     Testen ob der Wertebereich zulaessig ist und in ein Datum 
//                                            konvertieren, sonst Laufzeitfehler
//                   Datumsstring             in eine Datum konvertieren (z.B. "1 May 1992 12:00 PM")
//                   ein Zahlstring           in eine Zahl konvertieren Wertebereich testen
//                   andere Strings           rufen Laufzeitfehler hervor
//                   Boolscher Wert           True = -1 -> 29.12.1899
//                                            False = 0 -> 0:00:00
//                   Empty                    0:00:00 also als 0 interpretiert
//
//                 Mit der IsDate-Funktion koennen Sie bestimmen, ob Datum in ein Datum oder eine Uhrzeit
//                 umgewandelt werden kann. CDate erkennt sowohl Datums- und Zeitwerte als auch bestimmte
//                 Zahlen, die im zulaessigen Bereich fuer ein Datum liegen. Beim Umwandeln einer Zahl in ein
//                 Datumsformat wird der ganzzahlige Teil fuer das Datum verwendet. Nachkommastellen der Zahl
//                 werden in eine Tageszeit (beginnend bei 0:00) umgewandelt.
//                 CDate erkennt alle Datumsformate, die im Gebietsschema des Systems ausgewaehlt werden
//                 koennen. Die richtige Reihenfolge von Tag, Monat und Jahr kann nicht immer bestimmt
//                 werden, wenn sich das Datumsformat von den im Gebietsschema verfuegbaren Formaten 
//                 unterscheidet. Auerdem wird ein langes Datumsformat nicht erkannt, auch wenn es eine
//                 Zeichenfolge fuer den Wochentag enthaelt.
//                 Hinweis:   Die CVDate-Funktion ist aus Gruenden der Kompatibilitaet zu frueheren Visual
//                 Basic-Versionen verfuegbar. Durch den integrierten Datentyp Date (seit VB40) wird CVDate
//                 jedoch nicht mehr benoetigt. Die Syntax der CVDate-Funktion entspricht genau der Syntax
//                 der CDate-Funktion. Der einzige Unterschied besteht darin, da CVDate nicht den Datentyp
//                 Date liefert, sondern den Datentyp Variant mit dem Untertyp Date. Dies koennen Sie aber
//                 auch erreichen, indem Sie einen Ausdruck in den Datentyp Date umwandeln und ihn dann
//                 einer Variablen vom Typ Variant zuweisen. Mit demselben Verfahren koennen auch alle
//                 anderen integrierten Datentypen in die entsprechenden Untertypen des Datentyps Variant
//                 umgewandelt werden. 
///////////////////////////////////////////////////////////////////////////////////////////////////////////*/
 // als Parameter entweder Variant oder Double und String vgl. B_cCur
 // aus VB30 Online Hilfe:  
 //         Bei Stringausdruecken beachtet CVDate das Datumsformat das in der Internainal Abschnitt
 //         in der WIN.INI (bei Win31) eingestellt ist. Jedoch kann CVDate keinen String im langen
 //         Datumsformat akzeptieren auch wenn der Wochentag ausgeschrieben ist (-> in Lib erlauben)
 //         Wenn expression ein Zahlstring ist der nur Zahlenwerte enthaelt kann CVDate das Datum
 //         nicht korrekt konvertieren wenn das Format nicht der aktuellen einstellung in der WIN.INI
 //         Datei entspricht.
 // B_Date B_cvDate();

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function::  B_cDate                                                               (C) CiceroSoft 1995
//                 Wandelt einen Ausdruck in den Typ Date um.
//
// See also:     B_cvDate, B_dateSerial, B_isDate, alle B_c*-Fkt.
//
// Parameter:      expression:
//                    beliebiger gueltiger Datumsausdruck
//
// Return Value:  
//
// Errorhandling:  (in VB Laufzeitfehler(94) Verwendung von Null unzulaessig)
//                      - Wenn Null als Parameter uebergeben wird
//                 (in VB Laufzeitfehler(6) Ueberlauf
//                     - Wenn Wertebereich einer Double oder Single variabeln ueber den gueltigen
//                       Datumswertebereich hinausgeht -> vgl. Scheiss Meldung in VB30 bei CVdate
//                 (in VB Laufzeitfehler(13) Datentypen unvertraeglich)
//                     - wenn expression uebergabewert von Error() oder CVErr() erhaelt
//                     - Wenn kein Zahl oder Datumstring uebergeben wird, z.B. ein Zeichenstring
//                     - Wemm Error oder CVError angegeben wird.
//                 Bei uebergabe von Arrays immer Meldung von VB IDE, egal welchen Typ die Arrayelemente 
//                 haben
//
// Notes:      Mit der IsDate-Funktion koennen Sie bestimmen, ob Datum in ein Datum oder eine Uhrzeit 
//                 umgewandelt werden kann. CDate erkennt sowohl Datums- und Zeitwerte als auch bestimmte 
//                 Zahlen, die im zulaessigen Bereich fuer ein Datum liegen. Beim Umwandeln einer Zahl in ein
//                 Datumsformat wird der ganzzahlige Teil fuer das Datum verwendet. Nachkommastellen der Zahl
//                 werden in eine Tageszeit (beginnend bei 0:00) umgewandelt.
//                 CDate erkennt alle Datumsformate, die im Gebietsschema des Systems ausgewaehlt werden
//                 koennen. Die richtige Reihenfolge von Tag, Monat und Jahr kann nicht immer bestimmt werden,
//                 wenn sich das Datumsformat von den im Gebietsschema verfuegbaren Formaten unterscheidet. 
//                 Auerdem wird ein langes Datumsformat nicht erkannt, wenn es auch eine Zeichenfolge fuer 
//                 den Wochentag enthaelt.
//                 Hinweis: Die CVDate-Funktion ist ebenfalls aus Gruenden der Kompatibilitaet zu frueheren 
//                          Visual Basic-Versionen (vor VB40) verfuegbar. Durch den integrierten Datentyp
//                          Date wird CVDate jedoch nicht mehr benoetigt. Die Syntax der CVDate-Funktion
//                          entspricht genau der Syntax der CDate-Funktion. Der einzige Unterschied besteht 
//                          darin, da CVDate nicht den Datentyp Date liefert, sondern den Datentyp Variant
//                          mit dem Untertyp Date. Dies koennen Sie aber auch erreichen, indem Sie einen 
//                          Ausdruck in den Datentyp Date umwandeln und ihn dann einer Variablen vom Typ 
//                          Variant zuweisen. Mit demselben Verfahren koennen auch alle anderen integrierten 
//                          Datentypen in die entsprechenden Untertypen des Datentyps Variant umgewandelt
//                          werden.
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////*/
 // Parameter vgl. B_cvDate
 // Superbible: dat = CDate(Dbl#)
 //             Ist der WErt numerisch im Bereich von -657434 bis 2958465 -> Date
 //             Numerischer Bereich nicht in obigen                       -> LZF(13) Type Mismatch
 //             Datumstring                                               -> Date
 //             Numerische Strings                                        -> Date, wenn Die Nummer im gueltigen
 //                                                                          bereich ist. sonst LZF(13)
 //             Andere Stringausdruecke                                   -> LZF(13)
 // B_Date B_cDate();
    
////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function::  B_cBool                                                               (C) CiceroSoft 1995
//                 Wandelt einen Ausdruck in den Typ Boolean um.
//
// See also:     alle B_c*-Fkt.
//
// Parameter:      expression:
//                    ein beliebiger zulaessiger numerischer Ausdruck oder Zeichenfolgenausdruck.
//
// Return Value:  Ein boolescher Wert der in abhaengigkeit von dem Wert den expression liefert True oder
//                 False ist.
//
// Errorhandling: (in VB Laufzeitfehler(13) Datentypen unvertraeglich)
//                    - Wenn Error als Parameter uebergeben wird
//                    - Wenn kein reiner Zahlstring uebergeben wird
//                (in VB Laufzeitfehler(94) Null unzulaessig)
//                    - Wenn Null uebergeben wird
//
// Notes:     Wenn Ausdruck gleich Null ist, wird False zurueckgegeben, ansonsten wird True zurueckgegeben.
//                Wenn Ausdruck nicht als numerischer Wert interpretiert werden kann, tritt ein Laufzeit-
//                fehler auf. 
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////*/
 // PArameter vgl B_cCur
 // Superbible: boolean = CBool(Int%)
 // B_Boolean B_cBool(); 

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function::  B_cByte                                                               (C) CiceroSoft 1995
//                 Wandelt einen Ausdruck in den Typ Byte um.
//
// See also:     
//
// Parameter:      expression:
//                     ein beliebiger zulaessiger numerischer Ausdruck oder Zeichenfolgenausdruck.
//
// Return Value:  
//
// Errorhandling:  (in VB Laufzeitfehler(94) Null unzulaessig)
//                    - Wenn Null uebergeben wird
//                 (in VB Lafzeitfehler(6) Ueberlauf)
//                    - Wenn Integer, Currency, Single, Long, Double Werte den Wertebereich von Byte
//                      ueberschreiten.
//                    - Wenn der in einen Double konvertierte Wert fuer ein Datum nicht mehr in den
//                      Wertebereich von B_Byte passt.
//                    - Wenn der Return Value von CVerr ausserhalb des Wertebereiches von Byte liegt
//                 (in VB Laufzeitfehler(13) Datentypen unvertraeglich)
//                    - Wenn Error als Parameter uebergeben wird
//                    - Wenn kein reiner Zahlstring uebergeben wird
//                 bergabewert vom Typ Array werden von der VB IDE erkannt und als unvertraegliche 
//                 Datentypengemeldet.
//
// Notes:      Im allgemeinen koennen Sie Ihren Code mit den Funktionen zur Datentypumwandlung 
//                 dokumentieren, um zu zeigen, da das Ergebnis einer Operation nicht mit dem Standard-
//                 Datentyp, sondern mit einem bestimmten Datentyp ausgedrueckt werden soll. Verwenden Sie
//                 beispielsweise CByte, um Berechnungen auf Byte-Basis zu erzwingen, wenn normalerweise
//                 Berechnungen auf Basis von Waehrungsbetraegen, Werten einfacher oder doppelter Genauigkeit
//                 oder ganzzahligen Werten erfolgen wuerden.
//                 Sie sollten die CByte-Funktion anstelle von Val verwenden, um international gueltige 
//                 Umwandlungen von einem beliebigen anderen Datentyp in den Datentyp Byte vorzunehmen.
//                 Zum Beispiel werden unterschiedliche Dezimaltrennzeichen entsprechend der Gebietsschema-
//                 Einstellung Ihres Systems richtig erkannt, genau wie unterschiedliche Tausendertrenn-
//                 zeichen.
//                 Wenn Ausdruck auerhalb des zulaessigen Bereichs fuer den Datentyp Byte liegt, tritt ein
//                 Fehler auf.
//                 Wenn der boolsche Wert True (-1) uebergeben wird tritt keine Fehlermeldung auf obwohl der 
//                 Wertebereich ueberschritten wird. Stattdessen wird True zu 255. False bleibt 0.
///////////////////////////////////////////////////////////////////////////////////////////////////////////*/
 // Parameter s. B_CCur
 // Superbible:  byte = CByte(Int%)
 // B_Byte B_cByte();

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function::  B_val                                                                 (C) CiceroSoft 1995
//                 Gibt die in einer Zeichenfolge enthaltenen Zahlen zurueck.
//
// See also:     B_str, B_cDbl
//
// Parameter:      expression:
//                    ein beliebiger zulaessiger Zeichenfolgenausdruck oder numerischer Wert.
//                    numersiche Werte werden in einen String konvertiert.
//
// Return Value:  
//
// Errorhandling:  (in VB Laufzeitfehler(13) Datentypen unvertraeglich)
//                     - Wenn CVerr() als Argument uebergeben wird
//                 (in VB Laufzeitfehler(94) Null unzulaessig)
//                     - Wenn Null uebergeben wird
//                 bergabewert vom Typ Array werden von der VB IDE erkannt und als unvertraegliche 
//                 Datentypengemeldet.
//
// Notes:      Die Val-Funktion liest die Zeichen einer Zeichenfolge bis zum ersten Zeichen, das nicht
//                 als Teil einer Zahl interpretiert werden kann. Symbole und Zeichen, die oft als Teil 
//                 eines numerischen Wertes erkannt werden (z.B. Waehrungssymbole und Kommas), werden von Val
//                 nicht erkannt. Die Funktion erkennt allerdings die Praefixe &O (fuer oktal) und &H (fuer 
//                 hexadezimal) zur Angabe der Zahlenbasis. Leerzeichen, Tabulatoren und Zeilenvorschub-
//                 zeichen werden aus dem Argument entfernt.
//                 Das folgende Beispiel gibt den Wert 44137 zurueck:
//                   Val("    44137 Dortmund, Westenhellweg 112")
//                 Weiter Beispiele:
//                   Typ      expression     Return Value
//                   5        "&HFFFF"       -1 
//                   5        "123"           123
//                   5        "123abc"        123
//                   5        "123abc456"     123
//                   5        "abc 123"       0
//                   5        "123E+45"       123E+45
//                   5        "123,45"        123
//                   5        "123.45"        123,45
//                   4,5,6    123.456         123
//                   2,3      125             125
//                   0        empty           0
//                   10       Error           0
//                   11       True, False     0               da vorher in String konvertiert mit True/False,
//                                                            Wahr/Falsch
//                 Hinweis: Die Val-Funktion erkennt nur den Punkt (.) als gueltiges Dezimalzeichen. Wenn 
//                          Sie ein anderes Dezimalzeichen verwenden moechten (zum Beispiel das in
//                          Deutschland uebliche Komma), sollten Sie stattdessen die Funktion CDbl verwenden,
//                          um eine Zeichenfolge in eine Zahl umzuwandeln.
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////*/
 // Parameter s. B_cCur gleiche Problematik da er im Prinzip alles annehmen muss
 // B_String B_val()

//--------------------------------------------------------------------------------------------------
//  Gibt eine Zeichenfolge mit der Hexadezimaldarstellung einer Zahl zurck.
//  Hex(Zahl)
//  Das Argument Zahl ist ein beliebiger numerischer Ausdruck oder Zeichenfolgenausdruck.
//  Anmerkungen
//  Wenn Zahl keine ganze Zahl ist, rundet die Funktion den Wert vor der Auswertung auf die nchste 
//  ganze Zahl. 
//  Zahl 	   Rckgabewert von Hex 
//  Null     Null
//  Empty	   Null (0)
//  Andere Zahl	Bis zu acht hexadezimale Zeichen. Dies entsprich tdme Wert eines unsigned long 4.294.967.295
//  Fliesskommawerte werden gerundet auf Ganzahlwerte ( <0.5 und >=0.5)
//  Ein ueberlauf tritt auf wenn der Wertebereich von B_Long (+(- 2.147.483.647 ueberschritten wird
//  als ist der maximale Wert 7FFFFFFF
//  ACHTUNG : In VB werden ...... Sie knnen Hexadezimalzahlen direkt angeben, indem Sie vor eine Zahl im zulssigen Bereich die 
//  Zeichen &H setzen. &H10 entspricht zum Beispiel der Dezimalzahl 16 in Hexadezimaldarstellung.
/*
B_String B_hex(const B_Double&);
B_String B_hex(const B_Single&);
B_String B_hex(const B_Currency&);
B_String B_hex(const B_Long&);
B_String B_hex(const B_Integer&);
B_String B_hex(const B_Byte&);
B_String B_hex(const B_Empty&);
B_String B_hex(const B_Boolean&);
B_String B_hex(const B_String&); // nur Zahlstrings
B_String B_hex(const B_Date&);
B_Null   B_hex(const B_Null&);
*/

//--------------------------------------------------------------------------------------------------
// Gibt eine Zeichenfolge mit der Oktaldarstellung einer Zahl zurck.
// Oct(Zahl)
// Das Argument Zahl ist ein beliebiger gltiger numerischer Ausdruck oder Zeichenfolgenausdruck.
// Anmerkungen
// Wenn Zahl keine ganze Zahl ist, rundet die Funktion den Wert vor der Auswertung auf die nchste 
// ganze Zahl.
// Zahl     	Rckgabewert von Oct
// Null     	Null
// Empty	    Null (0)
// Andere Zahl	Bis zu 11 oktale Zeichen. Dies entspricht einem unsigned Long intern  4.294.967.295
// Fliesskommawerte werden gerundet auf Ganzahlwerte ( <0.5 und >=0.5)
// Ein ueberlauf tritt auf wenn der Wertebereich von B_Long (+(- 2.147.483.647 ueberschritten wird
// also ist der maximale Wert 17777777777
// Sie knnen Oktalzahlen direkt angeben, indem Sie &O vor eine Zahl im zulssigen Bereich setzen. 
// &O10 entspricht zum Beispiel der Dezimalzahl 8 in Oktaldarstellung.
/*
B_String B_oct(const B_Double&);
B_String B_oct(const B_Single&);
B_String B_oct(const B_Currency&);
B_String B_oct(const B_Long&);
B_String B_oct(const B_Integer&);
B_String B_oct(const B_Byte&);
B_String B_oct(const B_Empty&);
B_String B_oct(const B_Boolean&);
B_String B_oct(const B_String&); // nur Zahlstrings
B_String B_oct(const B_Date&);
B_Null   B_oct(const B_Null&); 
*/

//================================ Inlinefunktionen =========================================================
//
#if defined _DEBUG || defined B_VB_CONTRACT
  #define B_NOINLINE
#else
  #undef B_NOINLINE
#endif

#ifndef B_NOINLINE                 // festlegen wenn Debuger keine Inlines debuggen kann
  #define B_INLINE inline  
  #ifndef  BI_CONVT_HPP
    #include "bi_convt.hpp"
  #endif
#endif


//===========================================================================================================
//================================ End Of Header ============================================================
#endif // BF_CONVT_HPP

