#ifndef BF_MATH_HPP
#define BF_MATH_HPP
//////////////////////////////////////////////////////////////////////////////////////////////////////////
// Project:                 VB nach C/C++ Konverter Klassenbibliothek
// Modul:                   Mathematische Grundfunktionen von VB
// Filename:                bf_math.hpp
// Version:                 0.0
// Status:                  alpha
//
// Copyright:               (c)
//                          Cicerosoft GmbH, 
//                          Ballweilerstr. 7,
//                          66399 Erfweiler-Ehlingen
//
// Operating Systems:       Win95 
// Compiler:                VC++ 4.1
//
// Author:                  Jochen Diener JD
// Creation Date:           24.08.96
// last Modification:       JD 24.08.96
//
// Description:             Beinhaltet alle mathematischen Prozeduren die VB zur Verfgung stellt
//                          ( Die finanzmathematischen funktionen sind einer eigenen DLL untergebracht)
//                          
// Dependencies:            bi_math.hpp    ->  Inline-Funktionen
//                          bf_math.cpp    ->  Implementierung der Funktionen
//
// Remarks:            
//
// Revision(s):   
//                          
///////////////////////////////////////////////////////////////////////////////////////////////////////////


#ifndef BD_INTYP_HPP       
  #include "bd_intyp.hpp"       // alias der Standardatentypen
#endif

#ifndef BC_NULL_HPP
  #include "bc_null.hpp"
#endif

#ifndef BC_EMPTY_HPP
  #include "bc_empty.hpp"
#endif

#ifndef BC_BOOL_HPP
  #include "bc_bool.hpp"
#endif

#ifndef BC_BYTE_HPP
  #include "bc_byte.hpp"
#endif

#ifndef BC_INT_HPP
  #include "bc_int.hpp"
#endif

#ifndef BC_LONG_HPP
  #include "bc_long.hpp"
#endif

#ifndef BC_SNGLE_HPP
  #include "bc_sngle.hpp"
#endif

#ifndef BC_DBL_HPP
  #include "bc_dbl.hpp"
#endif

#ifndef BC_CURR_HPP
  #include "bc_curr.hpp"
#endif

#ifndef BC_STR_HPP
  #include "bc_str.hpp"
#endif

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function:                                                                             (C) CiceroSoft 1996
//     B_rnd()   Generiert eine Zufallszahl zwischen 0 und 1
// See also:
//     B_randomize
// Parameter:
//     cond
//         ein beliebiger arithmetischer Ausdruck
//         Die Rnd-Funktion gibt einen Wert im Bereich von 0 (eingeschlossen) bis 1 (nicht eingeschlossen) 
//         zurck.
//         Der Wert von cond bestimmt, wie Rnd eine Zufallszahl generiert:
//            Wert von cond	     Generierte Zufallszahl
//            < 0	                 Immer dieselbe Zahl, mit Zahl als Startwert.
//            > 0	                 Die nchste Zufallszahl der Folge.
//            == 0	               Die zuletzt generierte Zahl.
//            Nicht angegeben      Die nchste Zufallszahl der Folge.
//         Fr jeden gegebenen Startwert wird dieselbe Zufallszahlenfolge generiert, weil bei jedem Aufruf 
//         der Rnd-Funktion die vorherige Zahl als Startwert fr die nchste Zahl in der Folge dient.
//         Bevor Sie Rnd aufrufen, verwenden Sie die Randomize-Anweisung ohne Argument, um den Zufallszahlen-
//         generator mit einem Startwert zu initialisieren, der auf dem Systemzeitgeber basiert.
//         Verwenden Sie die folgende Formel, um ganzzahlige Zufallszahlen innerhalb eines bestimmten Bereichs
//         zu erzeugen:
//            Int((Obergrenze - Untergrenze + 1) * Rnd + Untergrenze)
//         Obergrenze steht fr die grte und Untergrenze fr die kleinste Zahl des Bereichs.
// Return Value:
//     Zufallszahl
// Errorhandling:
//     Null als Parameter -> B_InvalidNull
// Notes:  
//     Wenn Sie Folgen von Zufallszahlen wiederholen mchten, rufen Sie Rnd mit einem negativen Argument auf, 
//     unmittelbar bevor Sie Randomize mit einem numerischen Argument verwenden. Durch Verwenden von Randomize
//     mit demselben Wert fr Zahl wird die vorherige Folge nicht wiederholt.
//     Empty und Boolsche Werte sind erlaubt.
#ifdef B_VB_COMPATIBLE

DLL_DECL B_Single B_rnd(const B_Single& = B_Single(1.0F, B_explicit));
DLL_DECL B_Single B_rnd(const B_Null&) throw(B_InvalidNull);

#else // B_VB_COMPATIBLE

DLL_DECL B_Flt B_rnd(const B_Flt = 1.0F);
DLL_DECL B_Flt B_rnd(const B_Null&) throw(B_InvalidNull);

#endif //B_VB_COMPATIBLE

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function:                                                                             (C) CiceroSoft 1996
//     B_sqr()   gibt die Quadratwurzel einer Zahl zurueck
// See also:
// Parameter:
//     radikant
//         bel. numerischer Ausdruck >= 0
// Return Value:
//     Wurzel des Radikanten
// Errorhandling:
// Notes:  
//     Parameter               Ruckegabewert
//     Null                    Null
//     Empty, Bool, Integer    Integer
//     Long                    Long
//     Single, Double          Double
//
#ifdef B_VB_COMPATIBLE

DLL_DECL B_Double B_abs(const B_Double&);

#else // B_VB_COMPATIBLE

DLL_DECL B_Dbl B_abs(const B_Dbl);
DLL_DECL B_Lng B_abs(const B_Lng);

#endif // B_VB_COMPATIBLE

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function:                                                                             (C) CiceroSoft 1996
//     B_sqr()   gibt die Quadratwurzel einer Zahl zurueck
// See also:
// Parameter:
//     radikant
//         bel. numerischer Ausdruck >= 0
// Return Value:
//     Wurzel des Radikanten
// Errorhandling:
//     Null als Parameter -> B_InvalidNull
//     True -> InvalidArgument, da -1
// Notes:  
//     Empty und False erlaubt
#ifdef B_VB_COMPATIBLE

DLL_DECL B_Double B_sqr(const B_Double&) throw(B_InvalidArgument);
DLL_DECL B_Double B_sqr(const B_Null&) throw(B_InvalidNull);

#else // B_VB_COMPATIBLE

DLL_DECL B_Dbl B_sqr(const B_Dbl) throw(B_InvalidArgument);
DLL_DECL B_Dbl B_sqr(const B_Null&) throw(B_InvalidNull);

#endif // B_VB_COMPATIBLE

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function:                                                                             (C) CiceroSoft 1996
//     B_fix()   gibt den ganzahligen Anteil einer Zahl zurueck
// See also:
//     B_int(), B_cInt()
// Parameter:
//     number
//         bel. numerischer Ausdruck 
// Return Value:
// Errorhandling:
// Notes:  
//
DLL_DECL B_Currency B_fix(B_Currency);
DLL_DECL B_Null     B_fix(const B_Null&);

#ifdef B_VB_COMPATIBLE

DLL_DECL B_Double   B_fix(B_Double);
DLL_DECL B_Single   B_fix(B_Single);
DLL_DECL B_Long     B_fix(const B_Long&);
DLL_DECL B_Integer  B_fix(const B_Integer&);
DLL_DECL B_Byte     B_fix(const B_Byte&);
DLL_DECL B_Double   B_fix(const B_String&);
DLL_DECL B_Integer  B_fix(const B_Boolean&); // in jeweiligen Zahlwert
DLL_DECL B_Integer  B_fix(const B_Empty&);   // in 0

#else // B_VB_COMPATIBLE

DLL_DECL B_Dbl  B_fix(B_Dbl);
DLL_DECL B_Flt  B_fix(B_Flt);
DLL_DECL B_Lng  B_fix(const B_Lng);
DLL_DECL B_Int  B_fix(const B_Int);
DLL_DECL B_Dbl  B_fix(const B_String&);
DLL_DECL B_Int  B_fix(const B_Boolean&); // in jeweiligen Zahlwert
DLL_DECL B_Int  B_fix(const B_Empty&);   // in 0

#endif // B_VB_COMPATIBLE


////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function:                                                                             (C) CiceroSoft 1996
//     B_int()   gibt den ganzahligen Anteil einer Zahl zurueck
// See also:
//     B_fix(), B_cInt()
// Parameter:
//     number
//         bel. numerischer Ausdruck 
// Return Value:
// Errorhandling:
// Notes:  
//
DLL_DECL B_Currency B_int(B_Currency);
DLL_DECL B_Null     B_int(const B_Null&);

#ifdef B_VB_COMPATIBLE

DLL_DECL B_Double   B_int(B_Double);
DLL_DECL B_Single   B_int(B_Single);
DLL_DECL B_Long     B_int(const B_Long&);
DLL_DECL B_Integer  B_int(const B_Integer&);
DLL_DECL B_Byte     B_int(const B_Byte&);
DLL_DECL B_Double   B_int(const B_String&);
DLL_DECL B_Integer  B_int(const B_Boolean&); // in jeweiligen Zahlwert
DLL_DECL B_Integer  B_int(const B_Empty&);   // in 0

#else // B_VB_COMPATIBLE

DLL_DECL B_Dbl   B_int(B_Dbl);
DLL_DECL B_Flt   B_int(B_Flt);
DLL_DECL B_Lng   B_int(const B_Lng);
DLL_DECL B_Int   B_int(const B_Int);
DLL_DECL B_Dbl   B_int(const B_String&);
DLL_DECL B_Int   B_int(const B_Boolean&); // in jeweiligen Zahlwert
DLL_DECL B_Int   B_int(const B_Empty&);   // in 0

#endif // B_VB_COMPATIBLE


#ifndef B_VB_COMPATIBLE
DLL_DECL B_Dbl B_Pow(const B_Dbl lhs, const B_Dbl rhs);
#endif // B_VB_COMPATIBLE

//================================ Inlinefunktionen =========================================================
//
#if defined _DEBUG || defined B_VB_CONTRACT
  #define B_NOINLINE
#else
  #undef B_NOINLINE
#endif

#ifndef B_NOINLINE                 // festlegen wenn Debuger keine Inlines debuggen kann
  #define B_INLINE inline  
  #ifndef  BI_MATH_HPP
    #include "bi_math.hpp"
  #endif
#endif

//===========================================================================================================
//================================ End Of Header ============================================================
#endif // BF_MATH_HPP

