#ifndef BF_STR_HPP
#define BF_STR_HPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Project:                 VB nach C/C++ Konverter Klassenbibliothek
// Modul:                   VB Funktionen zur Stringbearbeitung
// Filename:                bf_str.hpp
// Version:                 0.0
// Status:                  alpha
//
// Copyright:               (c)
//                          Cicerosoft GmbH, 
//                          Ballweilerstr. 7,
//                          66399 Erfweiler-Ehlingen
//
// Operating System:        Win95
// Compiler:                VC++ 4.1
//
// Author:                  Jochen Diener JD
// Creation Date:           14.11.95
// Last Modification:       JD 24.08.96
//
// Description:             Beinhaltet alle Klassen / Hilfskt. die im Zusammenhang mit der VB 
//                          Konvertierung, fuer die Bereitstellung des Datentyps String benoetigt 
//                          werden. 
//                          
// Dependencies:            bc_str.hpp    ->  Deklaration der Klasse B_String
//                          bi_str.hpp    ->  Inlinefkt. der Klasse B_String
//                          bc_str.cpp    ->  Implementierung der Member/Hilfsfkt. B_String
//
//
// Remarks:             
//
// Revisions:
//                          
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//==========================================================================================================
//================================= Defintionsabhaengigkeiten ===============================================
//
#ifndef BD_INTYP_HPP       
  #include "bd_intyp.hpp"       // alias der Standardatentypen
#endif

#ifndef BC_NULL_HPP
  #include "bc_null.hpp"
#endif

#ifndef BC_EMPTY_HPP
  #include "bc_empty.hpp"
#endif

#ifndef BC_BOOL_HPP
  #include "bc_bool.hpp"
#endif

#ifndef BC_BYTE_HPP
  #include "bc_byte.hpp"
#endif

#ifndef BC_INT_HPP
  #include "bc_int.hpp"
#endif

#ifndef BC_LONG_HPP
  #include "bc_long.hpp"
#endif

#ifndef BC_SNGLE_HPP
  #include "bc_sngle.hpp"
#endif

#ifndef BC_DBL_HPP
  #include "bc_dbl.hpp"
#endif

#ifndef BC_CURR_HPP
  #include "bc_curr.hpp"
#endif

#ifndef BC_STR_HPP
  #include "bc_str.hpp"
#endif

#include "bc_str.hpp"

//==========================================================================================================
//================================= Export =================================================================
//
//Option Compare Binary ->  A < B < C < a < b < c    Case Sensitiv, Binaere Sortierreihenfolge   
// Option Compare Text  ->  A=a; B=b; C=c            nicht Case Sensitiv, Text Sortierfolge


//================================= Hilfsfunktionen fuer den Konverter =====================================
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_len
//         Bestimmt die Laenge eines Strings und gibt diese zurueck. Bei benutzer-
//         definierten Typen wird die Laenge des Benutzerdefinierten Typs in Byte
//         zuruekgegeben die zum abspeichern desselben benoetigt wird.
// Siehe auch:
//     B_lenB                            
// Parameter: 
//     string:   
//         jeder gueltige Zeichenfolgenausdruck
// Rueckgabewert:
//     Laenge des Strings als Zahl, respektive die Anzahl der Bytes des Benutzerdefinierten
//     Typs
// Bemerkung:  
//     Hat string den Wert Null wird Null zurueckgegeben
//

#ifdef B_VB_COMPATIBLE
DLL_DECL B_Long B_len(const B_String& string);
#else
DLL_DECL B_Lng B_len(const B_String&);
#ifdef THALESPLUS
DLL_DECL B_Lng B_len(const B_TStringStruct);
#endif // THALESPLUS
#endif // B_VB_COMPATIBLE

/*
// B_Long    B_len(const B_CString); // nur aus Performancegruenden sonst ueber Konstruktor von B_String konvertieren
B_Variant B_len(const B_Variant& string);
template<class T> B_Long B_len(T anyVariable);
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_lenB
//         Gibt die Anzahl Bytes zurueck die die Uebergebene Variable hat. (z.B. Laenge des Strings
//         in Byte, groesse einer Benutzerdefinierten Datenstruktur in Byte)
// Siehe auch:  
//     B_len                            
// Parameter: 
//     string:
//         jeder gueltige Zeichenfolgenausdruck
// Rueckgabewert:
// Bemerkung: 
//     Hat string den Wert Null wird Null zurueckgegeben
//
/* 
B_Long    B_lenB(const B_String& string);
// B_Long    B_lenB(const B_CString); 
B_Variant B_lenB(const B_Variant& string);
template<class T> B_Long B_lenB(T anyVariable); 
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_strComp
//         Vergleicht zwei Zeichenketten miteinander. Die Art des Vergleiches kann ueber den 
//         compareType Parameter festgelegt werden
// Siehe auch:                                
// Parameter:
//     string1: 
//         jeder gueltige Zeichenfolgenausdruck
//     string2:  
//         jeder gueltige Zeichenfolgenausdruck
//     compareType:
//         0 = binaer -> der Zeichenfolgenvgl. basiert auf der Sortierreihenfolge, die 
//             durch die interne binaere Darstellung der Zeichen festgelegt ist.
//             Die Sortierreihenfolge wird durch die Code-Seite festgelegt.
//             (case sensitiv)
//         1 = text   -> fuehrt zu einem Zeichenfolgenvgl. bei denen die im Gebietsschema
//             des Systems festgelegte Sortierreihenfolge fuer Zeichen verwendet
//             wird (ohne Unterschiede in der Gro- oder Kleinschreibung zu 
//             beruecksichtigen). 
//         wird compareType nicht explizit angegeben wird die aktuelle Einstellung
//         von Option Compare genommen (Standardeinstellung 0)
//                               
// Rueckgabewert:
//     -1     string1 (<) liegt im vor Alphabet string2
//      0     string1 (=) entspricht string2
//      1     string1 (>) liegt im Alphabet hinter string2
//     Null   string1 oder string2 hat den Wert Null
// Bemerkung:
//     Ob string1 im Alphabet vor oder hinter string2 liegt ist abhaengig vom Parameter 
//     compareType
//
#ifdef B_VB_COMPATIBLE
DLL_DECL B_Integer B_strComp(const B_String& string1,  const B_String& string2, B_Long compare = B_OPTION_COMPARE);
DLL_DECL B_Integer B_strComp(const B_String& string1,  const B_String& string2, B_Lng compare);
#else
DLL_DECL B_Int B_strComp(const B_String& string1, const B_String& string2, B_Lng compare = B_OPTION_COMPARE);
#endif

/* 
B_Variant B_strComp(const B_Variant& string1, const B_Variant& string2, const B_Char& compare=B_CharMax);
B_Variant B_strComp(const B_Variant& string1, const B_String&  string2, const B_Char& compare=B_CharMax);
B_Variant B_strComp(const B_String&  string1, const B_Variant& string2, const B_Char& compare=B_CharMax);
// aus Performancegruenden evtl. CString, ansonsten konvertierung ueber Konstruktor von B_String
// B_String  B_strComp(B_String&  string1, B_CString& string2, B_Char compare=B_CharMax);
// B_String  B_strComp(B_CString& string1, B_String&  string2, B_Char compare=B_CharMax);
*/


///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//    B_strConv
//         Konvertiert einen uebergebenen String in abhaengigkeit vom Parameter convertInType
//         in z.B. Grossbuchstaben oder Unicode und liefert als Rueckgabewert den konvertierten
//         String.  
// Siehe auch:                                
// Parameter: 
//    stringToConvert:
//        der zu konvertierende String. Ist dieser Null liefert die Funktion Null als 
//        Rueckgabewert.   
//    convertInType:
//        legt die Art der Konvertierung fest, Null ist nicht als Wert erlaubt
//            vbUpperCase     1   Zeichenfolge in Grobuchstaben umwandeln
//            vbLowerCase     2   Zeichenfolge in Kleinbuchstaben umwandeln
//            vbProperCase    3   Anfangsbuchstaben in jedem Wort der Zeichenfolge in 
//                                Grobuchstaben umwandeln
//            vbWide	        4   Schmale Zeichen (Einzelbyte) in der Zeichenfolge in 
//                                breite Zeichen (Doppelbyte) umwandeln
//            vbNarrow 	      8   Breite Zeichen (Doppelbyte) in der Zeichenfolge in 
//                                schmale Zeichen (Einzelbyte) umwandeln
//            vbUnicode	     64   Zeichenfolge unter Verwendung der Standard-Codeseite
//                                des Systems in Unicode umwandeln
//            vbFromUnicode 128   Zeichenfolge unter Verwendung der Standard-Codeseite 
//                                des Systems aus Unicode umwandeln
//
// Rueckgabewert:  
//     Ein String der den konvertierten stringToConvert beinhaltet
// Bemerkung: 
//     Die Konstanten vbKatakana und vbHiragana werden vorerst nicht implementiert
//     Konstanten koennen Kombiniert werden wenn die Kombination nicht widerspruechlich ist 
//     z.B. vbUpperCase + vbLowerCase
//     Die folgenden Zeichen sind zur Trennung von Woertern zulaessig: Null (Chr$(0)), 
//     horizontaler Tabulator (Chr$(9)), Zeilenumbruch (Chr$(10)), vertikaler Tabulator (Chr$(11)),
//     Seitenvorschub (Chr$(12)), Wagenruecklauf (Chr$(13)),Leerzeichen (Einzelbyte) (Chr$(32)). 
//     Der tatsaechliche Wert fuer ein Leerzeichen haengt bei Doppelbyte-Zeichensaetzen  (DBCS)
//     vom jeweiligen Land ab.
//
/*
B_String  B_strConv(const B_String&  stringToConvert, const B_Integer& convertInType);
   // nur wenn Performance Probleme sonst Konstruktor von B_String(char*)
// B_String  B_strConv(B_CString  stringToConvert, B_Integer convertInType);
   // nicht wegen Null sonder fuer Variant(string) d.h nur wenn Performance Probleme sonst Konvertierungsoperator 
   // von B_Variant moeglich da kein Null erlaubt
// B_Variant B_strConv(B_Variant& stringToConvert, B_Integer convertInType); // 
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_str/B_strS 
//          Konvertiert einen numerischen Wert in einen String der von der Funktion zurueckgegeben 
//          wird. Im Unterschied zu B_Str hat B_strS hat als Rueckgabewert einen B_String und keinen 
//          B_Variant. 
// Siehe auch:                                
// Parameter:
//     numberToConvert:
//          der numerische Wert der in eine string konvertiert werden soll.
//          Ist der Parameter Null wird ein Leerstring zurueckgegeben es erfolgt
//          in VB keine Fehlermeldung bei uns wird eine Warnung ausgegeben.
// Rueckgabewert:
//     Ein String der die konvertierte numberToConvert beinhaltet
// Bemerkung:
//     B_str erkennt nur den Punkt (.) als zulaessiges Dezimalzeichen. Wenn die Moeglichkeit zur 
//     Verwendung eines anderen Dezimalzeichens bestehen mu (zum Beispiel in internationalen 
//     Anwendungen), sollten Sie Zahlen mit der CStr-Funktion in eine Zeichenfolge umwandeln.
//     Innerhalb von Zeichenfolgen gelten die Einstellungen der Systemsteuerung.
//     Beim Umwandeln von Zahlen in Zeichenfolgen wird immer ein fuehrendes Leerzeichen fuer das 
//     Vorzeichen von Zahl reserviert. Ist Zahl positiv, so enthaelt die zurueckgegebene 
//     Zeichenfolge ein fuehrendes Leerzeichen (das positive Vorzeichen ist implizit).
/* 
B_Variant B_str(const B_Variant&  numberToConvert);
B_String  B_strS(const B_Variant& numberToConvert);
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_asc
//          Liefert den Zeichencode zurueck den das erste Zeichen im Parameter string entspricht.
//          Ist string keine gueltige Zeichenkette (insb. Null) fuehrt dies zu einem Laufzeitfehler.
// Siehe auch:  
//     B_ascB, B_ascW                           
// Parameter: 
//     string: 
//          Zeichenkette deren Zeichencode des ersten Buchstabens zurueckgegeben wird
// Rueckgabewert: 
//     Der Zeichencode des ersten Buchstabens von string
// Bemerkung:      
//     Zahldatentypen und Boolsche Datentypen werden in Strings konvertiert. Null == LZF(94), Empty == LZF(5)
#ifdef B_VB_COMPATIBLE
DLL_DECL B_Integer B_asc(const B_String&  string);
DLL_DECL B_Integer B_asc(B_cCStr);
#else	//ab.. 5.2.97 Rueckgabe geaendert von B_Int nach B_UChar
DLL_DECL B_UChar B_asc(const B_String& string);
DLL_DECL B_UChar B_asc(B_cCStr);
#ifdef THALESPLUS
DLL_DECL B_UChar B_asc(const B_TStringStruct);
#endif // THALESPLUS
#endif

// B_Integer B_asc(const B_Empty&) throw(B_InvalidArgument)
// B_Integer B_asc(const B_Null&) throw(B_InvalidNull);
/*
// B_Integer B_asc(const B_CString  string); // nur wenn Performance Probleme sonst Konstruktor von B_String(char*)
// B_Integer B_asc(const B_Variant& string); // nur wenn Performance Probleme sonst Konvertierungsoperator von B_Variant
                                             // moeglich da kein Null erlaubt
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_ascB 
//          Liefert das erste Byte zurueck das der Parameter string enthaelt
//          Ist string keine gueltige Zeichenkette (insb.Null) fuehrt dies zu einem Laufzeitfehler.
// Siehe auch:
//     B_asc, B_ascW                           
// Parameter: 
//     string:  
//          Zeichenkette deren erstes Byte zurueckgeliefert wird
// Rueckgabewert:
//     Das erste Byte der Zeichenkette string
// Bemerkung:      
/*
B_Byte B_ascB(const B_String&  string);
// B_Byte B_ascB(const B_CString  string);
// B_Byte B_ascB(const B_Variant& string);
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_ascW
//          Liefert 'native' (=Laenderspezifischen) Code des ersten Zeichens im String. D.h dass sich
//          z.B. Unicode und ANSI unterscheiden 
//          Ist string keine gueltige Zeichenkette (insb.Null) fuehrt dies zu einem Laufzeitfehler.
// Siehe auch:  
//     B_asc, B_ascB                           
// Parameter: 
//     string: 
//          Zeichenkette deren erstes Zeichen zurueckgeliefert wird
// Rueckgabewert:  
//     Das erste native Zeichen der Zeichenkette string
// Bemerkung:      
/*
B_Integer B_ascW(const B_String&  string);
// B_Integer B_ascW(const B_CString  string); 
// B_Integer B_ascW(const B_Variant& string); 
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_chr
//         Gibt das Zeichen fuer den angegebenen charcode (Zeichencode) im (ANSI)Zeichensatz zurueck
// Siehe auch: 
//     B_chrB, B_chrW                           
// Parameter:  
//     charcode:
//         Zahl die dem Zeichencode=Charcode des gesuchten zeichens entspricht
// Rueckgabewert:
//     Ein 1Zeichenstring der das durch charcode spezifizierte Zeichen enthaelt
// Bemerkung: 
//     Die Zahlen von 0 bis 31 entsprechen den (nicht darstellbaren) Standard-ASCII-Codes. 
//     Chr(10) gibt zum Beispiel ein Zeilenvorschubzeichen zurueck. 
//     Trotz Long Datentyp und Unicde nur bis 255 erlaubt. Zahlstrings sind erlaubt.
#ifdef B_VB_COMPATIBLE

DLL_DECL B_String B_chrS(B_Long charcode); // reicht das aus fuer Zahlstringkonv. ? oder eigene Ueberladung
DLL_DECL B_String B_chrS(B_Lng charcode);

#else

DLL_DECL B_String B_chrS(B_ULng charcode);

#endif
// B_String B_chr(const B_Boolean&) throw(B_InvalidArgument);
// B_String B_chr(const B_Null&) throw(B_InvalidNull);


///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_chr
//         Gibt einen String zurueck der das einzelene, mit charcode, spezifizierte Byte enthaelt
// Siehe auch:  
//     B_chr, B_chrW                           
// Parameter: 
//     charbyte:
//         Zahl die dem Byte entspricht, das in der Zeichenkette zurueckgegeben wird
// Rueckgabewert: 
//     Ein 1Zeichenstring der das durch charcode spezifizierte Zeichen enthaelt
// Bemerkung:      
/*
B_String B_chrB(const B_Byte& charbyte);
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_chrW 
//          Gibt einen String zurueck der das 'native' = Zeichen im Landescode (ANSI/Unicode) 
//          zurueckgibt
// Siehe auch:
//     B_chr, B_chrB                           
// Parameter: 
//     charcode:
//         Zahl die dem native Zeichen entpsricht das in der Zeichenkette zurueckgegeben wird
// Rueckgabewert:
//     Ein 1Zeichenstring der das durch charcode spezifizierte Zeichen enthaelt
// Bemerkung:      
/*
B_String B_chrW(const B_Long& charcode);
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_lSet
//         Fuegt eine einzufuegende Zeichenkette=insString linksbuendig in den Zielzeichenkette=
//         destString ein.
//         Ist insString laenger als destString, so werden in destString nur die ersten Zeichen von
//         insString(von links) bis zur Laenge von destString eingefuegt.
// Siehe auch:
//     B_rSet                       
// Parameter: 
//     insString:
//         Zeichenkette deren Zeichen linksbuendig in destString eingefuegt werden
//     destString:
//         Zeichenkette in die insString eingefuegt wird
// Rueckgabewert:
// Bemerkung:  
//     Null ist fuer keinen der Parameter erlaubt.
//     Ist ein Parameter ein Variatn mit numerischem Untertyp ist wird eine LAufzeitfehlermeldung
//     ausgegeben.
//     Bei uebergabe von Benutzerdefinierten Typen an die Funktion B_lSet wird eine Flatcopy
//     (=byte fuer byte Kopie) druchgefuehrt. ACHTUNG: Pruefen sie sehr genau welche Folgen
//     dies haben kann.
DLL_DECL void B_lSet(B_String& destString,  const B_String& insString);

#ifdef THALESPLUS
DLL_DECL void B_lSet(B_TStringStruct destString,  const B_TStringStruct insString);
DLL_DECL void B_lSet(B_TStringStruct destString,  const B_String& insString);
DLL_DECL void B_lSet(B_String& destString,  const B_TStringStruct insString);
#endif // THALESPLUS

/*
void B_lSet(B_Variant& destString, const B_Variant& insString); 
template<class T> void B_lSet(T destVariable, const T insVariable);
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_rSet
//         Fuegt eine einzufuegende Zeichenkette=insString rechtsbuendig in den Zielzeichenkette=
//         destString ein.
//         Ist insString laenger als destString, so werden in destString nur die ersten Zeichen von
//         insString(von links) bis zur Laenge von destString eingefuegt.
// Siehe auch:
//     B_lSet                       
// Parameter:
//     insString:
//         Zeichenkette deren Zeichen rechtsbuendig in destString eingefuegt werden
//         destString:
//         Zeichenkette in die insString eingefuegt wird
// Rueckgabewert:
// Bemerkung: 
//     Null ist fuer keinen der Parameter erlaubt.
//     Ist ein Parameter ein Variatn mit numerischem Untertyp ist wird eine LAufzeitfehlermeldung
//     ausgegeben.
//     Es koennen keine Benutzerdefinierten Typen uebergeben werden wie bei B_lSet
DLL_DECL void B_rSet(B_String& destString,  const B_String& insString);

#ifdef THALESPLUS
DLL_DECL void B_rSet(B_TStringStruct destString,  const B_TStringStruct insString);
DLL_DECL void B_rSet(B_String& destString,  const B_TStringStruct insString);
DLL_DECL void B_rSet(B_TStringStruct destString,  const B_String& insString);
#endif // THALESPLUS

/*
void B_rSet(B_Variant& destString, const B_Variant& insString);
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_inStr  
//            Die Funktion sucht eine Zeichenkette=searchForString einer anderen Zeichenkette=
//                 searchIntoString
// Siehe auch: 
//     B_like             
// Parameter:
//     searchIntoString:
//         Die Zeichenkette in der searchForString gesucht wird
//     searchForString: 
//         Die Zeichenkette die in searchIntoString gesucht wird  
//     startPos: 
//         Legt das Zeichen in searchIntoString fest ab dem searchForString gesucht wird. Wird 
//         startPos nicht festgelegt wird ab dem ersten Zeichen in searchIntoString mit der 
//         Suche begonnen. 
//     compareType: 
//         legt fest wie gesucht wird 
//         0 = textueller Vergleich
//         1 = textueller Vergleich ohne beruecksichtigung der Gross/Kleinschreibung       
// Rueckgabewert: 
//     Der Rueckgabewert gibt auskunft ueber das Ergebnis der Suche
//         searchIntoString hat die Laenge Null               ->  0
//         searchIntoString ist Null	                        ->  Null
//         searchForString  hat die Laenge Null               ->  start
//         searchForString ist Null	                          ->  Null
//         searchForString ist nicht vorhanden                ->  0
//         searchForString ist in searchIntoString enthalten  ->  Position, an der Uebereinstimmung beginnt
//        start > string2	                                    ->  0
// Bemerkung:   
//     Es werden keine Matchcodes gesucht vgl. B_like.
//     Das erste Zeichen bei startPos bedeutet = 1 (nicht wie in C/C++ ueblich mit 0)
//     startPos und compareType duerfen nicht Null sein -> Laufzeitfehler
#ifdef B_VB_COMPATIBLE
DLL_DECL B_Long B_inStr(const B_String& searchIntoString,  const B_String& searchForString, B_Long startPos = B_Long(1,B_explicit), B_Byte compareType = B_Byte(0, B_explicit));
DLL_DECL B_Long B_inStr(B_cCStr searchIntoString,  const B_String& searchForString, B_Long startPos = B_Long(1,B_explicit), B_Byte compareType = B_Byte(0, B_explicit));
DLL_DECL B_Long B_inStr(const B_String& searchIntoString,  B_cCStr searchForString, B_Long startPos = B_Long(1,B_explicit), B_Byte compareType = B_Byte(0, B_explicit));
#else
DLL_DECL B_Lng B_inStr(const B_String& searchIntoString,  const B_String& searchForString, B_Lng startPos = 1, B_UChar = 0);
DLL_DECL B_Lng B_inStr(B_cCStr searchIntoString,  const B_String& searchForString, B_Lng startPos = 1, B_UChar = 0);
DLL_DECL B_Lng B_inStr(const B_String& searchIntoString,  B_cCStr searchForString, B_Lng startPos = 1, B_UChar = 0);

#ifdef THALESPLUS
DLL_DECL B_Lng B_inStr(const B_TStringStruct searchIntoString,  const B_TStringStruct searchForString, B_Lng startPos = 1, B_UChar = 0);
DLL_DECL B_Lng B_inStr(const B_String&       searchIntoString,  const B_TStringStruct searchForString, B_Lng startPos = 1, B_UChar = 0);
DLL_DECL B_Lng B_inStr(const B_TStringStruct searchIntoString,	const B_String&       searchForString, B_Lng startPos = 1, B_UChar = 0);
DLL_DECL B_Lng B_inStr(      B_cCStr		 searchIntoString,	const B_TStringStruct searchForString, B_Lng startPos = 1, B_UChar = 0);
DLL_DECL B_Lng B_inStr(const B_TStringStruct searchIntoString,        B_cCStr		  searchForString, B_Lng startPos = 1, B_UChar = 0);
#endif // THALESPLUS

#endif
/* 
B_Variant B_inStr(const B_Variant& searchIntoString,  const B_Variant& searchForString, const B_Long& startPos = 1, const B_Byte& compareType = 0);
B_Variant B_inStrB(const B_Variant& searchIntoString, const B_Variant& searchForString, const B_Long& startPos = 1, const B_Byte& compareType = 0);
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_space / B_spaceS
//         Fuellt eine String mit einer angegebenen Anzahl=nbrSpaces Leerzeichen=Spaces 
//         Wird B_space() einer Zeichenkette fester Laenge zugewiesen wird diese immer komplett mit 
//         Leerzeichen gefuellt, egal ab der Wert anzSpaces groesser oder kleiner der Laenge der
//         fixen Zeichenkette ist. Es wird hierbei von VB keine Fehlermeldung ausgegeben.
//         Im Unterschied zu B_space und B_spaceS liegt darin, dass B_space eine Zeichenkette vom
//         Typ B_Variant und B_stringS eine Zeichenkette vom Typ B_String zurueckgibt.
// Siehe auch:    
//     B_string / B_stringS            
// Parameter:  
//     nbrSpaces:
//         Anzahl Leerezeichen aus denen die Rueckgabezeichenkette bestehen soll
// Rueckgabewert:
//     Eine Zeichenkette die nbrSpaces Leerezeichen enthaelt
// Bemerkung:  
//     Die Null als Wert fuer den Parameter nbrSpaces fuehrt zu einem Laufzeitfehler
/*
B_Variant B_space(const B_Long& nbrSpaces);
//-----------------------------------------------------------------------------------------------------------
B_String B_spaceS(const B_Long& nbrSpaces);
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_string / B_stringS 
//          Gibt eine Zeichenfolge in der angegebenen Laenge=nbrCharacters mit einem sich wieder-
//          holenden Zeichen=charcode zurueck.
//          Wird das Ergebnis einer Zeichenkette fester Laenge zugewiesen und die generierten 
//          Zeichenkette ist kleiner als diese, so wird mit Leerzeichen aufgefuellt. Ist die 
//          Anzahl nbrCharacters groesser so wird abgeschnitten auf die fixe Laenge der Zielzeichenkette.
// Siehe auch:   
//     B_space / B_spaceS
// Parameter:   
//     nbrCharacters:
//         Die Anzahl wie oft das Zeichen charcode in der zurueckgegebenen Zeichen-
//         kette enthalten sein soll
//     charcode:  
//         Wenn ein numerischer Wert angegeben wird, dann wird das entsprechende Zeichen aus 
//         dem Zeichensatz genommen. Ist der numerische Wert groesser als die Zeichencodegroesse 
//         dann wird eine Modulo-Operation ausgefuehrt. Ist charcode eine Zeichenkette wird das
//         erste Zeichen der Zeichenkette als Fuellzeichen verwandt.
// Rueckgabewert: 
//     Eine Zeichenkette mit der Anzahl nbrCharacters gefuellten Zeichen vom Typ charcode 
// Bemerkung:  
//    Fuer nbrCharacter ist keine Null erlaubt. Fuer charcode nur bei B_string, nicht bei B_stringS.
//    Wenn eine Modulo-Operation beim charcode Parameter durchgefuehrt wird geben wir im Gegensatz zu VB eine
//    Warnung aus
#ifdef B_VB_COMPATIBLE

DLL_DECL B_String B_stringS(const B_Long& nbrCharacters, const B_Byte& charcode);
DLL_DECL B_String B_stringS(const B_Long& nbrCharacters, B_cCStr charcode);

#else // B_VB_COMPATIBLE

DLL_DECL B_String B_stringS(const B_Lng nbrCharacters, B_cCStr charcode);

#ifdef THALESPLUS
DLL_DECL B_String B_stringS(const B_Lng nbrCharacters, const B_TStringStruct charcode);
#endif // THALESPLUS

#endif // B_VB_COMPATIBLE

DLL_DECL B_String B_stringS(B_Lng nbrCharacters, B_Shrt charcode);

/*
B_Variant B_string(const B_Long& nbrCharacters, const B_Variant& charcode);
//-----------------------------------------------------------------------------------------------------------
B_String B_stringS(const B_Long& nbrCharacters, const B_Variant& charcode);
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_lCase / B_lCaseS 
//         Gibt eine Zeichenfolge zurueck, in der alle Buchstaben in Kleinbuchstaben umgewandelt 
//         worden sind. Lediglich Grossbuchstaben werden in Kleinbuchstaben umgewandelt. Alle 
//         Kleinbuchstaben und sonstigen Zeichen bleiben unveraendert.
// Siehe auch:  
//     B_uCase / B_uCaseS
// Parameter: 
//     stringToLower: 
//         Zeichenkette die in Kleinbuchstaben umgewandelt werden soll.
// Rueckgabewert:  
// Bemerkung:   
//     Wird Null fuer strinToLower uebergeben so wird bei B_lCase Null zurueckgegeben und bei
//     B_lCaseS eine Fehlermeldung.
DLL_DECL B_String B_lCaseS(const B_String& stringToLower);
DLL_DECL B_String B_lCaseS(		 B_cCStr   stringToLower);

#ifdef THALESPLUS
DLL_DECL B_String B_lCaseS(const B_TStringStruct stringToLower);
#endif // THALESPLUS

/*
B_Variant B_lCase(const B_Variant& stringToLower);
//-----------------------------------------------------------------------------------------------------------
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_uCase / B_uCaseS 
//         Gibt eine Zeichenfolge zurueck, in der alle Buchstaben in Grossbuchstaben umgewandelt 
//         worden sind. Lediglich Kleinbuchstaben werden in Grossbuchstaben umgewandelt. Alle 
//         Grossbuchstaben und sonstigen Zeichen bleiben unveraendert.
// Siehe auch:
//     B_lCase / B_lCaseS
// Parameter: 
//     stringToUpper:
//         Zeichenkette die in Grossbuchstaben umgewandelt werden soll.
// Rueckgabewert:  
// Bemerkung:  
//     Wird Null fuer strinToUpper uebergeben so wird bei B_uCase Null zurueckgegeben und bei
//     B_uCaseS eine Fehlermeldung.
DLL_DECL B_String B_uCaseS(const B_String& stringToUpper);
DLL_DECL B_String B_uCaseS(		 B_cCStr   stringToLower);

#ifdef THALESPLUS
DLL_DECL B_String B_uCaseS(const B_TStringStruct stringToLower);
#endif // THALESPLUS

/*
B_Variant B_uCase (const B_Variant& stringToUpper);
//-----------------------------------------------------------------------------------------------------------
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_left / B_leftS
//         Kopiert aus einer Zeichenkette=cutString die Anzahl=nbrCutCharacters Zeichen, von links
//         in cutString beginnend, und gibt den so erzeugten Teilstring als Rueckgabewert zurueck.
//         Zeichenketten fixer leange, denen der Rueckgabewert zugewiesen wird werden mit Leerzeichen 
//         aufgefuellt oder abgeschnitten wenn nbrCutCharacters zu klein/gross gewaehlt wurde.
//         Der Unterschied von B_left und B_leftS liegt im Rueckgabewert und dass B_leftS keine
//         Null als Parameter akzeptiert.
// Siehe auch:   
//     B_right / B_rightS
// Parameter:   
//     cutString:  
//         Zeichenfolgenausdruck, aus dem die ersten nbrCutCharacters (linken) Zeichen 
//         zurueckgegeben werden. Wenn cutString den Wert Null enthaelt, wird Null zurueckgegeben. 
//     nbrCutCharacters:
//         Numerischer Ausdruck, der die Anzahl der zurueckzugebenden Zeichen angibt. 0 fuehrt zur
//         Rueckgabe einer Null-Zeichenfolge. Ist nbrCutCharacters groeer oder gleich der Zeichen-
//         anzahl in cutString, so wird die gesamte Zeichenfolge zurueckgegeben.
// Rueckgabewert:  
//     Zeichenkette die die aus cutString kopierten Zeichen enthaelt
// Bemerkung:     
/*
B_Variant B_left(const B_Variant& cutString, const B_Long& nbrCutCharacters);
//-----------------------------------------------------------------------------------------------------------
*/
#ifdef B_VB_COMPATIBLE
DLL_DECL B_String B_leftS(const B_String& cutString, B_Long nbrCutCharacters);

#else // B_VB_COMPATIBLE

#ifdef THALESPLUS
DLL_DECL B_String B_leftS(const B_TStringStruct cutString, B_Lng nbrCutCharacters);
#endif // THALESPLUS

#endif // B_VB_COMPATIBLE

DLL_DECL B_String B_leftS(const B_String& cutString, B_Lng nbrCutCharacters);



///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_leftB
//         Kopiert aus einer Zeichenkette=cutString die Anzahl=nbrCutBytes Bytes, von links in
//         cutString beginnend, und gibt den so erzeugten Teilstring als Rueckgabewert zurueck.
// Siehe auch: 
//     B_rightB
// Parameter: 
//     cutString:
//         Zeichenfolgenausdruck, aus dem die ersten nbrCutBytes (linken) Bytes zurueckgegeben 
//         werden. Wenn cutString den Wert Null enthaelt, wird Null zurueckgegeben. 
//     nbrCutBytes: 
//         Numerischer Ausdruck, der die Anzahl der zurueckzugebenden Bytes angibt. 0 fuehrt zur
//         Rueckgabe einer Null-Zeichenfolge. Ist nbrCutBytes groeer oder gleich der Byteanzahl 
//         in cutString, so wird die gesamte Zeichenfolge zurueckgegeben.
// Rueckgabewert: 
//     Zeichenkette die die aus cutString kopierten Bytes besteht
// Bemerkung:     
/*
B_Variant B_leftB(const B_Variant& cutString, const B_Long& nbrCutBytes);
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_right / B_rightS
//         Kopiert aus einer Zeichenkette=cutString die Anzahl=nbrCutCharacters Zeichen, von rechts
//         in cutString beginnend, und gibt den so erzeugten Teilstring als Rueckgabewert zurueck.
//         Zeichenketten fixer laenge, denen der Rueckgabewert zugewiesen wird werden mit Leerzeichen 
//         aufgefuellt oder abgeschnitten wenn nbrCutCharacters zu klein/gross gewaehlt wurde.
//         Der Unterschied von B_right und B_rightS liegt im Rueckgabewert und dass B_right keine
//         Null als Parameter akzeptiert.
// Siehe auch:
//     B_left / B_leftS
// Parameter:    
//     cutString:  
//         Zeichenfolgenausdruck, aus dem die ersten nbrCutCharacters (rechten) Zeichen 
//         zurueckgegeben werden. Wenn cutString den Wert Null enthaelt, wird Null zurueckgegeben. 
//     nbrCutCharacters:
//         Numerischer Ausdruck, der die Anzahl der zurueckzugebenden Zeichen angibt. 0 fuehrt zur
//         Rueckgabe einer Null-Zeichenfolge. Ist nbrCutCharacters groeer oder gleich der Zeichen-
//         anzahl in cutString, so wird die gesamte Zeichenfolge zurueckgegeben.
// Rueckgabewert: 
//     Zeichenkette die die aus cutString kopierten Zeichen enthaelt
// Bemerkung:     
/*
B_Variant B_right(const B_Variant& cutString,  const B_Long& nbrCutCharacters);
//-----------------------------------------------------------------------------------------------------------
*/
#ifdef B_VB_COMPATIBLE
DLL_DECL B_String B_rightS(const B_String& cutString, B_Long nbrCutCharacters);

#else // B_VB_COMPATIBLE

#ifdef THALESPLUS
DLL_DECL B_String B_rightS(const B_TStringStruct cutString, B_Lng nbrCutCharacters);
#endif // THALESPLUS

#endif // B_VB_COMPATIBLE

DLL_DECL B_String B_rightS(const B_String& cutString, B_Lng nbrCutCharacters);




///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_rightB 
//         Kopiert aus einer Zeichenkette=cutString die Anzahl=nbrCutBytes Bytes, von rechts in
//         cutString beginnend, und gibt den so erzeugten Teilstring als Rueckgabewert zurueck.
// Siehe auch:   
//     B_leftB
// Parameter:  
//     cutString:
//         Zeichenfolgenausdruck, aus dem die ersten nbrCutBytes (rechts) Bytes zurueckgegeben 
//         werden. Wenn cutString den Wert Null enthaelt, wird Null zurueckgegeben. 
//     nbrCutBytes: 
//         Numerischer Ausdruck, der die Anzahl der zurueckzugebenden Bytes angibt. 0 fuehrt zur
//         Rueckgabe einer Null-Zeichenfolge. Ist nbrCutBytes groeer oder gleich der Byteanzahl 
//         in cutString, so wird die gesamte Zeichenfolge zurueckgegeben.
// Rueckgabewert: 
//     Zeichenkette die die aus cutString kopierten Bytes besteht
// Bemerkung:     
/*
B_Variant B_rightB(const B_Variant& cutString, const B_Long& nbrCutBytes);
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_mid / B_midS
//         Die B_mid Funktione gibt ab der Position startPos die Anzahl cutLength Zeichen aus der
//         Zeichenkette cutString in einer Zeichenkette als Rueckgabewert zurueck.
// Siehe auch:    
// Parameter: 
//     cutString:
//         Zeichenkette aus der ein Teilstring extrahiert werden soll. Wird Null an B_mid ueber-
//         geben so wird Null zurueckgegeben. Eine uebergabe von Null an B_midS ist nicht
//         moeglich -> Laufzeitfehler.
//     startPos:
//         Position des Zeichens in cutString ab dem der Teilstring extrahiert wird. 
//     cutLength:
//         Anzahl der Zeichen die ab startPos aus cutString rauskopiert werden sollen. Wird
//         cutLength nicht angegeben wird bis zum Ende von cutString kopiert. Ist cutLength 
//         groesser als die maximale Stringlaenge von cutString wird bis zum ende von cutString
//         ohne Fehlermeldung kopiert.
// Rueckgabewert:
//     Zeichenkette die eine Kopie der aus cutString extrahierten Zeichen enthaelt.
// Bemerkung:  
//     Fuer cutLength und startPos ist keine Null als Uebergabewert erlaubt.
/*
B_Variant B_mid(const B_Variant& cutString,  const B_Long& startPos, const B_Long& cutLength = 0);
//-----------------------------------------------------------------------------------------------------------
*/
#ifdef B_VB_COMPATIBLE

DLL_DECL B_String B_midS(const B_String&  cutString,  B_Long startPos, B_Long cutLength = B_Integer(0,B_explicit));

#else

#ifdef THALESPLUS
DLL_DECL B_String B_midS(const B_TStringStruct  cutString,  B_Lng startPos, B_Lng cutLength = 0);
#endif // THALESPLUS

#endif

DLL_DECL B_String B_midS(const B_String&  cutString, B_Lng startPos, B_Lng cutLength = 0);



///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_midB
//          Die midB Funktion steht fuer Byte-Daten zur Verfuegung, die in einer Zeichenfolge enthalten
//          sind. Die Argumente geben nicht die Anzahl der Zeichen an, sondern die Anzahl=nbrCutBytes
//          der Bytes die von der Position startPos an aus cutString herauskopiert werden.
// Siehe auch:    
// Parameter: 
//     cutString:
//         Zeichenkette aus der ein Bytekette extrahiert werden soll. Wird Null an B_midB ueber-
//         geben so wird Null zurueckgegeben. 
//     startPos:
//         Position des Bytes in cutString ab dem die Bytekette extrahiert wird. 
//     nbrCutBytes:
//         Anzahl der Bytes die ab startPos aus cutString rauskopiert werden sollen. Wird
//         nbrCutBytes nicht angegeben wird bis zum Ende von cutString kopiert. Ist nbrCutBytes 
//         groesser als die maximale Bytelaenge von cutString wird bis zum ende von cutString
//         ohne Fehlermeldung kopiert.
// Rueckgabewert:
//     Zeichenkette die eine Kopie der aus cutString extrahierten Bytes enthaelt.
// Bemerkung:
//     Fuer nbrCutBytes und startPos ist keine Null als Uebergabewert erlaubt.
/*
B_Variant B_midB(const B_Variant& cutString, const B_Long& startPos, const B_Long& nbrCutBytes = 0);
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_lTrim / B_lTrimS
//         Gibt eine Kopie der Zeichenfolge stringToTrim zurueck, die keine fuehrenden Leerzeichen
//         enthaelt.
//         Wird Null angegeben wird Null zurueckgegeben. Null darf nur an B_lTrim uebergeben werden.
//         Bei Uebergabe von Null an B_lTrimS fuehrt dies zu einem Laufzeitfehler
// Siehe auch: 
//     B_rTrim[S] / B_trim[S]
// Parameter:   
//     stringToTrim:
//         Zeichenkette bei der die fuehrenden Leerzeichen entfernt werden sollen
// Rueckgabewert: 
//     Eine Kopie der Zeichenkette stringToTrim die keine fuehrenden Leerzeichen mehr enthaelt
// Bemerkung:     
/*
B_Variant B_lTrim (const B_Variant& stringToTrim);
//-----------------------------------------------------------------------------------------------------------
*/
DLL_DECL B_String  B_lTrimS(const B_String&  stringToTrim);

#ifdef THALESPLUS
DLL_DECL B_String  B_lTrimS(const B_TStringStruct stringToTrim);
#endif // THALESPLUS

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_rTrim / B_rTrimS
//         Gibt eine Kopie der Zeichenfolge stringToTrim zurueck, die keine nachfolgenden Leerzeichen
//         enthaelt.
//         Wird Null angegeben wird Null zurueckgegeben. Null darf nur an B_rTrim uebergeben werden.
//         Bei Uebergabe von Null an B_rTrimS fuehrt dies zu einem Laufzeitfehler
// Siehe auch:
//     B_lTrim[S] / B_trim[S]
// Parameter:  
//     stringToTrim:
//         Zeichenkette bei der die nachfolgenden Leerzeichen entfernt werden sollen
// Rueckgabewert: 
//     Eine Kopie der Zeichenkette stringToTrim die keine nachfolgenden Leerzeichen mehr enthaelt
// Bemerkung:     
/*
B_Variant B_rTrim (const B_Variant stringToTrim);
//-----------------------------------------------------------------------------------------------------------
*/
DLL_DECL B_String  B_rTrimS(const B_String& stringToTrim);

#ifdef THALESPLUS
DLL_DECL B_String  B_rTrimS(const B_TStringStruct stringToTrim);
#endif // THALESPLUS


///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_trim / B_trimS 
//         Gibt eine Kopie der Zeichenfolge stringToTrim zurueck, die weder fuehrende noch 
//         nachfolgenden Leerzeichen enthaelt.
//         Wird Null angegeben wird Null zurueckgegeben. Null darf nur an B_trim uebergeben werden.
//         Bei Uebergabe von Null an B_trimS fuehrt dies zu einem Laufzeitfehler
// Siehe auch: 
//     B_lTrim[S] / B_rTrim[S]
// Parameter: 
//     stringToTrim:
//         Zeichenkette bei der die fuehrenden und nachfolgenden Leerzeichen entfernt werden sollen
// Rueckgabewert:
//     Eine Kopie der Zeichenkette stringToTrim die keine fuehrenden und nachfolgenden Leerzeichen mehr enthaelt
// Bemerkung:     
/*
B_Variant B_trim  (const B_Variant& stringToTrim);
//-----------------------------------------------------------------------------------------------------------
*/
DLL_DECL B_String B_trimS(const B_String&  stringToTrim);

#ifdef THALESPLUS
DLL_DECL B_String  B_trimS(const B_TStringStruct stringToTrim);
#endif // THALESPLUS

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_midncpy
//         Kopiert B_String/B_TStringStruct-Quelle auf bestimmte Stelle im B_String/B_TStringStruct-Ziel
// Parameter: 
// Rueckgabewert:
// Bemerkung:     
/*
//-----------------------------------------------------------------------------------------------------------
*/
DLL_DECL void B_midncpy(B_String&  zielstring, B_Int platz, B_Int anzahl, const B_String& quellstring);
DLL_DECL void B_midncpy(B_String&  zielstring, B_Int platz, B_Int anzahl, const B_cCStr& quellstring);
#ifdef THALESPLUS
DLL_DECL void B_midncpy(B_String&  zielstring, B_Int platz, B_Int anzahl, const B_TStringStruct quellstring);
DLL_DECL void B_midncpy(B_TStringStruct  zielstring, B_Int platz, B_Int anzahl, const B_String& quellstring);
DLL_DECL void B_midncpy(B_TStringStruct  zielstring, B_Int platz, B_Int anzahl, const B_TStringStruct quellstring);
DLL_DECL void B_midncpy(B_TStringStruct  zielstring, B_Int platz, B_Int anzahl, const B_cCStr& quellstring);
#endif // THALESPLUS

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_like
//         Dient zum Vergleich einer Zeichenfolge und eines Musters. 
//         Das Verhalten des Operators Like haengt von der Option Compare-Anweisung ab. Die 
//         voreingestellte Methode fuer den Zeichenfolgenvergleich in allen Modulen ist Option 
//        Compare Binary, sofern nicht anders angegeben. Ausfuehrliche Infos vgl. VB OnlineHilfe
// Siehe auch:    
// Parameter:  
//     searchString: 
//        Zeichenkette in der nach Muster=patternToSearch gesucht werden soll
//     patternToSearch:
//        Das Muster, in form einer Zeichenkette, nach dem in searchString gesucht werden soll.
// Rueckgabewert:  
//     Ist vom Typ B_Boolean
//         patternToSearch ist in searchString enthalten           -> True
//         patternToSearch und searchString stimmen nicht ueberein -> False
//         patternToSearch oder searchString Null                  -> Null  
// Bemerkung:     
/*
B_Boolean B_like(const B_Variant& searchString, const B_Variant& patternToSearch);
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Funktionsname:                                                                           (C) CiceroSoft 1995
//     B_format
//         Formatiert einen Ausdruck entsprechend den Anweisungen in einem Formatausdruck.
// Siehe auch:    
// Parameter:
//     expression:
//         Ein gltiger Ausdruck.
//     formatExpression:
//       	 Ein gltiger benannter oder benutzerdefinierter Formatausdruck.
//     firstDayOfWeek:
//         Eine Konstante, die den ersten Wochentag angibt.
//         Konstante   	Wert	  Beschreibung
//         vbUseSystem	  0	      NLS API-Einstellung verwenden
//         vbSunday	    1	      Sonntag (Voreinstellung)
//         vbMonday	    2      	Montag
//         vbTuesday	    3     	Dienstag
//         vbWednesday	  4     	Mittwoch
//         vbThursday	  5      	Donnerstag
//         vbFriday	    6	      Freitag
//         vbSaturday	  7      	Samstag
//      frstWeekOfYear:
//         Eine Konstante, die die erste Woche im Jahr angibt
//         Konstante	       Wert	    Beschreibung
//         vbUseSystem	     0      	NLS API-Einstellung verwenden
//         vbFirstJan1	     1       	Mit der Woche beginnen, in die der 1. Januar fllt 
//                                   (Voreinstellung)
//         vbFirstFourDays	 2	      Mit der ersten Woche im Jahr beginnen, die mindestens
//                                   4 Tage hat
//         vbFirstFullWeek  3	      Mit der ersten vollstndigen Woche im Jahr beginnen
// Rueckgabewert:
// Bemerkung:
//      Dei Formatoptionen sind sehr umfangreich und dem VB REference MAnual vollstaendig
//      zu entnehmen
//      Zahlen	         Verwenden Sie vordefinierte benannte numerische Formate, oder erstellen
//                       Sie benutzerdefinierte numerische Formate.
//      Datum/Zeit       Verwenden Sie vordefinierte benannte Datums- und Zeitformate, oder
//                       erstellen Sie benutzerdefinierte Datums- und Zeitformate.
//      Fortlaufende Nr. 
//      fr Datum/Zeit	 Verwenden Sie Datums- und Zeitformate oder numerische Formate.
//      Zeichenfolgen	   Erstellen Sie eigene benutzerdefinierte Zeichenfolgenformate.
//      Beim Formatieren einer Zahl ohne die Angabe von Format liefert Format dasselbe
//      Ergebnis wie die Str-Funktion. Allerdings fehlt bei positiven Zahlen, die mit
//      Format als Zeichenfolgen formatiert wurden, das fr Vorzeichen reservierte 
//      fhrende Leerzeichen. Die mit Str umgewandelten Zahlen behalten das fhrende
//      Leerzeichen.
/*
 // expression kann alles sein deshalb Variant
 B_Variant B_format(const B_Variant& expression, const B_String& formatExpression = "", 
                    const B_Integer firstDayOfWeek = vbSunday, const B_Integer& firstWeekOfYear = vbFistJan1);
 B_String  B_formatS(const B_Variant& expression, const B_String& formatExpression = "", 
                    const B_Integer firstDayOfWeek = vbSunday, const B_Integer& firstWeekOfYear = vbFistJan1);
*/ 


//================================ Inlinefunktionen =========================================================
//
#if defined _DEBUG || defined B_VB_CONTRACT
  #define B_NOINLINE
#else
  #undef B_NOINLINE
#endif

#ifndef B_NOINLINE                 // festlegen wenn Debuger keine Inlines debuggen kann
  #define B_INLINE inline
  #ifndef BFI_STR_HPP
    #include "bfi_str.hpp"
  #endif
#endif

//===========================================================================================================
//================================ End Of Header ============================================================
#endif // BF_STR_HPP

