#ifndef BI_ARRAY_HPP
#define BI_ARRAY_HPP
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Project:                 VB nach C/C++ Konverter Klassenbibliothek
// Modul:                   VB Klasse fuer VB Arrays
// Filename:                bi_array.cpp
// Version:                 1.0
// Status:                  beta
//
// Copyright:               Copyright (C) 1995
//                          Cicerosoft GmbH, 
//                          Ballweilerstr. 7,
//                          66399 Erfweiler-Ehlingen
//
// Operating System(s):     Win95
// Compiler:                VC++ 4.1
//
// Author:                  Jochen Diener JD
// Creation Date:           JD 14.03.95
// Last Modification:       JD 15.07.96
//
// Description:             Beinhaltet alle Klassen / Hilfsfkt. die im Zusammenhang mit der VB 
//                          Konvertierung, fuer die Bereitstellung von Arrays benoetigt 
//                          werden. 
//                          
// Dependencies:            bc_array.cpp    ->  Klassendef.
//                          bc_array.hpp    ->  Klassendekl.
//
// Remarks:                 Die Beschreibung zu den Klassenmethoden werden so kurz als moeglich
//                          gehalten um die Uebersichtlichkeit zu erhoehen, Eine genauere Beschreibung
//                          laesst sich ggf. in den VB Handbuechern finden
//
// Revision(s):   
//                          
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

//-- C-Header
#include <math.h>
#include <string.h> // fuer memcpy usw.

//-- Lib-Header
#ifndef BC_DBG_HPP
  #include "bc_dbg.hpp"
#endif

const B_Lng ARR_ALLOC_SIZE = 8192; // immer 8K-Bloecke oder vielfache davon
extern DLL_DECL void* allocArray(size_t objectSize, size_t nbrElements);
extern DLL_DECL void  deallocArray(void* arrPtr);
extern DLL_DECL void  copyArray(size_t objectSize, unsigned long maxElements, void* destPtr, void* srcPtr);
extern DLL_DECL void* reallocArray(size_t objectSize, void* srcPtr, size_t newSize);

//===========================================================================================

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_ArrayUnchecked
// Author: 
//     JD 17.07.96
// Dependencies:
// Impl. Remarks
// Revision(s):
//
B_INLINE
void 
B_ArrayUnchecked::dimRange(B_Shrt actDim, B_Shrt nbrDims) {}

B_INLINE
void
B_ArrayUnchecked::boundRange(B_Lng lower, B_Lng upper) {}

B_INLINE
void 
B_ArrayUnchecked::idxRange(B_Lng idx, B_Lng lower, B_Lng upper) {}

B_INLINE
void
B_ArrayUnchecked::elementRange(B_ULng aktPos, B_ULng maxPos) {}

B_INLINE
void
B_ArrayUnchecked::validAssignment(B_ULng aktDimArr1, B_ULng nbrDimsArr1, B_ULng aktDimArr2, B_ULng nbrDimsArr2) {}

//===========================================================================================

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T>::initArray()
// Author: 
//     JD 15.07.96
// Dependencies:
// Impl. Remarks
//     Die initialisierung erfolgt ueber ein statisches Element, respektive den default Konstruktor. somit
//     werden auch strukturen korrekt initialisiert.
// Revision(s):
//   
template <class T, class ArrayContract>
void 
B_Array<T, ArrayContract>::initArray()
  {
  B_TRACE(B_Trace::lvl2, ("B_Array<T>::initArray()"));
  static T dummy; // defaultKonstruktor, respektive static-Variablen werden korrekt initialisiert
  T* pCnt = pElements;
  // Pointerarithmetik schneller als Subskription  
  if (initFlag != B_FALSE) // nur wenn nicht bei Konstruktion verboten
    {
    for (B_ULng i=0UL; i < nbrElements; i++)
      {
      *pCnt = dummy;
      pCnt++; 
      }
    }
  } // B_Array<T>::initArray()

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T>::initEl()
// Author: 
//     JD 15.07.96
// Dependencies:
// Impl. Remarks
//     Die initialisierung erfolgt ueber ein statisches Element, respektive den default Konstruktor. somit
//     werden auch strukturen korrekt initialisiert.
// Revision(s):
//   
template <class T, class ArrayContract>
void 
B_Array<T, ArrayContract>::initEl(T& el)
  {
  B_TRACE(B_Trace::lvl2, ("B_Array<T>::initEl(T& el)"));
  static T dummy; 
  if (initFlag != B_FALSE) // nur wenn nicht durch Konstruktor verboten
    {
    el = dummy;
    }
  } // B_Array<T>::initEl(T& el)



/////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Bounds<T>::B_Bounds()
// Author: 
//     JD 15.07.96
// Dependencies:
// Impl. Remarks
// Revision(s):
//   
template <class T, class ArrayContract>
B_Bounds<T, ArrayContract>::B_Bounds()
  : pArr(NULL),
    pBoundValues(NULL),
    elementPos(0),
    aktDim(0)
  {
  B_TRACE(B_Trace::lvl2, ("B_Bounds<T>::B_Bounds()"));
  } // B_Bounds<T>::B_Bounds()


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Bounds<T>::~B_Bounds()
// Author: 
//     JD 15.07.96
// Dependencies:
// Impl. Remarks
// Revision(s):
//   
template <class T, class ArrayContract>
B_Bounds<T, ArrayContract>::~B_Bounds()
  {
  B_TRACE(B_Trace::lvl2, ("B_Bounds<T>::~B_Bounds()"));
  // beide in B_Array Destruktor destruiert
  pArr = NULL;
  pBoundValues = NULL;
  } // B_Bounds<T>::~B_Bounds()

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Bounds<T>::operator T() 
// Author: 
//     JD 15.07.96
// Dependencies:
// Impl. Remarks
// Revision(s):
//   
template <class T, class ArrayContract> 
B_Bounds<T, ArrayContract>::operator T() 
  { 
  B_TRACE(B_Trace::lvl2, ("B_Bounds<T>::operator T() "));
  return (pArr->pElements)[elementPos];
  } // B_Bounds<T>::operator T() 

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Bounds<T>::operator T() 
// Author: 
//     JD 15.07.96
// Dependencies:
// Impl. Remarks
// Revision(s):
//   --keine Auswertung von 'option base' da dies konverter macht und spaeter
//     bei Aenderungen im C++ Code die Upper/Lower-Bounds explizit angegeben
//     werden muessen
//   --Die Berechnung geht nur weil in VB immer alle Dimensionen angegeben werden muessen
//     fuer C-Zugriffe muesste der Algorithmus aufwendiger sein.
//   --Alle Indexberechnungen beruhen auf der Tatsache dass der Wert fuer die untere Grenze (=lBound) in
//     der Dimension n 0 ist (wie in C ueblich) angenommen wird. Sonst fuehrt die Berechnung zu ungueltigen 
//     Indexwerten in einem Eindimensionalen c-B_Array.
#ifdef B_VB_COMPATIBLE

template <class T, class ArrayContract> 
B_Bounds<T, ArrayContract>
B_Bounds<T, ArrayContract>::operator[](const B_Long& idx) 
  {
  B_TRACE(B_Trace::lvl2, ("B_Bounds<T>& B_Bounds<T>::operator[](B_Lng idx) "));
  ++aktDim;
  //-- die akt. Dimension muss kleiner gleich der maximalen sein die bei der Deklaration der 
  //   Variabeln vom typ B_Array-Klasse in deren Konstruktor als Parameter angegeben wurde.
  // B_REQUIRE_LVL(nbrDims != 1 && aktDim <= (nbrDims - 1), B_Assure::lvl2); 
  ArrayContract::dimRange(aktDim, pArr->nbrDims);

  //-- der Wert x im Subsrkiptionsoperator ([x]) darf nicht ausserhalb der Grenzen liegen die bei der 
  //   Deklaration angegeben wurden liegen.
  ArrayContract::idxRange(idx.getValue(), pBoundValues[aktDim].lower, pBoundValues[aktDim].upper);
  
	/* JD190996
  //-- bevor letzte Dimension erreicht ist wird immer fuer die aktuelle Dimension n bestimmt 
  //   wieviele Elemente der Dimension n+1 sie durch den Wert idx anspricht (entspricht der 
  //   Berechnung der Zeilen). Diese Werte sind noetig fuer die Umrechnung ins Eindimensionale 
  //   fuer die B_Array-Klasse.
  if (aktDim < (pArr->nbrDims - 1))
    {
    //-- Die Elemente der Dimension n (= idx - lBound[aktDim]) werden zu der Anzahl der Elemente x der 
    //   Dimension n (=aktuelle) die die Dimension n-1 aufgrund ihres Werte idx errechnet (= arr_dim)
    //   hat hinzuaddiert. Ergibt 0mal fuer jeweils erstes Element einer Dimension.
    elementPos += idx.getValue() - pBoundValues[aktDim].lower;

    //-- In n+1 Dimension ist bekannt wieviele Element die n+2 Dimension hat.
    //   Hier werden die Feldelemente der n (=aktuellen) Dimension mit der Anzahl an
    //   Feldelementen die die n+1 Dimension hat multipliziert.
    //   Auf diese Weise ist fuer die Dimension n, in Abhaengigkeit von idx, die darin 
    //   enthaltene Anzahl an Elementen fuer die n+1 Dimension bestimmt
    elementPos *= pBoundValues[aktDim+1].range;
    }
  else if (aktDim == (pArr->nbrDims-1)) // letzte Dimension
    {
    //-- Da die n-1 Dimension die Anzahl Elemente der Dimension n bestimmt hat
    //   brauchen in der letzten Dimension nur noch das durch idx angebene x Element
    //   zu der Anzahl m (=arr_dim) an Elementen der Dimension n hinzuaddiert zu 
    //   werden ohne die Anzahl der Elemente der n+1 Dimension zu bestimmen.
    elementPos += (idx.getValue() - pBoundValues[aktDim].lower);
    }
  */
  elementPos += idx.getValue() - pBoundValues[aktDim].lower;
  if (aktDim < (pArr->nbrDims - 1))
		{
    elementPos *= pBoundValues[aktDim+1].range;
		}

  return *this;
  } // B_Bounds<T>& B_Bounds<T>::operator[](long idx)

#endif // B_VB_COMPATIBLE


//-- Sonderimpl fr B_Lng als idx rechnet sich wenn die Indexe als C-Datentypen genommen 
//   werden ca. 1/3 schneller als ueber B_Long konvertierung jedoch ohne VB-Fehlerchecks
//   Kommentare s. B_Long Version
template <class T, class ArrayContract> 
B_Bounds<T, ArrayContract>
B_Bounds<T, ArrayContract>::operator[](const B_Lng idx) 
  {
  B_TRACE(B_Trace::lvl2, ("B_Bounds<T>& B_Bounds<T>::operator[](B_Lng idx) "));
  ++aktDim;
  ArrayContract::dimRange(aktDim, pArr->nbrDims);
  ArrayContract::idxRange(idx, pBoundValues[aktDim].lower, pBoundValues[aktDim].upper);

  /* JD190996
  if (aktDim < (pArr->nbrDims - 1))
    {
    elementPos += idx - pBoundValues[aktDim].lower;
    elementPos *= pBoundValues[aktDim+1].range;
    }
  else if (aktDim == (pArr->nbrDims-1)) // letzte Dimension
    {
    elementPos += idx - pBoundValues[aktDim].lower;
    }
	*/
  elementPos += idx - pBoundValues[aktDim].lower;

	if (aktDim < (pArr->nbrDims - 1))
		{
    elementPos *= pBoundValues[aktDim+1].range;
		}

  return *this;
  } // B_Bounds<T>& B_Bounds<T>::operator[](long idx)

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Bounds<T>& B_Bounds<T>::operator=(const T& rhs) 
// Author: 
//     JD 15.07.96
// Dependencies:
// Impl. Remarks
// Revision(s):
//   
template <class T, class ArrayContract> 
B_Bounds<T, ArrayContract>& 
B_Bounds<T, ArrayContract>::operator=(const T& rhs) 
  {
  B_TRACE(B_Trace::lvl2, ("B_Bounds<T>& B_Bounds<T>::operator=(const T& rhs) "));
  ArrayContract::elementRange(elementPos, pArr->nbrElements);
  (pArr->pElements)[elementPos] = rhs;
  return *this;
  } // B_Bounds<T>& B_Bounds<T>::operator=(const T& rhs) 

template <class T, class ArrayContract> 
B_Bounds<T, ArrayContract>& 
B_Bounds<T, ArrayContract>::operator+=(const T& rhs) 
  {
  B_TRACE(B_Trace::lvl2, ("B_Bounds<T>& B_Bounds<T>::operator+=(const T& rhs) "));
  ArrayContract::elementRange(elementPos, pArr->nbrElements);
  (pArr->pElements)[elementPos] += rhs;
  return *this;
  } // B_Bounds<T>& B_Bounds<T>::operator+=(const T& rhs) 

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Bounds<T>::B_Bounds (B_Bounds& rhs)
// Author: 
//     JD 15.07.96
// Dependencies:
// Impl. Remarks
//     Achtung Aliasing gefahr, deshalb NULL an pArr geht weil CopyConstrukor nur !
//     fuer Elementzuweisungen gebraucht wird um EH wegen Zuweisung an sich selbst
//     zu umgehen !! 
//     Bei erweiterungen der Klasse muss peinlich genau darauf geachtet werden dass
//     auer operaotr= sonst keine den Copykonstruktor benoetigt.
// Revision(s):
//   
template <class T, class ArrayContract> 
B_Bounds<T, ArrayContract>::B_Bounds (const B_Bounds& rhs)
  {
  B_TRACE(B_Trace::lvl2, ("B_Bounds<T>::B_Bounds (B_Bounds& rhs)"));
  pBoundValues = rhs.pBoundValues;
  pArr         = rhs.pArr;
  elementPos   = rhs.elementPos;
  aktDim       = rhs.aktDim;
  } // B_Bounds<T>::B_Bounds (B_Bounds& rhs)

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Bounds<T>& B_Bounds<T>::operator=(const B_Bounds& rhs) 
// Author: 
//     JD 15.07.96
// Dependencies:
// Impl. Remarks
//     Hier werden Elemente des Arrays einander zugewiesen und keine Boundklasse oder Array !!!!
//     PArameter by Ref geht nicht wegen Zuweisung an sich selbst die haeufig vorkommen kann
// Revision(s):
//   
template <class T, class ArrayContract>
B_Bounds<T, ArrayContract>& 
B_Bounds<T, ArrayContract>::operator=(const B_Bounds& rhs) 
  {
  B_TRACE(B_Trace::lvl2, ("B_Bounds<T>& B_Bounds<T>::operator=(const B_Bounds& rhs) "));

  ArrayContract::validAssignment(aktDim, pArr->nbrDims, rhs.aktDim, (rhs.pArr)->nbrDims);  
  if (this == &rhs)
    {
    return *this;
    }
  else
    {
    (pArr->pElements)[elementPos] = (rhs.pArr)->pElements[rhs.elementPos];
    }
 
  return *this;
  } // B_Bounds<T>& B_Bounds<T>::operator=(const B_Bounds& rhs) 


//===========================================================================================================
//===========================================================================================================


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T>::B_Array()	CT161296	Fuer dynamische Array's
// Author: 
//     JD 15.07.96
// Dependencies:
// Impl. Remarks
//     Die allokation erfolgt immer ueber die ARR_ALLOC_SIZE Blockgroesse damit nicht zu oft allokiert/
//     deallokiert werden muss, wegen der Speicherfragmentierung
// Revision(s):
//
template <class T, class ArrayContract>
B_Array<T, ArrayContract>::B_Array()
  : nbrElements(1L),
    initFlag(B_TRUE)
  {
  B_TRACE(B_Trace::lvl2, ("B_Array<T>::B_Array()"));

  capacity = (unsigned long)ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)));
  pElements = new T[capacity]; 
  if (pElements == 0)
    {
    throw B_OutOfMem();
    }
  // initArray(); hat schon new erledigt, ist C++ nicht schoen :-)

  //-- Boundwerte
  // Bei Dim oder static max. 8 Dimensionen in VB, im Moment keine Rolle da bei Boudns immer reallokiert
  pBoundValues = new BoundPair[1];
  if (pBoundValues == 0)
    {
    delete[] pElements;
    throw B_OutOfMem();
    }
  bounds.pArr         = this;
  bounds.pBoundValues = pBoundValues;
  bounds.elementPos   = 0;
  bounds.aktDim       = 0;
  nbrDims             = 1;
  staticFlag		  = B_FALSE;	// CT161296	Als dynamisches Array gekennzeichnet

  } // B_Array<T>::B_Array()


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T>::B_Array(B_Boolean&, B_Boolean&)
// Author: 
//     JD 15.07.96
// Dependencies:
// Impl. Remarks
//     Die beiden Booleanwerte sindnur noetig um keine ungewollte PArameteruebergabe zu erlauben
//     wenn explicit verfuegbar nur noch ein PArameter noetig dann auch als Defaultparameter fuer
//     Defaultkonstruktor
// Revision(s):
//
template <class T, class ArrayContract>
B_Array<T, ArrayContract>::B_Array(const B_Boolean& lhs, const B_Boolean& rhs)
  : nbrElements(1L) // immer mind. 1
  {
  B_TRACE(B_Trace::lvl2, ("B_Array<T>::B_Array(const B_Boolean&, const B_Boolean&)"));
  capacity = ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)));
  pElements = new T[capacity]; 
  if (pElements == 0)
    {
    throw B_OutOfMem();
    }
  bounds.pArr = this;
  if (lhs.getValue() == 0 && rhs.getValue() == 0)
    {
    initFlag = B_FALSE;
    }
  // initArray(); hat schon new erledigt

  nbrDims = 1;
  
  bounds.aktDim     = 0;
  bounds.elementPos = 0;
  pBoundValues      = new BoundPair[nbrDims];
  if (pBoundValues == 0)
    {
    delete[] pElements;
    throw B_OutOfMem();
    }
  bounds.pBoundValues = pBoundValues;
  staticFlag		  = B_FALSE;	// CT161296	Als dynamisches Array gekennzeichnet

  } // B_Array<T>::B_Array(B_Boolean, B_Boolean)


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T>::B_Array(const VARIANT FAR&)
// Author: 
//     JD 15.07.96
// Dependencies:
// Impl. Remarks
//     Die beiden Booleanwerte sindnur noetig um keine ungewollte PArameteruebergabe zu erlauben
//     wenn explicit verfuegbar nur noch ein PArameter noetig dann auch als Defaultparameter fuer
//     Defaultkonstruktor
// Revision(s):
//
template <class T, class ArrayContract>
B_Array<T, ArrayContract>::B_Array(const VARIANT FAR& rhs)
  {
  B_TRACE(B_Trace::lvl2, ("B_Array<T, ArrayContract>::B_Array(const VARIANT FAR& rhs)"));
  if ( (rhs.vt & VT_ARRAY) == VT_ARRAY) // TEST geht das so ?
    {
    // convertArray(B_VarType(T), rhs); 
    // B_TRACE(B_Trace::lvl4, ("    vor aufruf convertArray")); // TEST !!!
    convertArray(2, rhs); // solange 2 bis B_VarType implementiert -> nur TEST fuer Integerarrays moeglich !!!
    }
  else
    {
    #ifdef B_VB_CONTRACT
      throw B_InvalidArgument();
    #endif
    #ifdef B_CPP_CONTRACT
      B_REQUIRE_LVL(B_FALSE, B_Assure::lvl2);
    #endif
    }
  } // B_Array<T, ArrayContract>::B_Array(const VARIANT FAR& rhs)

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T>::~B_Array()
// Author: 
//     JD 15.07.96
// Dependencies:
// Impl. Remarks
// Revision(s):
//
template <class T, class ArrayContract>
B_Array<T, ArrayContract>::~B_Array()
  {
  B_TRACE(B_Trace::lvl2, ("B_Array<T>::~B_Array()"));
  if (pElements != NULL)
    {
    delete[] pElements; 
    pElements = NULL;
    }
  if (bounds.pBoundValues != NULL)
    {
    delete[] pBoundValues; 
    pBoundValues = NULL;
    }
  } // B_Array<T>::~B_Array()

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T>::B_Array(B_Array& rhs) 
// Author: 
//     JD 15.07.96
// Dependencies:
// Impl. Remarks
// Revision(s):
//
template <class T, class ArrayContract> 
B_Array<T, ArrayContract>::B_Array(const B_Array& rhs) 
  {
  B_TRACE(B_Trace::lvl2, ("B_Array<T>::B_Array(const B_Array& rhs)"));
  B_WARNING(B_Warning::lvl1, "B_Array(const B_Array& rhs); Die groesse von *this wird ggf. an die von rhs angepasst"); 
  nbrElements = rhs.nbrElements;
  nbrDims     = rhs.nbrDims;
  
  //-- Bound-Element kopieren
  bounds.aktDim     = rhs.bounds.aktDim;
  bounds.elementPos = rhs.bounds.elementPos;
  // Neue groesse allokieren in C++ kein realloc
  delete[] pBoundValues;
  pBoundValues = new BoundPair[nbrDims];
  if (pBoundValues == 0)
    {
    throw B_OutOfMem();
    }
  copyArray(sizeof(BoundPair), nbrDims, pBoundValues, rhs.pBoundValues); // da nun beide gleich Flat-Copy
  bounds.pArr = this;   // In Bounds, Arrayzeiger aktualisieren geht nur hier und nicht in Bounds
  bounds.pBoundValues = pBoundValues;

  //-- Reallokation noetig ?
  if ( (nbrElements * sizeof(T)) > capacity )
    {
    capacity = ceil( nbrElements /  ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)))) * ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)));
    delete[] pElements;
    pElements = new T[capacity];
    if (pElements == 0)
      {
      delete[] pBoundValues;
      throw B_OutOfMem();
      }
    }
  copyArray(sizeof(T), nbrElements, pElements, rhs.pElements); // Flat-Copy
  } // B_Array<T>::B_Array(const B_Array& rhs) 

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T, ArrayContract>::operator=(const B_Array& rhs)
// Author: 
//     JD 15.07.96
// Dependencies:
// Impl. Remarks
// Revision(s):
//
template <class T, class ArrayContract>
B_Array<T, ArrayContract>&
B_Array<T, ArrayContract>::operator=(const B_Array& rhs)
  {
  B_TRACE(B_Trace::lvl2, ("B_Array<T>::operator=(const B_Array& rhs)"));
  B_WARNING(B_Warning::lvl1, "array1 = array2; hierbei kann die Groesse von arr1 veraendert werden");
  if (this == &rhs)
    {
    return *this;
    }

  B_ULng oldNbrElements = nbrElements;
  nbrElements           = rhs.nbrElements;

  //-- Bounds aktualisieren
  nbrDims    = rhs.nbrDims;
  // aktDim     = rhs.aktDim;     // nicht notwendig
  // elementPos = rhs.elementPos; // nicht notwendig
  bounds.pArr  = this;
  
  // Neue groesse allokieren in C++ kein realloc
  delete[] pBoundValues;
  pBoundValues = new BoundPair[nbrDims];
  if (pBoundValues == 0)
    {
    throw B_OutOfMem();
    }
  copyArray(sizeof(BoundPair), nbrDims, pBoundValues, rhs.pBoundValues); // da nun beide gleich Flat-Copy
  bounds.pBoundValues = pBoundValues;

  //-- Reallokation ?
  if ( capacity < (nbrElements * sizeof(T)) )
    {
    capacity = ceil( rhs.nbrElements /  ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)))) * ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)));
    delete[] pElements;
    pElements = new T[capacity];
    if (pElements == 0)
      {
      delete[] pBoundValues;
      throw B_OutOfMem();
      }
    }
  copyArray(sizeof(T), nbrElements, pElements, rhs.pElements); // Flat-Copy

  return *this;
  } // B_Array<T>::operator=(const B_Array& rhs)


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T, ArrayContract>::operator=(const B_Array& rhs)
// Author: 
//     JD 15.07.96
// Dependencies:
// Impl. Remarks
// Revision(s):
//
template <class T, class ArrayContract>
B_Array<T, ArrayContract>&
B_Array<T, ArrayContract>::operator=(const VARIANT FAR& rhs)
  {
  B_TRACE(B_Trace::lvl3, ("B_Array<T, ArrayContract>::operator=(const VARIANT FAR& rhs)"));
  if ( (rhs.vt & VT_ARRAY) == VT_ARRAY) // TEST geht das so mit log. Und?
    {
    // convertArray(B_VarType(T), rhs);
    convertArray(2, rhs); // TEST solange bis B_VarType impl.
    }
  else
    {
    #ifdef B_VB_CONTRACT
      throw B_InvalidArgument();
    #endif
    #ifdef B_CPP_CONTRACT
      B_REQUIRE_LVL(B_FALSE, B_Assure::lvl2);
    #endif
    }
  return *this;
  } // B_Array<T, ArrayContract>::operator=(const VARIANT FAR& rhs)


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T>::B_Array(unsigned char nbrDims,...)	CT161296	Fuer statische Array's 
// Author: 
//     JD 15.07.96
// Dependencies:
// Impl. Remarks
//     Byte geht da nur 60 erlaubt in VB -> reicht fuer Bereichscheck
// Revision(s):
//
template <class T, class ArrayContract>
B_Array<T, ArrayContract>::B_Array(const B_Shrt dims,...)
  : nbrElements(1L),
    initFlag(B_TRUE)
  {
  B_TRACE(B_Trace::lvl2, ("B_Array<T>::B_Array(const B_Byte& nbrDims,...)"));
  // kein Check moeglich fuer Boundangabe. Anwender koennte nbrDims = 2 angeben
  // jedoch nur zwei optionale argumente oder Mehr als 4 !!! Nicht testbar
  B_REQUIRE_LVL( (dims > 0 && dims <= 255), B_Assure::lvl2); // wird schon von Byte-Klasse getestet
  nbrDims = dims;

  //-- Bounds anlegen
  bounds.aktDim = 0;
  bounds.elementPos = 0;
  bounds.elementPos = 0;
  pBoundValues = new BoundPair[nbrDims];
  if (pBoundValues == 0)
    {
    throw B_OutOfMem();
    }
  bounds.pBoundValues = pBoundValues;
  bounds.pArr = this;


  //-- Ellipse auswerten
  va_list pPara;
    
  // Anzahl der Arrayelemente berechnen
  va_start(pPara, dims);
  for(B_Shrt cnt = 0; cnt < (B_Shrt)nbrDims; cnt++)
    {
    pBoundValues[cnt].lower = va_arg(pPara, B_Lng); // B_Long nur moeglich wenn in Ellipse alle Zahl-Literale in B_Long gecastet wuerden -> zu umstaendlich, daher aber auch keine Typpruefung
                                                    // auch dies ginge nicht da Ellipse Klassen weder vor Ellipse noch in Ellipse korrekt handhabt
    pBoundValues[cnt].upper = va_arg(pPara, B_Lng); 
    pBoundValues[cnt].range = pBoundValues[cnt].upper - pBoundValues[cnt].lower + 1;
    ArrayContract::boundRange(pBoundValues[cnt].lower, pBoundValues[cnt].upper);
    nbrElements *= pBoundValues[cnt].range; // In VB muss Upperbound groesser als Lowerbound sein
    }
  va_end(pPara);
  
  //-- Allokation
  capacity = (unsigned long)(ceil( nbrElements /  ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)))) * ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T))));
  pElements = new T[capacity];
  if (pElements == 0)
    {
    delete[] pBoundValues;
    throw B_OutOfMem();
    }

  staticFlag = B_TRUE;	// CT161296 Array als statisch kennzeichnen
 
  // initArray(); Elemente mit ihrem defaultwert initialisieren, wie in VB ueblich hat schon new gemacht
  } // B_Array<T>::B_Array(unsigned char nbrDims,...) 

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T>::B_Array(unsigned char nbrDims,...)	CT161296	Fuer statische Array's 
// Author: 
//     JD 15.07.96
// Dependencies:
// Impl. Remarks
//     Byte geht da nur 60 erlaubt in VB -> reicht fuer Bereichscheck
// Revision(s):
//
#if 0	//ab.. Compiler Fehler 10.6.97
template <class T, class ArrayContract> 
B_Array<T, ArrayContract>::B_Array(T initVal, struct B_PreInitArr&, const B_Shrt dims, ...) 
  : nbrElements(1L),
    initFlag(B_TRUE)
  {
  B_TRACE(B_Trace::lvl2, ("B_Array<T>::B_Array(const B_Byte& nbrDims,...)"));
  // kein Check moeglich fuer Boundangabe. Anwender koennte nbrDims = 2 angeben
  // jedoch nur zwei optionale argumente oder Mehr als 4 !!! Nicht testbar
  B_REQUIRE_LVL( (dims > 0 && dims <= 255), B_Assure::lvl2); // wird schon von Byte-Klasse getestet
  nbrDims = dims;

  //-- Bounds anlegen
  bounds.aktDim = 0;
  bounds.elementPos = 0;
  bounds.elementPos = 0;
  pBoundValues = new BoundPair[nbrDims];
  if (pBoundValues == 0)
    {
    throw B_OutOfMem();
    }
  bounds.pBoundValues = pBoundValues;
  bounds.pArr = this;


  //-- Ellipse auswerten
  va_list pPara;
    
  // Anzahl der Arrayelemente berechnen
  va_start(pPara, dims);
  for(B_Shrt cnt = 0; cnt < (B_Shrt)nbrDims; cnt++)
    {
    pBoundValues[cnt].lower = va_arg(pPara, B_Lng); // B_Long nur moeglich wenn in Ellipse alle Zahl-Literale in B_Long gecastet wuerden -> zu umstaendlich, daher aber auch keine Typpruefung
                                                           // auch dies ginge nicht da Ellipse Klassen weder vor Ellipse noch in Ellipse korrekt handhabt
    pBoundValues[cnt].upper = va_arg(pPara, B_Lng); 
    pBoundValues[cnt].range = pBoundValues[cnt].upper - pBoundValues[cnt].lower + 1;
    ArrayContract::boundRange(pBoundValues[cnt].lower, pBoundValues[cnt].upper);
    nbrElements *= pBoundValues[cnt].range; // In VB muss Upperbound groesser als Lowerbound sein
    }
  va_end(pPara);
  
  //-- Allokation
  capacity = ceil( nbrElements /  ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)))) * ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)));
  pElements = new T[capacity];
  if (pElements == 0)
    {
    delete[] pBoundValues;
    throw B_OutOfMem();
    }
 
  // Initialisieren der Arraelemente mit einem speziellen Vorgabewert.
  for (B_Lng i = 0; i < nbrElements; ++i)
    {
    pElements[i] = initVal;
    }

  staticFlag = B_TRUE;	// CT161296 Array als statisch kennzeichnen

  } // B_Array<T>::B_Array(T, const B_Shrt nbrDims,...) 
#endif
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Bounds<T>& B_Array<T>::operator[](long idx)
// Author: 
//     JD 16.07.96
// Dependencies:
// Impl. Remarks
// Revision(s):
//
#ifdef B_VB_COMPATIBLE

template <class T, class ArrayContract> 
B_Bounds<T, ArrayContract> 
B_Array<T, ArrayContract>::operator[](const B_Long& idx) // Typpruefung in B_Long
  {
  B_TRACE(B_Trace::lvl2, ("B_Bounds<T>& B_Array<T>::operator[](long idx)"));
  bounds.aktDim = 0; // kein Test noetig da Array immer mind. Eine Dimension hat
  //-- der Wert x im Subsrkiptionsoperator ([x]) darf nicht ausserhalb der Grenzen liegen die bei der 
  //   Deklaration angegeben wurden liegen.
  ArrayContract::idxRange(idx.getValue(),  pBoundValues[bounds.aktDim].lower, pBoundValues[bounds.aktDim].upper);
  //-- Die Elemente der Dimension n (= i - lBound[dim]) werden zu der Anzahl der Elemente x der 
  //   Dimension n (=aktuelle) die die Dimension n-1 aufgrund ihres Werte i errechnet (= arr_dim)
  //   hat hinzuaddiert. Ergibt 0mal fuer jeweils erstes Element einer Dimension.
  /* JD190996
  if (bounds.aktDim < (nbrDims - 1))
    {
    bounds.elementPos =  idx.getValue() - pBoundValues[bounds.aktDim].lower;
    bounds.elementPos *= pBoundValues[bounds.aktDim+1].range;
    }
  else if (bounds.aktDim == (nbrDims-1)) // letzte Dimension
    {
    bounds.elementPos = idx.getValue() - pBoundValues[bounds.aktDim].lower;
    }
	*/
  bounds.elementPos = idx.getValue() - pBoundValues[bounds.aktDim].lower;
  if (bounds.aktDim < (nbrDims - 1))
    {
    bounds.elementPos *= pBoundValues[bounds.aktDim+1].range;
    }

  return bounds;
  } // B_Bounds<T>& B_Array<T>::operator[](long idx)

#endif // B_VB_COMPATIBLE

template <class T, class ArrayContract> 
B_Bounds<T, ArrayContract> 
B_Array<T, ArrayContract>::operator[](const B_Lng idx)
  {
  B_TRACE(B_Trace::lvl2, ("B_Bounds<T>& B_Array<T>::operator[](long idx)"));
  bounds.aktDim = 0; // kein Test noetig da Array immer mind. Eine Dimension hat
  ArrayContract::idxRange(idx,  pBoundValues[bounds.aktDim].lower, pBoundValues[bounds.aktDim].upper);
  /*
  if (bounds.aktDim < (nbrDims - 1))
    {
    bounds.elementPos =  idx - pBoundValues[bounds.aktDim].lower;
    bounds.elementPos *= pBoundValues[bounds.aktDim+1].range;
    }
  else if (bounds.aktDim == (nbrDims-1)) // letzte Dimension
    {
    bounds.elementPos = (idx - pBoundValues[bounds.aktDim].lower);
    }
	*/
  bounds.elementPos = idx - pBoundValues[bounds.aktDim].lower; 

	if (bounds.aktDim < (nbrDims-1)) // letzte Dimension
    {
    bounds.elementPos *= pBoundValues[bounds.aktDim+1].range; 
    }

  return bounds;
  } // B_Bounds<T>& B_Array<T>::operator[](long idx)

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     Subskriptionsop. als Funktion -> Zugriff auf Members eines TYP in einem Array
// Parameter:  
// Return Value:	Template Member des Arrray-Indexes
// Errorhandling:
// Note:	(Anzahl Dimensionen, Index/Dimension, ...)
//			Implementierung nur fuer B_Lng da Ellypsen mit Klassen nicht moeglich sind.
//     
template<class T, class ArrayContract>
T& 
B_Array<T, ArrayContract>::gA(const B_Lng dims, ...)
{
	B_Lng		idx;					// Aktuell berechneter Index

	B_TRACE(B_Trace::lvl2, ("T& B_Array<T>::gA(long dims, long idx_D1, ...)"));
	
	bounds.elementPos = 0;				// Startwert fuer Indexberechnung
	va_list pPara;						// Ellipse auswerten
	va_start(pPara, dims);				// Anzahl der Arrayelemente berechnen
	for(bounds.aktDim = 0; bounds.aktDim < dims; bounds.aktDim++)
	{
		idx = va_arg(pPara, B_Lng);		// aktuellen Index vom Stack holen
		if(bounds.aktDim > 0)			// ab Dimension 2 bis n Dimensionen pruefen
			ArrayContract::dimRange(bounds.aktDim, nbrDims);
		ArrayContract::idxRange(idx,  pBoundValues[bounds.aktDim].lower, pBoundValues[bounds.aktDim].upper);
										// Indexberechnung pro Dimension
		bounds.elementPos += idx - pBoundValues[bounds.aktDim].lower; 

		if(bounds.aktDim < (nbrDims-1))	// letzte Dimension
		{
			bounds.elementPos *= pBoundValues[bounds.aktDim+1].range; 
		}
	}
	va_end(pPara);						// Stack schliessen

	return pElements[bounds.elementPos];
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T>::redim(unsigned char nbrDims, ...)
// Author: 
//     JD 16.07.96
// Dependencies:
// Impl. Remarks
// Revision(s):
//
template<class T, class ArrayContract>
void
B_Array<T, ArrayContract>::redim(const B_UChar dims, ...)
  {
  B_TRACE(B_Trace::lvl3, ("B_Array<T>::redim(unsigned char nbrDims, ...)"));
  nbrElements = 1L; // Gesamtanzahl an Elementen fuer Elementanzahl zu bestimmen die allokiert werden muss
  nbrDims     = dims;  
  va_list pPara;
  delete[] pBoundValues;
  pBoundValues = new BoundPair[nbrDims]; // exakte Groesse reicht bei Bounds
  if (pBoundValues == 0)
    {
    throw B_OutOfMem();
    }
  bounds.pBoundValues = pBoundValues;
  bounds.pArr = this;
  
  // B_REQUIRE_LVL( (nbrDims > 0 && nbrDims <= 255), B_Assure::lvl2); // testet scoon B_Byte ab

  va_start(pPara,dims);
  for(B_Shrt cnt = 0; cnt < (B_Shrt)nbrDims; ++cnt)
    {
    pBoundValues[cnt].lower = va_arg(pPara, B_Lng);    
    pBoundValues[cnt].upper = va_arg(pPara, B_Lng);
    pBoundValues[cnt].range = pBoundValues[cnt].upper - pBoundValues[cnt].lower + 1;
    ArrayContract::boundRange(pBoundValues[cnt].lower, pBoundValues[cnt].upper);
    nbrElements *= pBoundValues[cnt].range; // In VB muss Upperbound groesser als Lowerbound sein
    }
  va_end(pPara);

  //-- Reallokation noetig ?
#ifndef THALESPLUS	//ab.. 7.4.97	fuer QBASIC Abfrage weg, damit Types bei redim auf
					//jeden Fall initialisiert werden
  if (capacity < (nbrElements * sizeof(T)) )
#endif
    {
    capacity = (unsigned long)(ceil( nbrElements /  ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)))) * ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T))));
    delete[] pElements;
    pElements = new T[capacity];
    if (pElements == 0)
      {
      delete[] pBoundValues;
      throw B_OutOfMem();
      }
    }

  // initArray(); schon new erledigt
  } // B_Array<T>::redim(unsigned char nbrDims, ...)

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// NEU: CT171296	Implementiert fuer streamr() + dynamic Array
// Method:
//     redim   Redimensionierung des Arrays
// Parameter:  
//     der erste Parameter dims, bestimmt die Anzahl der Dimensionen. Fuer jede Dimension muss
//     dann die zugehoerigen Lower- und UpperBounds und Range angegeben werden
// Return Value:
// Errorhandling:
// Note:      
//     Enspricht der VB-Prozedur Redim()
template<class T, class ArrayContract>
void
B_Array<T, ArrayContract>::redim(const B_Shrt dims, BoundPair * bPair)
  {
  B_TRACE(B_Trace::lvl3, ("B_Array<T>::redim(unsigned char nbrDims, ...)"));
  nbrElements = 1L; // Gesamtanzahl an Elementen fuer Elementanzahl zu bestimmen die allokiert werden muss
  nbrDims     = dims;  
 
  delete[] pBoundValues;
  pBoundValues = new BoundPair[nbrDims]; // exakte Groesse reicht bei Bounds
  if (pBoundValues == 0)
    {
    throw B_OutOfMem();
    }
  bounds.pBoundValues = pBoundValues;
  bounds.pArr = this;

  for(B_Shrt cnt = 0; cnt < (B_Shrt)nbrDims; ++cnt)
    {
    pBoundValues[cnt].lower = bPair[cnt].lower;
    pBoundValues[cnt].upper = bPair[cnt].upper;
    pBoundValues[cnt].range = bPair[cnt].range;
    ArrayContract::boundRange(pBoundValues[cnt].lower, pBoundValues[cnt].upper);
    nbrElements *= pBoundValues[cnt].range; // In VB muss Upperbound groesser als Lowerbound sein
    }

  //-- Reallokation noetig ?
#ifndef THALESPLUS	//ab.. 7.4.97	fuer QBASIC Abfrage weg, damit Types bei redim auf
					//jeden Fall initialisiert werden
  if (capacity < (nbrElements * sizeof(T)) )
#endif
    {
    capacity = ceil( nbrElements /  ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)))) * ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)));
    delete[] pElements;
    pElements = new T[capacity];
    if (pElements == 0)
      {
      delete[] pBoundValues;
      throw B_OutOfMem();
      }
    }
  // initArray(); schon new erledigt
  } // B_Array<T>::redim(const B_Shrt dims, BoundPair * bPair)
    
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T>::pRedim(B_Lng)
// Author: 
//     JD 16.07.96
// Dependencies:
// Impl. Remarks
// Revision(s):
//
#ifdef B_VB_COMPATIBLE

template<class T, class ArrayContract>
void
B_Array<T, ArrayContract>::pRedim(const B_Long& uBoundLastDim)
  {
  B_TRACE(B_Trace::lvl3, ("B_Array<T>::pRedim(B_Lng uBoundLastDim)"));
  B_ULng nbrAddEl; // anzahl neuer hinzukommender Elemente in letzter Dimension
  B_ULng nbrOldEl; // anzhal alter Elemente in letzter Dimension
  B_ULng oldNbrElements = nbrElements;
  B_ULng oldMax;
  B_ULng newIdx;
  B_ULng cnt;
  B_ULng oldIdx;

  ArrayContract::boundRange(pBoundValues[nbrDims - 1].lower, uBoundLastDim.getValue());

  // Hilfswerte ermitteln
  nbrAddEl = abs(uBoundLastDim.getValue() - pBoundValues[nbrDims - 1].upper); // wieviele sind dazugekommen
  nbrOldEl = pBoundValues[nbrDims - 1].range;  
  
  // Elemntanzahl aktualisieren
  nbrElements /=  nbrOldEl; // Alte Letzte Dimension rausnehmen

  pBoundValues[nbrDims - 1].upper = uBoundLastDim.getValue(); // Neuer Upper Bound setzen  
  pBoundValues[nbrDims - 1].range = pBoundValues[nbrDims - 1].upper - pBoundValues[nbrDims - 1].lower + 1;
  // neue Elemntanzhal bestimmen
  nbrElements *= (uBoundLastDim.getValue() - pBoundValues[nbrDims - 1].lower) + 1; 

  // Hilfsarray mit neuer Elementanzahl allokieren zum Umkopieren
  T* pNewElements = new T[nbrElements]; // genau anlegen da nur Temporaer 
  if (pNewElements == 0)
    {
    throw B_OutOfMem();
    }

  // Alte Elemente an korrekter Stelle im neuen Array einfuegen
  oldMax = nbrOldEl; // weil zaehlung mit 0 beginnt
  for (newIdx = 0, oldIdx = 0; newIdx < nbrElements; ++newIdx, ++oldIdx)
    {
    if (newIdx < oldMax)
      {
      pNewElements[newIdx] = pElements[oldIdx]; // alle alten Elemente kopieren
      }
    else
      {
      // Alle Werte die in Letzer Dimension hinzukammen Initialisieren
      for (cnt = newIdx; cnt < (newIdx + nbrAddEl); ++cnt) 
        {
        initEl(pNewElements[cnt]);
        }
      newIdx = cnt - 1;        // Index anpassen da ja neue erledigt
      --oldIdx;                // Bei Abbruch while eins zu hoch   
      oldMax = cnt + nbrOldEl; // Neue Grenzwert fuer Kopieren alter Elemente bis neue kommen
      }
    }

  //-- Reallokation noetig ?
  if (capacity < (nbrElements * sizeof(T)) )
    {
    capacity = ceil( nbrElements /  ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)))) * ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)));
    delete[] pElements;
    pElements = new T[capacity];
    if (pElements == 0)
      {
      throw B_OutOfMem();
      }
    }
 
  //-- neue Werte uebertragen und temporaeren Speicherplatz freigeben
  copyArray(sizeof(T), nbrElements, pElements, pNewElements); // Flat-Copy

  // Speicher aufraeumen
  delete[] pNewElements; 
  } // B_Array<T>::pRedim(B_Long uBoundLastDim)

#endif // B_VB_COMPATIBLE

template<class T, class ArrayContract>
void
B_Array<T, ArrayContract>::pRedim(const B_Lng uBoundLastDim)
  {
  B_TRACE(B_Trace::lvl3, ("B_Array<T>::pRedim(B_Lng uBoundLastDim)"));
  B_ULng nbrAddEl; // anzahl neuer hinzukommender Elemente in letzter Dimension
  B_ULng nbrOldEl; // anzhal alter Elemente in letzter Dimension
  B_ULng oldNbrElements = nbrElements;
  B_ULng oldMax;
  B_ULng newIdx;
  B_ULng cnt;
  B_ULng oldIdx;

  ArrayContract::boundRange(pBoundValues[nbrDims - 1].lower, uBoundLastDim);

  // Hilfswerte ermitteln
  nbrAddEl = abs(uBoundLastDim - pBoundValues[nbrDims - 1].upper); // wieviele sind dazugekommen
  nbrOldEl = pBoundValues[nbrDims - 1].range;  
  
  // Elemntanzahl aktualisieren
  nbrElements /=  nbrOldEl; // Alte Letzte Dimension rausnehmen

  pBoundValues[nbrDims - 1].upper = uBoundLastDim; // Neuer Upper Bound setzen  
  pBoundValues[nbrDims - 1].range = pBoundValues[nbrDims - 1].upper - pBoundValues[nbrDims - 1].lower + 1;
  // neue Elemntanzhal bestimmen
  nbrElements *= (uBoundLastDim - pBoundValues[nbrDims - 1].lower) + 1; 

  // Hilfsarray mit neuer Elementanzahl allokieren zum Umkopieren
  T* pNewElements = new T[nbrElements]; // genau anlegen da nur Temporaer 
  if (pNewElements == 0)
    {
    throw B_OutOfMem();
    }

  // Alte Elemente an korrekter Stelle im neuen Array einfuegen
  oldMax = nbrOldEl; // weil zaehlung mit 0 beginnt
  for (newIdx = 0, oldIdx = 0; newIdx < nbrElements; ++newIdx, ++oldIdx)
    {
    if (newIdx < oldMax)
      {
      pNewElements[newIdx] = pElements[oldIdx]; // alle alten Elemente kopieren
      }
    else
      {
      // Alle Werte die in Letzer Dimension hinzukammen Initialisieren
      for (cnt = newIdx; cnt < (newIdx + nbrAddEl); ++cnt) 
        {
        initEl(pNewElements[cnt]);
        }
      newIdx = cnt - 1;        // Index anpassen da ja neue erledigt
      --oldIdx;                // Bei Abbruch while eins zu hoch   
      oldMax = cnt + nbrOldEl; // Neue Grenzwert fuer Kopieren alter Elemente bis neue kommen
      }
    }

  //-- Reallokation noetig ?
  if (capacity < (nbrElements * sizeof(T)) )
    {
    capacity = ceil( nbrElements /  ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)))) * ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)));
    delete[] pElements;
    pElements = new T[capacity];
    if (pElements == 0)
      {
      throw B_OutOfMem();
      }
    }
 
  //-- neue Werte uebertragen und temporaeren Speicherplatz freigeben
  copyArray(sizeof(T), nbrElements, pElements, pNewElements); // Flat-Copy

  // Speicher aufraeumen
  delete[] pNewElements; 
  } // B_Array<T>::pRedim(B_Lng uBoundLastDim)

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Version mit gleichen Parametern wie DIM und der static-Konstruktor
// CT240197
template<class T, class ArrayContract>
void
B_Array<T, ArrayContract>::pRedim(const B_UChar dims, ...)
  {
  B_TRACE(B_Trace::lvl3, ("B_Array<T, ArrayContract>::pRedim(const B_UChar dims, ...)"));
  B_ULng nbrAddEl; // anzahl neuer hinzukommender Elemente in letzter Dimension
  B_ULng nbrOldEl; // anzhal alter Elemente in letzter Dimension
  B_ULng oldNbrElements = nbrElements;
  B_ULng oldMax;
  B_ULng newIdx;
  B_ULng cnt;
  B_ULng oldIdx;

  B_Lng	dummyLower;
  B_Lng uBoundLastDim;

  //-- Ellipse auswerten
  va_list pPara;
    
  // UBound der letzten Dimension aus Ellipse lesen
  va_start(pPara, dims);
#if 1	//ab.. 4.4.97 Warningfreiheit
  for(cnt = 0; cnt < (B_ULng)nbrDims; cnt++)
#else
  for(cnt = 0; cnt < (B_Shrt)nbrDims; cnt++)
#endif
    {
    dummyLower	  = va_arg(pPara, B_Lng); // B_Long nur moeglich wenn in Ellipse alle Zahl-Literale in B_Long gecastet wuerden -> zu umstaendlich, daher aber auch keine Typpruefung
    uBoundLastDim = va_arg(pPara, B_Lng); // auch dies ginge nicht da Ellipse Klassen weder vor Ellipse noch in Ellipse korrekt handhabt
    }
  va_end(pPara);

  // UBound der letzten Dimension ermitteln
  uBoundLastDim = pBoundValues[cnt - 1].upper;

  ArrayContract::boundRange(pBoundValues[nbrDims - 1].lower, uBoundLastDim);

  // Hilfswerte ermitteln
  nbrAddEl = abs((int)(uBoundLastDim - pBoundValues[nbrDims - 1].upper)); // wieviele sind dazugekommen
  nbrOldEl = pBoundValues[nbrDims - 1].range;  
  
  // Elemntanzahl aktualisieren
  nbrElements /=  nbrOldEl; // Alte Letzte Dimension rausnehmen

  pBoundValues[nbrDims - 1].upper = uBoundLastDim; // Neuer Upper Bound setzen  
  pBoundValues[nbrDims - 1].range = pBoundValues[nbrDims - 1].upper - pBoundValues[nbrDims - 1].lower + 1;
  // neue Elemntanzhal bestimmen
  nbrElements *= (uBoundLastDim - pBoundValues[nbrDims - 1].lower) + 1; 

  // Hilfsarray mit neuer Elementanzahl allokieren zum Umkopieren
  T* pNewElements = new T[nbrElements]; // genau anlegen da nur Temporaer 
  if (pNewElements == 0)
    {
    throw B_OutOfMem();
    }

  // Alte Elemente an korrekter Stelle im neuen Array einfuegen
  oldMax = nbrOldEl; // weil zaehlung mit 0 beginnt
  for (newIdx = 0, oldIdx = 0; newIdx < nbrElements; ++newIdx, ++oldIdx)
    {
    if (newIdx < oldMax)
      {
      pNewElements[newIdx] = pElements[oldIdx]; // alle alten Elemente kopieren
      }
    else
      {
      // Alle Werte die in Letzer Dimension hinzukammen Initialisieren
      for (cnt = newIdx; cnt < (newIdx + nbrAddEl); ++cnt) 
        {
        initEl(pNewElements[cnt]);
        }
      newIdx = cnt - 1;        // Index anpassen da ja neue erledigt
      --oldIdx;                // Bei Abbruch while eins zu hoch   
      oldMax = cnt + nbrOldEl; // Neue Grenzwert fuer Kopieren alter Elemente bis neue kommen
      }
    }

  //-- Reallokation noetig ?
  if (capacity < (nbrElements * sizeof(T)) )
    {
    capacity = (unsigned long)(ceil( nbrElements /  ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)))) * ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T))));
    delete[] pElements;
    pElements = new T[capacity];
    if (pElements == 0)
      {
      throw B_OutOfMem();
      }
    }
 
  //-- neue Werte uebertragen und temporaeren Speicherplatz freigeben
  copyArray(sizeof(T), nbrElements, pElements, pNewElements); // Flat-Copy

  // Speicher aufraeumen
  delete[] pNewElements; 
  } // B_Array<T>::pRedim(B_Lng uBoundLastDim)

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T>::dimStatic()
// Author: 
//     JD 16.07.96
// Dependencies:
// Impl. Remarks
//     dimStatic	Fall: DIM for COMMON in Includdatei
// Revision(s):
//
template <class T, class ArrayContract>
void
B_Array<T, ArrayContract>::dimStatic(const B_UChar dims, ...)
  {
  B_TRACE(B_Trace::lvl2, ("B_Array<T, ArrayContract>::dimStatic(const B_UChar dims, ...)"));
  // kein Check moeglich fuer Boundangabe. Anwender koennte nbrDims = 2 angeben
  // jedoch nur zwei optionale argumente oder Mehr als 4 !!! Nicht testbar
  B_REQUIRE_LVL( (dims > 0 && dims <= 255), B_Assure::lvl2); // wird schon von Byte-Klasse getestet
  nbrDims = dims;

  //-- Bounds anlegen
  bounds.aktDim = 0;
  bounds.elementPos = 0;
  bounds.elementPos = 0;
  pBoundValues = new BoundPair[nbrDims];
  if (pBoundValues == 0)
    {
    throw B_OutOfMem();
    }
  bounds.pBoundValues = pBoundValues;
  bounds.pArr = this;


  //-- Ellipse auswerten
  va_list pPara;
    
  // Anzahl der Arrayelemente berechnen
  va_start(pPara, dims);
  for(B_Shrt cnt = 0; cnt < (B_Shrt)nbrDims; cnt++)
    {
    pBoundValues[cnt].lower = va_arg(pPara, B_Lng); // B_Long nur moeglich wenn in Ellipse alle Zahl-Literale in B_Long gecastet wuerden -> zu umstaendlich, daher aber auch keine Typpruefung
                                                    // auch dies ginge nicht da Ellipse Klassen weder vor Ellipse noch in Ellipse korrekt handhabt
    pBoundValues[cnt].upper = va_arg(pPara, B_Lng); 
    pBoundValues[cnt].range = pBoundValues[cnt].upper - pBoundValues[cnt].lower + 1;
    ArrayContract::boundRange(pBoundValues[cnt].lower, pBoundValues[cnt].upper);
    nbrElements *= pBoundValues[cnt].range; // In VB muss Upperbound groesser als Lowerbound sein
    }
  va_end(pPara);
  
  //-- Allokation
  capacity = (unsigned long)(ceil( nbrElements /  ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)))) * ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T))));
  pElements = new T[capacity];
  if (pElements == 0)
    {
    delete[] pBoundValues;
    throw B_OutOfMem();
    }

  staticFlag = B_TRUE;	// CT161296 Array als statisch kennzeichnen
 
  // initArray(); Elemente mit ihrem defaultwert initialisieren, wie in VB ueblich hat schon new gemacht
  } // B_Array<T, ArrayContract>::dimStatic(const B_UChar dims, ...)

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T>::dimDynamic()
// Author: 
//     JD 16.07.96
// Dependencies:
// Impl. Remarks
//     dimDynamic	Fall: COMMON ohne DIM in Icludedatei aber DIM im *.BAS - Code
// Revision(s):
//
template <class T, class ArrayContract>
void
B_Array<T, ArrayContract>::dimDynamic(const B_UChar dims, ...)
  {
  B_TRACE(B_Trace::lvl2, ("B_Array<T, ArrayContract>::dimDynamic(const B_UChar dims, ...)"));
  // kein Check moeglich fuer Boundangabe. Anwender koennte nbrDims = 2 angeben
  // jedoch nur zwei optionale argumente oder Mehr als 4 !!! Nicht testbar
  B_REQUIRE_LVL( (dims > 0 && dims <= 255), B_Assure::lvl2); // wird schon von Byte-Klasse getestet
  nbrDims = dims;

  //-- Bounds anlegen
  bounds.aktDim = 0;
  bounds.elementPos = 0;
  bounds.elementPos = 0;
  pBoundValues = new BoundPair[nbrDims];
  if (pBoundValues == 0)
    {
    throw B_OutOfMem();
    }
  bounds.pBoundValues = pBoundValues;
  bounds.pArr = this;


  //-- Ellipse auswerten
  va_list pPara;
    
  // Anzahl der Arrayelemente berechnen
  va_start(pPara, dims);
  for(B_Shrt cnt = 0; cnt < (B_Shrt)nbrDims; cnt++)
    {
    pBoundValues[cnt].lower = va_arg(pPara, B_Lng); // B_Long nur moeglich wenn in Ellipse alle Zahl-Literale in B_Long gecastet wuerden -> zu umstaendlich, daher aber auch keine Typpruefung
                                                    // auch dies ginge nicht da Ellipse Klassen weder vor Ellipse noch in Ellipse korrekt handhabt
    pBoundValues[cnt].upper = va_arg(pPara, B_Lng); 
    pBoundValues[cnt].range = pBoundValues[cnt].upper - pBoundValues[cnt].lower + 1;
    ArrayContract::boundRange(pBoundValues[cnt].lower, pBoundValues[cnt].upper);
    nbrElements *= pBoundValues[cnt].range; // In VB muss Upperbound groesser als Lowerbound sein
    }
  va_end(pPara);
  
  //-- Allokation
  capacity = (unsigned long)(ceil( nbrElements /  ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)))) * ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T))));
  pElements = new T[capacity];
  if (pElements == 0)
    {
    delete[] pBoundValues;
    throw B_OutOfMem();
    }

  staticFlag = B_FALSE;	// CT161296 Array als dynamisch kennzeichnen
 
  // initArray(); Elemente mit ihrem defaultwert initialisieren, wie in VB ueblich hat schon new gemacht
  } // B_Array<T, ArrayContract>::dimDynamic(const B_UChar dims, ...)

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T>::getSize() const
// Author: 
//     JD 16.07.96
// Dependencies:
// Impl. Remarks
// Revision(s):
//
#if 0	//ab.. Compiler Fehler 10.6.97
template <class T, class ArrayContract>
void
B_Array<T, ArrayContract>::copyToOle(const VARIANT FAR& rhs) const
  {
  B_TRACE(B_Trace::lvl3, ("B_Array<T, ArrayContract>::copyToOle(const VARIANT FAR& rhs) const"));
  if ( (rhs.vt & VT_ARRAY) == VT_ARRAY) // TEST geht das so mit log. Und?
    {
    toOle(B_VarType(T), rhs);
    }
  else
    {
    #ifdef B_VB_CONTRACT
      throw B_InvalidArgument();
    #endif
    #ifdef B_CPP_CONTRACT
      B_REQUIRE_LVL(B_FALSE, B_Assure::lvl2);
    #endif
    }
  } // B_Array<T, ArrayContract>::copyToOle(const VARIANT FAR& rhs) const
#endif

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T>::getSize() const
// Author: 
//     JD 16.07.96
// Dependencies:
// Impl. Remarks
// Revision(s):
//
template <class T, class ArrayContract>
B_Lng
B_Array<T, ArrayContract>::getSize() const
  {
  B_TRACE(B_Trace::lvl3, ("B_Array<T>::getSize() const"));
  return sizeof(nbrElements) + (nbrElements * sizeof(T)) + (nbrDims * sizeof(BoundPair)) + 
         sizeof(capacity) + sizeof(nbrDims) + sizeof(bounds);
  } // B_Array<T>::getSize() const

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T>::getValueSize() const
// Author: 
//     JD 16.07.96
// Dependencies:
// Impl. Remarks
// Revision(s):
//
template <class T, class ArrayContract>
B_Lng 
B_Array<T, ArrayContract>::getValueSize() const
  {
  B_TRACE(B_Trace::lvl3, ("B_Array<T>::getValueSize() const"));
  return sizeof(T) * nbrElements;
  } // B_Array<T>::getValueSize() const


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T>::dimensions() const
// Author: 
//     JD 16.07.96
// Dependencies:
// Impl. Remarks
// Revision(s):
//
template <class T, class ArrayContract>
B_Shrt
B_Array<T, ArrayContract>::dimensions() const
  {
  B_TRACE(B_Trace::lvl3, ("B_Array<T>::dimensions() const"));
  return nbrDims;
  } // B_Array<T>::dimensions() const

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T>::elements() const
// Author: 
//     JD 16.07.96
// Dependencies:
// Impl. Remarks
// Revision(s):
//
template <class T, class ArrayContract>
B_ULng
B_Array<T, ArrayContract>::elements() const
  {
  B_TRACE(B_Trace::lvl3, ("B_Array<T>::elements() const"));
  return nbrElements;
  } // B_Array<T>::elements() const

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T>::lBound(B_UChar dim) const
// Author: 
//     JD 16.07.96
// Dependencies:
// Impl. Remarks
//     Parameter vom Typ B_Integer damit Ueberlauffehler wie in VB
// Revision(s):
//

#ifdef B_VB_COMPATIBLE
template <class T, class ArrayContract>
B_Long
B_Array<T, ArrayContract>::lBound(B_Integer dim) const
  {
  B_TRACE(B_Trace::lvl3, ("B_Array<T>::lBound(B_Integer dim) const"));
  ArrayContract::dimRange( dim.getValue() , nbrDims);
  return B_Long(pBoundValues[dim.getValue() - 1].lower, B_explicit);
  } // B_Array<T>::lBound(B_Integer dim) const

template <class T, class ArrayContract>
B_Long
B_Array<T, ArrayContract>::lBound(B_Int dim) const
  {
  B_TRACE(B_Trace::lvl3, ("B_Array<T>::lBound(B_Int dim) const"));
  ArrayContract::dimRange( dim, nbrDims);
  return  B_Long(pBoundValues[dim - 1].lower, B_explicit);
  } // B_Array<T>::lBound(B_Int dim) const

#else // B_VB_COMPATIBLE

template <class T, class ArrayContract>
B_Lng
B_Array<T, ArrayContract>::lBound(B_Int dim) const
  {
  B_TRACE(B_Trace::lvl3, ("B_Array<T>::lBound(B_Int dim) const"));
  ArrayContract::dimRange( dim , nbrDims);
  return pBoundValues[dim - 1].lower;
  } // B_Array<T>::lBound(B_Int dim) const

#endif // B_VB_COMPATIBLE


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T>::uBound(B_UChar dim) const
// Author: 
//     JD 16.07.96
// Dependencies:
// Impl. Remarks
//     Parameter vom Typ B_Integer damit Ueberlauffehler wie in VB
// Revision(s):
//
#ifdef B_VB_COMPATIBLE

template <class T, class ArrayContract>
B_Long
B_Array<T, ArrayContract>::uBound(B_Integer dim) const
  {
  B_TRACE(B_Trace::lvl3, ("B_Array<T>::uBound(B_Integer dim) const"));
  ArrayContract::dimRange( dim.getValue() , nbrDims);
  return B_Long(pBoundValues[dim.getValue() - 1].upper, B_explicit);
  } // B_Array<T>::uBound(B_Integer dim) const

template <class T, class ArrayContract>
B_Long
B_Array<T, ArrayContract>::uBound(B_Int dim) const
  {
  B_TRACE(B_Trace::lvl3, ("B_Array<T>::uBound(B_Int dim) const"));
  ArrayContract::dimRange( dim , nbrDims);
  return B_Long(pBoundValues[dim - 1].upper, B_explicit);
  } // B_Array<T>::uBound(B_Int dim) const

#else // B_VB_COMPATIBLE

template <class T, class ArrayContract>
B_Lng
B_Array<T, ArrayContract>::uBound(B_Int dim) const
  {
  B_TRACE(B_Trace::lvl3, ("B_Array<T>::uBound(B_Int dim) const"));
  ArrayContract::dimRange( dim , nbrDims);
  return pBoundValues[dim - 1].upper;
  } // B_Array<T>::uBound(B_Int dim) const

#endif // B_VB_COMPATIBLE

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array<T>::erase()
// Author: 
//     CT 16.12.1996
// Dependencies:
// Impl. Remarks
// Revision(s):
// Note:	Ueberpruefen ob statisches oder dynamisches Array.
//			Bei dynamischen:	freigeben des Speichers
//			Bei statischen:		initialisieren alle Member
//

template <class T, class ArrayContract>
void
B_Array<T, ArrayContract>::erase(void)
{
	if(staticFlag == B_TRUE)			// statisches Array
	{
		B_ULng count;

		B_TRACE(B_Trace::lvl3, ("B_Array<T>::erase() : static Array"));

		for(count = 0; count < nbrElements; count++)
		{								// "NULL"-setzen aller Member
			initEl(pElements[count]);
		}
	}else{								// dynamisches Array
		B_TRACE(B_Trace::lvl3, ("B_Array<T>::erase() : dynamic Array"));

		if (pElements != NULL)			// internen Speicher freigeben
		{
			delete[] pElements; 
			pElements = NULL;
		}
		if (bounds.pBoundValues != NULL)
		{
			delete[] pBoundValues; 
			pBoundValues = NULL;
		}								// interne Varaiblen auf default setzen
		bounds.pBoundValues = pBoundValues;
		bounds.elementPos   = 0;
		bounds.aktDim       = 0;
		nbrDims             = 1;
		capacity			= 0;
		nbrElements			= 0;
	}
} // B_Array<T>::erase(B_Integer dim) const


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array::convertArray()
// Author: 
//     JD 19.07.96
// Dependencies:
// Impl. Remarks
// Revision(s):
//
template <class T, class ArrayContract>
void
B_Array<T, ArrayContract>::convertArray(B_Int varType, const VARIANT FAR& rhs)
  {
  B_TRACE(B_Trace::lvl1, ("B_Array::convertArray(B_Int varType, void* pElements, const VARIANT FAR& oleArr)"));
  B_BOOLEAN typFlag = B_FALSE;
  B_BOOLEAN variantFlag = B_FALSE;
  B_ULng el; // element-Counter als Abbruchkriterium der Arraykonvertierung
  VARIANT vaChange; // Zum konvertieren der Variantdatentypen in einen C-Datentyp wenn noetig
  HRESULT hresult;
  // Huge-Pointer fuer Zugriff auf Elemente eines Safearrays
  B_UChar  HUGEP* pByte;
  B_Shrt   HUGEP* pInt;
  B_Lng    HUGEP* pLong; 
  B_Flt    HUGEP* pFloat;
  B_Dbl    HUGEP* pDouble;
  BOOL     HUGEP* pBool;
  DATE     HUGEP* pDate;
  CURRENCY HUGEP* pCurr;
  VARIANT  HUGEP* pVar;
  BSTR     HUGEP* pStr;
 // ofstream cvtarr("cvtarr.dat", ios::out);

  bounds.aktDim = 0;
  nbrDims = (*rhs.pparray)->cDims; // oder (long)SafeArrayGetDim(*rhs.pparray);
  delete[] pBoundValues;
  pBoundValues = new  B_Bounds<T, ArrayContract>::BoundPair[nbrDims]; // exakte Groesse reicht bei Bounds  
  if (pBoundValues == 0)
    {
    throw B_OutOfMem();
    }
  bounds.pBoundValues = pBoundValues;
  bounds.pArr = this;
  nbrElements = 1L;
  B_Lng* pDimIdx = new B_Lng[nbrDims]; // Array zur Emulation der Spaltenindexe
  if (pDimIdx == 0)
    {
    throw B_OutOfMem();
    }


  //-- Check ob gueltiges Array und zugriff auf elemente
  //
  switch (rhs.vt)
    {
    case VT_ARRAY | VT_BYREF | VT_UI1:
      if (varType == VT_UI1)  // Test ob Typ des B_Array == Typ des Safearray
        {
        typFlag = B_TRUE;
        hresult = SafeArrayAccessData(*rhs.pparray, (void HUGEP* FAR*)&pByte);
        }
      break;
    case VT_ARRAY | VT_BYREF | VT_I2:
      if (varType == VT_I2) 
        {
        // cvtarr << " in case VT_ARRAY | VT_BYREF | VT_I2: vor SafeArrayAccessData" << endl; // TEST !!!
        typFlag = B_TRUE;
        hresult = SafeArrayAccessData(*rhs.pparray, (void HUGEP* FAR*)&pInt);
        }
      break;
    case VT_ARRAY | VT_BYREF | VT_I4:
      if (varType == VT_I4)
        {
        typFlag = B_TRUE;
        hresult = SafeArrayAccessData(*rhs.pparray, (void HUGEP* FAR*)&pLong);
        }
      break;
    case VT_ARRAY | VT_BYREF | VT_R4:
      if (varType == VT_R4) 
        {
        typFlag = B_TRUE;
        hresult = SafeArrayAccessData(*rhs.pparray, (void HUGEP* FAR*)&pFloat);
        }
      break;
    case VT_ARRAY | VT_BYREF | VT_R8:
      if (varType == VT_R8) 
        {
        typFlag = B_TRUE;
        hresult = SafeArrayAccessData(*rhs.pparray, (void HUGEP* FAR*)&pDouble);
        }
      break;
    case VT_ARRAY | VT_BYREF | VT_BOOL:
      if (varType == VT_BOOL) 
        {
        typFlag = B_TRUE;
        hresult = SafeArrayAccessData(*rhs.pparray, (void HUGEP* FAR*)&pBool);
        }
      break;
    case VT_ARRAY | VT_BYREF | VT_CY:
      if (varType == VT_CY)
        {
        typFlag = B_TRUE;
        hresult = SafeArrayAccessData(*rhs.pparray, (void HUGEP* FAR*)&pCurr);
        }
      break;
    case VT_ARRAY | VT_BYREF | VT_DATE:
      if (varType == VT_DATE) 
        {
        typFlag = B_TRUE;
        hresult = SafeArrayAccessData(*rhs.pparray, (void HUGEP* FAR*)&pDate);
        }
      break;
    case VT_ARRAY | VT_BYREF | VT_BSTR:
      if (varType == VT_BSTR) 
        {
        typFlag = B_TRUE;
        hresult = SafeArrayAccessData(*rhs.pparray, (void HUGEP* FAR*)&pStr);
        }
      break;
    // Variant-Array
    case VT_ARRAY | VT_BYREF | VT_VARIANT:
      typFlag = B_TRUE;
      variantFlag = B_TRUE; // kann Heterogen sein daher erst in Schleife auslesbar 
      hresult = SafeArrayAccessData(*rhs.pparray, (void HUGEP* FAR*)&pVar);
      break;
    default:
      typFlag = B_FALSE; // kein Array
    } // switch(rhs.vt)

    //-- Testauswertung ob Zugriff auf Safearray gueltig
    //
    if (typFlag == B_FALSE) // kein gueltiger Datentyp
      {
      delete[] pElements; // noetig da Konstruktor nocht nicht beendet -> kein Destruktor aufruf bei EH
      delete[] pDimIdx;
      pElements = NULL;
      #ifdef B_VB_CONTRACT
        throw B_InvalidArgument();
      #endif
      #ifdef B_CPP_CONTRACT
        B_REQUIRE_LVL(B_FALSE, B_Assure::lvl2);
      #endif
      } // if (typFlag == B_FALSE)
    if (typFlag == B_TRUE) // SafeArray zugriff testen
      {
      if (FAILED(hresult))
        {
        delete[] pElements; // noetig da Konstruktor nocht nicht beendet -> kein Destruktor aufruf bei EH
        delete[] pDimIdx;
        pElements = NULL;
        #ifdef B_VB_CONTRACT
          throw B_ArrayError();
        #endif
        #ifdef B_CPP_CONTRACT
          B_REQUIRE_LVL(B_FALSE, B_Assure::lvl2);
        #endif
        }
      } // if (typFlag == B_TRUE)


  //-- Boundsinformation in B_Bounds einlesen
  //
  // cvtarr << " typFlag==True && FAILED(hresult)==TRUE)" << endl; // TEST !!!
  for(B_Shrt cnt = 0; cnt < nbrDims; ++cnt)
    {
    hresult = SafeArrayGetLBound(*rhs.pparray, (cnt + 1), &(pBoundValues[cnt].lower) );
    if (FAILED(hresult))
      {
      delete[] pElements; // noetig da Konstruktor nocht nicht beendet -> kein Destruktor aufruf bei EH
      delete[] pDimIdx;
      #ifdef B_VB_CONTRACT
        throw B_SubscriptError();
      #endif
      #ifdef B_CPP_CONTRACT
        B_REQUIRE_LVL(B_FALSE, B_Assure::lvl2);
      #endif
      }
    pDimIdx[cnt] = pBoundValues[cnt].lower; // Indexarray mit Lowerbound initialisieren
    hresult = SafeArrayGetUBound(*rhs.pparray, (cnt + 1), &(pBoundValues[cnt].upper) );
    if (FAILED(hresult))
      {
      delete[] pElements; // noetig da Konstruktor nocht nicht beendet -> kein Destruktor aufruf bei EH
      delete[] pDimIdx;
      #ifdef B_VB_CONTRACT
        throw B_SubscriptError();
      #endif
      #ifdef B_CPP_CONTRACT
        B_REQUIRE_LVL(B_FALSE, B_Assure::lvl2);
      #endif
      }
    ArrayContract::boundRange(pBoundValues[cnt].lower, pBoundValues[cnt].upper);
    // cvtarr << "Lowerbound " << pBoundValues[cnt].lower << endl; // TEST !!!
    // cvtarr << "Upperbound " << pBoundValues[cnt].upper << endl; // TEST !!!
    nbrElements *= (pBoundValues[cnt].upper - pBoundValues[cnt].lower) + 1; // In VB muss Upperbound groesser als Lowerbound sein
    } // for ( cnt < nbrDims)

  // cvtarr << "Anz.El.: " << nbrElements << endl;
  //-- Speicherplatz fuer Elemente und initialisieren da ja Klassen und keine Standarddatentpyen
  //   somit kann der Aufruf des Konstruktors erforderlich sein um Speicheraufbau zu realisieren
  if ( nbrElements < ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T))) )
    {
    capacity =  ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)));
    pElements = new T[capacity];
    if (pElements == 0)
      {
      throw B_OutOfMem()
      }
    }
  else
    {
    capacity = ceil( nbrElements /  ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)))) * ceil(ARR_ALLOC_SIZE / static_cast<B_Dbl>(sizeof(T)));
    pElements = new T[capacity];
    if (pElements == 0)
      {
      throw B_OutOfMem()
      }
    }
  // initArray(); // Elemente mit ihrem defaultwert initialisieren, wie in VB ueblich erledigt new

  //-- Das Spaltenweise sortierte Array das in OLE-VARIANT steckt in Zeilenweise organisiertes
  //   B_Array transferieren und dann Elemente ihrer neuen Position in B_Array zuweisen.
  //   Trivialloesung mit For-Schleifen nicht moeglich da beliebige Dimensionen
  for (el = 0; el < nbrElements; el++)
    {
    //-- Die bounds-Klasse wird verwendet um Variablen und Speicherplatz zu sparen. In ihr werden
    //   die Berechnungen fuer das Safearray vorgenommen
    elementPos = 0L;
    for (bounds.aktDim = 0; bounds.aktDim < nbrDims; bounds.aktDim++)
      {
      if (bounds.aktDim < (nbrDims - 1))
        {
        bounds.elementPos += ( pDimIdx[bounds.aktDim]  - pBoundValues[bounds.aktDim].lower);
        bounds.elementPos *= pBoundValues[bounds.aktDim+1].range;
        }
      else if (bounds.aktDim == (nbrDims-1)) // letzte Dimension
        {
        bounds.elementPos += ( pDimIdx[bounds.aktDim] - pBoundValues[bounds.aktDim].lower + 1);
        }
      }

    --bounds.elementPos; // C-Array ist 1.element an Feldposition 0

    //-- Eintragen des Arrayelements von Safearray an seine position im B_Array, in abhaengigkeit 
    //   seines Typs. Test ob richtige Arraytypen und Element an richtige Position schreiben
    switch (rhs.vt)
      {
      case VT_ARRAY | VT_BYREF | VT_UI1:
        pElements[elementPos] = pByte[el];
        break;
      case VT_ARRAY | VT_BYREF | VT_I2:
        pElements[elementPos] = pInt[el];
        // cvtarr << "Integer: " << pInt[el] << " " << (int)pElements[elementPos] << "an pos:" << bounds.elementPos << endl; // TEST !!!
        break;
      case VT_ARRAY | VT_BYREF | VT_I4:
        pElements[elementPos] = pLong[el];
        break;
      case VT_ARRAY | VT_BYREF | VT_R4:
        pElements[elementPos] = pFloat[el];
        break;
      case VT_ARRAY | VT_BYREF | VT_R8:
        pElements[elementPos] = pDouble[el];
        break;
      case VT_ARRAY | VT_BYREF | VT_BOOL:
        pElements[elementPos] = pBool[el];
        break;
      case VT_ARRAY | VT_BYREF | VT_CY:
        // Currency erst in Double umwandeln
        VariantClear(&vaChange);
        VariantChangeType(&vaChange, &vaChange, 0, VT_CY);
        vaChange.cyVal.Hi = pCurr[el].Hi;        
        vaChange.cyVal.Lo = pCurr[el].Lo;
        VariantChangeType(&vaChange, &vaChange, 0, VT_R8);
        pElements[bounds.elementPos] = vaChange.dblVal;
        break;
      case VT_ARRAY | VT_BYREF | VT_DATE:
        // Date ist schon double
        pElements[bounds.elementPos] = pDate[el];
        break;
      case VT_ARRAY | VT_BYREF | VT_BSTR:
        // ACHTUNG testen wenn Arrayklasse fertig ob so geht
        // normal nochh konvertierung: 
        //    strcpy(szStr1, ConvertToAnsi((pStr[k])));
        //    cfout << "      Element(" << k << ") = " << szStr1 << endl;
        //    SysReAllocString(&pStr[k], ConvertToUnicode("reuckgabe string"));
        // pElements[bounds.elementPos].value = pStr[el];
        break;
      // Variant-Array
      case VT_ARRAY | VT_BYREF | VT_VARIANT:
        //-- kann Heterogen sein daher muss jedes Element einzelen der Typ abgefragt werden
        //   A`CHTUNG: nocht testn ob Variantklasse so konverieren kann
        // cvtarr << "VT_ARRAY | VT_BYREF | VT_VARIANT" << endl;    
        /*
        switch (pVar[el].vt)
          {
          case VT_UI1: // Byte
            pElements[bounds.elementPos] = pVar[el].bVal
            break;
          case VT_I2: // Short
            pElements[bounds.elementPos] = pVar[el].iVal;
            break;
          case VT_I4: // Long
            pElements[bounds.elementPos] = pVar[el].lVal;
            break;
          case VT_R4: // Float
            pElements[bounds.elementPos] = pVar[el].fltVal;
            break;
          case VT_R8: // Double
            pElements[bounds.elementPos] = pVar[el].dblVal;
            break;
          case VT_ERROR: //SCODE
            pElements[bounds.elementPos] = pVar[el].scode            
            // Hierbei ist -2146828287 = CVErr(1); -2146828278 = CVErr(10) usw.
            // nur CVErr(0) ist 0
            break;
          case VT_BOOL: //Boolean
            pVar[k].bool = 0;
            break;
          case VT_CY: // Currency
            VariantChangeType(rhs.pvarVal, rhs.pvarVal, 0, VT_R8);
            pElements[bounds.elementPos] = pVar[el].dblVal
            VariantChangeType(rhs.pvarVal, rhs.pvarVal, 0, VT_CY);
            break;
          case VT_DATE: // Datum und Zeit
            pElements[bounds.elementPos] = pVar[el].date;
            break;
          case VT_BSTR: // BSTR = Strings
            // strcpy(szStr1, ConvertToAnsi(pVar[k].bstrVal) );
            // cfout << "            para = " << szStr1 << endl;
            // SysReAllocString(&pVar[k].bstrVal, ConvertToUnicode("Jetzt is was aneres drin"));
            pElements[bounds.elementPos] = pVar[el].bstrVal;
            break;
          case VT_EMPTY: // Empty
            pElements[bounds.elementPos] = B_empty;
            break;
          case VT_NULL: // Null
            pElements[bounds.elementPos] = B_null;
            break;
          default:
            //cfout << "      kein bekannter Typ" << endl;
          } // switch(pVar[el].vt)
          */
        break;
      default:
        typFlag = B_FALSE; // kein Array
      } // switch(rhs.vt)
      //---- Ende Element in Array eintragen

    //-- Indexverlauf emulieren so dass Spaltenweise Array (so kommt es bei OLE) ausgelesen werden kann
    //
    for (bounds.aktDim = 0; bounds.aktDim < nbrDims; bounds.aktDim++)
      {
      if ( (pDimIdx[bounds.aktDim] == 0) || (pDimIdx[bounds.aktDim] % bounds.pBoundValues[bounds.aktDim].upper) != 0)
        {
        pDimIdx[bounds.aktDim]++ ;
        break;
        }
      else
        {
        pDimIdx[bounds.aktDim] = bounds.pBoundValues[bounds.aktDim].lower; // wieder zuruecksetzten
        }
      }
    
   } //  for (el = 0; el < nbrElements; el++)

    //-- Aufraeumarbeiten
    delete[] pDimIdx;
    hresult = SafeArrayUnaccessData(*rhs.pparray); // freigabe des gelockten Arrays

  } // B_Array::convertArray(B_Int varType, const VARIANT FAR& oleArr)

  
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Method:                                                                                (C) CiceroSoft 1996
//     B_Array::toOle()
// Author: 
//     JD 19.07.96
// Dependencies:
// Impl. Remarks
// Revision(s):
//
template <class T, class ArrayContract>
void
B_Array<T, ArrayContract>::toOle(B_Int varType, const VARIANT FAR& oleArr)
  {
  B_TRACE(B_Trace::lvl1, ("B_Array<T, ArrayContract>::toOle(B_Int varType, const VARIANT FAR& oleArr)"));
  B_BOOLEAN typFlag = B_FALSE;
  B_BOOLEAN variantFlag = B_FALSE;
  B_ULng el; // element-Counter als Abbruchkriterium der Arraykonvertierung
  B_Lng FAR aktBound;
  VARINAT vaChange; // Zum konvertieren der Variantdatentypen in einen C-Datentyp wenn noetig
  // Huge-Pointer fuer Zugriff auf Elemente eines Safearrays
  B_UChar  HUGEP* pByte;
  B_Shrt   HUGEP* pInt;
  B_Lng    HUGEP* pLong; 
  B_Flt    HUGEP* pFloat;
  B_Dbl    HUGEP* pDouble;
  BOOL     HUGEP* pBool;
  DATE     HUGEP* pDate;
  CURRENCY HUGEP* pCurr;
  VARIANT  HUGEP* pVar;
  BSTR     HUGEP* pStr;

  // Array muessen gleiche groesse == gleiche Dimesionsanzahl und gl. Boundwerte haben
  if (nbrDims != (*rhs.pparray)->cDims) // oder (long)SafeArrayGetDim(*rhs.pparray);
    {
    #ifdef B_VB_CONTRACT
      throw B_ArrayError();
    #endif
    #ifdef B_CPP_CONTRACT
      B_REQUIRE_LVL(B_FALSE, B_Assure::lvl2);
    #endif
    }

  B_Lng* pDimIdx = new B_Lng[nbrDims]; // Array zur Emulation der Spaltenindexe
  if (pDimIdx == 0)
    {
    throw B_OutOfMem()
    }

  //-- Check ob gueltiges Array und zugriff auf elemente
  //
  switch (rhs.vt)
    {
    case VT_ARRAY | VT_BYREF | VT_UI1:
      if (varType == VT_UI1)  // Test ob Typ des B_Array == Typ des Safearray
        {
        typFlag = B_TRUE;
        hresult = SafeArrayAccessData(*rhs.pparray, (void HUGEP* FAR*)&pByte);
        }
      break;
    case VT_ARRAY | VT_BYREF | VT_I2:
      if (varType == VT_I2) 
        {
        typFlag = B_TRUE;
        hresult = SafeArrayAccessData(*rhs.pparray, (void HUGEP* FAR*)&pInt);
        }
      break;
    case VT_ARRAY | VT_BYREF | VT_I4:
      if (varType == VT_I4)
        {
        typFlag = B_TRUE;
        hresult = SafeArrayAccessData(*rhs.pparray, (void HUGEP* FAR*)&pLong);
        }
      break;
    case VT_ARRAY | VT_BYREF | VT_R4:
      if (varType == VT_R4) 
        {
        typFlag = B_TRUE;
        hresult = SafeArrayAccessData(*rhs.pparray, (void HUGEP* FAR*)&pFloat);
        }
      break;
    case VT_ARRAY | VT_BYREF | VT_R8:
      if (varType == VT_R8) 
        {
        typFlag = B_TRUE;
        hresult = SafeArrayAccessData(*rhs.pparray, (void HUGEP* FAR*)&pDouble);
        }
      break;
    case VT_ARRAY | VT_BYREF | VT_BOOL:
      if (varType == VT_BOOL) 
        {
        typFlag = B_TRUE;
        hresult = SafeArrayAccessData(*rhs.pparray, (void HUGEP* FAR*)&pBool);
        }
      break;
    case VT_ARRAY | VT_BYREF | VT_CY:
      if (varType == VT_CY)
        {
        typFlag = B_TRUE;
        hresult = SafeArrayAccessData(*rhs.pparray, (void HUGEP* FAR*)&pCurr);
        }
      break;
    case VT_ARRAY | VT_BYREF | VT_DATE:
      if (varType == VT_DATE) 
        {
        typFlag = B_TRUE;
        hresult = SafeArrayAccessData(*rhs.pparray, (void HUGEP* FAR*)&pDate);
        }
      break;
    case VT_ARRAY | VT_BYREF | VT_BSTR:
      if (varType == VT_BSTR) 
        {
        typFlag = B_TRUE;
        hresult = SafeArrayAccessData(*rhs.pparray, (void HUGEP* FAR*)&pStr);
        }
      break;
    // Variant-Array
    case VT_ARRAY | VT_BYREF | VT_VARIANT:
      typFlag = B_TRUE
      variantFlag = B_TRUE; // kann Heterogen sein daher erst in Schleife auslesbar 
      hresult = SafeArrayAccessData(*rhs.pparray, (void HUGEP* FAR*)&pVar);
      break;
    default:
      typFlag = B_FALSE; // kein Array
    } // switch(rhs.vt)

    //-- Testauswertung ob Zugriff auf Safearray gueltig
    //
    if (typFlag == B_FALSE) // kein gueltiger Datentyp
      {
      delete[] pDimIdx;
      #ifdef B_VB_CONTRACT
        throw B_InvalidArgument();
      #endif
      #ifdef B_CPP_CONTRACT
        B_REQUIRE_LVL(B_FALSE, B_Assure::lvl2);
      #endif
      } // if (typFlag == B_FALSE)
    if (typFlag == B_TRUE) // SafeArray zugriff testen
      {
      if (FAILED(hresult))
        {
        delete[] pDimIdx;
        #ifdef B_VB_CONTRACT
          throw B_ArrayError();
        #endif
        #ifdef B_CPP_CONTRACT
          B_REQUIRE_LVL(B_FALSE, B_Assure::lvl2);
        #endif
        }
      } // if (typFlag == B_TRUE)


  //-- Boundsinformation auf gleichheit Testen
  //
  for(B_Shrt cnt = 0; cnt < nbrDims; ++cnt)
    {
    hresult = SafeArrayGetLBound(*rhs.pparray, (cnt + 1), &aktBound);
    if (FAILED(hresult) || (aktBound != pBoundValues[cnt].lower) )
      {
      delete[] pDimIdx;
      #ifdef B_VB_CONTRACT
        throw B_SubcriptError();
      #endif
      #ifdef B_CPP_CONTRACT
        B_REQUIRE_LVL(B_FALSE, B_Assure::lvl2);
      #endif
      }
    pDimIdx[cnt] = pBoundValues[cnt].lower; // Indexarray mit Lowerbound initialisieren
    hresult = SafeArrayGetUBound(*rhs.pparray, (cnt + 1), &aktBound);
    if (FAILED(hresult) || (aktBound != pBoundValues[cnt].upper) )
      {
      delete[] pDimIdx;
      #ifdef B_VB_CONTRACT
        throw B_SubcriptError();
      #endif
      #ifdef B_CPP_CONTRACT
        B_REQUIRE_LVL(B_FALSE, B_Assure::lvl2);
      #endif
      }
    ArrayContract::boundRange(pBoundValues[cnt].lower, pBoundValues[cnt].upper);
    nbrElements *= pBoundValues[cnt].range; // In VB muss Upperbound groesser als Lowerbound sein
    } // for ( cnt < nbrDims)

  //-- Das Spaltenweise sortierte Array das in OLE-VARIANT steckt in Zeilenweise organisiertes
  //   B_Array transferieren und dann Elemente ihrer neuen Position in B_Array zuweisen.
  //   Trivialloesung mit For-Schleifen nicht moeglich da beliebige Dimensionen
  for (el = 0; el < nbrElements; el++)
    {
    //-- Die bounds-Klasse wird verwendet um Variablen und Speicherplatz zu sparen. In ihr werden
    //   die Berechnungen fuer das Safearray vorgenommen
    bounds.elementPos = 0L;
    for (bounds.aktDim = 0; bounds.aktDim < nbrDims; bounds.aktDim++)
      {
      if (bounds.aktDim < (nbrDims-1))
        {
        bounds.elementPos += pDimIdx[bounds.aktDim]  - pBoundValues[bounds.aktDim].lower;
        bounds.elementPos *= pBoundValues[bounds.aktDim+1].range;
        }
      else if (bounds.aktDim == (nbrDims-1)) // letzte Dimension
        {
        bounds.elementPos += pDimIdx[bounds.aktDim] - pBoundValues[bounds.aktDim].lower + 1;
        }
      }

    --bounds.elementPos; // C-Array ist 1.element an Feldposition 0

    //-- Eintragen des Arrayelements von Safearray an seine position im B_Array, in abhaengigkeit 
    //   seines Typs. Test ob richtige Arraytypen und Element an richtige Position schreiben
    switch (rhs.vt)
      {
      case VT_ARRAY | VT_BYREF | VT_UI1:
        pByte[el] = pElements[elementPos].value;
        break;
      case VT_ARRAY | VT_BYREF | VT_I2:
        pInt[el] = pElements[elementPos].value;
        break;
      case VT_ARRAY | VT_BYREF | VT_I4:
        pLong[el] = pElements[elementPos].value;
        break;
      case VT_ARRAY | VT_BYREF | VT_R4:
        pFloat[el] = pElements[elementPos].value;
        break;
      case VT_ARRAY | VT_BYREF | VT_R8:
        pDouble[el] = pElements[elementPos].value;
        break;
      case VT_ARRAY | VT_BYREF | VT_BOOL:
        pBool[el] = pElements[elementPos].value;
        break;
      case VT_ARRAY | VT_BYREF | VT_CY:
        // Currency erst in Double umwandeln
        VariantClear(&vaChange);
        VariantChangeType(&vaChange, &vaChange, 0, VT_R8);
        vaChange.dblVal = pElements[elementPos].value;
        VariantChangeType(&vaChange, &vaChange, 0, VT_CY);
        pCurr[el].Hi = vaChange.cyVal.Hi;        
        pCurr[el].Lo = vaChange.cyVal.Lo;
        break;
      case VT_ARRAY | VT_BYREF | VT_DATE:
        // Date ist schon double
        pDate[el] = pElements[elementPos].value;
        break;
      case VT_ARRAY | VT_BYREF | VT_BSTR:
        // ACHTUNG testen wenn Arrayklasse fertig ob so geht
        // normal nochh konvertierung: 
        //    strcpy(szStr1, ConvertToAnsi((pStr[k])));
        //    cfout << "      Element(" << k << ") = " << szStr1 << endl;
        //    SysReAllocString(&pStr[k], ConvertToUnicode("reuckgabe string"));
        // pElements[bounds.elementPos].value = pStr[el];
        break;
      // Variant-Array
      case VT_ARRAY | VT_BYREF | VT_VARIANT:
        //-- kann Heterogen sein daher muss jedes Element einzelen der Typ abgefragt werden
        //   A`CHTUNG: nocht testn ob Variantklasse so konverieren kann
        /*
        switch (pVar[el].vt)
          {
          case VT_UI1: // Byte
            pElements[bounds.elementPos] = pVar[el].bVal
            break;
          case VT_I2: // Short
            pElements[bounds.elementPos] = pVar[el].iVal;
            break;
          case VT_I4: // Long
            pElements[bounds.elementPos] = pVar[el].lVal;
            break;
          case VT_R4: // Float
            pElements[bounds.elementPos] = pVar[el].fltVal;
            break;
          case VT_R8: // Double
            pElements[bounds.elementPos] = pVar[el].dblVal;
            break;
          case VT_ERROR: //SCODE
            pElements[bounds.elementPos] = pVar[el].scode            
            // Hierbei ist -2146828287 = CVErr(1); -2146828278 = CVErr(10) usw.
            // nur CVErr(0) ist 0
            break;
          case VT_BOOL: //Boolean
            pVar[k].bool = 0;
            break;
          case VT_CY: // Currency
            VariantChangeType(rhs.pvarVal, rhs.pvarVal, 0, VT_R8);
            pElements[bounds.elementPos] = pVar[el].dblVal
            VariantChangeType(rhs.pvarVal, rhs.pvarVal, 0, VT_CY);
            break;
          case VT_DATE: // Datum und Zeit
            pElements[bounds.elementPos] = pVar[el].date;
            break;
          case VT_BSTR: // BSTR = Strings
            // strcpy(szStr1, ConvertToAnsi(pVar[k].bstrVal) );
            // cfout << "            para = " << szStr1 << endl;
            // SysReAllocString(&pVar[k].bstrVal, ConvertToUnicode("Jetzt is was aneres drin"));
            pElements[bounds.elementPos] = pVar[el].bstrVal;
            break;
          case VT_EMPTY: // Empty
            pElements[bounds.elementPos] = B_empty;
            break;
          case VT_NULL: // Null
            pElements[bounds.elementPos] = B_null;
            break;
          default:
            //cfout << "      kein bekannter Typ" << endl;
          } // switch(pVar[el].vt)
          */
        break;
      default:
        typFlag = B_FALSE; // kein Array
      } // switch(rhs.vt)
    //---- Ende Element in Array eintragen

    //-- Indexverlauf emulieren so dass Spaltenweise Array (so kommt es bei OLE) ausgelesen werden kann
    //
    for (bounds.aktDim = 0; bounds.aktDim < nbrDims; bounds.aktDim++)
      {
      if ( (pDimIdx[bounds.aktDim] == 0) || (pDimIdx[bounds.aktDim] % pBoundValues[bounds.aktDim].upper) != 0)
        {
        ++pDimIdx[bounds.aktDim];
        break;
        }
      else
        {
        pDimIdx[bounds.aktDim] = pBoundValues[bounds.aktDim].lower; // wieder zuruecksetzten
        }
      }
    } //  for (el = 0; el < nbrElements; el++)

    //-- Aufraeumarbeiten
    delete[] pDimIdx;
    hresult = SafeArrayUnaccessData(*rhs.pparray); // freigabe des gelockten Arrays

  } // B_Array::toOle(B_Int varType, const VARIANT FAR& oleArr)

//===========================================================================================================
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif // BI_ARRAY_HPP