/*************************************************************
* Projekt:	VBNC				
* Modul:	BC_TARR.CPP	Klasse	Array in Typen
* Autor:	Claus Tucher		CiceroSoft GmbH
* Datum:	26.11.1996
**/

#ifndef BI_TARR_HPP
# define BI_TARR_HPP

/*************************************************************
* I N C L U D E
*/

#ifndef BD_INTYP_HPP
  #include "bd_intyp.hpp"       // alias der Standardatentypen
#endif

#ifndef BC_DBG_HPP
  #include "bc_dbg.hpp"
#endif


/*************************************************************
* D E F I N E
*/


/*************************************************************
* F U N K T I O N E N
* M O D U L S C H N I T T S T E L L E
*/


/*************************************************************
* F U N K T I O N E N
* M O D U L S C H N I T T S T E L L E
* C O D E
*/
/*************************************************************
* C L A S S E S
* M E T H O D E N
*/
// Dimention 1 -----------------------------------------------
// 1D [] als gA() Funktion
// Bemerkung: "B_Lng dims" ist nur zur Kompatibilitt mit der
//			  B_Array-Klasseeingefuegt worden
template <class T, int low1D, int up1D>
#if 1	//ab.. 4.4.97 Warningfreiheit
T& B_T1DArray<T, low1D, up1D>::gA(B_Lng, B_Lng index)
#else
T& B_T1DArray<T, low1D, up1D>::gA(B_Lng dims, B_Lng index)
#endif
{
	B_TRACE(B_Trace::lvl1, ("B_T1DArray<T, low1D, up1D>::gA(B_Lng dims, B_Lng index)"));

	if(checkBoundery(index) == B_FALSE)
	{
	#ifdef B_VB_CONTRACT
		throw B_SubscriptError();
	#endif
	#ifdef B_CPP_CONTRACT
		B_REQUIRE_LVL(checkBoundery(index) == B_TRUE, B_Assure::lvl2);
		return a1D[0];
	#endif
	}

	return a1D[index - low1D];
}

// 1D = Operator
template <class T, int low1D, int up1D>
B_T1DArray<T, low1D, up1D>& B_T1DArray<T, low1D, up1D>::operator= (B_T1DArray& rhs )
{
	B_TRACE(B_Trace::lvl1, ("B_T1DArray<T, low1D, up1D>::operator= (B_T1DArray& rhs )"));

	if(this == &rhs)
    {
		return *this;
    }

	memcpy(a1D, rhs.a1D, sizeof(a1D));

	return *this;
}

// 1D [] Operator
template <class T, int low1D, int up1D>
T& B_T1DArray<T, low1D, up1D>::operator[] (B_Lng index)
{
	B_TRACE(B_Trace::lvl1, ("B_T1DArray<T, low1D, up1D>::operator[] (B_Lng index)"));

	if(checkBoundery(index) == B_FALSE)
	{
	#ifdef B_VB_CONTRACT
		throw B_SubscriptError();
	#endif
	#ifdef B_CPP_CONTRACT
		B_REQUIRE_LVL(checkBoundery(index) == B_TRUE, B_Assure::lvl2);
		return a1D[0];
	#endif
	}

	return a1D[index - low1D];
}

// 1D [] const Operator
template <class T, int low1D, int up1D>
const T& B_T1DArray<T, low1D, up1D>::operator[] (B_Lng index) const
{
	B_TRACE(B_Trace::lvl1, ("B_T1DArray<T, low1D, up1D>::operator[] (B_Lng index)"));

	if(checkBoundery(index) == B_FALSE)
	{
	#ifdef B_VB_CONTRACT
		throw B_SubscriptError();
	#endif
	#ifdef B_CPP_CONTRACT
		B_REQUIRE_LVL(checkBoundery(index) == B_TRUE, B_Assure::lvl2);
		return a1D[0];
	#endif
	}

	return a1D[index - low1D];
}


// Check Boundery
template <class T, int low1D, int up1D>
B_Shrt B_T1DArray<T, low1D, up1D>::checkBoundery(B_Lng index)
{
	B_TRACE(B_Trace::lvl1, ("int B_T1DArray<T, low1D, up1D>::checkBoundery(B_Lng index)"));

	if(index < low1D || index > up1D)
		return B_FALSE;
	else
		return B_TRUE;
}

// Dimention 2 -----------------------------------------------

// 2D [] als gA() Funktion
// Bemerkung: "B_Lng dims" ist nur zur Kompatibilitt mit der
//			  B_Array-Klasseeingefuegt worden
template <class T, int low1D, int up1D, int low2D, int up2D>
T& B_T2DArray<T, low1D, up1D, low2D, up2D>::gA(B_Lng dims, B_Lng index, B_Lng index2D)
{
	B_TRACE(B_Trace::lvl1, ("B_T2DArray<T, low1D, up1D, low2D, up2D>::gA(B_Lng dims, B_Lng index, B_Lng index2D)"));

	if(checkBoundery(index, index2D) == B_FALSE)
	{
	#ifdef B_VB_CONTRACT
		throw B_SubscriptError();
	#endif
	#ifdef B_CPP_CONTRACT
		B_REQUIRE_LVL(checkBoundery(index, index2D) == B_TRUE, B_Assure::lvl2);
	#endif
	}

	return &a2D[index - low1D][index2D - low2D];
}

// 2D = Operator
template <class T, int low1D, int up1D, int low2D, int up2D>
B_T2DArray<T, low1D, up1D, low2D, up2D>& B_T2DArray<T, low1D, up1D, low2D, up2D>::operator= (B_T2DArray& rhs )
{
	B_TRACE(B_Trace::lvl1, ("B_T2DArray<T, low1D, up1D, low2D, up2D>::operator= (B_T2DArray& rhs )"));

	if(this == &rhs)
    {
		return *this;
    }

	memcpy(a2D, rhs.a2D, sizeof(a2D));

	return *this;
}
#if 0	//ab.. 14.5.97 wird ueber gA erledigt
// 2D [] Operator
template <class T, int low1D, int up1D, int low2D, int up2D>
T& B_T2DArray<T, low1D, up1D, low2D, up2D>::operator[] (B_Lng index, B_Lng index2D)
{
	B_TRACE(B_Trace::lvl1, ("B_T2DArray<T, low1D, up1D, low2D, up2D>::operator[] (B_Lng index, B_Lng index2D)"));

	if(checkBoundery(index, index2D) == B_FALSE)
	{
	#ifdef B_VB_CONTRACT
		throw B_SubscriptError();
	#endif
	#ifdef B_CPP_CONTRACT
		B_REQUIRE_LVL(checkBoundery(index, index2D) == B_TRUE, B_Assure::lvl2);
	#endif
	}

	return &a2D[index - low1D][index2D - low2D];
}

// 2D [] const Operator
template <class T, int low1D, int up1D, int low2D, int up2D>
const T& B_T2DArray<T, low1D, up1D, low2D, up2D>::operator[] (B_Lng index, B_Lng index2D) const
{
	B_TRACE(B_Trace::lvl1, ("B_T2DArray<T, low1D, up1D, low2D, up2D>::operator[] (B_Lng index, B_Lng index2D)"));

	if(checkBoundery(index, index2D) == B_FALSE)
	{
	#ifdef B_VB_CONTRACT
		throw B_SubscriptError();
	#endif
	#ifdef B_CPP_CONTRACT
		B_REQUIRE_LVL(checkBoundery(index, index2D) == B_TRUE, B_Assure::lvl2);
	#endif
	}

	return &a2D[index - low1D][index2D - low2D];
}
#endif
// Check Boundery
template <class T, int low1D, int up1D, int low2D, int up2D>
B_Shrt B_T2DArray<T, low1D, up1D, low2D, up2D>::checkBoundery(B_Lng index, B_Lng index2D)
{
	B_TRACE(B_Trace::lvl1, ("B_T2DArray<T, low1D, up1D, low2D, up2D>::checkBoundery(B_Lng index, B_Lng index2D)"));

	if((index   < low1D || index   > up1D) || \
	   (index2D < low2D || index2D > up2D))
		return B_FALSE;
	else
		return B_TRUE;
}

// Dimention 3 -----------------------------------------------

// 3D [] als gA() Funktion
// Bemerkung: "B_Lng dims" ist nur zur Kompatibilitt mit der
//			  B_Array-Klasseeingefuegt worden
template <class T, int low1D, int up1D, int low2D, int up2D, int low3D, int up3D>
#if 1   //ab.. 21.7.97 Compilerfehler bei 5.0
T& B_T3DArray<T, low1D, up1D, low2D, up2D, low3D, up3D>::gA(B_Lng dims, B_Lng index, B_Lng index2D, B_Lng index3D)
#else
T& B_T3DArray<T, low1D, up1D>::gA(B_Lng dims, B_Lng index, B_Lng index2D, B_Lng index3D)
#endif
{
	B_TRACE(B_Trace::lvl1, ("B_T3DArray<T, low1D, up1D>::gA(B_Lng dims, B_Lng index, B_Lng index2D, B_Lng index3D)"));
	
	if(checkBoundery(index, index2D, index3D) == B_FALSE)
	{
	#ifdef B_VB_CONTRACT
		throw B_SubscriptError();
	#endif
	#ifdef B_CPP_CONTRACT
		B_REQUIRE_LVL(checkBoundery(index, index2D, index3D) == B_TRUE, B_Assure::lvl2);
		return a1D[0][0][0];
	#endif
	}

	return &a2D[index - low1D][index2D - low2D][index3D - low3D];
}

// 3D = Operator
template <class T, int low1D, int up1D, int low2D, int up2D, int low3D, int up3D>
#if 1   
B_T3DArray<T, low1D, up1D, low2D, up2D, low3D, up3D>& 
B_T3DArray<T, low1D, up1D, low2D, up2D, low3D, up3D>::operator= (B_T3DArray& rhs )
#else
B_T3DArray<T, low1D, up1D>& B_T3DArray<T, low1D, up1D>::operator= (B_T3DArray& rhs )
#endif
{
	B_TRACE(B_Trace::lvl1, ("B_T3DArray<T, low1D, up1D>::operator= (B_T3DArray& rhs )"));

	if(this == &rhs)
    {
		return *this;
    }

	memcpy(a3D, rhs.a3D, sizeof(a3D));

	return *this;
}
#if 0	//ab.. 14.5.97 wird ueber gA erledigt
// 3D [] Operator
template <class T, int low1D, int up1D, int low2D, int up2D, int low3D, int up3D>
T& B_T3DArray<T, low1D, up1D>::operator[] (B_Lng index, B_Lng index2D, B_Lng index3D)
{
	B_TRACE(B_Trace::lvl1, ("B_T3DArray<T, low1D, up1D>::operator[] (B_Lng index, B_Lng index2D, B_Lng index3D)"));
	
	if(checkBoundery(index, index2D, index3D) == B_FALSE)
	{
	#ifdef B_VB_CONTRACT
		throw B_SubscriptError();
	#endif
	#ifdef B_CPP_CONTRACT
		B_REQUIRE_LVL(checkBoundery(index, index2D, index3D) == B_TRUE, B_Assure::lvl2);
		return a1D[0][0][0];
	#endif
	}

	return &a2D[index - low1D][index2D - low2D][index3D - low3D];
}

// 3D [] const Operator
template <class T, int low1D, int up1D, int low2D, int up2D, int low3D, int up3D>
#if 1   
const T& B_T3DArray<T, low1D, up1D, low2D, up2D, low3D, up3D>::operator[] (B_Lng index, B_Lng index2D, B_Lng index3D) const
#else
const T& B_T3DArray<T, low1D, up1D>::operator[] (B_Lng index, B_Lng index2D, B_Lng index3D) const
#endif
{
	B_TRACE(B_Trace::lvl1, ("B_T3DArray<T, low1D, up1D>::operator[] (B_Lng index, B_Lng index2D, B_Lng index3D)"));
	
	if(checkBoundery(index, index2D, index3D) == B_FALSE)
	{
	#ifdef B_VB_CONTRACT
		throw B_SubscriptError();
	#endif
	#ifdef B_CPP_CONTRACT
		B_REQUIRE_LVL(checkBoundery(index, index2D, index3D) == B_TRUE, B_Assure::lvl2);
		return a1D[0][0][0];
	#endif
	}

	return &a2D[index - low1D][index2D - low2D][index3D - low3D];
}
#endif
// Check Boundery
template <class T, int low1D, int up1D, int low2D, int up2D, int low3D, int up3D>
#if 1   
B_Shrt B_T3DArray<T, low1D, up1D, low2D, up2D, low3D, up3D>::checkBoundery(B_Lng index, B_Lng index2D, B_Lng index3D)
#else
B_Shrt B_T3DArray<T, low1D, up1D>::checkBoundery(B_Lng index, B_Lng index2D, B_Lng index3D)
#endif
{
	B_TRACE(B_Trace::lvl1, ("B_T3DArray<T, low1D, up1D>::checkBoundery(B_Lng index, B_Lng index2D, B_Lng index3D)"));
	
	if((index   < low1D || index   > up1D) || \
	   (index2D < low2D || index2D > up2D) || \
	   (index3D < low3D || index3D > up3D))
		return B_FALSE;
	else
		return B_TRUE;
}

#endif	//BI_TARR_HPP
/* E O F */