/*************************************************************
* Projekt:	VBNC				
* Modul:	BC_TSTR.CPP	Methoden, Operatoren	Strings in Typen
* Autor:	Claus Tucher		CiceroSoft GmbH
* Datum:	26.11.1996
**/
#ifndef BI_TSTR_HPP
# define BI_TSTR_HPP

/*************************************************************
* I N C L U D E
*/

#ifndef BD_INTYP_HPP
  #include "bd_intyp.hpp"       // alias der Standardatentypen
#endif

#ifndef BC_DBG_HPP
  #include "bc_dbg.hpp"
#endif




/*************************************************************
* F U N K T I O N E N
* M O D U L S C H N I T T S T E L L E
*/

/*
* Konstruktoren,
* De-, 
* Operatoren
* M E T H O D E N
*/

//**************** Konstruktor ********************************************
template <int groesse>
B_TString<groesse>::B_TString()
{
	B_TRACE(B_Trace::lvl1, ("B_TString<groesse>::B_TString()"));

	memset(value, 0x00, groesse);
}
template <int groesse>
B_TString<groesse>::B_TString(B_cCStr s)
{
	B_TRACE(B_Trace::lvl1, ("B_TString<groesse>::B_TString(B_cCStr s)"));
 
	if (checkOverflow(s) == B_FALSE)
	{
		B_Int	sLen = strlen(s); 

		memcpy(value,		  s,    sLen);
		memset(value + sLen, 0x20, groesse - sLen);
	}
	else
		memcpy(value, s, groesse);
}
template <int groesse>
B_TString<groesse>::B_TString(const B_TStringStruct& s)
{
	B_TRACE(B_Trace::lvl1, ("B_TString<groesse>::B_TString(const B_TStringStruct& s)"));
 
	if (checkOverflow(s) == B_FALSE)
	{
		memcpy(value,			   s.value, s.groesse);
		memset(value + s.groesse, 0x20,    groesse - s.groesse);
	}
	else
		memcpy(value, s.value, groesse);
}


//**************** Ueberlauf **********************************************
template <int groesse>
B_Shrt B_TString<groesse>::checkOverflow(const B_String& s)
{
	B_TRACE(B_Trace::lvl1, ("B_TString<groesse>::checkOverflow(const B_String& s)"));

	if(s.nbrChar > groesse)
		return B_TRUE;
	else
		return B_FALSE;
}
template <int groesse>
B_Shrt B_TString<groesse>::checkOverflow(B_cCStr s)
{
	B_TRACE(B_Trace::lvl1, ("B_TString<groesse>::checkOverflow(B_cCStr s)"));

	if(strlen(s) > groesse)
		return B_TRUE;
	else
		return B_FALSE;
}
template <int groesse>
B_Shrt B_TString<groesse>::checkOverflow(const B_TStringStruct& s)
{
	B_TRACE(B_Trace::lvl1, ("B_TString<groesse>::checkOverflow(const B_TStringStruct& s)"));

	if(s.groesse > groesse)
		return B_TRUE;
	else
		return B_FALSE;
}


//**************** LEN(String) ********************************************
template <int groesse>
B_Lng B_TString<groesse>::len() const
{
	B_TRACE(B_Trace::lvl1, ("B_TString<groesse>::len() const"));
	return groesse;
}


//**************** + Operatoren *******************************************
// CT060197	alle Verkettungen haben einen B_String als Ergebnis
//			verlagert nach (BC/BI)_STR.HPP


//**************** == Operatoren ******************************************
// CT100197 alle Vergleichsoperatoren verwenden intern die 
//			B_String-Methode "compare()" und sind deshalb nach
//			(BC/BI)_STR.HPP verlagert


//**************** = Operatoren *******************************************
template <int groesse>
B_TString<groesse>& B_TString<groesse>::operator=(const B_String& rhs)
{
	B_TRACE(B_Trace::lvl1, ("B_TString<groesse>::operator=(const B_String& rhs)"));

	if (checkOverflow(rhs) == B_FALSE)
	{
		memcpy(value,				 rhs.value, rhs.nbrChar);
		memset(value + rhs.nbrChar, 0x20,		groesse - rhs.nbrChar);
	}
	else
		memcpy(value, rhs.value, groesse);
	return *this;
}
template <int groesse>
B_TString<groesse>& B_TString<groesse>::operator=(B_cCStr rhs)
{
	B_TRACE(B_Trace::lvl1, ("B_TString<groesse>::operator=(B_cCStr rhs)"));

	if (checkOverflow(rhs) == B_FALSE)
	{
		B_Int	sLen = strlen(rhs); 

		memcpy(value,		  rhs,  sLen);
		memset(value + sLen, 0x20, groesse - sLen);
	}
	else
		memcpy(value, rhs,  groesse);
	return *this;
}
template <int groesse>
B_TString<groesse>& B_TString<groesse>::operator=(const B_TStringStruct rhs)
{
	B_TRACE(B_Trace::lvl1, ("B_TString<groesse>::operator=(const B_TStringStruct rhs)"));

	if (checkOverflow(rhs) == B_FALSE)
	{
		memcpy(value,				 rhs.value, rhs.groesse);
		memset(value + rhs.groesse, 0x20,		groesse - rhs.groesse);
	}
	else
		memcpy(value, rhs.value, groesse);
	return *this;
}


//********************* Konvertierungsoperatoren ***********************
template <int groesse>
B_TString<groesse>::operator B_TStringStruct() const
{
	B_TRACE(B_Trace::lvl1, ("B_TString<groesse>::operator B_TStringStruct() const"));
	return B_TStringStruct((B_Lng) groesse, (B_CStr)value);
}


#endif //BI_TSTR_HPP
/* E O F */